//===- TableGen'erated file -------------------------------------*- C++ -*-===//
//
// DAG Instruction Selector for the X86 target
//
// Automatically generated file, do not edit!
//
//===----------------------------------------------------------------------===//

// *** NOTE: This file is #included into the middle of the target
// *** instruction selector class.  These functions are really methods.

// Instruction selector priority queue:
std::vector<SDNode*> ISelQueue;
/// Keep track of nodes which have already been added to queue.
unsigned char *ISelQueued;
/// Keep track of nodes which have already been selected.
unsigned char *ISelSelected;
/// IsChainCompatible - Returns true if Chain is Op or Chain does
/// not reach Op.
static bool IsChainCompatible(SDNode *Chain, SDNode *Op) {
  if (Chain->getOpcode() == ISD::EntryToken)
    return true;
  else if (Chain->getOpcode() == ISD::TokenFactor)
    return false;
  else if (Chain->getNumOperands() > 0) {
    SDOperand C0 = Chain->getOperand(0);
    if (C0.getValueType() == MVT::Other)
      return C0.Val != Op && IsChainCompatible(C0.Val, Op);
  }
  return true;
}
/// Sorting functions for the selection queue.
struct isel_sort : public std::binary_function<SDNode*, SDNode*, bool> {
  bool operator()(const SDNode* left, const SDNode* right) const {
    return (left->getNodeId() > right->getNodeId());
  }
};

inline void setQueued(int Id) {
  ISelQueued[Id / 8] |= 1 << (Id % 8);
}
inline bool isQueued(int Id) {
  return ISelQueued[Id / 8] & (1 << (Id % 8));
}
inline void setSelected(int Id) {
  ISelSelected[Id / 8] |= 1 << (Id % 8);
}
inline bool isSelected(int Id) {
  return ISelSelected[Id / 8] & (1 << (Id % 8));
}

void AddToISelQueue(SDOperand N) DISABLE_INLINE {
  int Id = N.Val->getNodeId();
  if (Id != -1 && !isQueued(Id)) {
    ISelQueue.push_back(N.Val);
    std::push_heap(ISelQueue.begin(), ISelQueue.end(), isel_sort());
    setQueued(Id);
  }
}

class VISIBILITY_HIDDEN ISelQueueUpdater :
  public SelectionDAG::DAGUpdateListener {
    std::vector<SDNode*> &ISelQueue;
    bool HadDelete;
  public:
    explicit ISelQueueUpdater(std::vector<SDNode*> &isq)
      : ISelQueue(isq), HadDelete(false) {}
    
    bool hadDelete() const { return HadDelete; }
    
    virtual void NodeDeleted(SDNode *N) {
      ISelQueue.erase(std::remove(ISelQueue.begin(), ISelQueue.end(), N),
                      ISelQueue.end());
      HadDelete = true;
    }
    
    // Ignore updates.
    virtual void NodeUpdated(SDNode *N) {}
  };
inline void UpdateQueue(const ISelQueueUpdater &ISQU) {
  if (ISQU.hadDelete())
    std::make_heap(ISelQueue.begin(), ISelQueue.end(),isel_sort());
}

void ReplaceUses(SDOperand F, SDOperand T) DISABLE_INLINE {
  ISelQueueUpdater ISQU(ISelQueue);
  CurDAG->ReplaceAllUsesOfValueWith(F, T, &ISQU);
  setSelected(F.Val->getNodeId());
  UpdateQueue(ISQU);
}
void ReplaceUses(SDNode *F, SDNode *T) DISABLE_INLINE {
  unsigned FNumVals = F->getNumValues();
  unsigned TNumVals = T->getNumValues();
  ISelQueueUpdater ISQU(ISelQueue);
  if (FNumVals != TNumVals) {
    for (unsigned i = 0, e = std::min(FNumVals, TNumVals); i < e; ++i)
      CurDAG->ReplaceAllUsesOfValueWith(SDOperand(F, i), SDOperand(T, i), &ISQU);
  } else {
    CurDAG->ReplaceAllUsesWith(F, T, &ISQU);
  }
  setSelected(F->getNodeId());
  UpdateQueue(ISQU);
}

// SelectRoot - Top level entry to DAG isel.
SDOperand SelectRoot(SDOperand Root) {
  SelectRootInit();
  unsigned NumBytes = (DAGSize + 7) / 8;
  ISelQueued   = new unsigned char[NumBytes];
  ISelSelected = new unsigned char[NumBytes];
  memset(ISelQueued,   0, NumBytes);
  memset(ISelSelected, 0, NumBytes);

  // Create a dummy node (which is not added to allnodes), that adds
  // a reference to the root node, preventing it from being deleted,
  // and tracking any changes of the root.
  HandleSDNode Dummy(CurDAG->getRoot());
  ISelQueue.push_back(CurDAG->getRoot().Val);
  while (!ISelQueue.empty()) {
    SDNode *Node = ISelQueue.front();
    std::pop_heap(ISelQueue.begin(), ISelQueue.end(), isel_sort());
    ISelQueue.pop_back();
    if (!isSelected(Node->getNodeId())) {
      SDNode *ResNode = Select(SDOperand(Node, 0));
      if (ResNode != Node) {
        if (ResNode)
          ReplaceUses(Node, ResNode);
        if (Node->use_empty()) { // Don't delete EntryToken, etc.
          ISelQueueUpdater ISQU(ISelQueue);
          CurDAG->RemoveDeadNode(Node, &ISQU);
          UpdateQueue(ISQU);
        }
      }
    }
  }

  delete[] ISelQueued;
  ISelQueued = NULL;
  delete[] ISelSelected;
  ISelSelected = NULL;
  return Dummy.getValue();
}

// Node transformations.
inline SDOperand Transform_MMX_SHUFFLE_get_shuf_imm(SDNode *N) {

  return getI8Imm(X86::getShuffleSHUFImmediate(N));

}
inline SDOperand Transform_PSxLDQ_imm(SDNode *inN) {
  ConstantSDNode *N = cast<ConstantSDNode>(inN);

  // Transformation function: imm >> 3
  return getI32Imm(N->getValue() >> 3);

}
inline SDOperand Transform_SHUFFLE_get_pshufhw_imm(SDNode *N) {

  return getI8Imm(X86::getShufflePSHUFHWImmediate(N));

}
inline SDOperand Transform_SHUFFLE_get_pshuflw_imm(SDNode *N) {

  return getI8Imm(X86::getShufflePSHUFLWImmediate(N));

}
inline SDOperand Transform_SHUFFLE_get_shuf_imm(SDNode *N) {

  return getI8Imm(X86::getShuffleSHUFImmediate(N));

}

// Predicate functions.
inline bool Predicate_MMX_MOVL_shuffle_mask(SDNode *N) {

  return X86::isMOVLMask(N);

}
inline bool Predicate_MMX_PSHUFW_shuffle_mask(SDNode *N) {

  return X86::isPSHUFDMask(N);

}
inline bool Predicate_MMX_UNPCKH_shuffle_mask(SDNode *N) {

  return X86::isUNPCKHMask(N);

}
inline bool Predicate_MMX_UNPCKH_v_undef_shuffle_mask(SDNode *N) {

  return X86::isUNPCKH_v_undef_Mask(N);

}
inline bool Predicate_MMX_UNPCKL_shuffle_mask(SDNode *N) {

  return X86::isUNPCKLMask(N);

}
inline bool Predicate_MMX_UNPCKL_v_undef_shuffle_mask(SDNode *N) {

  return X86::isUNPCKL_v_undef_Mask(N);

}
inline bool Predicate_MOVHLPS_shuffle_mask(SDNode *N) {

  return X86::isMOVHLPSMask(N);

}
inline bool Predicate_MOVHLPS_v_undef_shuffle_mask(SDNode *N) {

  return X86::isMOVHLPS_v_undef_Mask(N);

}
inline bool Predicate_MOVHP_shuffle_mask(SDNode *N) {

  return X86::isMOVHPMask(N);

}
inline bool Predicate_MOVLP_shuffle_mask(SDNode *N) {

  return X86::isMOVLPMask(N);

}
inline bool Predicate_MOVL_shuffle_mask(SDNode *N) {

  return X86::isMOVLMask(N);

}
inline bool Predicate_MOVSHDUP_shuffle_mask(SDNode *N) {

  return X86::isMOVSHDUPMask(N);

}
inline bool Predicate_MOVSLDUP_shuffle_mask(SDNode *N) {

  return X86::isMOVSLDUPMask(N);

}
inline bool Predicate_PSHUFD_binary_shuffle_mask(SDNode *N) {

  return X86::isSHUFPMask(N);

}
inline bool Predicate_PSHUFD_shuffle_mask(SDNode *N) {

  return X86::isPSHUFDMask(N);

}
inline bool Predicate_PSHUFHW_shuffle_mask(SDNode *N) {

  return X86::isPSHUFHWMask(N);

}
inline bool Predicate_PSHUFLW_shuffle_mask(SDNode *N) {

  return X86::isPSHUFLWMask(N);

}
inline bool Predicate_SHUFP_shuffle_mask(SDNode *N) {

  return X86::isSHUFPMask(N);

}
inline bool Predicate_SHUFP_unary_shuffle_mask(SDNode *N) {

  return X86::isPSHUFDMask(N);

}
inline bool Predicate_SSE_splat_lo_mask(SDNode *N) {

  return X86::isSplatLoMask(N);

}
inline bool Predicate_SSE_splat_mask(SDNode *N) {

  return X86::isSplatMask(N);

}
inline bool Predicate_UNPCKH_shuffle_mask(SDNode *N) {

  return X86::isUNPCKHMask(N);

}
inline bool Predicate_UNPCKH_v_undef_shuffle_mask(SDNode *N) {

  return X86::isUNPCKH_v_undef_Mask(N);

}
inline bool Predicate_UNPCKL_shuffle_mask(SDNode *N) {

  return X86::isUNPCKLMask(N);

}
inline bool Predicate_UNPCKL_v_undef_shuffle_mask(SDNode *N) {

  return X86::isUNPCKL_v_undef_Mask(N);

}
inline bool Predicate_alignedload(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::NON_EXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getAlignment() >= 16;
  return false;

}
inline bool Predicate_alignedstore(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N))
    return !ST->isTruncatingStore() &&
           ST->getAddressingMode() == ISD::UNINDEXED &&
           ST->getAlignment() >= 16;
  return false;

}
inline bool Predicate_and_su(SDNode *N) {

  return N->hasOneUse();

}
inline bool Predicate_atomic_las_16(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i16;
  return false;

}
inline bool Predicate_atomic_las_32(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i32;
  return false;

}
inline bool Predicate_atomic_las_64(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i64;
  return false;

}
inline bool Predicate_atomic_las_8(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i8;
  return false;

}
inline bool Predicate_atomic_lcs_16(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i16;
  return false;

}
inline bool Predicate_atomic_lcs_32(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i32;
  return false;

}
inline bool Predicate_atomic_lcs_64(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i64;
  return false;

}
inline bool Predicate_atomic_lcs_8(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i8;
  return false;

}
inline bool Predicate_atomic_swap_16(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i16;
  return false;

}
inline bool Predicate_atomic_swap_32(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i32;
  return false;

}
inline bool Predicate_atomic_swap_64(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i64;
  return false;

}
inline bool Predicate_atomic_swap_8(SDNode *N) {

  if (AtomicSDNode* V = dyn_cast<AtomicSDNode>(N))
        return V->getVT() == MVT::i8;
  return false;

}
inline bool Predicate_extloadf32(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::EXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::f32;
  return false;

}
inline bool Predicate_extloadf64(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::EXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::f64;
  return false;

}
inline bool Predicate_extloadi1(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::EXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i1;
  return false;

}
inline bool Predicate_extloadi16(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::EXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i16;
  return false;

}
inline bool Predicate_extloadi32(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::EXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i32;
  return false;

}
inline bool Predicate_extloadi8(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::EXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i8;
  return false;

}
inline bool Predicate_fp32imm0(SDNode *inN) {
  ConstantFPSDNode *N = cast<ConstantFPSDNode>(inN);

  return N->isExactlyValue(+0.0);

}
inline bool Predicate_fpimm0(SDNode *inN) {
  ConstantFPSDNode *N = cast<ConstantFPSDNode>(inN);

  return N->isExactlyValue(+0.0);

}
inline bool Predicate_fpimm1(SDNode *inN) {
  ConstantFPSDNode *N = cast<ConstantFPSDNode>(inN);

  return N->isExactlyValue(+1.0);

}
inline bool Predicate_fpimmneg0(SDNode *inN) {
  ConstantFPSDNode *N = cast<ConstantFPSDNode>(inN);

  return N->isExactlyValue(-0.0);

}
inline bool Predicate_fpimmneg1(SDNode *inN) {
  ConstantFPSDNode *N = cast<ConstantFPSDNode>(inN);

  return N->isExactlyValue(-1.0);

}
inline bool Predicate_i16immSExt8(SDNode *inN) {
  ConstantSDNode *N = cast<ConstantSDNode>(inN);

  // i16immSExt8 predicate - True if the 16-bit immediate fits in a 8-bit
  // sign extended field.
  return (int16_t)N->getValue() == (int8_t)N->getValue();

}
inline bool Predicate_i32immSExt8(SDNode *inN) {
  ConstantSDNode *N = cast<ConstantSDNode>(inN);

  // i32immSExt8 predicate - True if the 32-bit immediate fits in a 8-bit
  // sign extended field.
  return (int32_t)N->getValue() == (int8_t)N->getValue();

}
inline bool Predicate_i64immFFFFFFFF(SDNode *inN) {
  ConstantSDNode *N = cast<ConstantSDNode>(inN);

  // i64immFFFFFFFF - True if this is a specific constant we can't write in
  // tblgen files.
  return N->getValue() == 0x00000000FFFFFFFFULL;

}
inline bool Predicate_i64immSExt32(SDNode *inN) {
  ConstantSDNode *N = cast<ConstantSDNode>(inN);

  // i64immSExt32 predicate - True if the 64-bit immediate fits in a 32-bit
  // sign extended field.
  return (int64_t)N->getValue() == (int32_t)N->getValue();

}
inline bool Predicate_i64immSExt8(SDNode *inN) {
  ConstantSDNode *N = cast<ConstantSDNode>(inN);

  // i64immSExt8 predicate - True if the 64-bit immediate fits in a 8-bit
  // sign extended field.
  return (int64_t)N->getValue() == (int8_t)N->getValue();

}
inline bool Predicate_i64immZExt32(SDNode *inN) {
  ConstantSDNode *N = cast<ConstantSDNode>(inN);

  // i64immZExt32 predicate - True if the 64-bit immediate fits in a 32-bit
  // unsignedsign extended field.
  return (uint64_t)N->getValue() == (uint32_t)N->getValue();

}
inline bool Predicate_immAllOnes(SDNode *inN) {
  ConstantSDNode *N = cast<ConstantSDNode>(inN);
 return N->isAllOnesValue(); 
}
inline bool Predicate_immAllOnesV(SDNode *N) {

  return ISD::isBuildVectorAllOnes(N);

}
inline bool Predicate_immAllOnesV_bc(SDNode *N) {

  return ISD::isBuildVectorAllOnes(N);

}
inline bool Predicate_immAllZerosV(SDNode *N) {

  return ISD::isBuildVectorAllZeros(N);

}
inline bool Predicate_immAllZerosV_bc(SDNode *N) {

  return ISD::isBuildVectorAllZeros(N);

}
inline bool Predicate_load(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::NON_EXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED;
  return false;

}
inline bool Predicate_memop(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::NON_EXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getAlignment() >= 16;
  return false;

}
inline bool Predicate_memop64(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::NON_EXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getAlignment() >= 8;
  return false;

}
inline bool Predicate_post_store(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return !ST->isTruncatingStore() &&
            (AM == ISD::POST_INC || AM == ISD::POST_DEC);
  }
  return false;

}
inline bool Predicate_post_truncstf32(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::POST_INC || AM == ISD::POST_DEC) &&
           ST->isTruncatingStore() && ST->getMemoryVT() == MVT::f32;
  }
  return false;

}
inline bool Predicate_post_truncsti1(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::POST_INC || AM == ISD::POST_DEC) &&
           ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i1;
  }
  return false;

}
inline bool Predicate_post_truncsti16(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::POST_INC || AM == ISD::POST_DEC) &&
           ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i16;
  }
  return false;

}
inline bool Predicate_post_truncsti32(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::POST_INC || AM == ISD::POST_DEC) &&
           ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i32;
  }
  return false;

}
inline bool Predicate_post_truncsti8(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::POST_INC || AM == ISD::POST_DEC) &&
           ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i8;
  }
  return false;

}
inline bool Predicate_pre_store(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::PRE_INC || AM == ISD::PRE_DEC) &&
           !ST->isTruncatingStore();
  }
  return false;

}
inline bool Predicate_pre_truncstf32(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::PRE_INC || AM == ISD::PRE_DEC) &&
           ST->isTruncatingStore() && ST->getMemoryVT() == MVT::f32;
  }
  return false;

}
inline bool Predicate_pre_truncsti1(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::PRE_INC || AM == ISD::PRE_DEC) &&
           ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i1;
  }
  return false;

}
inline bool Predicate_pre_truncsti16(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::PRE_INC || AM == ISD::PRE_DEC) &&
           ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i16;
  }
  return false;

}
inline bool Predicate_pre_truncsti32(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::PRE_INC || AM == ISD::PRE_DEC) &&
           ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i32;
  }
  return false;

}
inline bool Predicate_pre_truncsti8(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N)) {
    ISD::MemIndexedMode AM = ST->getAddressingMode();
    return (AM == ISD::PRE_INC || AM == ISD::PRE_DEC) &&
           ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i8;
  }
  return false;

}
inline bool Predicate_sextloadi1(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::SEXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i1;
  return false;

}
inline bool Predicate_sextloadi16(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::SEXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i16;
  return false;

}
inline bool Predicate_sextloadi32(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::SEXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i32;
  return false;

}
inline bool Predicate_sextloadi8(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::SEXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i8;
  return false;

}
inline bool Predicate_store(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N))
    return !ST->isTruncatingStore() &&
           ST->getAddressingMode() == ISD::UNINDEXED;
  return false;

}
inline bool Predicate_truncstoref32(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N))
    return ST->isTruncatingStore() && ST->getMemoryVT() == MVT::f32 &&
           ST->getAddressingMode() == ISD::UNINDEXED;
  return false;

}
inline bool Predicate_truncstoref64(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N))
    return ST->isTruncatingStore() && ST->getMemoryVT() == MVT::f64 &&
           ST->getAddressingMode() == ISD::UNINDEXED;
  return false;

}
inline bool Predicate_truncstorei16(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N))
    return ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i16 &&
           ST->getAddressingMode() == ISD::UNINDEXED;
  return false;

}
inline bool Predicate_truncstorei32(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N))
    return ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i32 &&
           ST->getAddressingMode() == ISD::UNINDEXED;
  return false;

}
inline bool Predicate_truncstorei8(SDNode *N) {

  if (StoreSDNode *ST = dyn_cast<StoreSDNode>(N))
    return ST->isTruncatingStore() && ST->getMemoryVT() == MVT::i8 &&
           ST->getAddressingMode() == ISD::UNINDEXED;
  return false;

}
inline bool Predicate_vtFP(SDNode *inN) {
  VTSDNode *N = cast<VTSDNode>(inN);
 return MVT::isFloatingPoint(N->getVT()); 
}
inline bool Predicate_vtInt(SDNode *inN) {
  VTSDNode *N = cast<VTSDNode>(inN);
 return MVT::isInteger(N->getVT()); 
}
inline bool Predicate_zextloadi1(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::ZEXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i1;
  return false;

}
inline bool Predicate_zextloadi16(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::ZEXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i16;
  return false;

}
inline bool Predicate_zextloadi32(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::ZEXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i32;
  return false;

}
inline bool Predicate_zextloadi8(SDNode *N) {

  if (LoadSDNode *LD = dyn_cast<LoadSDNode>(N))
    return LD->getExtensionType() == ISD::ZEXTLOAD &&
           LD->getAddressingMode() == ISD::UNINDEXED &&
           LD->getMemoryVT() == MVT::i8;
  return false;

}


SDNode *Emit_0(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_1(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_2(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0);
}
SDNode *Emit_3(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N1)->getValue()), MVT::i8);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, Tmp1);
}
SDNode *Emit_4(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N1);
}
SDNode *Select_ISD_ADD_i8(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (add:i8 GR8:i8:$src1, (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADD8rm:i8 GR8:i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADD8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (add:i8 (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>, GR8:i8:$src1)
    // Emits: (ADD8rm:i8 GR8:i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADD8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (add:i8 GR8:i8:$src, 1:i8)
      // Emits: (INC8r:i8 GR8:i8:$src)
      // Pattern complexity = 8  cost = 1  size = 2
      if (CN0 == 1) {
        return Emit_2(N, X86::INC8r, MVT::i8);
      }

      // Pattern: (add:i8 GR8:i8:$src, -1:i8)
      // Emits: (DEC8r:i8 GR8:i8:$src)
      // Pattern complexity = 8  cost = 1  size = 2
      if (CN0 == -1) {
        return Emit_2(N, X86::DEC8r, MVT::i8);
      }
    }

    // Pattern: (add:i8 GR8:i8:$src1, (imm:i8):$src2)
    // Emits: (ADD8ri:i8 GR8:i8:$src1, (imm:i8):$src2)
    // Pattern complexity = 6  cost = 1  size = 3
    if (N1.getOpcode() == ISD::Constant) {
      return Emit_3(N, X86::ADD8ri, MVT::i8);
    }
  }

  // Pattern: (add:i8 GR8:i8:$src1, GR8:i8:$src2)
  // Emits: (ADD8rr:i8 GR8:i8:$src1, GR8:i8:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::ADD8rr, MVT::i8);
}

SDNode *Emit_5(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N1)->getValue()), MVT::i16);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, Tmp1);
}
SDNode *Select_ISD_ADD_i16(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (add:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADD16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADD16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (add:i16 (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, GR16:i16:$src1)
    // Emits: (ADD16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADD16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((!Subtarget->is64Bit())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (add:i16 GR16:i16:$src, 1:i16)
      // Emits: (INC16r:i16 GR16:i16:$src)
      // Pattern complexity = 8  cost = 1  size = 1
      if (CN0 == 1) {
        return Emit_2(N, X86::INC16r, MVT::i16);
      }

      // Pattern: (add:i16 GR16:i16:$src, -1:i16)
      // Emits: (DEC16r:i16 GR16:i16:$src)
      // Pattern complexity = 8  cost = 1  size = 1
      if (CN0 == -1) {
        return Emit_2(N, X86::DEC16r, MVT::i16);
      }
    }
  }
  if ((Subtarget->is64Bit())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (add:i16 GR16:i16:$src, 1:i16)
      // Emits: (INC64_16r:i16 GR16:i16:$src)
      // Pattern complexity = 8  cost = 1  size = 2
      if (CN0 == 1) {
        return Emit_2(N, X86::INC64_16r, MVT::i16);
      }

      // Pattern: (add:i16 GR16:i16:$src, -1:i16)
      // Emits: (DEC64_16r:i16 GR16:i16:$src)
      // Pattern complexity = 8  cost = 1  size = 2
      if (CN0 == -1) {
        return Emit_2(N, X86::DEC64_16r, MVT::i16);
      }
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (add:i16 GR16:i16:$src1, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2)
      // Emits: (ADD16ri8:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i16immSExt8(N1.Val)) {
        return Emit_5(N, X86::ADD16ri8, MVT::i16);
      }

      // Pattern: (add:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Emits: (ADD16ri:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_5(N, X86::ADD16ri, MVT::i16);
    }
  }

  // Pattern: (add:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Emits: (ADD16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::ADD16rr, MVT::i16);
}

SDNode *Emit_6(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3 };
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, Ops0, 4);
}
SDNode *Emit_7(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N10);
}
SDNode *Emit_8(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  AddToISelQueue(N0);
  AddToISelQueue(N10);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N10);
}
SDNode *Emit_9(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N1, N00);
}
SDNode *Emit_10(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N1);
  AddToISelQueue(N00);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N1, N00);
}
SDNode *Emit_11(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, Tmp1);
}
SDNode *Select_ISD_ADD_i32(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (add:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADD32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADD32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (add:i32 (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, GR32:i32:$src1)
    // Emits: (ADD32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADD32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: lea32addr:i32:$src
  // Emits: (LEA64_32r:i32 lea32addr:i32:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  if ((Subtarget->is64Bit())) {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA64_32r, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: lea32addr:i32:$src
  // Emits: (LEA32r:i32 lea32addr:i32:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  if ((!Subtarget->is64Bit())) {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA32r, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == X86ISD::Wrapper) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (add:i32 GR32:i32:$src1, (X86Wrapper:i32 (tconstpool:i32):$src2))
        // Emits: (ADD32ri:i32 GR32:i32:$src1, (tconstpool:i32):$src2)
        // Pattern complexity = 9  cost = 1  size = 3
        if (N10.getOpcode() == ISD::TargetConstantPool) {
          return Emit_7(N, X86::ADD32ri, MVT::i32);
        }

        // Pattern: (add:i32 GR32:i32:$src1, (X86Wrapper:i32 (tjumptable:i32):$src2))
        // Emits: (ADD32ri:i32 GR32:i32:$src1, (tjumptable:i32):$src2)
        // Pattern complexity = 9  cost = 1  size = 3
        if (N10.getOpcode() == ISD::TargetJumpTable) {
          return Emit_8(N, X86::ADD32ri, MVT::i32);
        }

        // Pattern: (add:i32 GR32:i32:$src1, (X86Wrapper:i32 (tglobaladdr:i32):$src2))
        // Emits: (ADD32ri:i32 GR32:i32:$src1, (tglobaladdr:i32):$src2)
        // Pattern complexity = 9  cost = 1  size = 3
        if (N10.getOpcode() == ISD::TargetGlobalAddress) {
          return Emit_7(N, X86::ADD32ri, MVT::i32);
        }

        // Pattern: (add:i32 GR32:i32:$src1, (X86Wrapper:i32 (texternalsym:i32):$src2))
        // Emits: (ADD32ri:i32 GR32:i32:$src1, (texternalsym:i32):$src2)
        // Pattern complexity = 9  cost = 1  size = 3
        if (N10.getOpcode() == ISD::TargetExternalSymbol) {
          return Emit_7(N, X86::ADD32ri, MVT::i32);
        }
      }
    }
    if (N0.getOpcode() == X86ISD::Wrapper) {
      SDOperand N00 = N0.getOperand(0);

      // Pattern: (add:i32 (X86Wrapper:i32 (tconstpool:i32):$src2), GR32:i32:$src1)
      // Emits: (ADD32ri:i32 GR32:i32:$src1, (tconstpool:i32):$src2)
      // Pattern complexity = 9  cost = 1  size = 3
      if (N00.getOpcode() == ISD::TargetConstantPool) {
        return Emit_9(N, X86::ADD32ri, MVT::i32);
      }

      // Pattern: (add:i32 (X86Wrapper:i32 (tjumptable:i32):$src2), GR32:i32:$src1)
      // Emits: (ADD32ri:i32 GR32:i32:$src1, (tjumptable:i32):$src2)
      // Pattern complexity = 9  cost = 1  size = 3
      if (N00.getOpcode() == ISD::TargetJumpTable) {
        return Emit_10(N, X86::ADD32ri, MVT::i32);
      }

      // Pattern: (add:i32 (X86Wrapper:i32 (tglobaladdr:i32):$src2), GR32:i32:$src1)
      // Emits: (ADD32ri:i32 GR32:i32:$src1, (tglobaladdr:i32):$src2)
      // Pattern complexity = 9  cost = 1  size = 3
      if (N00.getOpcode() == ISD::TargetGlobalAddress) {
        return Emit_9(N, X86::ADD32ri, MVT::i32);
      }

      // Pattern: (add:i32 (X86Wrapper:i32 (texternalsym:i32):$src2), GR32:i32:$src1)
      // Emits: (ADD32ri:i32 GR32:i32:$src1, (texternalsym:i32):$src2)
      // Pattern complexity = 9  cost = 1  size = 3
      if (N00.getOpcode() == ISD::TargetExternalSymbol) {
        return Emit_9(N, X86::ADD32ri, MVT::i32);
      }
    }
  }
  if ((!Subtarget->is64Bit())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (add:i32 GR32:i32:$src, 1:i32)
      // Emits: (INC32r:i32 GR32:i32:$src)
      // Pattern complexity = 8  cost = 1  size = 1
      if (CN0 == 1) {
        return Emit_2(N, X86::INC32r, MVT::i32);
      }

      // Pattern: (add:i32 GR32:i32:$src, -1:i32)
      // Emits: (DEC32r:i32 GR32:i32:$src)
      // Pattern complexity = 8  cost = 1  size = 1
      if (CN0 == -1) {
        return Emit_2(N, X86::DEC32r, MVT::i32);
      }
    }
  }
  if ((Subtarget->is64Bit())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (add:i32 GR32:i32:$src, 1:i32)
      // Emits: (INC64_32r:i32 GR32:i32:$src)
      // Pattern complexity = 8  cost = 1  size = 2
      if (CN0 == 1) {
        return Emit_2(N, X86::INC64_32r, MVT::i32);
      }

      // Pattern: (add:i32 GR32:i32:$src, -1:i32)
      // Emits: (DEC64_32r:i32 GR32:i32:$src)
      // Pattern complexity = 8  cost = 1  size = 2
      if (CN0 == -1) {
        return Emit_2(N, X86::DEC64_32r, MVT::i32);
      }
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (add:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (ADD32ri8:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val)) {
        return Emit_11(N, X86::ADD32ri8, MVT::i32);
      }

      // Pattern: (add:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (ADD32ri:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_11(N, X86::ADD32ri, MVT::i32);
    }
  }

  // Pattern: (add:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Emits: (ADD32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::ADD32rr, MVT::i32);
}

SDNode *Emit_12(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N1)->getValue()), MVT::i64);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, Tmp1);
}
SDNode *Select_ISD_ADD_i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (add:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADD64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADD64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (add:i64 (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, GR64:i64:$src1)
    // Emits: (ADD64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADD64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: lea64addr:i64:$src
  // Emits: (LEA64r:i64 lea64addr:i64:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA64r, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (add:i64 GR64:i64:$src, 1:i64)
      // Emits: (INC64r:i64 GR64:i64:$src)
      // Pattern complexity = 8  cost = 1  size = 2
      if (CN0 == 1) {
        return Emit_2(N, X86::INC64r, MVT::i64);
      }

      // Pattern: (add:i64 GR64:i64:$src, -1:i64)
      // Emits: (DEC64r:i64 GR64:i64:$src)
      // Pattern complexity = 8  cost = 1  size = 2
      if (CN0 == -1) {
        return Emit_2(N, X86::DEC64r, MVT::i64);
      }
    }
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (add:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Emits: (ADD64ri32:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt32(N1.Val)) {
        return Emit_12(N, X86::ADD64ri32, MVT::i64);
      }

      // Pattern: (add:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (ADD64ri8:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val)) {
        return Emit_12(N, X86::ADD64ri8, MVT::i64);
      }
    }
  }

  // Pattern: (add:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Emits: (ADD64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::ADD64rr, MVT::i64);
}

SDNode *Emit_13(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain10 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_14(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Select_ISD_ADD_v8i8(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (add:v8i8 VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PADDBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_load(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_13(N, X86::MMX_PADDBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (add:v8i8 (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), VR64:v8i8:$src1)
      // Emits: (MMX_PADDBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      if (N0.getOpcode() == ISD::BIT_CONVERT &&
          N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_load(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N1 = N.getOperand(1);
              if (N00.Val->getValueType(0) == MVT::v1i64) {
                return Emit_14(N, X86::MMX_PADDBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }

    // Pattern: (add:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
    // Emits: (MMX_PADDBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PADDBrr, MVT::v8i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ADD_v4i16(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (add:v4i16 VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PADDWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_load(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_13(N, X86::MMX_PADDWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (add:v4i16 (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), VR64:v4i16:$src1)
      // Emits: (MMX_PADDWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      if (N0.getOpcode() == ISD::BIT_CONVERT &&
          N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_load(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N1 = N.getOperand(1);
              if (N00.Val->getValueType(0) == MVT::v1i64) {
                return Emit_14(N, X86::MMX_PADDWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }

    // Pattern: (add:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
    // Emits: (MMX_PADDWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PADDWrr, MVT::v4i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ADD_v2i32(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (add:v2i32 VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PADDDrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_load(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_13(N, X86::MMX_PADDDrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (add:v2i32 (bitconvert:v2i32 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), VR64:v2i32:$src1)
      // Emits: (MMX_PADDDrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      if (N0.getOpcode() == ISD::BIT_CONVERT &&
          N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_load(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N1 = N.getOperand(1);
              if (N00.Val->getValueType(0) == MVT::v1i64) {
                return Emit_14(N, X86::MMX_PADDDrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }

    // Pattern: (add:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
    // Emits: (MMX_PADDDrr:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PADDDrr, MVT::v2i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ADD_v1i64(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (add:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PADDQrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_load(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_13(N, X86::MMX_PADDQrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (add:v1i64 (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), VR64:v1i64:$src1)
      // Emits: (MMX_PADDQrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      if (N0.getOpcode() == ISD::BIT_CONVERT &&
          N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_load(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N1 = N.getOperand(1);
              if (N00.Val->getValueType(0) == MVT::v1i64) {
                return Emit_14(N, X86::MMX_PADDQrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }

    // Pattern: (add:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
    // Emits: (MMX_PADDQrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PADDQrr, MVT::v1i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ADD_v16i8(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (add:v16i8 VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PADDBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_13(N, X86::PADDBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (add:v16i8 (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), VR128:v16i8:$src1)
      // Emits: (PADDBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      if (N0.getOpcode() == ISD::BIT_CONVERT &&
          N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_memop(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N1 = N.getOperand(1);
              if (N00.Val->getValueType(0) == MVT::v2i64) {
                return Emit_14(N, X86::PADDBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }

    // Pattern: (add:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
    // Emits: (PADDBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::PADDBrr, MVT::v16i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ADD_v8i16(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (add:v8i16 VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PADDWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_13(N, X86::PADDWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (add:v8i16 (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), VR128:v8i16:$src1)
      // Emits: (PADDWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      if (N0.getOpcode() == ISD::BIT_CONVERT &&
          N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_memop(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N1 = N.getOperand(1);
              if (N00.Val->getValueType(0) == MVT::v2i64) {
                return Emit_14(N, X86::PADDWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }

    // Pattern: (add:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
    // Emits: (PADDWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::PADDWrr, MVT::v8i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ADD_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (add:v4i32 VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PADDDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_13(N, X86::PADDDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (add:v4i32 (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), VR128:v4i32:$src1)
      // Emits: (PADDDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      if (N0.getOpcode() == ISD::BIT_CONVERT &&
          N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_memop(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N1 = N.getOperand(1);
              if (N00.Val->getValueType(0) == MVT::v2i64) {
                return Emit_14(N, X86::PADDDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }

    // Pattern: (add:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
    // Emits: (PADDDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::PADDDrr, MVT::v4i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ADD_v2i64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (add:v2i64 VR128:v2i64:$src1, (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (PADDQrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::PADDQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (add:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, VR128:v2i64:$src1)
      // Emits: (PADDQrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::PADDQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (add:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
    // Emits: (PADDQrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::PADDQrr, MVT::v2i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_15(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, MVT::Flag, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  SDOperand InFlag(ResNode, 2);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  return ResNode;
}
SDNode *Emit_16(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, MVT::Flag, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  SDOperand InFlag(ResNode, 2);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  return ResNode;
}
SDNode *Emit_17(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Flag, N0, Tmp1);
  SDOperand InFlag(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  return ResNode;
}
SDNode *Emit_18(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Flag, N0, N1);
  SDOperand InFlag(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  return ResNode;
}
SDNode *Select_ISD_ADDC_i32(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (addc:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADD32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_15(N, X86::ADD32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (addc:i32 (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, GR32:i32:$src1)
    // Emits: (ADD32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_16(N, X86::ADD32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (addc:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (ADD32ri8:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val)) {
        return Emit_17(N, X86::ADD32ri8, MVT::i32);
      }

      // Pattern: (addc:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (ADD32ri:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_17(N, X86::ADD32ri, MVT::i32);
    }
  }

  // Pattern: (addc:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Emits: (ADD32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_18(N, X86::ADD32rr, MVT::i32);
}

SDNode *Emit_19(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N1)->getValue()), MVT::i64);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Flag, N0, Tmp1);
  SDOperand InFlag(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  return ResNode;
}
SDNode *Select_ISD_ADDC_i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (addc:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADD64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_15(N, X86::ADD64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (addc:i64 (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, GR64:i64:$src1)
    // Emits: (ADD64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_16(N, X86::ADD64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (addc:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Emits: (ADD64ri32:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt32(N1.Val)) {
        return Emit_19(N, X86::ADD64ri32, MVT::i64);
      }

      // Pattern: (addc:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (ADD64ri8:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val)) {
        return Emit_19(N, X86::ADD64ri8, MVT::i64);
      }
    }
  }

  // Pattern: (addc:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Emits: (ADD64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_18(N, X86::ADD64rr, MVT::i64);
}

SDNode *Emit_20(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand InFlag = N.getOperand(2);
  AddToISelQueue(InFlag);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, MVT::Flag, Ops0, 8);
  Chain1 = SDOperand(ResNode, 1);
  InFlag = SDOperand(ResNode, 2);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  return ResNode;
}
SDNode *Emit_21(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand InFlag = N.getOperand(2);
  AddToISelQueue(InFlag);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, MVT::Flag, Ops0, 8);
  Chain0 = SDOperand(ResNode, 1);
  InFlag = SDOperand(ResNode, 2);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  return ResNode;
}
SDNode *Emit_22(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  SDOperand InFlag = N.getOperand(2);
  AddToISelQueue(InFlag);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Flag, N0, Tmp1, InFlag);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  return ResNode;
}
SDNode *Emit_23(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  SDOperand InFlag = N.getOperand(2);
  AddToISelQueue(InFlag);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Flag, N0, N1, InFlag);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  return ResNode;
}
SDNode *Select_ISD_ADDE_i32(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (adde:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADC32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_20(N, X86::ADC32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (adde:i32 (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, GR32:i32:$src1)
    // Emits: (ADC32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_21(N, X86::ADC32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (adde:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (ADC32ri8:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val)) {
        return Emit_22(N, X86::ADC32ri8, MVT::i32);
      }

      // Pattern: (adde:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (ADC32ri:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_22(N, X86::ADC32ri, MVT::i32);
    }
  }

  // Pattern: (adde:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Emits: (ADC32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_23(N, X86::ADC32rr, MVT::i32);
}

SDNode *Emit_24(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N1)->getValue()), MVT::i64);
  SDOperand InFlag = N.getOperand(2);
  AddToISelQueue(InFlag);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Flag, N0, Tmp1, InFlag);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  return ResNode;
}
SDNode *Select_ISD_ADDE_i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (adde:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADC64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_20(N, X86::ADC64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (adde:i64 (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, GR64:i64:$src1)
    // Emits: (ADC64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_21(N, X86::ADC64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (adde:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Emits: (ADC64ri32:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt32(N1.Val)) {
        return Emit_24(N, X86::ADC64ri32, MVT::i64);
      }

      // Pattern: (adde:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (ADC64ri8:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val)) {
        return Emit_24(N, X86::ADC64ri8, MVT::i64);
      }
    }
  }

  // Pattern: (adde:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Emits: (ADC64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_23(N, X86::ADC64rr, MVT::i64);
}

SDNode *Select_ISD_AND_i8(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (and:i8 GR8:i8:$src1, (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (AND8rm:i8 GR8:i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::AND8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (and:i8 (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>, GR8:i8:$src1)
    // Emits: (AND8rm:i8 GR8:i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::AND8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (and:i8 GR8:i8:$src1, (imm:i8):$src2)
    // Emits: (AND8ri:i8 GR8:i8:$src1, (imm:i8):$src2)
    // Pattern complexity = 6  cost = 1  size = 3
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {
      return Emit_3(N, X86::AND8ri, MVT::i8);
    }
  }

  // Pattern: (and:i8 GR8:i8:$src1, GR8:i8:$src2)
  // Emits: (AND8rr:i8 GR8:i8:$src1, GR8:i8:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::AND8rr, MVT::i8);
}

SDNode *Select_ISD_AND_i16(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (and:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (AND16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::AND16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (and:i16 (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, GR16:i16:$src1)
    // Emits: (AND16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::AND16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (and:i16 GR16:i16:$src1, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2)
      // Emits: (AND16ri8:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i16immSExt8(N1.Val)) {
        return Emit_5(N, X86::AND16ri8, MVT::i16);
      }

      // Pattern: (and:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Emits: (AND16ri:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_5(N, X86::AND16ri, MVT::i16);
    }
  }

  // Pattern: (and:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Emits: (AND16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::AND16rr, MVT::i16);
}

SDNode *Emit_25(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Select_ISD_AND_i32(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    {
      SDOperand N1 = N.getOperand(1);
      if (isa<ConstantSDNode>(N1)) {

        // Pattern: (and:i32 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>, 255:i32)
        // Emits: (MOVZX32rm8:i32 addr:iPTR:$src)
        // Pattern complexity = 27  cost = 1  size = 3
        if (CheckAndMask(N0, cast<ConstantSDNode>(N1), 255) &&
            N0.getOpcode() == ISD::LOAD &&
            N0.hasOneUse() &&
            CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
          SDOperand Chain0 = N0.getOperand(0);
          if (Predicate_load(N0.Val)) {
            SDOperand N01 = N0.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_25(N, X86::MOVZX32rm8, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }

        // Pattern: (and:i32 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>, 65535:i32)
        // Emits: (MOVZX32rm16:i32 addr:iPTR:$src)
        // Pattern complexity = 27  cost = 1  size = 3
        if (CheckAndMask(N0, cast<ConstantSDNode>(N1), 65535) &&
            N0.getOpcode() == ISD::LOAD &&
            N0.hasOneUse() &&
            CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
          SDOperand Chain0 = N0.getOperand(0);
          if (Predicate_load(N0.Val)) {
            SDOperand N01 = N0.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_25(N, X86::MOVZX32rm16, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (and:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (AND32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::AND32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (and:i32 (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, GR32:i32:$src1)
    // Emits: (AND32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::AND32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (and:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (AND32ri8:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val)) {
        return Emit_11(N, X86::AND32ri8, MVT::i32);
      }

      // Pattern: (and:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (AND32ri:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_11(N, X86::AND32ri, MVT::i32);
    }
  }

  // Pattern: (and:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Emits: (AND32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::AND32rr, MVT::i32);
}

SDNode *Emit_26(const SDOperand &N, unsigned Opc0, unsigned Opc1, MVT::ValueType VT0, MVT::ValueType VT1) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Tmp0 = CurDAG->getTargetConstant(0x0ULL, MVT::i64);
  AddToISelQueue(N0);
  SDOperand Tmp2 = CurDAG->getTargetConstant(0x3ULL, MVT::i32);
  SDOperand Tmp3(CurDAG->getTargetNode(Opc0, VT0, N0, Tmp2), 0);
  SDOperand Tmp4 = CurDAG->getTargetConstant(0x3ULL, MVT::i32);
  return CurDAG->SelectNodeTo(N.Val, Opc1, VT1, Tmp0, Tmp3, Tmp4);
}
SDNode *Select_ISD_AND_i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (and:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (AND64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::AND64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (and:i64 (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, GR64:i64:$src1)
    // Emits: (AND64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::AND64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (and:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Emits: (AND64ri32:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt32(N1.Val)) {
        return Emit_12(N, X86::AND64ri32, MVT::i64);
      }

      // Pattern: (and:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (AND64ri8:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val)) {
        return Emit_12(N, X86::AND64ri8, MVT::i64);
      }

      // Pattern: (and:i64 GR64:i64:$src, (imm:i64)<<P:Predicate_i64immFFFFFFFF>>)
      // Emits: (SUBREG_TO_REG:i64 0:i64, (EXTRACT_SUBREG:i32 GR64:i64:$src, 3:i32), 3:i32)
      // Pattern complexity = 7  cost = 2  size = 0
      if (Predicate_i64immFFFFFFFF(N1.Val)) {
        return Emit_26(N, TargetInstrInfo::EXTRACT_SUBREG, TargetInstrInfo::SUBREG_TO_REG, MVT::i32, MVT::i64);
      }
    }
  }

  // Pattern: (and:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Emits: (AND64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::AND64rr, MVT::i64);
}

SDNode *Emit_27(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N010 = N01.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N00);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N00, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_28(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N000 = N00.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N01);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_29(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N110 = N11.getOperand(0);
  AddToISelQueue(N10);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N10, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_30(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N11);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_31(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N00);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N00, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_32(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N01);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_33(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N10);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N10, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_34(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N11);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_35(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N010 = N01.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N00);
  AddToISelQueue(N1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N00, N1);
}
SDNode *Emit_36(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N000 = N00.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N01);
  AddToISelQueue(N1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N01, N1);
}
SDNode *Emit_37(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N110 = N11.getOperand(0);
  AddToISelQueue(N10);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N10, N0);
}
SDNode *Emit_38(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N11);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N11, N0);
}
SDNode *Emit_39(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N00);
  AddToISelQueue(N1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N00, N1);
}
SDNode *Emit_40(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N01);
  AddToISelQueue(N1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N01, N1);
}
SDNode *Emit_41(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N10);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N10, N0);
}
SDNode *Emit_42(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N11);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N11, N0);
}
SDNode *Select_ISD_AND_v1i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      {
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N010 = N01.getOperand(0);

          // Pattern: (and:v1i64 (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (build_vector:v2i32)<<P:Predicate_immAllOnesV>>)), (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
          // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
          // Pattern complexity = 32  cost = 1  size = 3
          if (N010.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N010.Val)) {
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::LOAD &&
                N1.hasOneUse() &&
                CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
              SDOperand Chain1 = N1.getOperand(0);
              if (Predicate_load(N1.Val)) {
                SDOperand N11 = N1.getOperand(1);
                SDOperand CPTmp0;
                SDOperand CPTmp1;
                SDOperand CPTmp2;
                SDOperand CPTmp3;
                if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                    N010.Val->getValueType(0) == MVT::v2i32) {
                  return Emit_27(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
          if (N010.getOpcode() == ISD::BIT_CONVERT &&
              Predicate_immAllOnesV_bc(N010.Val)) {
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::LOAD &&
                N1.hasOneUse() &&
                CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
              SDOperand Chain1 = N1.getOperand(0);
              if (Predicate_load(N1.Val)) {
                SDOperand N11 = N1.getOperand(1);
                SDOperand CPTmp0;
                SDOperand CPTmp1;
                SDOperand CPTmp2;
                SDOperand CPTmp3;
                if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

                  // Pattern: (and:v1i64 (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (bitconvert:v4i16)<<P:Predicate_immAllOnesV_bc>>)), (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
                  // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
                  // Pattern complexity = 32  cost = 1  size = 3
                  if (N010.Val->getValueType(0) == MVT::v4i16) {
                    return Emit_27(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (and:v1i64 (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (bitconvert:v8i8)<<P:Predicate_immAllOnesV_bc>>)), (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
                  // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
                  // Pattern complexity = 32  cost = 1  size = 3
                  if (N010.Val->getValueType(0) == MVT::v8i8) {
                    return Emit_27(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (and:v1i64 (xor:v1i64 (bitconvert:v1i64 (build_vector:v2i32)<<P:Predicate_immAllOnesV>>), VR64:v1i64:$src1), (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 32  cost = 1  size = 3
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::LOAD &&
              N1.hasOneUse() &&
              CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
            SDOperand Chain1 = N1.getOperand(0);
            if (Predicate_load(N1.Val)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N000.Val->getValueType(0) == MVT::v2i32) {
                return Emit_28(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::XOR) {
            SDOperand N10 = N1.getOperand(0);

            // Pattern: (and:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (build_vector:v2i32)<<P:Predicate_immAllOnesV>>)))
            // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::BIT_CONVERT) {
                SDOperand N110 = N11.getOperand(0);
                if (N110.getOpcode() == ISD::BUILD_VECTOR &&
                    Predicate_immAllOnesV(N110.Val) &&
                    N110.Val->getValueType(0) == MVT::v2i32) {
                  return Emit_29(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }

            // Pattern: (and:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, (xor:v1i64 (bitconvert:v1i64 (build_vector:v2i32)<<P:Predicate_immAllOnesV>>), VR64:v1i64:$src1))
            // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            if (N10.getOpcode() == ISD::BIT_CONVERT) {
              SDOperand N100 = N10.getOperand(0);
              if (N100.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_immAllOnesV(N100.Val)) {
                SDOperand N11 = N1.getOperand(1);
                if (N100.Val->getValueType(0) == MVT::v2i32) {
                  return Emit_30(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }

    // Pattern: (and:v1i64 (xor:v1i64 (bitconvert:v1i64 (bitconvert:v4i16)<<P:Predicate_immAllOnesV_bc>>), VR64:v1i64:$src1), (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 32  cost = 1  size = 3
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BIT_CONVERT &&
            Predicate_immAllOnesV_bc(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::LOAD &&
              N1.hasOneUse() &&
              CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
            SDOperand Chain1 = N1.getOperand(0);
            if (Predicate_load(N1.Val)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N000.Val->getValueType(0) == MVT::v4i16) {
                return Emit_28(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::XOR) {
            SDOperand N10 = N1.getOperand(0);

            // Pattern: (and:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (bitconvert:v4i16)<<P:Predicate_immAllOnesV_bc>>)))
            // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::BIT_CONVERT) {
                SDOperand N110 = N11.getOperand(0);
                if (N110.getOpcode() == ISD::BIT_CONVERT &&
                    Predicate_immAllOnesV_bc(N110.Val) &&
                    N110.Val->getValueType(0) == MVT::v4i16) {
                  return Emit_29(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }

            // Pattern: (and:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, (xor:v1i64 (bitconvert:v1i64 (bitconvert:v4i16)<<P:Predicate_immAllOnesV_bc>>), VR64:v1i64:$src1))
            // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            if (N10.getOpcode() == ISD::BIT_CONVERT) {
              SDOperand N100 = N10.getOperand(0);
              if (N100.getOpcode() == ISD::BIT_CONVERT &&
                  Predicate_immAllOnesV_bc(N100.Val)) {
                SDOperand N11 = N1.getOperand(1);
                if (N100.Val->getValueType(0) == MVT::v4i16) {
                  return Emit_30(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }

    // Pattern: (and:v1i64 (xor:v1i64 (bitconvert:v1i64 (bitconvert:v8i8)<<P:Predicate_immAllOnesV_bc>>), VR64:v1i64:$src1), (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 32  cost = 1  size = 3
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BIT_CONVERT &&
            Predicate_immAllOnesV_bc(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::LOAD &&
              N1.hasOneUse() &&
              CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
            SDOperand Chain1 = N1.getOperand(0);
            if (Predicate_load(N1.Val)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N000.Val->getValueType(0) == MVT::v8i8) {
                return Emit_28(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::XOR) {
            SDOperand N10 = N1.getOperand(0);

            // Pattern: (and:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (bitconvert:v8i8)<<P:Predicate_immAllOnesV_bc>>)))
            // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::BIT_CONVERT) {
                SDOperand N110 = N11.getOperand(0);
                if (N110.getOpcode() == ISD::BIT_CONVERT &&
                    Predicate_immAllOnesV_bc(N110.Val) &&
                    N110.Val->getValueType(0) == MVT::v8i8) {
                  return Emit_29(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }

            // Pattern: (and:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, (xor:v1i64 (bitconvert:v1i64 (bitconvert:v8i8)<<P:Predicate_immAllOnesV_bc>>), VR64:v1i64:$src1))
            // Emits: (MMX_PANDNrm:v1i64 VR64:v8i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            if (N10.getOpcode() == ISD::BIT_CONVERT) {
              SDOperand N100 = N10.getOperand(0);
              if (N100.getOpcode() == ISD::BIT_CONVERT &&
                  Predicate_immAllOnesV_bc(N100.Val)) {
                SDOperand N11 = N1.getOperand(1);
                if (N100.Val->getValueType(0) == MVT::v8i8) {
                  return Emit_30(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);

      // Pattern: (and:v1i64 (xor:v1i64 VR64:v1i64:$src1, (build_vector:v1i64)<<P:Predicate_immAllOnesV>>), (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (MMX_PANDNrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 29  cost = 1  size = 3
      {
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N01.Val)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::LOAD &&
              N1.hasOneUse() &&
              CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
            SDOperand Chain1 = N1.getOperand(0);
            if (Predicate_load(N1.Val)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                return Emit_31(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (and:v1i64 (xor:v1i64 (build_vector:v1i64)<<P:Predicate_immAllOnesV>>, VR64:v1i64:$src1), (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (MMX_PANDNrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 29  cost = 1  size = 3
      if (N00.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_immAllOnesV(N00.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_32(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::XOR) {
            SDOperand N10 = N1.getOperand(0);

            // Pattern: (and:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, (xor:v1i64 VR64:v1i64:$src1, (build_vector:v1i64)<<P:Predicate_immAllOnesV>>))
            // Emits: (MMX_PANDNrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 29  cost = 1  size = 3
            {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_immAllOnesV(N11.Val)) {
                return Emit_33(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }

            // Pattern: (and:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, (xor:v1i64 (build_vector:v1i64)<<P:Predicate_immAllOnesV>>, VR64:v1i64:$src1))
            // Emits: (MMX_PANDNrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 29  cost = 1  size = 3
            if (N10.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_immAllOnesV(N10.Val)) {
              return Emit_34(N, X86::MMX_PANDNrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (and:v1i64 VR64:v1i64:$src1, (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (MMX_PANDrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MMX_PANDrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (and:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, VR64:v1i64:$src1)
    // Emits: (MMX_PANDrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::MMX_PANDrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      {
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N010 = N01.getOperand(0);

          // Pattern: (and:v1i64 (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (build_vector:v2i32)<<P:Predicate_immAllOnesV>>)), VR64:v1i64:$src2)
          // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
          // Pattern complexity = 13  cost = 1  size = 3
          if (N010.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N010.Val)) {
            SDOperand N1 = N.getOperand(1);
            if (N010.Val->getValueType(0) == MVT::v2i32) {
              return Emit_35(N, X86::MMX_PANDNrr, MVT::v1i64);
            }
          }
          if (N010.getOpcode() == ISD::BIT_CONVERT &&
              Predicate_immAllOnesV_bc(N010.Val)) {
            SDOperand N1 = N.getOperand(1);

            // Pattern: (and:v1i64 (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (bitconvert:v4i16)<<P:Predicate_immAllOnesV_bc>>)), VR64:v1i64:$src2)
            // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
            // Pattern complexity = 13  cost = 1  size = 3
            if (N010.Val->getValueType(0) == MVT::v4i16) {
              return Emit_35(N, X86::MMX_PANDNrr, MVT::v1i64);
            }

            // Pattern: (and:v1i64 (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (bitconvert:v8i8)<<P:Predicate_immAllOnesV_bc>>)), VR64:v1i64:$src2)
            // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
            // Pattern complexity = 13  cost = 1  size = 3
            if (N010.Val->getValueType(0) == MVT::v8i8) {
              return Emit_35(N, X86::MMX_PANDNrr, MVT::v1i64);
            }
          }
        }
      }

      // Pattern: (and:v1i64 (xor:v1i64 (bitconvert:v1i64 (build_vector:v2i32)<<P:Predicate_immAllOnesV>>), VR64:v1i64:$src1), VR64:v1i64:$src2)
      // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 13  cost = 1  size = 3
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N000.Val->getValueType(0) == MVT::v2i32) {
            return Emit_36(N, X86::MMX_PANDNrr, MVT::v1i64);
          }
        }
      }
    }
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::XOR) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (and:v1i64 VR64:v1i64:$src2, (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (build_vector:v2i32)<<P:Predicate_immAllOnesV>>)))
        // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
        // Pattern complexity = 13  cost = 1  size = 3
        {
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N110 = N11.getOperand(0);
            if (N110.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_immAllOnesV(N110.Val) &&
                N110.Val->getValueType(0) == MVT::v2i32) {
              return Emit_37(N, X86::MMX_PANDNrr, MVT::v1i64);
            }
          }
        }

        // Pattern: (and:v1i64 VR64:v1i64:$src2, (xor:v1i64 (bitconvert:v1i64 (build_vector:v2i32)<<P:Predicate_immAllOnesV>>), VR64:v1i64:$src1))
        // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
        // Pattern complexity = 13  cost = 1  size = 3
        if (N10.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N100 = N10.getOperand(0);
          if (N100.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N100.Val)) {
            SDOperand N11 = N1.getOperand(1);
            if (N100.Val->getValueType(0) == MVT::v2i32) {
              return Emit_38(N, X86::MMX_PANDNrr, MVT::v1i64);
            }
          }
        }
      }
    }

    // Pattern: (and:v1i64 (xor:v1i64 (bitconvert:v1i64 (bitconvert:v4i16)<<P:Predicate_immAllOnesV_bc>>), VR64:v1i64:$src1), VR64:v1i64:$src2)
    // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
    // Pattern complexity = 13  cost = 1  size = 3
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BIT_CONVERT &&
            Predicate_immAllOnesV_bc(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N000.Val->getValueType(0) == MVT::v4i16) {
            return Emit_36(N, X86::MMX_PANDNrr, MVT::v1i64);
          }
        }
      }
    }
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::XOR) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (and:v1i64 VR64:v1i64:$src2, (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (bitconvert:v4i16)<<P:Predicate_immAllOnesV_bc>>)))
        // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
        // Pattern complexity = 13  cost = 1  size = 3
        {
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N110 = N11.getOperand(0);
            if (N110.getOpcode() == ISD::BIT_CONVERT &&
                Predicate_immAllOnesV_bc(N110.Val) &&
                N110.Val->getValueType(0) == MVT::v4i16) {
              return Emit_37(N, X86::MMX_PANDNrr, MVT::v1i64);
            }
          }
        }

        // Pattern: (and:v1i64 VR64:v1i64:$src2, (xor:v1i64 (bitconvert:v1i64 (bitconvert:v4i16)<<P:Predicate_immAllOnesV_bc>>), VR64:v1i64:$src1))
        // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
        // Pattern complexity = 13  cost = 1  size = 3
        if (N10.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N100 = N10.getOperand(0);
          if (N100.getOpcode() == ISD::BIT_CONVERT &&
              Predicate_immAllOnesV_bc(N100.Val)) {
            SDOperand N11 = N1.getOperand(1);
            if (N100.Val->getValueType(0) == MVT::v4i16) {
              return Emit_38(N, X86::MMX_PANDNrr, MVT::v1i64);
            }
          }
        }
      }
    }

    // Pattern: (and:v1i64 (xor:v1i64 (bitconvert:v1i64 (bitconvert:v8i8)<<P:Predicate_immAllOnesV_bc>>), VR64:v1i64:$src1), VR64:v1i64:$src2)
    // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
    // Pattern complexity = 13  cost = 1  size = 3
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BIT_CONVERT &&
            Predicate_immAllOnesV_bc(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N000.Val->getValueType(0) == MVT::v8i8) {
            return Emit_36(N, X86::MMX_PANDNrr, MVT::v1i64);
          }
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::XOR) {
      SDOperand N10 = N1.getOperand(0);

      // Pattern: (and:v1i64 VR64:v1i64:$src2, (xor:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (bitconvert:v8i8)<<P:Predicate_immAllOnesV_bc>>)))
      // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 13  cost = 1  size = 3
      {
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N110 = N11.getOperand(0);
          if (N110.getOpcode() == ISD::BIT_CONVERT &&
              Predicate_immAllOnesV_bc(N110.Val) &&
              N110.Val->getValueType(0) == MVT::v8i8) {
            return Emit_37(N, X86::MMX_PANDNrr, MVT::v1i64);
          }
        }
      }

      // Pattern: (and:v1i64 VR64:v1i64:$src2, (xor:v1i64 (bitconvert:v1i64 (bitconvert:v8i8)<<P:Predicate_immAllOnesV_bc>>), VR64:v1i64:$src1))
      // Emits: (MMX_PANDNrr:v1i64 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 13  cost = 1  size = 3
      if (N10.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N100 = N10.getOperand(0);
        if (N100.getOpcode() == ISD::BIT_CONVERT &&
            Predicate_immAllOnesV_bc(N100.Val)) {
          SDOperand N11 = N1.getOperand(1);
          if (N100.Val->getValueType(0) == MVT::v8i8) {
            return Emit_38(N, X86::MMX_PANDNrr, MVT::v1i64);
          }
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    {
      SDOperand N0 = N.getOperand(0);
      if (N0.getOpcode() == ISD::XOR) {
        SDOperand N00 = N0.getOperand(0);

        // Pattern: (and:v1i64 (xor:v1i64 VR64:v1i64:$src1, (build_vector:v1i64)<<P:Predicate_immAllOnesV>>), VR64:v1i64:$src2)
        // Emits: (MMX_PANDNrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
        // Pattern complexity = 10  cost = 1  size = 3
        {
          SDOperand N01 = N0.getOperand(1);
          if (N01.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N01.Val)) {
            return Emit_39(N, X86::MMX_PANDNrr, MVT::v1i64);
          }
        }

        // Pattern: (and:v1i64 (xor:v1i64 (build_vector:v1i64)<<P:Predicate_immAllOnesV>>, VR64:v1i64:$src1), VR64:v1i64:$src2)
        // Emits: (MMX_PANDNrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
        // Pattern complexity = 10  cost = 1  size = 3
        if (N00.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N00.Val)) {
          return Emit_40(N, X86::MMX_PANDNrr, MVT::v1i64);
        }
      }
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::XOR) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (and:v1i64 VR64:v1i64:$src2, (xor:v1i64 VR64:v1i64:$src1, (build_vector:v1i64)<<P:Predicate_immAllOnesV>>))
        // Emits: (MMX_PANDNrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
        // Pattern complexity = 10  cost = 1  size = 3
        {
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N11.Val)) {
            return Emit_41(N, X86::MMX_PANDNrr, MVT::v1i64);
          }
        }

        // Pattern: (and:v1i64 VR64:v1i64:$src2, (xor:v1i64 (build_vector:v1i64)<<P:Predicate_immAllOnesV>>, VR64:v1i64:$src1))
        // Emits: (MMX_PANDNrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
        // Pattern complexity = 10  cost = 1  size = 3
        if (N10.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N10.Val)) {
          return Emit_42(N, X86::MMX_PANDNrr, MVT::v1i64);
        }
      }
    }

    // Pattern: (and:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
    // Emits: (MMX_PANDrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PANDrr, MVT::v1i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_43(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N000 = N00.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N010 = N01.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N000);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N000, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_44(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N000 = N00.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N010 = N01.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N010);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N010, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_45(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N110 = N11.getOperand(0);
  AddToISelQueue(N100);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N100, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_46(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N110 = N11.getOperand(0);
  AddToISelQueue(N110);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N110, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_47(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N000 = N00.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N000);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N000, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_48(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N010 = N01.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N010);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N010, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_49(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N100);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N100, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_50(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N110 = N11.getOperand(0);
  AddToISelQueue(N110);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N110, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_51(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N00);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N00, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_52(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  AddToISelQueue(N10);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N10, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_53(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N000 = N00.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  AddToISelQueue(N000);
  AddToISelQueue(N10);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N000, N10);
}
SDNode *Emit_54(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N010 = N01.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  AddToISelQueue(N010);
  AddToISelQueue(N10);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N010, N10);
}
SDNode *Emit_55(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N100);
  AddToISelQueue(N00);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N100, N00);
}
SDNode *Emit_56(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N110 = N11.getOperand(0);
  AddToISelQueue(N110);
  AddToISelQueue(N00);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N110, N00);
}
SDNode *Emit_57(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  AddToISelQueue(N00);
  AddToISelQueue(N10);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N00, N10);
}
SDNode *Select_ISD_AND_v2i64(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);

        // Pattern: (and:v2i64 (xor:v2i64 (bitconvert:v2i64 VR128:v4f32:$src1), (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>)), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
        // Emits: (ANDNPSrm:v2i64 VR128:v4f32:$src1, addr:iPTR:$src2)
        // Pattern complexity = 35  cost = 1  size = 3
        {
          SDOperand N01 = N0.getOperand(1);
          if (N01.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N010 = N01.getOperand(0);
            if (N010.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_immAllOnesV(N010.Val)) {
              SDOperand N1 = N.getOperand(1);
              if (N1.getOpcode() == ISD::LOAD &&
                  N1.hasOneUse() &&
                  CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
                SDOperand Chain1 = N1.getOperand(0);
                if (Predicate_memop(N1.Val)) {
                  SDOperand N11 = N1.getOperand(1);
                  SDOperand CPTmp0;
                  SDOperand CPTmp1;
                  SDOperand CPTmp2;
                  SDOperand CPTmp3;
                  if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                      N000.Val->getValueType(0) == MVT::v4f32 &&
                      N010.Val->getValueType(0) == MVT::v4i32) {
                    return Emit_43(N, X86::ANDNPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }

        // Pattern: (and:v2i64 (xor:v2i64 (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>), (bitconvert:v2i64 VR128:v4f32:$src1)), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
        // Emits: (ANDNPSrm:v2i64 VR128:v4f32:$src1, addr:iPTR:$src2)
        // Pattern complexity = 35  cost = 1  size = 3
        if (N000.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          if (N01.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N010 = N01.getOperand(0);
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::LOAD &&
                N1.hasOneUse() &&
                CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
              SDOperand Chain1 = N1.getOperand(0);
              if (Predicate_memop(N1.Val)) {
                SDOperand N11 = N1.getOperand(1);
                SDOperand CPTmp0;
                SDOperand CPTmp1;
                SDOperand CPTmp2;
                SDOperand CPTmp3;
                if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                    N000.Val->getValueType(0) == MVT::v4i32 &&
                    N010.Val->getValueType(0) == MVT::v4f32) {
                  return Emit_44(N, X86::ANDNPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::XOR) {
            SDOperand N10 = N1.getOperand(0);
            if (N10.getOpcode() == ISD::BIT_CONVERT) {
              SDOperand N100 = N10.getOperand(0);

              // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 (bitconvert:v2i64 VR128:v4f32:$src1), (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>)))
              // Emits: (ANDNPSrm:v2i64 VR128:v4f32:$src1, addr:iPTR:$src2)
              // Pattern complexity = 35  cost = 1  size = 3
              {
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::BIT_CONVERT) {
                  SDOperand N110 = N11.getOperand(0);
                  if (N110.getOpcode() == ISD::BUILD_VECTOR &&
                      Predicate_immAllOnesV(N110.Val) &&
                      N100.Val->getValueType(0) == MVT::v4f32 &&
                      N110.Val->getValueType(0) == MVT::v4i32) {
                    return Emit_45(N, X86::ANDNPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }

              // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>), (bitconvert:v2i64 VR128:v4f32:$src1)))
              // Emits: (ANDNPSrm:v2i64 VR128:v4f32:$src1, addr:iPTR:$src2)
              // Pattern complexity = 35  cost = 1  size = 3
              if (N100.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_immAllOnesV(N100.Val)) {
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::BIT_CONVERT) {
                  SDOperand N110 = N11.getOperand(0);
                  if (N100.Val->getValueType(0) == MVT::v4i32 &&
                      N110.Val->getValueType(0) == MVT::v4f32) {
                    return Emit_46(N, X86::ANDNPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);

      // Pattern: (and:v2i64 (xor:v2i64 (bitconvert:v2i64 VR128:v2f64:$src1), (build_vector:v2i64)<<P:Predicate_immAllOnesV>>), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (ANDNPDrm:v2i64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 32  cost = 1  size = 3
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N01.Val)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::LOAD &&
              N1.hasOneUse() &&
              CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
            SDOperand Chain1 = N1.getOperand(0);
            if (Predicate_memop(N1.Val)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N000.Val->getValueType(0) == MVT::v2f64) {
                return Emit_47(N, X86::ANDNPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
      {
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N010 = N01.getOperand(0);
          if (N010.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N010.Val)) {
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::LOAD &&
                N1.hasOneUse() &&
                CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
              SDOperand Chain1 = N1.getOperand(0);
              if (Predicate_memop(N1.Val)) {
                SDOperand N11 = N1.getOperand(1);
                SDOperand CPTmp0;
                SDOperand CPTmp1;
                SDOperand CPTmp2;
                SDOperand CPTmp3;
                if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

                  // Pattern: (and:v2i64 (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>)), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
                  // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
                  // Pattern complexity = 32  cost = 1  size = 3
                  if (N010.Val->getValueType(0) == MVT::v4i32) {
                    return Emit_27(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (and:v2i64 (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v8i16)<<P:Predicate_immAllOnesV>>)), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
                  // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
                  // Pattern complexity = 32  cost = 1  size = 3
                  if (N010.Val->getValueType(0) == MVT::v8i16) {
                    return Emit_27(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (and:v2i64 (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v16i8)<<P:Predicate_immAllOnesV>>)), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
                  // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
                  // Pattern complexity = 32  cost = 1  size = 3
                  if (N010.Val->getValueType(0) == MVT::v16i8) {
                    return Emit_27(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (and:v2i64 (xor:v2i64 (build_vector:v2i64)<<P:Predicate_immAllOnesV>>, (bitconvert:v2i64 VR128:v2f64:$src1)), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (ANDNPDrm:v2i64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 32  cost = 1  size = 3
      if (N00.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_immAllOnesV(N00.Val)) {
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N010 = N01.getOperand(0);
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::LOAD &&
              N1.hasOneUse() &&
              CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
            SDOperand Chain1 = N1.getOperand(0);
            if (Predicate_memop(N1.Val)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N010.Val->getValueType(0) == MVT::v2f64) {
                return Emit_48(N, X86::ANDNPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::XOR) {
            SDOperand N10 = N1.getOperand(0);

            // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 (bitconvert:v2i64 VR128:v2f64:$src1), (build_vector:v2i64)<<P:Predicate_immAllOnesV>>))
            // Emits: (ANDNPDrm:v2i64 VR128:v2f64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            if (N10.getOpcode() == ISD::BIT_CONVERT) {
              SDOperand N100 = N10.getOperand(0);
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_immAllOnesV(N11.Val) &&
                  N100.Val->getValueType(0) == MVT::v2f64) {
                return Emit_49(N, X86::ANDNPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }

            // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 (build_vector:v2i64)<<P:Predicate_immAllOnesV>>, (bitconvert:v2i64 VR128:v2f64:$src1)))
            // Emits: (ANDNPDrm:v2i64 VR128:v2f64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            if (N10.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_immAllOnesV(N10.Val)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::BIT_CONVERT) {
                SDOperand N110 = N11.getOperand(0);
                if (N110.Val->getValueType(0) == MVT::v2f64) {
                  return Emit_50(N, X86::ANDNPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }

    // Pattern: (and:v2i64 (xor:v2i64 (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 32  cost = 1  size = 3
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::LOAD &&
              N1.hasOneUse() &&
              CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
            SDOperand Chain1 = N1.getOperand(0);
            if (Predicate_memop(N1.Val)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N000.Val->getValueType(0) == MVT::v4i32) {
                return Emit_28(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::XOR) {
            SDOperand N10 = N1.getOperand(0);

            // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>)))
            // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::BIT_CONVERT) {
                SDOperand N110 = N11.getOperand(0);
                if (N110.getOpcode() == ISD::BUILD_VECTOR &&
                    Predicate_immAllOnesV(N110.Val) &&
                    N110.Val->getValueType(0) == MVT::v4i32) {
                  return Emit_29(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }

            // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1))
            // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            if (N10.getOpcode() == ISD::BIT_CONVERT) {
              SDOperand N100 = N10.getOperand(0);
              if (N100.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_immAllOnesV(N100.Val)) {
                SDOperand N11 = N1.getOperand(1);
                if (N100.Val->getValueType(0) == MVT::v4i32) {
                  return Emit_30(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }

    // Pattern: (and:v2i64 (xor:v2i64 (bitconvert:v2i64 (build_vector:v8i16)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 32  cost = 1  size = 3
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::LOAD &&
              N1.hasOneUse() &&
              CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
            SDOperand Chain1 = N1.getOperand(0);
            if (Predicate_memop(N1.Val)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N000.Val->getValueType(0) == MVT::v8i16) {
                return Emit_28(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::XOR) {
            SDOperand N10 = N1.getOperand(0);

            // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v8i16)<<P:Predicate_immAllOnesV>>)))
            // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::BIT_CONVERT) {
                SDOperand N110 = N11.getOperand(0);
                if (N110.getOpcode() == ISD::BUILD_VECTOR &&
                    Predicate_immAllOnesV(N110.Val) &&
                    N110.Val->getValueType(0) == MVT::v8i16) {
                  return Emit_29(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }

            // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 (bitconvert:v2i64 (build_vector:v8i16)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1))
            // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            if (N10.getOpcode() == ISD::BIT_CONVERT) {
              SDOperand N100 = N10.getOperand(0);
              if (N100.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_immAllOnesV(N100.Val)) {
                SDOperand N11 = N1.getOperand(1);
                if (N100.Val->getValueType(0) == MVT::v8i16) {
                  return Emit_30(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }

    // Pattern: (and:v2i64 (xor:v2i64 (bitconvert:v2i64 (build_vector:v16i8)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 32  cost = 1  size = 3
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::LOAD &&
              N1.hasOneUse() &&
              CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
            SDOperand Chain1 = N1.getOperand(0);
            if (Predicate_memop(N1.Val)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N000.Val->getValueType(0) == MVT::v16i8) {
                return Emit_28(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::XOR) {
            SDOperand N10 = N1.getOperand(0);

            // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v16i8)<<P:Predicate_immAllOnesV>>)))
            // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::BIT_CONVERT) {
                SDOperand N110 = N11.getOperand(0);
                if (N110.getOpcode() == ISD::BUILD_VECTOR &&
                    Predicate_immAllOnesV(N110.Val) &&
                    N110.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_29(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }

            // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 (bitconvert:v2i64 (build_vector:v16i8)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1))
            // Emits: (PANDNrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 32  cost = 1  size = 3
            if (N10.getOpcode() == ISD::BIT_CONVERT) {
              SDOperand N100 = N10.getOperand(0);
              if (N100.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_immAllOnesV(N100.Val)) {
                SDOperand N11 = N1.getOperand(1);
                if (N100.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_30(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);

      // Pattern: (and:v2i64 (xor:v2i64 VR128:v2i64:$src1, (build_vector:v2i64)<<P:Predicate_immAllOnesV>>), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (PANDNrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 29  cost = 1  size = 3
      {
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N01.Val)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::LOAD &&
              N1.hasOneUse() &&
              CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
            SDOperand Chain1 = N1.getOperand(0);
            if (Predicate_memop(N1.Val)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                return Emit_31(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (and:v2i64 (xor:v2i64 (build_vector:v2i64)<<P:Predicate_immAllOnesV>>, VR128:v2i64:$src1), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (PANDNrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 29  cost = 1  size = 3
      if (N00.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_immAllOnesV(N00.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_32(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::XOR) {
            SDOperand N10 = N1.getOperand(0);

            // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 VR128:v2i64:$src1, (build_vector:v2i64)<<P:Predicate_immAllOnesV>>))
            // Emits: (PANDNrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 29  cost = 1  size = 3
            {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_immAllOnesV(N11.Val)) {
                return Emit_33(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }

            // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (xor:v2i64 (build_vector:v2i64)<<P:Predicate_immAllOnesV>>, VR128:v2i64:$src1))
            // Emits: (PANDNrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 29  cost = 1  size = 3
            if (N10.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_immAllOnesV(N10.Val)) {
              return Emit_34(N, X86::PANDNrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }

  // Pattern: (and:v2i64 (bitconvert:v2i64 VR128:v4f32:$src1), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
  // Emits: (ANDPSrm:v2i64 VR128:v4f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 25  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::v4f32) {
            return Emit_51(N, X86::ANDPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }

  // Pattern: (and:v2i64 (bitconvert:v2i64 VR128:v2f64:$src1), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
  // Emits: (ANDPDrm:v2i64 VR128:v2f64:$src1, addr:iPTR:$src2)
  // Pattern complexity = 25  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::v2f64) {
            return Emit_51(N, X86::ANDPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }

  // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (bitconvert:v2i64 VR128:v4f32:$src1))
  // Emits: (ANDPSrm:v2i64 VR128:v4f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 25  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N10 = N1.getOperand(0);
            if (N10.Val->getValueType(0) == MVT::v4f32) {
              return Emit_52(N, X86::ANDPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (bitconvert:v2i64 VR128:v2f64:$src1))
    // Emits: (ANDPDrm:v2i64 VR128:v2f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 25  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N10 = N1.getOperand(0);
            if (N10.Val->getValueType(0) == MVT::v2f64) {
              return Emit_52(N, X86::ANDPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (and:v2i64 VR128:v2i64:$src1, (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (PANDrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::PANDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (and:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, VR128:v2i64:$src1)
    // Emits: (PANDrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::PANDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);

      // Pattern: (and:v2i64 (xor:v2i64 (bitconvert:v2i64 VR128:v2f64:$src1), (build_vector:v2i64)<<P:Predicate_immAllOnesV>>), (bitconvert:v2i64 VR128:v2f64:$src2))
      // Emits: (ANDNPDrr:v2i64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 16  cost = 1  size = 3
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N01.Val)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N10 = N1.getOperand(0);
            if (N000.Val->getValueType(0) == MVT::v2f64 &&
                N10.Val->getValueType(0) == MVT::v2f64) {
              return Emit_53(N, X86::ANDNPDrr, MVT::v2i64);
            }
          }
        }
      }

      // Pattern: (and:v2i64 (xor:v2i64 (build_vector:v2i64)<<P:Predicate_immAllOnesV>>, (bitconvert:v2i64 VR128:v2f64:$src1)), (bitconvert:v2i64 VR128:v2f64:$src2))
      // Emits: (ANDNPDrr:v2i64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 16  cost = 1  size = 3
      if (N00.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_immAllOnesV(N00.Val)) {
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N010 = N01.getOperand(0);
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N10 = N1.getOperand(0);
            if (N010.Val->getValueType(0) == MVT::v2f64 &&
                N10.Val->getValueType(0) == MVT::v2f64) {
              return Emit_54(N, X86::ANDNPDrr, MVT::v2i64);
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::XOR) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (and:v2i64 (bitconvert:v2i64 VR128:v2f64:$src2), (xor:v2i64 (bitconvert:v2i64 VR128:v2f64:$src1), (build_vector:v2i64)<<P:Predicate_immAllOnesV>>))
        // Emits: (ANDNPDrr:v2i64 VR128:v2f64:$src1, VR128:v2f64:$src2)
        // Pattern complexity = 16  cost = 1  size = 3
        if (N10.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N100 = N10.getOperand(0);
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N11.Val) &&
              N00.Val->getValueType(0) == MVT::v2f64 &&
              N100.Val->getValueType(0) == MVT::v2f64) {
            return Emit_55(N, X86::ANDNPDrr, MVT::v2i64);
          }
        }

        // Pattern: (and:v2i64 (bitconvert:v2i64 VR128:v2f64:$src2), (xor:v2i64 (build_vector:v2i64)<<P:Predicate_immAllOnesV>>, (bitconvert:v2i64 VR128:v2f64:$src1)))
        // Emits: (ANDNPDrr:v2i64 VR128:v2f64:$src1, VR128:v2f64:$src2)
        // Pattern complexity = 16  cost = 1  size = 3
        if (N10.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N10.Val)) {
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N110 = N11.getOperand(0);
            if (N00.Val->getValueType(0) == MVT::v2f64 &&
                N110.Val->getValueType(0) == MVT::v2f64) {
              return Emit_56(N, X86::ANDNPDrr, MVT::v2i64);
            }
          }
        }
      }
    }
  }

  // Pattern: (and:v2i64 (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>)), VR128:v2i64:$src2)
  // Emits: (ANDNPSrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Pattern complexity = 13  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      if (N01.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N010 = N01.getOperand(0);
        if (N010.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N010.Val)) {
          SDOperand N1 = N.getOperand(1);
          if (N010.Val->getValueType(0) == MVT::v4i32) {
            return Emit_35(N, X86::ANDNPSrr, MVT::v2i64);
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      if (N01.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N010 = N01.getOperand(0);
        if (N010.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N010.Val)) {
          SDOperand N1 = N.getOperand(1);

          // Pattern: (and:v2i64 (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>)), VR128:v2i64:$src2)
          // Emits: (PANDNrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
          // Pattern complexity = 13  cost = 1  size = 3
          if (N010.Val->getValueType(0) == MVT::v4i32) {
            return Emit_35(N, X86::PANDNrr, MVT::v2i64);
          }

          // Pattern: (and:v2i64 (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v8i16)<<P:Predicate_immAllOnesV>>)), VR128:v2i64:$src2)
          // Emits: (PANDNrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
          // Pattern complexity = 13  cost = 1  size = 3
          if (N010.Val->getValueType(0) == MVT::v8i16) {
            return Emit_35(N, X86::PANDNrr, MVT::v2i64);
          }

          // Pattern: (and:v2i64 (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v16i8)<<P:Predicate_immAllOnesV>>)), VR128:v2i64:$src2)
          // Emits: (PANDNrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
          // Pattern complexity = 13  cost = 1  size = 3
          if (N010.Val->getValueType(0) == MVT::v16i8) {
            return Emit_35(N, X86::PANDNrr, MVT::v2i64);
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (and:v2i64 (xor:v2i64 (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1), VR128:v2i64:$src2)
    // Emits: (ANDNPSrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
    // Pattern complexity = 13  cost = 1  size = 3
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N000.Val->getValueType(0) == MVT::v4i32) {
            return Emit_36(N, X86::ANDNPSrr, MVT::v2i64);
          }
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::XOR) {
      SDOperand N10 = N1.getOperand(0);

      // Pattern: (and:v2i64 VR128:v2i64:$src2, (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>)))
      // Emits: (ANDNPSrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Pattern complexity = 13  cost = 1  size = 3
      {
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N110 = N11.getOperand(0);
          if (N110.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N110.Val) &&
              N110.Val->getValueType(0) == MVT::v4i32) {
            return Emit_37(N, X86::ANDNPSrr, MVT::v2i64);
          }
        }
      }

      // Pattern: (and:v2i64 VR128:v2i64:$src2, (xor:v2i64 (bitconvert:v2i64 (build_vector:v4i32)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1))
      // Emits: (ANDNPSrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Pattern complexity = 13  cost = 1  size = 3
      if (N10.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N100 = N10.getOperand(0);
        if (N100.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N100.Val)) {
          SDOperand N11 = N1.getOperand(1);
          if (N100.Val->getValueType(0) == MVT::v4i32) {
            return Emit_38(N, X86::ANDNPSrr, MVT::v2i64);
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (and:v2i64 (xor:v2i64 (bitconvert:v2i64 (build_vector:v8i16)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1), VR128:v2i64:$src2)
    // Emits: (PANDNrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
    // Pattern complexity = 13  cost = 1  size = 3
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N000.Val->getValueType(0) == MVT::v8i16) {
            return Emit_36(N, X86::PANDNrr, MVT::v2i64);
          }
        }
      }
    }
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::XOR) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (and:v2i64 VR128:v2i64:$src2, (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v8i16)<<P:Predicate_immAllOnesV>>)))
        // Emits: (PANDNrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
        // Pattern complexity = 13  cost = 1  size = 3
        {
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N110 = N11.getOperand(0);
            if (N110.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_immAllOnesV(N110.Val) &&
                N110.Val->getValueType(0) == MVT::v8i16) {
              return Emit_37(N, X86::PANDNrr, MVT::v2i64);
            }
          }
        }

        // Pattern: (and:v2i64 VR128:v2i64:$src2, (xor:v2i64 (bitconvert:v2i64 (build_vector:v8i16)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1))
        // Emits: (PANDNrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
        // Pattern complexity = 13  cost = 1  size = 3
        if (N10.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N100 = N10.getOperand(0);
          if (N100.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N100.Val)) {
            SDOperand N11 = N1.getOperand(1);
            if (N100.Val->getValueType(0) == MVT::v8i16) {
              return Emit_38(N, X86::PANDNrr, MVT::v2i64);
            }
          }
        }
      }
    }

    // Pattern: (and:v2i64 (xor:v2i64 (bitconvert:v2i64 (build_vector:v16i8)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1), VR128:v2i64:$src2)
    // Emits: (PANDNrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
    // Pattern complexity = 13  cost = 1  size = 3
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N000 = N00.getOperand(0);
        if (N000.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N000.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand N1 = N.getOperand(1);
          if (N000.Val->getValueType(0) == MVT::v16i8) {
            return Emit_36(N, X86::PANDNrr, MVT::v2i64);
          }
        }
      }
    }
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::XOR) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (and:v2i64 VR128:v2i64:$src2, (xor:v2i64 VR128:v2i64:$src1, (bitconvert:v2i64 (build_vector:v16i8)<<P:Predicate_immAllOnesV>>)))
        // Emits: (PANDNrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
        // Pattern complexity = 13  cost = 1  size = 3
        {
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N110 = N11.getOperand(0);
            if (N110.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_immAllOnesV(N110.Val) &&
                N110.Val->getValueType(0) == MVT::v16i8) {
              return Emit_37(N, X86::PANDNrr, MVT::v2i64);
            }
          }
        }

        // Pattern: (and:v2i64 VR128:v2i64:$src2, (xor:v2i64 (bitconvert:v2i64 (build_vector:v16i8)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src1))
        // Emits: (PANDNrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
        // Pattern complexity = 13  cost = 1  size = 3
        if (N10.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N100 = N10.getOperand(0);
          if (N100.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N100.Val)) {
            SDOperand N11 = N1.getOperand(1);
            if (N100.Val->getValueType(0) == MVT::v16i8) {
              return Emit_38(N, X86::PANDNrr, MVT::v2i64);
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::XOR) {
      SDOperand N00 = N0.getOperand(0);

      // Pattern: (and:v2i64 (xor:v2i64 VR128:v2i64:$src1, (build_vector:v2i64)<<P:Predicate_immAllOnesV>>), VR128:v2i64:$src2)
      // Emits: (PANDNrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Pattern complexity = 10  cost = 1  size = 3
      {
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N01.Val)) {
          return Emit_39(N, X86::PANDNrr, MVT::v2i64);
        }
      }

      // Pattern: (and:v2i64 (xor:v2i64 (build_vector:v2i64)<<P:Predicate_immAllOnesV>>, VR128:v2i64:$src1), VR128:v2i64:$src2)
      // Emits: (PANDNrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Pattern complexity = 10  cost = 1  size = 3
      if (N00.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_immAllOnesV(N00.Val)) {
        return Emit_40(N, X86::PANDNrr, MVT::v2i64);
      }
    }
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::XOR) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (and:v2i64 VR128:v2i64:$src2, (xor:v2i64 VR128:v2i64:$src1, (build_vector:v2i64)<<P:Predicate_immAllOnesV>>))
        // Emits: (PANDNrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
        // Pattern complexity = 10  cost = 1  size = 3
        {
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_immAllOnesV(N11.Val)) {
            return Emit_41(N, X86::PANDNrr, MVT::v2i64);
          }
        }

        // Pattern: (and:v2i64 VR128:v2i64:$src2, (xor:v2i64 (build_vector:v2i64)<<P:Predicate_immAllOnesV>>, VR128:v2i64:$src1))
        // Emits: (PANDNrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
        // Pattern complexity = 10  cost = 1  size = 3
        if (N10.getOpcode() == ISD::BUILD_VECTOR &&
            Predicate_immAllOnesV(N10.Val)) {
          return Emit_42(N, X86::PANDNrr, MVT::v2i64);
        }
      }
    }

    // Pattern: (and:v2i64 (bitconvert:v2i64 VR128:v2f64:$src1), (bitconvert:v2i64 VR128:v2f64:$src2))
    // Emits: (ANDPDrr:v2i64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Pattern complexity = 9  cost = 1  size = 3
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N10 = N1.getOperand(0);
        if (N00.Val->getValueType(0) == MVT::v2f64 &&
            N10.Val->getValueType(0) == MVT::v2f64) {
          return Emit_57(N, X86::ANDPDrr, MVT::v2i64);
        }
      }
    }
  }

  // Pattern: (and:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Emits: (ANDPSrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    return Emit_4(N, X86::ANDPSrr, MVT::v2i64);
  }

  // Pattern: (and:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Emits: (PANDrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    return Emit_4(N, X86::PANDrr, MVT::v2i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_58(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_59(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0);
}
SDNode *Select_ISD_ANY_EXTEND_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);

  // Pattern: (anyext:i16 (ld:i8 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (MOVZX16rm8:i16 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  if (N0.getOpcode() == ISD::LOAD &&
      N0.hasOneUse()) {
    SDOperand Chain0 = N0.getOperand(0);
    if (Predicate_load(N0.Val)) {
      SDOperand N01 = N0.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N0.Val->getValueType(0) == MVT::i8) {
        return Emit_58(N, X86::MOVZX16rm8, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }

  // Pattern: (anyext:i16 GR8:i8:$src)
  // Emits: (MOVZX16rr8:i16 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i8) {
    return Emit_59(N, X86::MOVZX16rr8, MVT::i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ANY_EXTEND_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  if (N0.getOpcode() == ISD::LOAD &&
      N0.hasOneUse()) {
    SDOperand Chain0 = N0.getOperand(0);
    if (Predicate_load(N0.Val)) {
      SDOperand N01 = N0.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

        // Pattern: (anyext:i32 (ld:i8 addr:iPTR:$src)<<P:Predicate_load>>)
        // Emits: (MOVZX32rm8:i32 addr:iPTR:$src)
        // Pattern complexity = 22  cost = 1  size = 3
        if (N0.Val->getValueType(0) == MVT::i8) {
          return Emit_58(N, X86::MOVZX32rm8, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (anyext:i32 (ld:i16 addr:iPTR:$src)<<P:Predicate_load>>)
        // Emits: (MOVZX32rm16:i32 addr:iPTR:$src)
        // Pattern complexity = 22  cost = 1  size = 3
        if (N0.Val->getValueType(0) == MVT::i16) {
          return Emit_58(N, X86::MOVZX32rm16, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (anyext:i32 GR8:i8:$src)
  // Emits: (MOVZX32rr8:i32 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i8) {
    return Emit_59(N, X86::MOVZX32rr8, MVT::i32);
  }

  // Pattern: (anyext:i32 GR16:i16:$src)
  // Emits: (MOVZX32rr16:i32 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i16) {
    return Emit_59(N, X86::MOVZX32rr16, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_60(const SDOperand &N, unsigned Opc0, unsigned Opc1, unsigned Opc2, MVT::ValueType VT0, MVT::ValueType VT1, MVT::ValueType VT2, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand Tmp0(CurDAG->getTargetNode(Opc0, VT0), 0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops1[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDOperand Tmp2(CurDAG->getTargetNode(Opc1, VT1, MVT::Other, Ops1, 6), 0);
  Chain0 = SDOperand(Tmp2.Val, 1);
  SDOperand Tmp3 = CurDAG->getTargetConstant(0x3ULL, MVT::i32);
  AddToISelQueue(Chain0);
  SDOperand Ops2[] = { Tmp0, Tmp2, Tmp3, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc2, VT2, MVT::Other, Ops2, 4);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_61(const SDOperand &N, unsigned Opc0, unsigned Opc1, MVT::ValueType VT0, MVT::ValueType VT1) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Tmp0(CurDAG->getTargetNode(Opc0, VT0), 0);
  AddToISelQueue(N0);
  SDOperand Tmp2 = CurDAG->getTargetConstant(0x3ULL, MVT::i32);
  return CurDAG->SelectNodeTo(N.Val, Opc1, VT1, Tmp0, N0, Tmp2);
}
SDNode *Select_ISD_ANY_EXTEND_i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  if (N0.getOpcode() == ISD::LOAD &&
      N0.hasOneUse()) {
    SDOperand Chain0 = N0.getOperand(0);
    if (Predicate_load(N0.Val)) {
      SDOperand N01 = N0.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

        // Pattern: (anyext:i64 (ld:i8 addr:iPTR:$src)<<P:Predicate_load>>)
        // Emits: (MOVZX64rm8:i64 addr:iPTR:$src)
        // Pattern complexity = 22  cost = 1  size = 3
        if (N0.Val->getValueType(0) == MVT::i8) {
          return Emit_58(N, X86::MOVZX64rm8, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (anyext:i64 (ld:i16 addr:iPTR:$src)<<P:Predicate_load>>)
        // Emits: (MOVZX64rm16:i64 addr:iPTR:$src)
        // Pattern complexity = 22  cost = 1  size = 3
        if (N0.Val->getValueType(0) == MVT::i16) {
          return Emit_58(N, X86::MOVZX64rm16, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (anyext:i64 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>)
        // Emits: (INSERT_SUBREG:i64 (IMPLICIT_DEF:i64), (MOV32rm:i32 addr:iPTR:$src), 3:i32)
        // Pattern complexity = 22  cost = 3  size = 3
        if (N0.Val->getValueType(0) == MVT::i32) {
          return Emit_60(N, TargetInstrInfo::IMPLICIT_DEF, X86::MOV32rm, TargetInstrInfo::INSERT_SUBREG, MVT::i64, MVT::i32, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (anyext:i64 GR8:i8:$src)
  // Emits: (MOVZX64rr8:i64 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i8) {
    return Emit_59(N, X86::MOVZX64rr8, MVT::i64);
  }

  // Pattern: (anyext:i64 GR16:i16:$src)
  // Emits: (MOVZX64rr16:i64 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i16) {
    return Emit_59(N, X86::MOVZX64rr16, MVT::i64);
  }

  // Pattern: (anyext:i64 GR32:i32:$src)
  // Emits: (INSERT_SUBREG:i64 (IMPLICIT_DEF:i64), GR32:i32:$src, 3:i32)
  // Pattern complexity = 3  cost = 2  size = 0
  if (N0.Val->getValueType(0) == MVT::i32) {
    return Emit_61(N, TargetInstrInfo::IMPLICIT_DEF, TargetInstrInfo::INSERT_SUBREG, MVT::i64, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_62(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N2);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N2, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Select_ISD_ATOMIC_LAS_i8(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  if (Predicate_atomic_las_8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_62(N, X86::LXADD8, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_LAS_i16(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  if (Predicate_atomic_las_16(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_62(N, X86::LXADD16, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_LAS_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  if (Predicate_atomic_las_32(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_62(N, X86::LXADD32, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_LAS_i64(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  if (Predicate_atomic_las_64(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_62(N, X86::LXADD64, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_LOAD_AND_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_62(N, X86::ATOMAND32, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_LOAD_MAX_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_62(N, X86::ATOMMAX32, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_LOAD_MIN_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_62(N, X86::ATOMMIN32, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_LOAD_OR_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_62(N, X86::ATOMOR32, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_LOAD_UMAX_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_62(N, X86::ATOMUMAX32, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_LOAD_UMIN_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_62(N, X86::ATOMUMIN32, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_LOAD_XOR_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_62(N, X86::ATOMXOR32, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_SWAP_i8(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  if (Predicate_atomic_swap_8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_62(N, X86::XCHG8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_SWAP_i16(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  if (Predicate_atomic_swap_16(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_62(N, X86::XCHG16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_SWAP_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  if (Predicate_atomic_swap_32(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_62(N, X86::XCHG32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ATOMIC_SWAP_i64(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  if (Predicate_atomic_swap_64(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_62(N, X86::XCHG64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_i32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f32) {
      return Emit_59(N, X86::MOVSS2DIrr, MVT::i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_i64(const SDOperand &N) {

  // Pattern: (bitconvert:i64 FR64:f64:$src)
  // Emits: (MOVSDto64rr:i64 FR64:f64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f64) {
      return Emit_59(N, X86::MOVSDto64rr, MVT::i64);
    }
  }
  SDOperand N0 = N.getOperand(0);

  // Pattern: (bitconvert:i64 VR64:v1i64:$src)
  // Emits: (MMX_MOVD64from64rr:i64 VR64:v8i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::v1i64) {
    return Emit_59(N, X86::MMX_MOVD64from64rr, MVT::i64);
  }

  // Pattern: (bitconvert:i64 VR64:v2i32:$src)
  // Emits: (MMX_MOVD64from64rr:i64 VR64:v8i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::v2i32) {
    return Emit_59(N, X86::MMX_MOVD64from64rr, MVT::i64);
  }

  // Pattern: (bitconvert:i64 VR64:v4i16:$src)
  // Emits: (MMX_MOVD64from64rr:i64 VR64:v8i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::v4i16) {
    return Emit_59(N, X86::MMX_MOVD64from64rr, MVT::i64);
  }

  // Pattern: (bitconvert:i64 VR64:v8i8:$src)
  // Emits: (MMX_MOVD64from64rr:i64 VR64:v8i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::v8i8) {
    return Emit_59(N, X86::MMX_MOVD64from64rr, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_f32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (bitconvert:f32 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>)
    // Emits: (MOVDI2SSrm:f32 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::i32) {
          return Emit_58(N, X86::MOVDI2SSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (bitconvert:f32 GR32:i32:$src)
    // Emits: (MOVDI2SSrr:f32 GR32:i32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i32) {
      return Emit_59(N, X86::MOVDI2SSrr, MVT::f32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (bitconvert:f64 (ld:i64 addr:iPTR:$src)<<P:Predicate_load>>)
    // Emits: (MOV64toSDrm:f64 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::i64) {
          return Emit_58(N, X86::MOV64toSDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (bitconvert:f64 GR64:i64:$src)
    // Emits: (MOV64toSDrr:f64 GR64:i64:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i64) {
      return Emit_59(N, X86::MOV64toSDrr, MVT::f64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_63(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  AddToISelQueue(N00);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N00);
}
SDNode *Emit_64(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  AddToISelQueue(N00);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N00);
}
SDNode *Emit_65(const SDOperand &N) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  AddToISelQueue(N0);
  ReplaceUses(N, N0);
  return NULL;
}
SDNode *Select_ISD_BIT_CONVERT_v8i8(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);

  // Pattern: (bitconvert:v8i8 (vector_extract:i64 VR128:v2i64:$src, 0:iPTR))
  // Emits: (MMX_MOVDQ2Qrr:v8i8 VR128:v16i8:$src)
  // Pattern complexity = 11  cost = 1  size = 3
  if (N0.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
    SDOperand N00 = N0.getOperand(0);
    SDOperand N01 = N0.getOperand(1);
    if (isa<ConstantSDNode>(N01)) {
      int64_t CN0 = cast<ConstantSDNode>(N01)->getSignExtended();
      if (CN0 == 0 &&
          N0.Val->getValueType(0) == MVT::i64 &&
          N00.Val->getValueType(0) == MVT::v2i64) {
        return Emit_63(N, X86::MMX_MOVDQ2Qrr, MVT::v8i8);
      }
    }
  }

  // Pattern: (bitconvert:v8i8 (scalar_to_vector:v2i32 GR32:i32:$src))
  // Emits: (MMX_MOVD64rr:v8i8 GR32:i32:$src)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR) {
    SDOperand N00 = N0.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::v2i32 &&
        N00.Val->getValueType(0) == MVT::i32) {
      return Emit_64(N, X86::MMX_MOVD64rr, MVT::v8i8);
    }
  }

  // Pattern: (bitconvert:v8i8 VR64:v1i64:$src)
  // Emits: VR64:v8i8:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v1i64) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v8i8 VR64:v2i32:$src)
  // Emits: VR64:v8i8:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v2i32) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v8i8 VR64:v4i16:$src)
  // Emits: VR64:v8i8:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v4i16) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v8i8 GR64:i64:$src)
  // Emits: (MMX_MOVD64to64rr:v8i8 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i64) {
    return Emit_59(N, X86::MMX_MOVD64to64rr, MVT::v8i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_v4i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);

  // Pattern: (bitconvert:v4i16 (vector_extract:i64 VR128:v2i64:$src, 0:iPTR))
  // Emits: (MMX_MOVDQ2Qrr:v4i16 VR128:v16i8:$src)
  // Pattern complexity = 11  cost = 1  size = 3
  if (N0.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
    SDOperand N00 = N0.getOperand(0);
    SDOperand N01 = N0.getOperand(1);
    if (isa<ConstantSDNode>(N01)) {
      int64_t CN0 = cast<ConstantSDNode>(N01)->getSignExtended();
      if (CN0 == 0 &&
          N0.Val->getValueType(0) == MVT::i64 &&
          N00.Val->getValueType(0) == MVT::v2i64) {
        return Emit_63(N, X86::MMX_MOVDQ2Qrr, MVT::v4i16);
      }
    }
  }

  // Pattern: (bitconvert:v4i16 (scalar_to_vector:v2i32 GR32:i32:$src))
  // Emits: (MMX_MOVD64rr:v4i16 GR32:i32:$src)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR) {
    SDOperand N00 = N0.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::v2i32 &&
        N00.Val->getValueType(0) == MVT::i32) {
      return Emit_64(N, X86::MMX_MOVD64rr, MVT::v4i16);
    }
  }

  // Pattern: (bitconvert:v4i16 VR64:v1i64:$src)
  // Emits: VR64:v4i16:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v1i64) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v4i16 VR64:v2i32:$src)
  // Emits: VR64:v4i16:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v2i32) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v4i16 VR64:v8i8:$src)
  // Emits: VR64:v4i16:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v8i8) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v4i16 GR64:i64:$src)
  // Emits: (MMX_MOVD64to64rr:v4i16 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i64) {
    return Emit_59(N, X86::MMX_MOVD64to64rr, MVT::v4i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_66(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N010 = N01.getOperand(0);
  SDOperand Chain010 = N010.getOperand(0);
  SDOperand N0101 = N010.getOperand(1);
  SDOperand N02 = N0.getOperand(2);
  AddToISelQueue(N0101);
  AddToISelQueue(N0101);
  SDOperand LSI_N010 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N010)->getMemOperand());
  AddToISelQueue(Chain010);
  SDOperand Ops0[] = { N0101, N0101, LSI_N010, Chain010 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 4);
  Chain010 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N010.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Select_ISD_BIT_CONVERT_v2i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);

  // Pattern: (bitconvert:v2i32 (vector_shuffle:v2i32 (build_vector:v2i32)<<P:Predicate_immAllZerosV>>, (scalar_to_vector:v2i32 (ld:v1i64 addr:iPTR:$src)<<P:Predicate_load>>), (build_vector:v2i32)<<P:Predicate_MMX_UNPCKL_shuffle_mask>>))
  // Emits: (MMX_PUNPCKLDQrm:v2i32 VR64:v8i8:$src, VR64:v8i8:$src)
  // Pattern complexity = 56  cost = 1  size = 3
  if (N0.getOpcode() == ISD::VECTOR_SHUFFLE &&
      N0.hasOneUse()) {
    SDOperand N00 = N0.getOperand(0);
    if (N00.getOpcode() == ISD::BUILD_VECTOR &&
        Predicate_immAllZerosV(N00.Val)) {
      SDOperand N01 = N0.getOperand(1);
      if (N01.getOpcode() == ISD::SCALAR_TO_VECTOR &&
          N01.hasOneUse()) {
        SDOperand N010 = N01.getOperand(0);
        if (N010.getOpcode() == ISD::LOAD &&
            N010.hasOneUse() &&
            CanBeFoldedBy(N010.Val, N01.Val, N.Val)) {
          SDOperand Chain010 = N010.getOperand(0);
          if (Predicate_load(N010.Val)) {
            SDOperand N0101 = N010.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N0101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N02 = N0.getOperand(2);
              if (N02.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_MMX_UNPCKL_shuffle_mask(N02.Val) &&
                  N0.Val->getValueType(0) == MVT::v2i32 &&
                  N010.Val->getValueType(0) == MVT::v1i64) {
                return Emit_66(N, X86::MMX_PUNPCKLDQrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (bitconvert:v2i32 (vector_extract:i64 VR128:v2i64:$src, 0:iPTR))
  // Emits: (MMX_MOVDQ2Qrr:v2i32 VR128:v16i8:$src)
  // Pattern complexity = 11  cost = 1  size = 3
  if (N0.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
    SDOperand N00 = N0.getOperand(0);
    SDOperand N01 = N0.getOperand(1);
    if (isa<ConstantSDNode>(N01)) {
      int64_t CN0 = cast<ConstantSDNode>(N01)->getSignExtended();
      if (CN0 == 0 &&
          N0.Val->getValueType(0) == MVT::i64 &&
          N00.Val->getValueType(0) == MVT::v2i64) {
        return Emit_63(N, X86::MMX_MOVDQ2Qrr, MVT::v2i32);
      }
    }
  }

  // Pattern: (bitconvert:v2i32 VR64:v1i64:$src)
  // Emits: VR64:v2i32:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v1i64) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v2i32 VR64:v4i16:$src)
  // Emits: VR64:v2i32:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v4i16) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v2i32 VR64:v8i8:$src)
  // Emits: VR64:v2i32:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v8i8) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v2i32 GR64:i64:$src)
  // Emits: (MMX_MOVD64to64rr:v2i32 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i64) {
    return Emit_59(N, X86::MMX_MOVD64to64rr, MVT::v2i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_v1i64(const SDOperand &N) {

  // Pattern: (bitconvert:v1i64 (vector_extract:i64 VR128:v2i64:$src, 0:iPTR))
  // Emits: (MMX_MOVDQ2Qrr:v1i64 VR128:v2i64:$src)
  // Pattern complexity = 11  cost = 1  size = 3
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      if (isa<ConstantSDNode>(N01)) {
        int64_t CN0 = cast<ConstantSDNode>(N01)->getSignExtended();
        if (CN0 == 0 &&
            N0.Val->getValueType(0) == MVT::i64 &&
            N00.Val->getValueType(0) == MVT::v2i64) {
          return Emit_63(N, X86::MMX_MOVDQ2Qrr, MVT::v1i64);
        }
      }
    }
  }
  SDOperand N0 = N.getOperand(0);

  // Pattern: (bitconvert:v1i64 VR64:v2i32:$src)
  // Emits: VR64:v1i64:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v2i32) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v1i64 VR64:v4i16:$src)
  // Emits: VR64:v1i64:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v4i16) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v1i64 VR64:v8i8:$src)
  // Emits: VR64:v1i64:$src
  // Pattern complexity = 3  cost = 0  size = 0
  if (N0.Val->getValueType(0) == MVT::v8i8) {
    return Emit_65(N);
  }

  // Pattern: (bitconvert:v1i64 GR64:i64:$src)
  // Emits: (MMX_MOVD64to64rr:v1i64 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i64) {
    return Emit_59(N, X86::MMX_MOVD64to64rr, MVT::v1i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_v16i8(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (bitconvert:v16i8 VR128:v2i64:$src)
    // Emits: VR128:v16i8:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v2i64) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v16i8 VR128:v4i32:$src)
    // Emits: VR128:v16i8:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v4i32) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v16i8 VR128:v8i16:$src)
    // Emits: VR128:v16i8:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v8i16) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v16i8 VR128:v2f64:$src)
    // Emits: VR128:v16i8:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v2f64) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v16i8 VR128:v4f32:$src)
    // Emits: VR128:v16i8:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v4f32) {
      return Emit_65(N);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_v8i16(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (bitconvert:v8i16 VR128:v2i64:$src)
    // Emits: VR128:v8i16:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v2i64) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v8i16 VR128:v4i32:$src)
    // Emits: VR128:v8i16:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v4i32) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v8i16 VR128:v16i8:$src)
    // Emits: VR128:v8i16:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v16i8) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v8i16 VR128:v2f64:$src)
    // Emits: VR128:v8i16:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v2f64) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v8i16 VR128:v4f32:$src)
    // Emits: VR128:v8i16:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v4f32) {
      return Emit_65(N);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (bitconvert:v4i32 VR128:v2i64:$src)
    // Emits: VR128:v4i32:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v2i64) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v4i32 VR128:v8i16:$src)
    // Emits: VR128:v4i32:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v8i16) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v4i32 VR128:v16i8:$src)
    // Emits: VR128:v4i32:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v16i8) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v4i32 VR128:v2f64:$src)
    // Emits: VR128:v4i32:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v2f64) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v4i32 VR128:v4f32:$src)
    // Emits: VR128:v4i32:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v4f32) {
      return Emit_65(N);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_v2i64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (bitconvert:v2i64 VR128:v4i32:$src)
    // Emits: VR128:v2i64:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v4i32) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v2i64 VR128:v8i16:$src)
    // Emits: VR128:v2i64:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v8i16) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v2i64 VR128:v16i8:$src)
    // Emits: VR128:v2i64:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v16i8) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v2i64 VR128:v2f64:$src)
    // Emits: VR128:v2i64:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v2f64) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v2i64 VR128:v4f32:$src)
    // Emits: VR128:v2i64:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v4f32) {
      return Emit_65(N);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (bitconvert:v4f32 VR128:v2i64:$src)
    // Emits: VR128:v4f32:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v2i64) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v4f32 VR128:v4i32:$src)
    // Emits: VR128:v4f32:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v4i32) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v4f32 VR128:v8i16:$src)
    // Emits: VR128:v4f32:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v8i16) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v4f32 VR128:v16i8:$src)
    // Emits: VR128:v4f32:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v16i8) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v4f32 VR128:v2f64:$src)
    // Emits: VR128:v4f32:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v2f64) {
      return Emit_65(N);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BIT_CONVERT_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (bitconvert:v2f64 VR128:v2i64:$src)
    // Emits: VR128:v2f64:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v2i64) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v2f64 VR128:v4i32:$src)
    // Emits: VR128:v2f64:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v4i32) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v2f64 VR128:v8i16:$src)
    // Emits: VR128:v2f64:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v8i16) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v2f64 VR128:v16i8:$src)
    // Emits: VR128:v2f64:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v16i8) {
      return Emit_65(N);
    }

    // Pattern: (bitconvert:v2f64 VR128:v4f32:$src)
    // Emits: VR128:v2f64:$src
    // Pattern complexity = 3  cost = 0  size = 0
    if (N0.Val->getValueType(0) == MVT::v4f32) {
      return Emit_65(N);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_67(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N1);
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, N1, Chain);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_ISD_BR(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::BasicBlock) {
    return Emit_67(N, X86::JMP);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_68(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N1.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain1);
  InChains.push_back(Chain1);
  Chain1 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 6);
  Chain1 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain1.Val, Chain1.ResNo));
  return ResNode;
}
SDNode *Select_ISD_BRIND(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::LOAD &&
      N1.hasOneUse() &&
      CanBeFoldedBy(N1.Val, N.Val, N.Val) &&
      (Chain.Val == N1.Val || IsChainCompatible(Chain.Val, N1.Val))) {
    SDOperand Chain1 = N1.getOperand(0);
    if (Predicate_load(N1.Val)) {
      SDOperand N11 = N1.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

        // Pattern: (brind:void (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>)
        // Emits: (JMP32m:void addr:iPTR:$dst)
        // Pattern complexity = 22  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::i32) {
          return Emit_68(N, X86::JMP32m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (brind:void (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>)
        // Emits: (JMP64m:void addr:iPTR:$dst)
        // Pattern complexity = 22  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::i64) {
          return Emit_68(N, X86::JMP64m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (brind:void GR32:i32:$dst)
  // Emits: (JMP32r:void GR32:i32:$dst)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i32) {
    return Emit_67(N, X86::JMP32r);
  }

  // Pattern: (brind:void GR64:i64:$dst)
  // Emits: (JMP64r:void GR64:i64:$dst)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i64) {
    return Emit_67(N, X86::JMP64r);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BSWAP_i32(const SDOperand &N) {
  return Emit_59(N, X86::BSWAP32r, MVT::i32);
}

SDNode *Select_ISD_BSWAP_i64(const SDOperand &N) {
  return Emit_59(N, X86::BSWAP64r, MVT::i64);
}

SDNode *Emit_69(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0);
}
SDNode *Select_ISD_BUILD_VECTOR_v8i8(const SDOperand &N) {
  if ((Subtarget->hasMMX()) &&
      Predicate_immAllZerosV(N.Val)) {
    return Emit_69(N, X86::MMX_V_SET0, MVT::v8i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BUILD_VECTOR_v4i16(const SDOperand &N) {
  if ((Subtarget->hasMMX()) &&
      Predicate_immAllZerosV(N.Val)) {
    return Emit_69(N, X86::MMX_V_SET0, MVT::v4i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BUILD_VECTOR_v2i32(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {

    // Pattern: (build_vector:v2i32)<<P:Predicate_immAllZerosV>>
    // Emits: (MMX_V_SET0:v2i32)
    // Pattern complexity = 4  cost = 1  size = 3
    if (Predicate_immAllZerosV(N.Val)) {
      return Emit_69(N, X86::MMX_V_SET0, MVT::v2i32);
    }

    // Pattern: (build_vector:v2i32)<<P:Predicate_immAllOnesV>>
    // Emits: (MMX_V_SETALLONES:v2i32)
    // Pattern complexity = 4  cost = 1  size = 3
    if (Predicate_immAllOnesV(N.Val)) {
      return Emit_69(N, X86::MMX_V_SETALLONES, MVT::v2i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BUILD_VECTOR_v1i64(const SDOperand &N) {
  if ((Subtarget->hasMMX()) &&
      Predicate_immAllZerosV(N.Val)) {
    return Emit_69(N, X86::MMX_V_SET0, MVT::v1i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BUILD_VECTOR_v16i8(const SDOperand &N) {
  if ((Subtarget->hasSSE1()) &&
      Predicate_immAllZerosV(N.Val)) {
    return Emit_69(N, X86::V_SET0, MVT::v16i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BUILD_VECTOR_v8i16(const SDOperand &N) {
  if ((Subtarget->hasSSE1()) &&
      Predicate_immAllZerosV(N.Val)) {
    return Emit_69(N, X86::V_SET0, MVT::v8i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BUILD_VECTOR_v4i32(const SDOperand &N) {

  // Pattern: (build_vector:v4i32)<<P:Predicate_immAllZerosV>>
  // Emits: (V_SET0:v4i32)
  // Pattern complexity = 4  cost = 1  size = 3
  if ((Subtarget->hasSSE1()) &&
      Predicate_immAllZerosV(N.Val)) {
    return Emit_69(N, X86::V_SET0, MVT::v4i32);
  }

  // Pattern: (build_vector:v4i32)<<P:Predicate_immAllOnesV>>
  // Emits: (V_SETALLONES:v4i32)
  // Pattern complexity = 4  cost = 1  size = 3
  if ((Subtarget->hasSSE2()) &&
      Predicate_immAllOnesV(N.Val)) {
    return Emit_69(N, X86::V_SETALLONES, MVT::v4i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BUILD_VECTOR_v2i64(const SDOperand &N) {
  if ((Subtarget->hasSSE1()) &&
      Predicate_immAllZerosV(N.Val)) {
    return Emit_69(N, X86::V_SET0, MVT::v2i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BUILD_VECTOR_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1()) &&
      Predicate_immAllZerosV(N.Val)) {
    return Emit_69(N, X86::V_SET0, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_BUILD_VECTOR_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE1()) &&
      Predicate_immAllZerosV(N.Val)) {
    return Emit_69(N, X86::V_SET0, MVT::v2f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_70(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  bool HasInFlag = (N.getOperand(N.getNumOperands()-1).getValueType() == MVT::Flag);
  SDOperand Tmp0 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N2)->getValue()), MVT::i32);
  AddToISelQueue(Chain);
  SDOperand InFlag(0, 0);
  if (HasInFlag) {
    InFlag = N.getOperand(N.getNumOperands()-1);
    AddToISelQueue(InFlag);
  }
  SDOperand Ops0[] = { Tmp0, Tmp1, Chain, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Ops0, HasInFlag ? 4 : 3);
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Select_ISD_CALLSEQ_END(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::Constant) {
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant) {
      return Emit_70(N, X86::ADJCALLSTACKUP);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_71(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Tmp0 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Tmp0, Chain);
  Chain = SDOperand(ResNode, 0);
  SDOperand InFlag(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Select_ISD_CALLSEQ_START(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::Constant) {
    return Emit_71(N, X86::ADJCALLSTACKDOWN);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_72(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Tmp0 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N)->getValue()), MVT::i8);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, Tmp0);
}
SDNode *Select_ISD_Constant_i8(const SDOperand &N) {

  // Pattern: 0:i8
  // Emits: (MOV8r0:i8)
  // Pattern complexity = 5  cost = 1  size = 3
  if (cast<ConstantSDNode>(N)->getSignExtended() == 0) {
    return Emit_69(N, X86::MOV8r0, MVT::i8);
  }

  // Pattern: (imm:i8):$src
  // Emits: (MOV8ri:i8 (imm:i8):$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_72(N, X86::MOV8ri, MVT::i8);
}

SDNode *Emit_73(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Tmp0 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N)->getValue()), MVT::i16);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, Tmp0);
}
SDNode *Select_ISD_Constant_i16(const SDOperand &N) {

  // Pattern: 0:i16
  // Emits: (MOV16r0:i16)
  // Pattern complexity = 5  cost = 1  size = 3
  if (cast<ConstantSDNode>(N)->getSignExtended() == 0) {
    return Emit_69(N, X86::MOV16r0, MVT::i16);
  }

  // Pattern: (imm:i16):$src
  // Emits: (MOV16ri:i16 (imm:i16):$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_73(N, X86::MOV16ri, MVT::i16);
}

SDNode *Emit_74(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Tmp0 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N)->getValue()), MVT::i32);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, Tmp0);
}
SDNode *Select_ISD_Constant_i32(const SDOperand &N) {

  // Pattern: 0:i32
  // Emits: (MOV32r0:i32)
  // Pattern complexity = 5  cost = 1  size = 3
  if (cast<ConstantSDNode>(N)->getSignExtended() == 0) {
    return Emit_69(N, X86::MOV32r0, MVT::i32);
  }

  // Pattern: (imm:i32):$src
  // Emits: (MOV32ri:i32 (imm:i32):$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_74(N, X86::MOV32ri, MVT::i32);
}

SDNode *Emit_75(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Tmp0 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N)->getValue()), MVT::i64);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, Tmp0);
}
SDNode *Select_ISD_Constant_i64(const SDOperand &N) {

  // Pattern: 0:i64
  // Emits: (MOV64r0:i64)
  // Pattern complexity = 6  cost = 1  size = 3
  if (cast<ConstantSDNode>(N)->getSignExtended() == 0) {
    return Emit_69(N, X86::MOV64r0, MVT::i64);
  }

  // Pattern: (imm:i64)<<P:Predicate_i64immZExt32>>:$src
  // Emits: (MOV64ri64i32:i64 (imm:i64):$src)
  // Pattern complexity = 5  cost = 1  size = 3
  if (Predicate_i64immZExt32(N.Val)) {
    return Emit_75(N, X86::MOV64ri64i32, MVT::i64);
  }

  // Pattern: (imm:i64)<<P:Predicate_i64immSExt32>>:$src
  // Emits: (MOV64ri32:i64 (imm:i64):$src)
  // Pattern complexity = 4  cost = 1  size = 3
  if (Predicate_i64immSExt32(N.Val)) {
    return Emit_75(N, X86::MOV64ri32, MVT::i64);
  }

  // Pattern: (imm:i64):$src
  // Emits: (MOV64ri:i64 (imm:i64):$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_75(N, X86::MOV64ri, MVT::i64);
}

SDNode *Emit_76(const SDOperand &N, unsigned Opc0, unsigned Opc1, MVT::ValueType VT0, MVT::ValueType VT1) DISABLE_INLINE {
  SDOperand Tmp0(CurDAG->getTargetNode(Opc0, VT0), 0);
  return CurDAG->SelectNodeTo(N.Val, Opc1, VT1, Tmp0);
}
SDNode *Select_ISD_ConstantFP_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {

    // Pattern: (fpimm:f32)<<P:Predicate_fpimm0>>
    // Emits: (LD_Fp032:f32)
    // Pattern complexity = 4  cost = 1  size = 0
    if (Predicate_fpimm0(N.Val)) {
      return Emit_69(N, X86::LD_Fp032, MVT::f32);
    }

    // Pattern: (fpimm:f32)<<P:Predicate_fpimm1>>
    // Emits: (LD_Fp132:f32)
    // Pattern complexity = 4  cost = 1  size = 0
    if (Predicate_fpimm1(N.Val)) {
      return Emit_69(N, X86::LD_Fp132, MVT::f32);
    }
  }

  // Pattern: (fpimm:f32)<<P:Predicate_fp32imm0>>
  // Emits: (FsFLD0SS:f32)
  // Pattern complexity = 4  cost = 1  size = 3
  if ((Subtarget->hasSSE1()) &&
      Predicate_fp32imm0(N.Val)) {
    return Emit_69(N, X86::FsFLD0SS, MVT::f32);
  }
  if ((!Subtarget->hasSSE1())) {

    // Pattern: (fpimm:f32)<<P:Predicate_fpimmneg0>>
    // Emits: (CHS_Fp32:f32 (LD_Fp032:f32))
    // Pattern complexity = 4  cost = 2  size = 0
    if (Predicate_fpimmneg0(N.Val)) {
      return Emit_76(N, X86::LD_Fp032, X86::CHS_Fp32, MVT::f32, MVT::f32);
    }

    // Pattern: (fpimm:f32)<<P:Predicate_fpimmneg1>>
    // Emits: (CHS_Fp32:f32 (LD_Fp132:f32))
    // Pattern complexity = 4  cost = 2  size = 0
    if (Predicate_fpimmneg1(N.Val)) {
      return Emit_76(N, X86::LD_Fp132, X86::CHS_Fp32, MVT::f32, MVT::f32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ConstantFP_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {

    // Pattern: (fpimm:f64)<<P:Predicate_fpimm0>>
    // Emits: (LD_Fp064:f64)
    // Pattern complexity = 4  cost = 1  size = 0
    if (Predicate_fpimm0(N.Val)) {
      return Emit_69(N, X86::LD_Fp064, MVT::f64);
    }

    // Pattern: (fpimm:f64)<<P:Predicate_fpimm1>>
    // Emits: (LD_Fp164:f64)
    // Pattern complexity = 4  cost = 1  size = 0
    if (Predicate_fpimm1(N.Val)) {
      return Emit_69(N, X86::LD_Fp164, MVT::f64);
    }
  }

  // Pattern: (fpimm:f64)<<P:Predicate_fpimm0>>
  // Emits: (FsFLD0SD:f64)
  // Pattern complexity = 4  cost = 1  size = 3
  if ((Subtarget->hasSSE2()) &&
      Predicate_fpimm0(N.Val)) {
    return Emit_69(N, X86::FsFLD0SD, MVT::f64);
  }
  if ((!Subtarget->hasSSE2())) {

    // Pattern: (fpimm:f64)<<P:Predicate_fpimmneg0>>
    // Emits: (CHS_Fp64:f64 (LD_Fp064:f64))
    // Pattern complexity = 4  cost = 2  size = 0
    if (Predicate_fpimmneg0(N.Val)) {
      return Emit_76(N, X86::LD_Fp064, X86::CHS_Fp64, MVT::f64, MVT::f64);
    }

    // Pattern: (fpimm:f64)<<P:Predicate_fpimmneg1>>
    // Emits: (CHS_Fp64:f64 (LD_Fp164:f64))
    // Pattern complexity = 4  cost = 2  size = 0
    if (Predicate_fpimmneg1(N.Val)) {
      return Emit_76(N, X86::LD_Fp164, X86::CHS_Fp64, MVT::f64, MVT::f64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ConstantFP_f80(const SDOperand &N) {

  // Pattern: (fpimm:f80)<<P:Predicate_fpimm0>>
  // Emits: (LD_Fp080:f80)
  // Pattern complexity = 4  cost = 1  size = 0
  if (Predicate_fpimm0(N.Val)) {
    return Emit_69(N, X86::LD_Fp080, MVT::f80);
  }

  // Pattern: (fpimm:f80)<<P:Predicate_fpimm1>>
  // Emits: (LD_Fp180:f80)
  // Pattern complexity = 4  cost = 1  size = 0
  if (Predicate_fpimm1(N.Val)) {
    return Emit_69(N, X86::LD_Fp180, MVT::f80);
  }

  // Pattern: (fpimm:f80)<<P:Predicate_fpimmneg0>>
  // Emits: (CHS_Fp80:f80 (LD_Fp080:f80))
  // Pattern complexity = 4  cost = 2  size = 0
  if (Predicate_fpimmneg0(N.Val)) {
    return Emit_76(N, X86::LD_Fp080, X86::CHS_Fp80, MVT::f80, MVT::f80);
  }

  // Pattern: (fpimm:f80)<<P:Predicate_fpimmneg1>>
  // Emits: (CHS_Fp80:f80 (LD_Fp180:f80))
  // Pattern complexity = 4  cost = 2  size = 0
  if (Predicate_fpimmneg1(N.Val)) {
    return Emit_76(N, X86::LD_Fp180, X86::CHS_Fp80, MVT::f80, MVT::f80);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_77(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  SDOperand Tmp0 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N2)->getValue()), MVT::i32);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N3)->getValue()), MVT::i32);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { Tmp0, Tmp1, Tmp2, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 4);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_ISD_DEBUG_LOC(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::Constant) {
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant) {
      SDOperand N3 = N.getOperand(3);
      if (N3.getOpcode() == ISD::Constant &&
          N1.Val->getValueType(0) == MVT::i32 &&
          N2.Val->getValueType(0) == MVT::i32 &&
          N3.Val->getValueType(0) == MVT::i32) {
        return Emit_77(N, X86::DWARF_LOC);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_78(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N00);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N00, Tmp1);
}
SDNode *Select_ISD_EXTRACT_VECTOR_ELT_i32(const SDOperand &N) {

  // Pattern: (extractelt:i32 (bitconvert:v4i32 VR128:v4f32:$src1), (imm:iPTR):$src2)
  // Emits: (EXTRACTPSrr:i32 VR128:v4f32:$src1, (imm:i32):$src2)
  // Pattern complexity = 9  cost = 1  size = 3
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::Constant &&
          N0.Val->getValueType(0) == MVT::v4i32 &&
          N00.Val->getValueType(0) == MVT::v4f32) {
        return Emit_78(N, X86::EXTRACTPSrr, MVT::i32);
      }
    }
  }

  // Pattern: (vector_extract:i32 VR128:v4i32:$src, 0:iPTR)
  // Emits: (MOVPDI2DIrr:i32 VR128:v4i32:$src)
  // Pattern complexity = 8  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
      if (CN0 == 0 &&
          N0.Val->getValueType(0) == MVT::v4i32) {
        return Emit_2(N, X86::MOVPDI2DIrr, MVT::i32);
      }
    }
  }

  // Pattern: (extractelt:i32 VR128:v4i32:$src1, (imm:iPTR):$src2)
  // Emits: (PEXTRDrr:i32 VR128:v4i32:$src1, (imm:i32):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant &&
        N0.Val->getValueType(0) == MVT::v4i32) {
      return Emit_11(N, X86::PEXTRDrr, MVT::i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_EXTRACT_VECTOR_ELT_i64(const SDOperand &N) {

  // Pattern: (vector_extract:i64 VR128:v2i64:$src, 0:iPTR)
  // Emits: (MOVPQIto64rr:i64 VR128:v2i64:$src)
  // Pattern complexity = 8  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
      if (CN0 == 0 &&
          N0.Val->getValueType(0) == MVT::v2i64) {
        return Emit_2(N, X86::MOVPQIto64rr, MVT::i64);
      }
    }
  }

  // Pattern: (extractelt:i64 VR128:v2i64:$src1, (imm:iPTR):$src2)
  // Emits: (PEXTRQrr:i64 VR128:v2i64:$src1, (imm:i32):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant &&
        N0.Val->getValueType(0) == MVT::v2i64) {
      return Emit_11(N, X86::PEXTRQrr, MVT::i64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_EXTRACT_VECTOR_ELT_f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
      if (CN0 == 0 &&
          N0.Val->getValueType(0) == MVT::v4f32) {
        return Emit_2(N, X86::MOVPS2SSrr, MVT::f32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_EXTRACT_VECTOR_ELT_f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
      if (CN0 == 0 &&
          N0.Val->getValueType(0) == MVT::v2f64) {
        return Emit_2(N, X86::MOVPD2SDrr, MVT::f64);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FABS_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {
    return Emit_59(N, X86::ABS_Fp32, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FABS_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    return Emit_59(N, X86::ABS_Fp64, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FABS_f80(const SDOperand &N) {
  return Emit_59(N, X86::ABS_Fp80, MVT::f80);
}

SDNode *Emit_79(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N12 = N1.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_80(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N02 = N0.getOperand(2);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Select_ISD_FADD_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (fadd:f32 RFP32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADD_Fp32m:f32 RFP32:f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 0
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADD_Fp32m, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fadd:f32 (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>, RFP32:f32:$src1)
    // Emits: (ADD_Fp32m:f32 RFP32:f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 0
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADD_Fp32m, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (fadd:f32 FR32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADDSSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADDSSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fadd:f32 (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>, FR32:f32:$src1)
    // Emits: (ADDSSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADDSSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((!Subtarget->hasSSE1())) {
    {
      SDOperand N0 = N.getOperand(0);
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == X86ISD::FILD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N12 = N1.getOperand(2);

            // Pattern: (fadd:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i16:Other))
            // Emits: (ADD_FpI16m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
            // Pattern complexity = 21  cost = 1  size = 0
            if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
              return Emit_79(N, X86::ADD_FpI16m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (fadd:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i32:Other))
            // Emits: (ADD_FpI32m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
            // Pattern complexity = 21  cost = 1  size = 0
            if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
              return Emit_79(N, X86::ADD_FpI32m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
      if (N0.getOpcode() == X86ISD::FILD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N02 = N0.getOperand(2);

          // Pattern: (fadd:f32 (X86fild:f32 addr:iPTR:$src2, i16:Other), RFP32:f32:$src1)
          // Emits: (ADD_FpI16m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N02)->getVT() == MVT::i16) {
            return Emit_80(N, X86::ADD_FpI16m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fadd:f32 (X86fild:f32 addr:iPTR:$src2, i32:Other), RFP32:f32:$src1)
          // Emits: (ADD_FpI32m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N02)->getVT() == MVT::i32) {
            return Emit_80(N, X86::ADD_FpI32m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fadd:f32 RFP32:f32:$src1, RFP32:f32:$src2)
    // Emits: (ADD_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 0
    return Emit_4(N, X86::ADD_Fp32, MVT::f32);
  }

  // Pattern: (fadd:f32 FR32:f32:$src1, FR32:f32:$src2)
  // Emits: (ADDSSrr:f32 FR32:f32:$src1, FR32:f32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    return Emit_4(N, X86::ADDSSrr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FADD_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);

        // Pattern: (fadd:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
        // Emits: (ADD_Fp64m:f64 RFP64:f64:$src1, addr:iPTR:$src2)
        // Pattern complexity = 22  cost = 1  size = 0
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADD_Fp64m, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }

        // Pattern: (fadd:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
        // Emits: (ADD_Fp64m32:f64 RFP64:f64:$src1, addr:iPTR:$src2)
        // Pattern complexity = 22  cost = 1  size = 0
        if (Predicate_extloadf32(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADD_Fp64m32, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);

      // Pattern: (fadd:f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>, RFP64:f64:$src1)
      // Emits: (ADD_Fp64m:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADD_Fp64m, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fadd:f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_extloadf32>>, RFP64:f64:$src1)
      // Emits: (ADD_Fp64m32:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADD_Fp64m32, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (fadd:f64 FR64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (ADDSDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADDSDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fadd:f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>, FR64:f64:$src1)
    // Emits: (ADDSDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADDSDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((!Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == X86ISD::FILD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N12 = N1.getOperand(2);

            // Pattern: (fadd:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i16:Other))
            // Emits: (ADD_FpI16m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 21  cost = 1  size = 0
            if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
              return Emit_79(N, X86::ADD_FpI16m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (fadd:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i32:Other))
            // Emits: (ADD_FpI32m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 21  cost = 1  size = 0
            if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
              return Emit_79(N, X86::ADD_FpI32m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
      if (N0.getOpcode() == X86ISD::FILD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N02 = N0.getOperand(2);

          // Pattern: (fadd:f64 (X86fild:f64 addr:iPTR:$src2, i16:Other), RFP64:f64:$src1)
          // Emits: (ADD_FpI16m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N02)->getVT() == MVT::i16) {
            return Emit_80(N, X86::ADD_FpI16m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fadd:f64 (X86fild:f64 addr:iPTR:$src2, i32:Other), RFP64:f64:$src1)
          // Emits: (ADD_FpI32m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N02)->getVT() == MVT::i32) {
            return Emit_80(N, X86::ADD_FpI32m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fadd:f64 RFP64:f64:$src1, RFP64:f64:$src2)
    // Emits: (ADD_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 0
    return Emit_4(N, X86::ADD_Fp64, MVT::f64);
  }

  // Pattern: (fadd:f64 FR64:f64:$src1, FR64:f64:$src2)
  // Emits: (ADDSDrr:f64 FR64:f64:$src1, FR64:f64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    return Emit_4(N, X86::ADDSDrr, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FADD_f80(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);

        // Pattern: (fadd:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
        // Emits: (ADD_Fp80m32:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 22  cost = 1  size = 0
        if (Predicate_extloadf32(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADD_Fp80m32, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }

        // Pattern: (fadd:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf64>>)
        // Emits: (ADD_Fp80m64:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 22  cost = 1  size = 0
        if (Predicate_extloadf64(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::ADD_Fp80m64, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);

      // Pattern: (fadd:f80 (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf32>>, RFP80:f80:$src1)
      // Emits: (ADD_Fp80m32:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADD_Fp80m32, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fadd:f80 (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf64>>, RFP80:f80:$src1)
      // Emits: (ADD_Fp80m64:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf64(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::ADD_Fp80m64, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == X86ISD::FILD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N12 = N1.getOperand(2);

          // Pattern: (fadd:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i16:Other))
          // Emits: (ADD_FpI16m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
            return Emit_79(N, X86::ADD_FpI16m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fadd:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i32:Other))
          // Emits: (ADD_FpI32m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
            return Emit_79(N, X86::ADD_FpI32m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
    if (N0.getOpcode() == X86ISD::FILD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        SDOperand N02 = N0.getOperand(2);

        // Pattern: (fadd:f80 (X86fild:f80 addr:iPTR:$src2, i16:Other), RFP80:f80:$src1)
        // Emits: (ADD_FpI16m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N02)->getVT() == MVT::i16) {
          return Emit_80(N, X86::ADD_FpI16m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (fadd:f80 (X86fild:f80 addr:iPTR:$src2, i32:Other), RFP80:f80:$src1)
        // Emits: (ADD_FpI32m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N02)->getVT() == MVT::i32) {
          return Emit_80(N, X86::ADD_FpI32m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fadd:f80 RFP80:f80:$src1, RFP80:f80:$src2)
  // Emits: (ADD_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
  // Pattern complexity = 3  cost = 1  size = 0
  return Emit_4(N, X86::ADD_Fp80, MVT::f80);
}

SDNode *Select_ISD_FADD_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (fadd:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (ADDPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::ADDPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (fadd:v4f32 (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>, VR128:v4f32:$src1)
      // Emits: (ADDPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::ADDPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fadd:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Emits: (ADDPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::ADDPSrr, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FADD_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (fadd:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (ADDPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::ADDPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (fadd:v2f64 (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>, VR128:v2f64:$src1)
      // Emits: (ADDPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::ADDPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fadd:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Emits: (ADDPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::ADDPDrr, MVT::v2f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FCOS_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {
    return Emit_59(N, X86::COS_Fp32, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FCOS_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    return Emit_59(N, X86::COS_Fp64, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FCOS_f80(const SDOperand &N) {
  return Emit_59(N, X86::COS_Fp80, MVT::f80);
}

SDNode *Select_ISD_FDIV_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

          // Pattern: (fdiv:f32 RFP32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
          // Emits: (DIV_Fp32m:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 22  cost = 1  size = 0
          {
            return Emit_0(N, X86::DIV_Fp32m, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fdiv:f32 RFP32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
          // Emits: (DIVR_Fp32m:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 22  cost = 1  size = 0
          return Emit_0(N, X86::DIVR_Fp32m, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fdiv:f32 FR32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
  // Emits: (DIVSSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::DIVSSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((!Subtarget->hasSSE1())) {
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == X86ISD::FILD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N12 = N1.getOperand(2);

          // Pattern: (fdiv:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i16:Other))
          // Emits: (DIV_FpI16m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
            return Emit_79(N, X86::DIV_FpI16m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fdiv:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i32:Other))
          // Emits: (DIV_FpI32m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
            return Emit_79(N, X86::DIV_FpI32m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fdiv:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i16:Other))
          // Emits: (DIVR_FpI16m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
            return Emit_79(N, X86::DIVR_FpI16m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fdiv:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i32:Other))
          // Emits: (DIVR_FpI32m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
            return Emit_79(N, X86::DIVR_FpI32m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fdiv:f32 RFP32:f32:$src1, RFP32:f32:$src2)
    // Emits: (DIV_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 0
    return Emit_4(N, X86::DIV_Fp32, MVT::f32);
  }

  // Pattern: (fdiv:f32 FR32:f32:$src1, FR32:f32:$src2)
  // Emits: (DIVSSrr:f32 FR32:f32:$src1, FR32:f32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    return Emit_4(N, X86::DIVSSrr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FDIV_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);

      // Pattern: (fdiv:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (DIV_Fp64m:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::DIV_Fp64m, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fdiv:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
      // Emits: (DIV_Fp64m32:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::DIV_Fp64m32, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fdiv:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (DIVR_Fp64m:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::DIVR_Fp64m, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fdiv:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
      // Emits: (DIVR_Fp64m32:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::DIVR_Fp64m32, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fdiv:f64 FR64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
  // Emits: (DIVSDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::DIVSDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((!Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == X86ISD::FILD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N12 = N1.getOperand(2);

          // Pattern: (fdiv:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i16:Other))
          // Emits: (DIV_FpI16m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
            return Emit_79(N, X86::DIV_FpI16m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fdiv:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i32:Other))
          // Emits: (DIV_FpI32m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
            return Emit_79(N, X86::DIV_FpI32m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fdiv:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i16:Other))
          // Emits: (DIVR_FpI16m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
            return Emit_79(N, X86::DIVR_FpI16m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fdiv:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i32:Other))
          // Emits: (DIVR_FpI32m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
            return Emit_79(N, X86::DIVR_FpI32m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fdiv:f64 RFP64:f64:$src1, RFP64:f64:$src2)
    // Emits: (DIV_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 0
    return Emit_4(N, X86::DIV_Fp64, MVT::f64);
  }

  // Pattern: (fdiv:f64 FR64:f64:$src1, FR64:f64:$src2)
  // Emits: (DIVSDrr:f64 FR64:f64:$src1, FR64:f64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    return Emit_4(N, X86::DIVSDrr, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FDIV_f80(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);

      // Pattern: (fdiv:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
      // Emits: (DIV_Fp80m32:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::DIV_Fp80m32, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fdiv:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf64>>)
      // Emits: (DIV_Fp80m64:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf64(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::DIV_Fp80m64, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fdiv:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
      // Emits: (DIVR_Fp80m32:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::DIVR_Fp80m32, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fdiv:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf64>>)
      // Emits: (DIVR_Fp80m64:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf64(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::DIVR_Fp80m64, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    if (N1.getOpcode() == X86ISD::FILD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      SDOperand N11 = N1.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        SDOperand N12 = N1.getOperand(2);

        // Pattern: (fdiv:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i16:Other))
        // Emits: (DIV_FpI16m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
          return Emit_79(N, X86::DIV_FpI16m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (fdiv:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i32:Other))
        // Emits: (DIV_FpI32m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
          return Emit_79(N, X86::DIV_FpI32m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (fdiv:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i16:Other))
        // Emits: (DIVR_FpI16m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
          return Emit_79(N, X86::DIVR_FpI16m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (fdiv:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i32:Other))
        // Emits: (DIVR_FpI32m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
          return Emit_79(N, X86::DIVR_FpI32m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fdiv:f80 RFP80:f80:$src1, RFP80:f80:$src2)
  // Emits: (DIV_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
  // Pattern complexity = 3  cost = 1  size = 0
  return Emit_4(N, X86::DIV_Fp80, MVT::f80);
}

SDNode *Select_ISD_FDIV_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (fdiv:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (DIVPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::DIVPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fdiv:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Emits: (DIVPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::DIVPSrr, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FDIV_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (fdiv:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (DIVPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::DIVPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fdiv:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Emits: (DIVPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::DIVPDrr, MVT::v2f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FMUL_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (fmul:f32 RFP32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (MUL_Fp32m:f32 RFP32:f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 0
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MUL_Fp32m, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fmul:f32 (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>, RFP32:f32:$src1)
    // Emits: (MUL_Fp32m:f32 RFP32:f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 0
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::MUL_Fp32m, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (fmul:f32 FR32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (MULSSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MULSSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fmul:f32 (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>, FR32:f32:$src1)
    // Emits: (MULSSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::MULSSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((!Subtarget->hasSSE1())) {
    {
      SDOperand N0 = N.getOperand(0);
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == X86ISD::FILD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N12 = N1.getOperand(2);

            // Pattern: (fmul:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i16:Other))
            // Emits: (MUL_FpI16m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
            // Pattern complexity = 21  cost = 1  size = 0
            if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
              return Emit_79(N, X86::MUL_FpI16m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (fmul:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i32:Other))
            // Emits: (MUL_FpI32m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
            // Pattern complexity = 21  cost = 1  size = 0
            if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
              return Emit_79(N, X86::MUL_FpI32m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
      if (N0.getOpcode() == X86ISD::FILD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N02 = N0.getOperand(2);

          // Pattern: (fmul:f32 (X86fild:f32 addr:iPTR:$src2, i16:Other), RFP32:f32:$src1)
          // Emits: (MUL_FpI16m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N02)->getVT() == MVT::i16) {
            return Emit_80(N, X86::MUL_FpI16m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fmul:f32 (X86fild:f32 addr:iPTR:$src2, i32:Other), RFP32:f32:$src1)
          // Emits: (MUL_FpI32m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N02)->getVT() == MVT::i32) {
            return Emit_80(N, X86::MUL_FpI32m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fmul:f32 RFP32:f32:$src1, RFP32:f32:$src2)
    // Emits: (MUL_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 0
    return Emit_4(N, X86::MUL_Fp32, MVT::f32);
  }

  // Pattern: (fmul:f32 FR32:f32:$src1, FR32:f32:$src2)
  // Emits: (MULSSrr:f32 FR32:f32:$src1, FR32:f32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    return Emit_4(N, X86::MULSSrr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FMUL_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);

        // Pattern: (fmul:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
        // Emits: (MUL_Fp64m:f64 RFP64:f64:$src1, addr:iPTR:$src2)
        // Pattern complexity = 22  cost = 1  size = 0
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MUL_Fp64m, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }

        // Pattern: (fmul:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
        // Emits: (MUL_Fp64m32:f64 RFP64:f64:$src1, addr:iPTR:$src2)
        // Pattern complexity = 22  cost = 1  size = 0
        if (Predicate_extloadf32(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MUL_Fp64m32, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);

      // Pattern: (fmul:f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>, RFP64:f64:$src1)
      // Emits: (MUL_Fp64m:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::MUL_Fp64m, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fmul:f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_extloadf32>>, RFP64:f64:$src1)
      // Emits: (MUL_Fp64m32:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::MUL_Fp64m32, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (fmul:f64 FR64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (MULSDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MULSDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fmul:f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>, FR64:f64:$src1)
    // Emits: (MULSDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::MULSDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((!Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == X86ISD::FILD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N12 = N1.getOperand(2);

            // Pattern: (fmul:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i16:Other))
            // Emits: (MUL_FpI16m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 21  cost = 1  size = 0
            if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
              return Emit_79(N, X86::MUL_FpI16m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (fmul:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i32:Other))
            // Emits: (MUL_FpI32m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 21  cost = 1  size = 0
            if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
              return Emit_79(N, X86::MUL_FpI32m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
      if (N0.getOpcode() == X86ISD::FILD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N02 = N0.getOperand(2);

          // Pattern: (fmul:f64 (X86fild:f64 addr:iPTR:$src2, i16:Other), RFP64:f64:$src1)
          // Emits: (MUL_FpI16m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N02)->getVT() == MVT::i16) {
            return Emit_80(N, X86::MUL_FpI16m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fmul:f64 (X86fild:f64 addr:iPTR:$src2, i32:Other), RFP64:f64:$src1)
          // Emits: (MUL_FpI32m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N02)->getVT() == MVT::i32) {
            return Emit_80(N, X86::MUL_FpI32m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fmul:f64 RFP64:f64:$src1, RFP64:f64:$src2)
    // Emits: (MUL_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 0
    return Emit_4(N, X86::MUL_Fp64, MVT::f64);
  }

  // Pattern: (fmul:f64 FR64:f64:$src1, FR64:f64:$src2)
  // Emits: (MULSDrr:f64 FR64:f64:$src1, FR64:f64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    return Emit_4(N, X86::MULSDrr, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FMUL_f80(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);

        // Pattern: (fmul:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
        // Emits: (MUL_Fp80m32:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 22  cost = 1  size = 0
        if (Predicate_extloadf32(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MUL_Fp80m32, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }

        // Pattern: (fmul:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf64>>)
        // Emits: (MUL_Fp80m64:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 22  cost = 1  size = 0
        if (Predicate_extloadf64(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MUL_Fp80m64, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);

      // Pattern: (fmul:f80 (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf32>>, RFP80:f80:$src1)
      // Emits: (MUL_Fp80m32:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::MUL_Fp80m32, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fmul:f80 (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf64>>, RFP80:f80:$src1)
      // Emits: (MUL_Fp80m64:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf64(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::MUL_Fp80m64, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == X86ISD::FILD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N12 = N1.getOperand(2);

          // Pattern: (fmul:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i16:Other))
          // Emits: (MUL_FpI16m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
            return Emit_79(N, X86::MUL_FpI16m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fmul:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i32:Other))
          // Emits: (MUL_FpI32m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
            return Emit_79(N, X86::MUL_FpI32m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
    if (N0.getOpcode() == X86ISD::FILD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        SDOperand N02 = N0.getOperand(2);

        // Pattern: (fmul:f80 (X86fild:f80 addr:iPTR:$src2, i16:Other), RFP80:f80:$src1)
        // Emits: (MUL_FpI16m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N02)->getVT() == MVT::i16) {
          return Emit_80(N, X86::MUL_FpI16m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (fmul:f80 (X86fild:f80 addr:iPTR:$src2, i32:Other), RFP80:f80:$src1)
        // Emits: (MUL_FpI32m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N02)->getVT() == MVT::i32) {
          return Emit_80(N, X86::MUL_FpI32m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fmul:f80 RFP80:f80:$src1, RFP80:f80:$src2)
  // Emits: (MUL_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
  // Pattern complexity = 3  cost = 1  size = 0
  return Emit_4(N, X86::MUL_Fp80, MVT::f80);
}

SDNode *Select_ISD_FMUL_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (fmul:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (MULPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::MULPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (fmul:v4f32 (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>, VR128:v4f32:$src1)
      // Emits: (MULPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::MULPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fmul:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Emits: (MULPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MULPSrr, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FMUL_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (fmul:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (MULPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::MULPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (fmul:v2f64 (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>, VR128:v2f64:$src1)
      // Emits: (MULPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::MULPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fmul:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Emits: (MULPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MULPDrr, MVT::v2f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FNEG_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {
    return Emit_59(N, X86::CHS_Fp32, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FNEG_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    return Emit_59(N, X86::CHS_Fp64, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FNEG_f80(const SDOperand &N) {
  return Emit_59(N, X86::CHS_Fp80, MVT::f80);
}

SDNode *Select_ISD_FP_EXTEND_f64(const SDOperand &N) {

  // Pattern: (fextend:f64 (ld:f32 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (CVTSS2SDrm:f64 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::f32) {
          return Emit_58(N, X86::CVTSS2SDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fextend:f64 RFP32:f32:$src)
  // Emits: (MOV_Fp3264:f64 RFP32:f32:$src)
  // Pattern complexity = 3  cost = 1  size = 0
  if ((!Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f32) {
      return Emit_59(N, X86::MOV_Fp3264, MVT::f64);
    }
  }

  // Pattern: (fextend:f64 FR32:f32:$src)
  // Emits: (CVTSS2SDrr:f64 FR32:f32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f32) {
      return Emit_59(N, X86::CVTSS2SDrr, MVT::f64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FP_EXTEND_f80(const SDOperand &N) {

  // Pattern: (fextend:f80 RFP32:f32:$src)
  // Emits: (MOV_Fp3280:f80 RFP32:f32:$src)
  // Pattern complexity = 3  cost = 1  size = 0
  if ((!Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f32) {
      return Emit_59(N, X86::MOV_Fp3280, MVT::f80);
    }
  }

  // Pattern: (fextend:f80 RFP64:f64:$src)
  // Emits: (MOV_Fp6480:f80 RFP64:f64:$src)
  // Pattern complexity = 3  cost = 1  size = 0
  if ((!Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f64) {
      return Emit_59(N, X86::MOV_Fp6480, MVT::f80);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FP_ROUND_f32(const SDOperand &N) {

  // Pattern: (fround:f32 (ld:f64 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (CVTSD2SSrm:f32 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::f64) {
          return Emit_58(N, X86::CVTSD2SSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((!Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (fround:f32 RFP64:f64:$src)
    // Emits: (MOV_Fp6432:f32 RFP64:f64:$src)
    // Pattern complexity = 3  cost = 1  size = 0
    if (N0.Val->getValueType(0) == MVT::f64) {
      return Emit_59(N, X86::MOV_Fp6432, MVT::f32);
    }

    // Pattern: (fround:f32 RFP80:f80:$src)
    // Emits: (MOV_Fp8032:f32 RFP80:f80:$src)
    // Pattern complexity = 3  cost = 1  size = 0
    if (N0.Val->getValueType(0) == MVT::f80) {
      return Emit_59(N, X86::MOV_Fp8032, MVT::f32);
    }
  }

  // Pattern: (fround:f32 FR64:f64:$src)
  // Emits: (CVTSD2SSrr:f32 FR64:f64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f64) {
      return Emit_59(N, X86::CVTSD2SSrr, MVT::f32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FP_ROUND_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f80) {
      return Emit_59(N, X86::MOV_Fp8064, MVT::f64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FP_TO_SINT_i32(const SDOperand &N) {

  // Pattern: (fp_to_sint:i32 (ld:f32 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (CVTTSS2SIrm:i32 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::f32) {
          return Emit_58(N, X86::CVTTSS2SIrm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fp_to_sint:i32 (ld:f64 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (CVTTSD2SIrm:i32 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::f64) {
          return Emit_58(N, X86::CVTTSD2SIrm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fp_to_sint:i32 FR32:f32:$src)
  // Emits: (CVTTSS2SIrr:i32 FR32:f32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f32) {
      return Emit_59(N, X86::CVTTSS2SIrr, MVT::i32);
    }
  }

  // Pattern: (fp_to_sint:i32 FR64:f64:$src)
  // Emits: (CVTTSD2SIrr:i32 FR64:f64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f64) {
      return Emit_59(N, X86::CVTTSD2SIrr, MVT::i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FP_TO_SINT_i64(const SDOperand &N) {

  // Pattern: (fp_to_sint:i64 (ld:f64 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (CVTTSD2SI64rm:i64 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::f64) {
          return Emit_58(N, X86::CVTTSD2SI64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fp_to_sint:i64 (ld:f32 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (CVTTSS2SI64rm:i64 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::f32) {
          return Emit_58(N, X86::CVTTSS2SI64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fp_to_sint:i64 FR64:f64:$src)
  // Emits: (CVTTSD2SI64rr:i64 FR64:f64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f64) {
      return Emit_59(N, X86::CVTTSD2SI64rr, MVT::i64);
    }
  }

  // Pattern: (fp_to_sint:i64 FR32:f32:$src)
  // Emits: (CVTTSS2SI64rr:i64 FR32:f32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::f32) {
      return Emit_59(N, X86::CVTTSS2SI64rr, MVT::i64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FP_TO_SINT_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::v4f32) {
      return Emit_59(N, X86::Int_CVTTPS2DQrr, MVT::v4i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FSIN_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {
    return Emit_59(N, X86::SIN_Fp32, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FSIN_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    return Emit_59(N, X86::SIN_Fp64, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FSIN_f80(const SDOperand &N) {
  return Emit_59(N, X86::SIN_Fp80, MVT::f80);
}

SDNode *Select_ISD_FSQRT_f32(const SDOperand &N) {

  // Pattern: (fsqrt:f32 (ld:f32 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (SQRTSSm:f32 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_58(N, X86::SQRTSSm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fsqrt:f32 RFP32:f32:$src)
  // Emits: (SQRT_Fp32:f32 RFP32:f32:$src)
  // Pattern complexity = 3  cost = 1  size = 0
  if ((!Subtarget->hasSSE1())) {
    return Emit_59(N, X86::SQRT_Fp32, MVT::f32);
  }

  // Pattern: (fsqrt:f32 FR32:f32:$src)
  // Emits: (SQRTSSr:f32 FR32:f32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    return Emit_59(N, X86::SQRTSSr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FSQRT_f64(const SDOperand &N) {

  // Pattern: (fsqrt:f64 (ld:f64 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (SQRTSDm:f64 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_58(N, X86::SQRTSDm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fsqrt:f64 RFP64:f64:$src)
  // Emits: (SQRT_Fp64:f64 RFP64:f64:$src)
  // Pattern complexity = 3  cost = 1  size = 0
  if ((!Subtarget->hasSSE2())) {
    return Emit_59(N, X86::SQRT_Fp64, MVT::f64);
  }

  // Pattern: (fsqrt:f64 FR64:f64:$src)
  // Emits: (SQRTSDr:f64 FR64:f64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    return Emit_59(N, X86::SQRTSDr, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FSQRT_f80(const SDOperand &N) {
  return Emit_59(N, X86::SQRT_Fp80, MVT::f80);
}

SDNode *Select_ISD_FSQRT_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (fsqrt:v4f32 (ld:v4f32 addr:iPTR:$src)<<P:Predicate_memop>>)
    // Emits: (SQRTPSm:v4f32 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse()) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_58(N, X86::SQRTPSm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fsqrt:v4f32 VR128:v4f32:$src)
    // Emits: (SQRTPSr:v4f32 VR128:v4f32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_59(N, X86::SQRTPSr, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FSQRT_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (fsqrt:v2f64 (ld:v2f64 addr:iPTR:$src)<<P:Predicate_memop>>)
    // Emits: (SQRTPDm:v2f64 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse()) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_58(N, X86::SQRTPDm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fsqrt:v2f64 VR128:v2f64:$src)
    // Emits: (SQRTPDr:v2f64 VR128:v2f64:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_59(N, X86::SQRTPDr, MVT::v2f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FSUB_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

          // Pattern: (fsub:f32 RFP32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
          // Emits: (SUB_Fp32m:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 22  cost = 1  size = 0
          {
            return Emit_0(N, X86::SUB_Fp32m, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fsub:f32 RFP32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
          // Emits: (SUBR_Fp32m:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 22  cost = 1  size = 0
          return Emit_0(N, X86::SUBR_Fp32m, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fsub:f32 FR32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
  // Emits: (SUBSSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::SUBSSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((!Subtarget->hasSSE1())) {
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == X86ISD::FILD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N12 = N1.getOperand(2);

          // Pattern: (fsub:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i16:Other))
          // Emits: (SUB_FpI16m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
            return Emit_79(N, X86::SUB_FpI16m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fsub:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i32:Other))
          // Emits: (SUB_FpI32m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
            return Emit_79(N, X86::SUB_FpI32m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fsub:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i16:Other))
          // Emits: (SUBR_FpI16m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
            return Emit_79(N, X86::SUBR_FpI16m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fsub:f32 RFP32:f32:$src1, (X86fild:f32 addr:iPTR:$src2, i32:Other))
          // Emits: (SUBR_FpI32m32:f32 RFP32:f32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
            return Emit_79(N, X86::SUBR_FpI32m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fsub:f32 RFP32:f32:$src1, RFP32:f32:$src2)
    // Emits: (SUB_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 0
    return Emit_4(N, X86::SUB_Fp32, MVT::f32);
  }

  // Pattern: (fsub:f32 FR32:f32:$src1, FR32:f32:$src2)
  // Emits: (SUBSSrr:f32 FR32:f32:$src1, FR32:f32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    return Emit_4(N, X86::SUBSSrr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FSUB_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);

      // Pattern: (fsub:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (SUB_Fp64m:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::SUB_Fp64m, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fsub:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
      // Emits: (SUB_Fp64m32:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::SUB_Fp64m32, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fsub:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (SUBR_Fp64m:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::SUBR_Fp64m, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fsub:f64 RFP64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
      // Emits: (SUBR_Fp64m32:f64 RFP64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::SUBR_Fp64m32, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fsub:f64 FR64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
  // Emits: (SUBSDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::SUBSDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((!Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == X86ISD::FILD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N12 = N1.getOperand(2);

          // Pattern: (fsub:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i16:Other))
          // Emits: (SUB_FpI16m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
            return Emit_79(N, X86::SUB_FpI16m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fsub:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i32:Other))
          // Emits: (SUB_FpI32m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
            return Emit_79(N, X86::SUB_FpI32m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fsub:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i16:Other))
          // Emits: (SUBR_FpI16m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
            return Emit_79(N, X86::SUBR_FpI16m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (fsub:f64 RFP64:f64:$src1, (X86fild:f64 addr:iPTR:$src2, i32:Other))
          // Emits: (SUBR_FpI32m64:f64 RFP64:f64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 21  cost = 1  size = 0
          if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
            return Emit_79(N, X86::SUBR_FpI32m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fsub:f64 RFP64:f64:$src1, RFP64:f64:$src2)
    // Emits: (SUB_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 0
    return Emit_4(N, X86::SUB_Fp64, MVT::f64);
  }

  // Pattern: (fsub:f64 FR64:f64:$src1, FR64:f64:$src2)
  // Emits: (SUBSDrr:f64 FR64:f64:$src1, FR64:f64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    return Emit_4(N, X86::SUBSDrr, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FSUB_f80(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);

      // Pattern: (fsub:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
      // Emits: (SUB_Fp80m32:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::SUB_Fp80m32, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fsub:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf64>>)
      // Emits: (SUB_Fp80m64:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf64(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::SUB_Fp80m64, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fsub:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf32>>)
      // Emits: (SUBR_Fp80m32:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf32(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::SUBR_Fp80m32, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (fsub:f80 RFP80:f80:$src1, (ld:f80 addr:iPTR:$src2)<<P:Predicate_extloadf64>>)
      // Emits: (SUBR_Fp80m64:f80 RFP80:f80:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 0
      if (Predicate_extloadf64(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_0(N, X86::SUBR_Fp80m64, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    if (N1.getOpcode() == X86ISD::FILD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      SDOperand N11 = N1.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        SDOperand N12 = N1.getOperand(2);

        // Pattern: (fsub:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i16:Other))
        // Emits: (SUB_FpI16m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
          return Emit_79(N, X86::SUB_FpI16m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (fsub:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i32:Other))
        // Emits: (SUB_FpI32m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
          return Emit_79(N, X86::SUB_FpI32m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (fsub:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i16:Other))
        // Emits: (SUBR_FpI16m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N12)->getVT() == MVT::i16) {
          return Emit_79(N, X86::SUBR_FpI16m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (fsub:f80 RFP80:f80:$src1, (X86fild:f80 addr:iPTR:$src2, i32:Other))
        // Emits: (SUBR_FpI32m80:f80 RFP80:f80:$src1, addr:iPTR:$src2)
        // Pattern complexity = 21  cost = 1  size = 0
        if (cast<VTSDNode>(N12)->getVT() == MVT::i32) {
          return Emit_79(N, X86::SUBR_FpI32m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (fsub:f80 RFP80:f80:$src1, RFP80:f80:$src2)
  // Emits: (SUB_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
  // Pattern complexity = 3  cost = 1  size = 0
  return Emit_4(N, X86::SUB_Fp80, MVT::f80);
}

SDNode *Select_ISD_FSUB_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (fsub:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (SUBPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::SUBPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fsub:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Emits: (SUBPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::SUBPSrr, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FSUB_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (fsub:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (SUBPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::SUBPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (fsub:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Emits: (SUBPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::SUBPDrr, MVT::v2f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FrameIndex_i32(const SDOperand &N) {

  // Pattern: lea32addr:i32:$src
  // Emits: (LEA64_32r:i32 lea32addr:i32:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  if ((Subtarget->is64Bit())) {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA64_32r, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: lea32addr:i32:$src
  // Emits: (LEA32r:i32 lea32addr:i32:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  if ((!Subtarget->is64Bit())) {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA32r, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_FrameIndex_i64(const SDOperand &N) {
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_6(N, X86::LEA64r, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_81(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N2)->getValue()), MVT::i32);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 8);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_82(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N2)->getValue()), MVT::i32);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N1, Tmp2);
}
SDNode *Select_ISD_INSERT_VECTOR_ELT_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (insertelt:v4i32 VR128:v4i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, (imm:iPTR):$src3)
    // Emits: (PINSRDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2, (imm:i32):$src3)
    // Pattern complexity = 25  cost = 1  size = 3
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::Constant) {
            return Emit_81(N, X86::PINSRDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (insertelt:v4i32 VR128:v4i32:$src1, GR32:i32:$src2, (imm:iPTR):$src3)
    // Emits: (PINSRDrr:v4i32 VR128:v4i32:$src1, GR32:i32:$src2, (imm:i32):$src3)
    // Pattern complexity = 6  cost = 1  size = 3
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant) {
      return Emit_82(N, X86::PINSRDrr, MVT::v4i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_INSERT_VECTOR_ELT_v2i64(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (insertelt:v2i64 VR128:v2i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, (imm:iPTR):$src3)
    // Emits: (PINSRQrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2, (imm:i32):$src3)
    // Pattern complexity = 25  cost = 1  size = 3
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::Constant) {
            return Emit_81(N, X86::PINSRQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (insertelt:v2i64 VR128:v2i64:$src1, GR64:i64:$src2, (imm:iPTR):$src3)
    // Emits: (PINSRQrr:v2i64 VR128:v2i64:$src1, GR64:i64:$src2, (imm:i32):$src3)
    // Pattern complexity = 6  cost = 1  size = 3
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant) {
      return Emit_82(N, X86::PINSRQrr, MVT::v2i64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_83(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N3);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 6);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_84(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 5);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_85(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Chain);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_86(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  SDOperand N4 = N.getOperand(4);
  AddToISelQueue(N2);
  AddToISelQueue(N3);
  AddToISelQueue(Chain);
  AddToISelQueue(N4);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::EDI, N4, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { N2, N3, Chain, InFlag };
  ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 4);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_87(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  SDOperand N4 = N.getOperand(4);
  AddToISelQueue(Chain);
  AddToISelQueue(N2);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::EAX, N2, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  AddToISelQueue(N3);
  ResNode = CurDAG->getCopyToReg(Chain, X86::ECX, N3, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  AddToISelQueue(N4);
  ResNode = CurDAG->getCopyToReg(Chain, X86::EDX, N4, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Chain, InFlag);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_88(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(Chain);
  AddToISelQueue(N2);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::ECX, N2, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  AddToISelQueue(N3);
  ResNode = CurDAG->getCopyToReg(Chain, X86::EAX, N3, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Chain, InFlag);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_ISD_INTRINSIC_VOID(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_void:void 555:iPTR, addr:iPTR:$dst, VR128:v4f32:$src)
      // Emits: (MOVUPSmr_Int:void addr:iPTR:$dst, VR128:v4f32:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 555) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_83(N, X86::MOVUPSmr_Int, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_void:void 545:iPTR, addr:iPTR:$dst, VR128:v4f32:$src)
      // Emits: (MOVNTPSmr:void addr:iPTR:$dst, VR128:v4f32:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 545) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_83(N, X86::MOVNTPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_void:void 538:iPTR, addr:iPTR:$src)
      // Emits: (LDMXCSR:void addr:iPTR:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 538) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_84(N, X86::LDMXCSR, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_void:void 554:iPTR, addr:iPTR:$dst)
      // Emits: (STMXCSR:void addr:iPTR:$dst)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 554) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_84(N, X86::STMXCSR, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_void:void 450:iPTR, addr:iPTR:$dst, VR128:v2f64:$src)
      // Emits: (MOVUPDmr_Int:void addr:iPTR:$dst, VR128:v2f64:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 450) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_83(N, X86::MOVUPDmr_Int, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_void:void 449:iPTR, addr:iPTR:$dst, VR128:v16i8:$src)
      // Emits: (MOVDQUmr_Int:void addr:iPTR:$dst, VR128:v16i8:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 449) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_83(N, X86::MOVDQUmr_Int, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_void:void 393:iPTR, addr:iPTR:$dst, VR128:v2f64:$src)
      // Emits: (MOVNTPDmr:void addr:iPTR:$dst, VR128:v2f64:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 393) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_83(N, X86::MOVNTPDmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_void:void 391:iPTR, addr:iPTR:$dst, VR128:v2i64:$src)
      // Emits: (MOVNTDQmr:void addr:iPTR:$dst, VR128:v2i64:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 391) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_83(N, X86::MOVNTDQmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_void:void 392:iPTR, addr:iPTR:$dst, GR32:i32:$src)
      // Emits: (MOVNTImr:void addr:iPTR:$dst, GR32:i32:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 392) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_83(N, X86::MOVNTImr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_void:void 353:iPTR, addr:iPTR:$src)
      // Emits: (CLFLUSH:void addr:iPTR:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 353) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_84(N, X86::CLFLUSH, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_void:void 448:iPTR, addr:iPTR:$dst, VR128:v4i32:$src)
      // Emits: (MOVLQ128mr:void addr:iPTR:$dst, VR128:v4i32:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 448) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_83(N, X86::MOVLQ128mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (intrinsic_void:void 306:iPTR, addr:iPTR:$dst, VR64:v1i64:$src)
  // Emits: (MMX_MOVNTQmr:void addr:iPTR:$dst, VR64:v1i64:$src)
  // Pattern complexity = 23  cost = 1  size = 3
  if ((Subtarget->hasMMX())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
      if (CN0 == 306) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_83(N, X86::MMX_MOVNTQmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (intrinsic_void:void 551:iPTR)
  // Emits: (SFENCE:void)
  // Pattern complexity = 8  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
      if (CN0 == 551) {
        return Emit_85(N, X86::SFENCE);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_void:void 384:iPTR, VR128:v16i8:$src, VR128:v16i8:$mask, EDI:i32)
      // Emits: (MASKMOVDQU:void VR128:v16i8:$src, VR128:v16i8:$mask)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 384) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand N4 = N.getOperand(4);
        if (N4.Val->getValueType(0) == MVT::i32) {
          return Emit_86(N, X86::MASKMOVDQU);
        }
      }

      // Pattern: (intrinsic_void:void 379:iPTR)
      // Emits: (LFENCE:void)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 379) {
        return Emit_85(N, X86::LFENCE);
      }

      // Pattern: (intrinsic_void:void 387:iPTR)
      // Emits: (MFENCE:void)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 387) {
        return Emit_85(N, X86::MFENCE);
      }
    }
  }
  if ((Subtarget->hasSSE3())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_void:void 467:iPTR, EAX:i32, ECX:i32, EDX:i32)
      // Emits: (MONITOR:void)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 467) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand N4 = N.getOperand(4);
        if (N2.Val->getValueType(0) == MVT::i32) {
          return Emit_87(N, X86::MONITOR);
        }
      }

      // Pattern: (intrinsic_void:void 468:iPTR, ECX:i32, EAX:i32)
      // Emits: (MWAIT:void)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 468) {
        return Emit_88(N, X86::MWAIT);
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_void:void 303:iPTR)
      // Emits: (MMX_EMMS:void)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 303) {
        return Emit_85(N, X86::MMX_EMMS);
      }

      // Pattern: (intrinsic_void:void 304:iPTR)
      // Emits: (MMX_FEMMS:void)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 304) {
        return Emit_85(N, X86::MMX_FEMMS);
      }

      // Pattern: (intrinsic_void:void 305:iPTR, VR64:v8i8:$src, VR64:v8i8:$mask, EDI:i32)
      // Emits: (MMX_MASKMOVQ:void VR64:v8i8:$src, VR64:v8i8:$mask)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 305) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand N4 = N.getOperand(4);
        if (N4.Val->getValueType(0) == MVT::i32) {
          return Emit_86(N, X86::MMX_MASKMOVQ);
        }
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_89(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_90(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N1);
}
SDNode *Select_ISD_INTRINSIC_WO_CHAIN_i32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:i32 531:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTSS2SIrm:i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 531) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTSS2SIrm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:i32 535:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTTSS2SIrm:i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 535) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTTSS2SIrm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:i32 368:iPTR, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTSD2SIrm:i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 368) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTSD2SIrm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:i32 376:iPTR, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTTSD2SIrm:i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 376) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTTSD2SIrm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:i32 531:iPTR, VR128:v4f32:$src)
      // Emits: (Int_CVTSS2SIrr:i32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 531) {
        return Emit_90(N, X86::Int_CVTSS2SIrr, MVT::i32);
      }

      // Pattern: (intrinsic_wo_chain:i32 535:iPTR, VR128:v4f32:$src)
      // Emits: (Int_CVTTSS2SIrr:i32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 535) {
        return Emit_90(N, X86::Int_CVTTSS2SIrr, MVT::i32);
      }

      // Pattern: (intrinsic_wo_chain:i32 544:iPTR, VR128:v4f32:$src)
      // Emits: (MOVMSKPSrr:i32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 544) {
        return Emit_90(N, X86::MOVMSKPSrr, MVT::i32);
      }

      // Pattern: (intrinsic_wo_chain:i32 390:iPTR, VR128:v2f64:$src)
      // Emits: (MOVMSKPDrr:i32 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 390) {
        return Emit_90(N, X86::MOVMSKPDrr, MVT::i32);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:i32 368:iPTR, VR128:v2f64:$src)
      // Emits: (Int_CVTSD2SIrr:i32 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 368) {
        return Emit_90(N, X86::Int_CVTSD2SIrr, MVT::i32);
      }

      // Pattern: (intrinsic_wo_chain:i32 376:iPTR, VR128:v2f64:$src)
      // Emits: (Int_CVTTSD2SIrr:i32 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 376) {
        return Emit_90(N, X86::Int_CVTTSD2SIrr, MVT::i32);
      }

      // Pattern: (intrinsic_wo_chain:i32 416:iPTR, VR128:v16i8:$src)
      // Emits: (PMOVMSKBrr:i32 VR128:v16i8:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 416) {
        return Emit_90(N, X86::PMOVMSKBrr, MVT::i32);
      }
    }
  }

  // Pattern: (intrinsic_wo_chain:i32 327:iPTR, VR64:v8i8:$src)
  // Emits: (MMX_PMOVMSKBrr:i32 VR64:v8i8:$src)
  // Pattern complexity = 8  cost = 1  size = 3
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 327) {
        return Emit_90(N, X86::MMX_PMOVMSKBrr, MVT::i32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_INTRINSIC_WO_CHAIN_i64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:i64 369:iPTR, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTSD2SI64rm:i64 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 369) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTSD2SI64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:i64 377:iPTR, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTTSD2SI64rm:i64 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 377) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTTSD2SI64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:i64 532:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTSS2SI64rm:i64 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 532) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTSS2SI64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:i64 536:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTTSS2SI64rm:i64 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 536) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTTSS2SI64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:i64 369:iPTR, VR128:v2f64:$src)
      // Emits: (Int_CVTSD2SI64rr:i64 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 369) {
        return Emit_90(N, X86::Int_CVTSD2SI64rr, MVT::i64);
      }

      // Pattern: (intrinsic_wo_chain:i64 377:iPTR, VR128:v2f64:$src)
      // Emits: (Int_CVTTSD2SI64rr:i64 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 377) {
        return Emit_90(N, X86::Int_CVTTSD2SI64rr, MVT::i64);
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:i64 532:iPTR, VR128:v4f32:$src)
      // Emits: (Int_CVTSS2SI64rr:i64 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 532) {
        return Emit_90(N, X86::Int_CVTSS2SI64rr, MVT::i64);
      }

      // Pattern: (intrinsic_wo_chain:i64 536:iPTR, VR128:v4f32:$src)
      // Emits: (Int_CVTTSS2SI64rr:i64 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 536) {
        return Emit_90(N, X86::Int_CVTTSS2SI64rr, MVT::i64);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_91(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain10 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_92(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N20 = N2.getOperand(0);
  SDOperand Chain20 = N20.getOperand(0);
  SDOperand N201 = N20.getOperand(1);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N20 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N20)->getMemOperand());
  AddToISelQueue(Chain20);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N20, Chain20 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain20 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N20.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_93(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N1);
  AddToISelQueue(N2);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N1, N2);
}
SDNode *Select_ISD_INTRINSIC_WO_CHAIN_v8i8(const SDOperand &N) {
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v8i8 563:iPTR, (bitconvert:v8i8 (ld:v8i8 addr:iPTR:$src)<<P:Predicate_memop64>>))
      // Emits: (PABSBrm64:v8i8 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 563) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop64(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v8i8) {
                return Emit_91(N, X86::PABSBrm64, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 587:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v8i8 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PSHUFBrm64:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 587) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v8i8) {
                return Emit_92(N, X86::PSHUFBrm64, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 589:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v8i8 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PSIGNBrm64:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 589) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v8i8) {
                return Emit_92(N, X86::PSIGNBrm64, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v8i8 310:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PADDSBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 310) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PADDSBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 312:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PADDUSBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 312) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PADDUSBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 348:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSUBSBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 348) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSUBSBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 350:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSUBUSBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 350) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSUBUSBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 314:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PAVGBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 314) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PAVGBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 326:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PMINUBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 326) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PMINUBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 324:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PMAXUBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 324) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PMAXUBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 316:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PCMPEQBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 316) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PCMPEQBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 319:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PCMPGTBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 319) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PCMPGTBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 308:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PACKSSWBrm:v8i8 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 308) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PACKSSWBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i8 309:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PACKUSWBrm:v8i8 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 309) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PACKUSWBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v8i8 563:iPTR, VR64:v8i8:$src)
      // Emits: (PABSBrr64:v8i8 VR64:v8i8:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 563) {
        return Emit_90(N, X86::PABSBrr64, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 587:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (PSHUFBrr64:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 587) {
        return Emit_93(N, X86::PSHUFBrr64, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 589:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (PSIGNBrr64:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 589) {
        return Emit_93(N, X86::PSIGNBrr64, MVT::v8i8);
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v8i8 310:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (MMX_PADDSBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 310) {
        return Emit_93(N, X86::MMX_PADDSBrr, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 312:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (MMX_PADDUSBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 312) {
        return Emit_93(N, X86::MMX_PADDUSBrr, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 348:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (MMX_PSUBSBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 348) {
        return Emit_93(N, X86::MMX_PSUBSBrr, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 350:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (MMX_PSUBUSBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 350) {
        return Emit_93(N, X86::MMX_PSUBUSBrr, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 314:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (MMX_PAVGBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 314) {
        return Emit_93(N, X86::MMX_PAVGBrr, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 326:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (MMX_PMINUBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 326) {
        return Emit_93(N, X86::MMX_PMINUBrr, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 324:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (MMX_PMAXUBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 324) {
        return Emit_93(N, X86::MMX_PMAXUBrr, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 316:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (MMX_PCMPEQBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 316) {
        return Emit_93(N, X86::MMX_PCMPEQBrr, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 319:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (MMX_PCMPGTBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 319) {
        return Emit_93(N, X86::MMX_PCMPGTBrr, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 308:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PACKSSWBrr:v8i8 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 308) {
        return Emit_93(N, X86::MMX_PACKSSWBrr, MVT::v8i8);
      }

      // Pattern: (intrinsic_wo_chain:v8i8 309:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PACKUSWBrr:v8i8 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 309) {
        return Emit_93(N, X86::MMX_PACKUSWBrr, MVT::v8i8);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_94(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N1);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N2)->getValue()), MVT::i32);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N1, Tmp2);
}
SDNode *Select_ISD_INTRINSIC_WO_CHAIN_v4i16(const SDOperand &N) {
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4i16 567:iPTR, (bitconvert:v4i16 (ld:v4i16 addr:iPTR:$src)<<P:Predicate_memop64>>))
      // Emits: (PABSWrm64:v4i16 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 567) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop64(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v4i16) {
                return Emit_91(N, X86::PABSWrm64, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 575:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v4i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PHADDWrm64:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 575) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i16) {
                return Emit_92(N, X86::PHADDWrm64, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 573:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v4i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PHADDSWrm64:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 573) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i16) {
                return Emit_92(N, X86::PHADDSWrm64, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 581:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v4i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PHSUBWrm64:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 581) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i16) {
                return Emit_92(N, X86::PHSUBWrm64, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 579:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v4i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PHSUBSWrm64:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 579) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i16) {
                return Emit_92(N, X86::PHSUBSWrm64, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 583:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v8i8 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PMADDUBSWrm64:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 583) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v8i8) {
                return Emit_92(N, X86::PMADDUBSWrm64, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 585:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v4i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PMULHRSWrm64:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 585) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i16) {
                return Emit_92(N, X86::PMULHRSWrm64, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 593:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v4i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PSIGNWrm64:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 593) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i16) {
                return Emit_92(N, X86::PSIGNWrm64, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4i16 311:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PADDSWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 311) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PADDSWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 313:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PADDUSWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 313) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PADDUSWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 349:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSUBSWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 349) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSUBSWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 351:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSUBUSWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 351) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSUBUSWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 328:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PMULHWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 328) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PMULHWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 329:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PMULHUWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 329) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PMULHUWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 315:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PAVGWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 315) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PAVGWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 325:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PMINSWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 325) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PMINSWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 323:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PMAXSWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 323) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PMAXSWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 331:iPTR, VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSADBWrm:v4i16 VR64:v8i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 331) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSADBWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 344:iPTR, VR64:v4i16:$src1, (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSRLWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 344) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSRLWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 334:iPTR, VR64:v4i16:$src1, (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSLLWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 334) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSLLWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 339:iPTR, VR64:v4i16:$src1, (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSRAWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 339) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSRAWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 318:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PCMPEQWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 318) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PCMPEQWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 321:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PCMPGTWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 321) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PCMPGTWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 307:iPTR, VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PACKSSDWrm:v4i16 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 307) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PACKSSDWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 347:iPTR, VR64:v4i16:$src1, (imm:i32):$src2)
      // Emits: (MMX_PSRLWri:v4i16 VR64:v4i16:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 347) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::MMX_PSRLWri, MVT::v4i16);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 337:iPTR, VR64:v4i16:$src1, (imm:i32):$src2)
      // Emits: (MMX_PSLLWri:v4i16 VR64:v4i16:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 337) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::MMX_PSLLWri, MVT::v4i16);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i16 341:iPTR, VR64:v4i16:$src1, (imm:i32):$src2)
      // Emits: (MMX_PSRAWri:v4i16 VR64:v4i16:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 341) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::MMX_PSRAWri, MVT::v4i16);
        }
      }
    }
  }
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4i16 567:iPTR, VR64:v4i16:$src)
      // Emits: (PABSWrr64:v4i16 VR64:v4i16:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 567) {
        return Emit_90(N, X86::PABSWrr64, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 575:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (PHADDWrr64:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 575) {
        return Emit_93(N, X86::PHADDWrr64, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 573:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (PHADDSWrr64:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 573) {
        return Emit_93(N, X86::PHADDSWrr64, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 581:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (PHSUBWrr64:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 581) {
        return Emit_93(N, X86::PHSUBWrr64, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 579:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (PHSUBSWrr64:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 579) {
        return Emit_93(N, X86::PHSUBSWrr64, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 583:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (PMADDUBSWrr64:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 583) {
        return Emit_93(N, X86::PMADDUBSWrr64, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 585:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (PMULHRSWrr64:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 585) {
        return Emit_93(N, X86::PMULHRSWrr64, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 593:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (PSIGNWrr64:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 593) {
        return Emit_93(N, X86::PSIGNWrr64, MVT::v4i16);
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4i16 311:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PADDSWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 311) {
        return Emit_93(N, X86::MMX_PADDSWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 313:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PADDUSWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 313) {
        return Emit_93(N, X86::MMX_PADDUSWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 349:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PSUBSWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 349) {
        return Emit_93(N, X86::MMX_PSUBSWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 351:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PSUBUSWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 351) {
        return Emit_93(N, X86::MMX_PSUBUSWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 328:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PMULHWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 328) {
        return Emit_93(N, X86::MMX_PMULHWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 329:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PMULHUWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 329) {
        return Emit_93(N, X86::MMX_PMULHUWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 315:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PAVGWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 315) {
        return Emit_93(N, X86::MMX_PAVGWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 325:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PMINSWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 325) {
        return Emit_93(N, X86::MMX_PMINSWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 323:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PMAXSWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 323) {
        return Emit_93(N, X86::MMX_PMAXSWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 331:iPTR, VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Emits: (MMX_PSADBWrr:v4i16 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 331) {
        return Emit_93(N, X86::MMX_PSADBWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 344:iPTR, VR64:v4i16:$src1, VR64:v1i64:$src2)
      // Emits: (MMX_PSRLWrr:v4i16 VR64:v4i16:$src1, VR64:v1i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 344) {
        return Emit_93(N, X86::MMX_PSRLWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 334:iPTR, VR64:v4i16:$src1, VR64:v1i64:$src2)
      // Emits: (MMX_PSLLWrr:v4i16 VR64:v4i16:$src1, VR64:v1i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 334) {
        return Emit_93(N, X86::MMX_PSLLWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 339:iPTR, VR64:v4i16:$src1, VR64:v1i64:$src2)
      // Emits: (MMX_PSRAWrr:v4i16 VR64:v4i16:$src1, VR64:v1i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 339) {
        return Emit_93(N, X86::MMX_PSRAWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 318:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PCMPEQWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 318) {
        return Emit_93(N, X86::MMX_PCMPEQWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 321:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PCMPGTWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 321) {
        return Emit_93(N, X86::MMX_PCMPGTWrr, MVT::v4i16);
      }

      // Pattern: (intrinsic_wo_chain:v4i16 307:iPTR, VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Emits: (MMX_PACKSSDWrr:v4i16 VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 307) {
        return Emit_93(N, X86::MMX_PACKSSDWrr, MVT::v4i16);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_INTRINSIC_WO_CHAIN_v2i32(const SDOperand &N) {
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i32 565:iPTR, (bitconvert:v2i32 (ld:v2i32 addr:iPTR:$src)<<P:Predicate_memop64>>))
      // Emits: (PABSDrm64:v2i32 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 565) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop64(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v2i32) {
                return Emit_91(N, X86::PABSDrm64, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 571:iPTR, VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v2i32 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PHADDDrm64:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 571) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i32) {
                return Emit_92(N, X86::PHADDDrm64, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 577:iPTR, VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v2i32 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PHSUBDrm64:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 577) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i32) {
                return Emit_92(N, X86::PHSUBDrm64, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 591:iPTR, VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v2i32 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PSIGNDrm64:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 591) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i32) {
                return Emit_92(N, X86::PSIGNDrm64, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i32 330:iPTR, VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PMULUDQrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 330) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PMULUDQrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 322:iPTR, VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PMADDWDrm:v2i32 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 322) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PMADDWDrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 342:iPTR, VR64:v2i32:$src1, (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSRLDrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 342) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSRLDrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 332:iPTR, VR64:v2i32:$src1, (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSLLDrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 332) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSLLDrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 338:iPTR, VR64:v2i32:$src1, (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSRADrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 338) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSRADrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 317:iPTR, VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PCMPEQDrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 317) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PCMPEQDrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 320:iPTR, VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PCMPGTDrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 320) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PCMPGTDrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i32 528:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTPS2PIrm:v2i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 528) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTPS2PIrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 534:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTTPS2PIrm:v2i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 534) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTTPS2PIrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i32 525:iPTR, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTPD2PIrm:v2i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 525) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTPD2PIrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 533:iPTR, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTTPD2PIrm:v2i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 533) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTTPD2PIrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i32 345:iPTR, VR64:v2i32:$src1, (imm:i32):$src2)
      // Emits: (MMX_PSRLDri:v2i32 VR64:v2i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 345) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::MMX_PSRLDri, MVT::v2i32);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 335:iPTR, VR64:v2i32:$src1, (imm:i32):$src2)
      // Emits: (MMX_PSLLDri:v2i32 VR64:v2i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 335) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::MMX_PSLLDri, MVT::v2i32);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i32 340:iPTR, VR64:v2i32:$src1, (imm:i32):$src2)
      // Emits: (MMX_PSRADri:v2i32 VR64:v2i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 340) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::MMX_PSRADri, MVT::v2i32);
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i32 528:iPTR, VR128:v4f32:$src)
      // Emits: (Int_CVTPS2PIrr:v2i32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 528) {
        return Emit_90(N, X86::Int_CVTPS2PIrr, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 534:iPTR, VR128:v4f32:$src)
      // Emits: (Int_CVTTPS2PIrr:v2i32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 534) {
        return Emit_90(N, X86::Int_CVTTPS2PIrr, MVT::v2i32);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i32 525:iPTR, VR128:v2f64:$src)
      // Emits: (Int_CVTPD2PIrr:v2i32 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 525) {
        return Emit_90(N, X86::Int_CVTPD2PIrr, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 533:iPTR, VR128:v2f64:$src)
      // Emits: (Int_CVTTPD2PIrr:v2i32 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 533) {
        return Emit_90(N, X86::Int_CVTTPD2PIrr, MVT::v2i32);
      }
    }
  }
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i32 565:iPTR, VR64:v2i32:$src)
      // Emits: (PABSDrr64:v2i32 VR64:v2i32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 565) {
        return Emit_90(N, X86::PABSDrr64, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 571:iPTR, VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Emits: (PHADDDrr64:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 571) {
        return Emit_93(N, X86::PHADDDrr64, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 577:iPTR, VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Emits: (PHSUBDrr64:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 577) {
        return Emit_93(N, X86::PHSUBDrr64, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 591:iPTR, VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Emits: (PSIGNDrr64:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 591) {
        return Emit_93(N, X86::PSIGNDrr64, MVT::v2i32);
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i32 330:iPTR, VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Emits: (MMX_PMULUDQrr:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 330) {
        return Emit_93(N, X86::MMX_PMULUDQrr, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 322:iPTR, VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Emits: (MMX_PMADDWDrr:v2i32 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 322) {
        return Emit_93(N, X86::MMX_PMADDWDrr, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 342:iPTR, VR64:v2i32:$src1, VR64:v1i64:$src2)
      // Emits: (MMX_PSRLDrr:v2i32 VR64:v2i32:$src1, VR64:v1i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 342) {
        return Emit_93(N, X86::MMX_PSRLDrr, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 332:iPTR, VR64:v2i32:$src1, VR64:v1i64:$src2)
      // Emits: (MMX_PSLLDrr:v2i32 VR64:v2i32:$src1, VR64:v1i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 332) {
        return Emit_93(N, X86::MMX_PSLLDrr, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 338:iPTR, VR64:v2i32:$src1, VR64:v1i64:$src2)
      // Emits: (MMX_PSRADrr:v2i32 VR64:v2i32:$src1, VR64:v1i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 338) {
        return Emit_93(N, X86::MMX_PSRADrr, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 317:iPTR, VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Emits: (MMX_PCMPEQDrr:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 317) {
        return Emit_93(N, X86::MMX_PCMPEQDrr, MVT::v2i32);
      }

      // Pattern: (intrinsic_wo_chain:v2i32 320:iPTR, VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Emits: (MMX_PCMPGTDrr:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 320) {
        return Emit_93(N, X86::MMX_PCMPGTDrr, MVT::v2i32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_95(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N20 = N2.getOperand(0);
  SDOperand Chain20 = N20.getOperand(0);
  SDOperand N201 = N20.getOperand(1);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp3 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N3)->getValue()), MVT::i16);
  SDOperand LSI_N20 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N20)->getMemOperand());
  AddToISelQueue(Chain20);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp3, LSI_N20, Chain20 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 8);
  Chain20 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N20.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_96(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N1);
  AddToISelQueue(N2);
  SDOperand Tmp3 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N3)->getValue()), MVT::i16);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N1, N2, Tmp3);
}
SDNode *Select_ISD_INTRINSIC_WO_CHAIN_v1i64(const SDOperand &N) {

  // Pattern: (intrinsic_wo_chain:v1i64 569:iPTR, VR64:v1i64:$src1, (bitconvert:v1i64 (ld:v2i32 addr:iPTR:$src2)<<P:Predicate_memop64>>), (imm:i16):$src3)
  // Emits: (PALIGNR64rm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2, (imm:i16):$src3)
  // Pattern complexity = 33  cost = 1  size = 3
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 569) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N3 = N.getOperand(3);
                if (N3.getOpcode() == ISD::Constant &&
                    N20.Val->getValueType(0) == MVT::v2i32) {
                  return Emit_95(N, X86::PALIGNR64rm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v1i64 343:iPTR, VR64:v1i64:$src1, (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSRLQrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 343) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSRLQrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v1i64 333:iPTR, VR64:v1i64:$src1, (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PSLLQrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 333) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v1i64) {
                return Emit_92(N, X86::MMX_PSLLQrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (intrinsic_wo_chain:v1i64 569:iPTR, VR64:v1i64:$src1, VR64:v1i64:$src2, (imm:i16):$src3)
  // Emits: (PALIGNR64rr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2, (imm:i16):$src3)
  // Pattern complexity = 11  cost = 1  size = 3
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 569) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::Constant) {
          return Emit_96(N, X86::PALIGNR64rr, MVT::v1i64);
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v1i64 346:iPTR, VR64:v1i64:$src1, (imm:i32):$src2)
      // Emits: (MMX_PSRLQri:v1i64 VR64:v1i64:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 346) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::MMX_PSRLQri, MVT::v1i64);
        }
      }

      // Pattern: (intrinsic_wo_chain:v1i64 336:iPTR, VR64:v1i64:$src1, (imm:i32):$src2)
      // Emits: (MMX_PSLLQri:v1i64 VR64:v1i64:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 336) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::MMX_PSLLQri, MVT::v1i64);
        }
      }

      // Pattern: (intrinsic_wo_chain:v1i64 343:iPTR, VR64:v1i64:$src1, VR64:v1i64:$src2)
      // Emits: (MMX_PSRLQrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 343) {
        return Emit_93(N, X86::MMX_PSRLQrr, MVT::v1i64);
      }

      // Pattern: (intrinsic_wo_chain:v1i64 333:iPTR, VR64:v1i64:$src1, VR64:v1i64:$src2)
      // Emits: (MMX_PSLLQrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 333) {
        return Emit_93(N, X86::MMX_PSLLQrr, MVT::v1i64);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_INTRINSIC_WO_CHAIN_v16i8(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v16i8 399:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PADDSBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 399) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PADDSBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v16i8 401:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PADDUSBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 401) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PADDUSBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v16i8 439:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSUBSBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 439) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSUBSBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v16i8 441:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSUBUSBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 441) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSUBUSBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v16i8 403:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PAVGBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 403) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PAVGBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v16i8 415:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMINUBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 415) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PMINUBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v16i8 413:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMAXUBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 413) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PMAXUBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v16i8 405:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PCMPEQBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 405) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PCMPEQBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v16i8 408:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PCMPGTBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 408) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PCMPGTBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v16i8 564:iPTR, (bitconvert:v16i8 (ld:v16i8 addr:iPTR:$src)<<P:Predicate_memop>>))
      // Emits: (PABSBrm128:v16i8 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 564) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v16i8) {
                return Emit_91(N, X86::PABSBrm128, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v16i8 588:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSHUFBrm128:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 588) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v16i8) {
                return Emit_92(N, X86::PSHUFBrm128, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v16i8 590:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSIGNBrm128:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 590) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v16i8) {
                return Emit_92(N, X86::PSIGNBrm128, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v16i8 399:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PADDSBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 399) {
        return Emit_93(N, X86::PADDSBrr, MVT::v16i8);
      }

      // Pattern: (intrinsic_wo_chain:v16i8 401:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PADDUSBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 401) {
        return Emit_93(N, X86::PADDUSBrr, MVT::v16i8);
      }

      // Pattern: (intrinsic_wo_chain:v16i8 439:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PSUBSBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 439) {
        return Emit_93(N, X86::PSUBSBrr, MVT::v16i8);
      }

      // Pattern: (intrinsic_wo_chain:v16i8 441:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PSUBUSBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 441) {
        return Emit_93(N, X86::PSUBUSBrr, MVT::v16i8);
      }

      // Pattern: (intrinsic_wo_chain:v16i8 403:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PAVGBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 403) {
        return Emit_93(N, X86::PAVGBrr, MVT::v16i8);
      }

      // Pattern: (intrinsic_wo_chain:v16i8 415:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PMINUBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 415) {
        return Emit_93(N, X86::PMINUBrr, MVT::v16i8);
      }

      // Pattern: (intrinsic_wo_chain:v16i8 413:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PMAXUBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 413) {
        return Emit_93(N, X86::PMAXUBrr, MVT::v16i8);
      }

      // Pattern: (intrinsic_wo_chain:v16i8 405:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PCMPEQBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 405) {
        return Emit_93(N, X86::PCMPEQBrr, MVT::v16i8);
      }

      // Pattern: (intrinsic_wo_chain:v16i8 408:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PCMPGTBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 408) {
        return Emit_93(N, X86::PCMPGTBrr, MVT::v16i8);
      }
    }
  }
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v16i8 564:iPTR, VR128:v16i8:$src)
      // Emits: (PABSBrr128:v16i8 VR128:v16i8:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 564) {
        return Emit_90(N, X86::PABSBrr128, MVT::v16i8);
      }

      // Pattern: (intrinsic_wo_chain:v16i8 588:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PSHUFBrr128:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 588) {
        return Emit_93(N, X86::PSHUFBrr128, MVT::v16i8);
      }

      // Pattern: (intrinsic_wo_chain:v16i8 590:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PSIGNBrr128:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 590) {
        return Emit_93(N, X86::PSIGNBrr128, MVT::v16i8);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_INTRINSIC_WO_CHAIN_v8i16(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v8i16 400:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PADDSWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 400) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PADDSWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 402:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PADDUSWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 402) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PADDUSWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 440:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSUBSWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 440) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSUBSWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 442:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSUBUSWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 442) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSUBUSWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 418:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMULHUWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 418) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PMULHUWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 417:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMULHWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 417) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PMULHWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 404:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PAVGWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 404) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PAVGWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 414:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMINSWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 414) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PMINSWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 412:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMAXSWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 412) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PMAXSWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 424:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSLLWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 424) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSLLWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 435:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSRLWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 435) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSRLWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 429:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSRAWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 429) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSRAWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 407:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PCMPEQWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 407) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PCMPEQWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 410:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PCMPGTWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 410) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PCMPGTWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 397:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PACKSSWBrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 397) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PACKSSWBrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 398:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PACKUSWBrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 398) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PACKUSWBrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v8i16 568:iPTR, (bitconvert:v8i16 (ld:v8i16 addr:iPTR:$src)<<P:Predicate_memop64>>))
      // Emits: (PABSWrm128:v8i16 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 568) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop64(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v8i16) {
                return Emit_91(N, X86::PABSWrm128, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 576:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v8i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PHADDWrm128:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 576) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v8i16) {
                return Emit_92(N, X86::PHADDWrm128, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 582:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v8i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PHSUBWrm128:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 582) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v8i16) {
                return Emit_92(N, X86::PHSUBWrm128, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 580:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v8i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PHSUBSWrm128:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 580) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v8i16) {
                return Emit_92(N, X86::PHSUBSWrm128, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 584:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMADDUBSWrm128:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 584) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v16i8) {
                return Emit_92(N, X86::PMADDUBSWrm128, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 586:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v8i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PMULHRSWrm128:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 586) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v8i16) {
                return Emit_92(N, X86::PMULHRSWrm128, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 594:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v8i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PSIGNWrm128:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 594) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v8i16) {
                return Emit_92(N, X86::PSIGNWrm128, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v8i16 427:iPTR, VR128:v8i16:$src1, (imm:i32):$src2)
      // Emits: (PSLLWri:v8i16 VR128:v8i16:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 427) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::PSLLWri, MVT::v8i16);
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 438:iPTR, VR128:v8i16:$src1, (imm:i32):$src2)
      // Emits: (PSRLWri:v8i16 VR128:v8i16:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 438) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::PSRLWri, MVT::v8i16);
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 431:iPTR, VR128:v8i16:$src1, (imm:i32):$src2)
      // Emits: (PSRAWri:v8i16 VR128:v8i16:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 431) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::PSRAWri, MVT::v8i16);
        }
      }

      // Pattern: (intrinsic_wo_chain:v8i16 400:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PADDSWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 400) {
        return Emit_93(N, X86::PADDSWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 402:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PADDUSWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 402) {
        return Emit_93(N, X86::PADDUSWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 440:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PSUBSWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 440) {
        return Emit_93(N, X86::PSUBSWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 442:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PSUBUSWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 442) {
        return Emit_93(N, X86::PSUBUSWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 418:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PMULHUWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 418) {
        return Emit_93(N, X86::PMULHUWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 417:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PMULHWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 417) {
        return Emit_93(N, X86::PMULHWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 404:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PAVGWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 404) {
        return Emit_93(N, X86::PAVGWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 414:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PMINSWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 414) {
        return Emit_93(N, X86::PMINSWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 412:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PMAXSWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 412) {
        return Emit_93(N, X86::PMAXSWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 424:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PSLLWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 424) {
        return Emit_93(N, X86::PSLLWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 435:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PSRLWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 435) {
        return Emit_93(N, X86::PSRLWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 429:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PSRAWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 429) {
        return Emit_93(N, X86::PSRAWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 407:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PCMPEQWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 407) {
        return Emit_93(N, X86::PCMPEQWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 410:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PCMPGTWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 410) {
        return Emit_93(N, X86::PCMPGTWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 397:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PACKSSWBrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 397) {
        return Emit_93(N, X86::PACKSSWBrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 398:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PACKUSWBrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 398) {
        return Emit_93(N, X86::PACKUSWBrr, MVT::v8i16);
      }
    }
  }
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v8i16 568:iPTR, VR128:v8i16:$src)
      // Emits: (PABSWrr128:v8i16 VR128:v8i16:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 568) {
        return Emit_90(N, X86::PABSWrr128, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 576:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PHADDWrr128:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 576) {
        return Emit_93(N, X86::PHADDWrr128, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 582:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PHSUBWrr128:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 582) {
        return Emit_93(N, X86::PHSUBWrr128, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 580:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PHSUBSWrr128:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 580) {
        return Emit_93(N, X86::PHSUBSWrr128, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 584:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PMADDUBSWrr128:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 584) {
        return Emit_93(N, X86::PMADDUBSWrr128, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 586:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PMULHRSWrr128:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 586) {
        return Emit_93(N, X86::PMULHRSWrr128, MVT::v8i16);
      }

      // Pattern: (intrinsic_wo_chain:v8i16 594:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PSIGNWrr128:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 594) {
        return Emit_93(N, X86::PSIGNWrr128, MVT::v8i16);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_INTRINSIC_WO_CHAIN_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4i32 411:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMADDWDrm:v4i32 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 411) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PMADDWDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 421:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSLLDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 421) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSLLDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 432:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSRLDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 432) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSRLDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 428:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSRADrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 428) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSRADrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 406:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PCMPEQDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 406) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PCMPEQDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 409:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PCMPGTDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 409) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PCMPGTDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 396:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PACKSSDWrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 396) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PACKSSDWrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4i32 566:iPTR, (bitconvert:v4i32 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_memop>>))
      // Emits: (PABSDrm128:v4i32 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 566) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v4i32) {
                return Emit_91(N, X86::PABSDrm128, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 572:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v4i32 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PHADDDrm128:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 572) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_92(N, X86::PHADDDrm128, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 574:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v8i16 addr:iPTR:$src2)<<P:Predicate_memop64>>))
      // Emits: (PHADDSWrm128:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 574) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v8i16) {
                return Emit_92(N, X86::PHADDSWrm128, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 578:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v4i32 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PHSUBDrm128:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 578) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_92(N, X86::PHSUBDrm128, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 592:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v4i32 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSIGNDrm128:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 592) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_92(N, X86::PSIGNDrm128, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4i32 366:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTPS2DQrm:v4i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 366) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTPS2DQrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 375:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTTPS2DQrm:v4i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 375) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTTPS2DQrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 364:iPTR, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTPD2DQrm:v4i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 364) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTPD2DQrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 374:iPTR, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTTPD2DQrm:v4i32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 374) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTTPD2DQrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 425:iPTR, VR128:v4i32:$src1, (imm:i32):$src2)
      // Emits: (PSLLDri:v4i32 VR128:v4i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 425) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::PSLLDri, MVT::v4i32);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 436:iPTR, VR128:v4i32:$src1, (imm:i32):$src2)
      // Emits: (PSRLDri:v4i32 VR128:v4i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 436) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::PSRLDri, MVT::v4i32);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 430:iPTR, VR128:v4i32:$src1, (imm:i32):$src2)
      // Emits: (PSRADri:v4i32 VR128:v4i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 430) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::PSRADri, MVT::v4i32);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4i32 366:iPTR, VR128:v4f32:$src)
      // Emits: (Int_CVTPS2DQrr:v4i32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 366) {
        return Emit_90(N, X86::Int_CVTPS2DQrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 375:iPTR, VR128:v4f32:$src)
      // Emits: (Int_CVTTPS2DQrr:v4i32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 375) {
        return Emit_90(N, X86::Int_CVTTPS2DQrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 364:iPTR, VR128:v2f64:$src)
      // Emits: (Int_CVTPD2DQrr:v4i32 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 364) {
        return Emit_90(N, X86::Int_CVTPD2DQrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 374:iPTR, VR128:v2f64:$src)
      // Emits: (Int_CVTTPD2DQrr:v4i32 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 374) {
        return Emit_90(N, X86::Int_CVTTPD2DQrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 411:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PMADDWDrr:v4i32 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 411) {
        return Emit_93(N, X86::PMADDWDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 421:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PSLLDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 421) {
        return Emit_93(N, X86::PSLLDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 432:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PSRLDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 432) {
        return Emit_93(N, X86::PSRLDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 428:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PSRADrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 428) {
        return Emit_93(N, X86::PSRADrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 406:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PCMPEQDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 406) {
        return Emit_93(N, X86::PCMPEQDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 409:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PCMPGTDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 409) {
        return Emit_93(N, X86::PCMPGTDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 396:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PACKSSDWrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 396) {
        return Emit_93(N, X86::PACKSSDWrr, MVT::v4i32);
      }
    }
  }
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4i32 566:iPTR, VR128:v4i32:$src)
      // Emits: (PABSDrr128:v4i32 VR128:v4i32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 566) {
        return Emit_90(N, X86::PABSDrr128, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 572:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PHADDDrr128:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 572) {
        return Emit_93(N, X86::PHADDDrr128, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 574:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PHADDSWrr128:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 574) {
        return Emit_93(N, X86::PHADDSWrr128, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 578:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PHSUBDrr128:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 578) {
        return Emit_93(N, X86::PHSUBDrr128, MVT::v4i32);
      }

      // Pattern: (intrinsic_wo_chain:v4i32 592:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PSIGNDrr128:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 592) {
        return Emit_93(N, X86::PSIGNDrr128, MVT::v4i32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_97(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N20 = N2.getOperand(0);
  SDOperand Chain20 = N20.getOperand(0);
  SDOperand N201 = N20.getOperand(1);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp3 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N3)->getValue()), MVT::i32);
  SDOperand LSI_N20 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N20)->getMemOperand());
  AddToISelQueue(Chain20);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp3, LSI_N20, Chain20 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 8);
  Chain20 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N20.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_98(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand Chain2 = N2.getOperand(0);
  SDOperand N21 = N2.getOperand(1);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N2 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N2)->getMemOperand());
  AddToISelQueue(Chain2);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N2, Chain2 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain2 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N2.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_99(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N1);
  AddToISelQueue(N2);
  SDOperand Tmp3 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N3)->getValue()), MVT::i32);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N1, N2, Tmp3);
}
SDNode *Emit_100(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N1);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N2)->getValue()), MVT::i32);
  SDOperand Tmp3 = Transform_PSxLDQ_imm(Tmp2.Val);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N1, Tmp3);
}
SDNode *Select_ISD_INTRINSIC_WO_CHAIN_v2i64(const SDOperand &N) {

  // Pattern: (intrinsic_wo_chain:v2i64 570:iPTR, VR128:v2i64:$src1, (bitconvert:v2i64 (ld:v4i32 addr:iPTR:$src2)<<P:Predicate_memop>>), (imm:i32):$src3)
  // Emits: (PALIGNR128rm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2, (imm:i32):$src3)
  // Pattern complexity = 33  cost = 1  size = 3
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 570) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N3 = N.getOperand(3);
                if (N3.getOpcode() == ISD::Constant &&
                    N20.Val->getValueType(0) == MVT::v4i32) {
                  return Emit_97(N, X86::PALIGNR128rm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i64 419:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMULUDQrm:v2i64 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 419) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PMULUDQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i64 420:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSADBWrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 420) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSADBWrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i64 423:iPTR, VR128:v2i64:$src1, (bitconvert:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSLLQrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 423) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSLLQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i64 434:iPTR, VR128:v2i64:$src1, (bitconvert:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PSRLQrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 434) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val)) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v2i64) {
                return Emit_92(N, X86::PSRLQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i64 443:iPTR, VR128:v2i64:$src1, (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (PUNPCKHQDQrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 443) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::PUNPCKHQDQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i64 444:iPTR, VR128:v2i64:$src1, (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (PUNPCKLQDQrm:v2i64 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 444) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::PUNPCKLQDQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i64 426:iPTR, VR128:v2i64:$src1, (imm:i32):$src2)
      // Emits: (PSLLQri:v2i64 VR128:v2i64:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 426) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::PSLLQri, MVT::v2i64);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i64 437:iPTR, VR128:v2i64:$src1, (imm:i32):$src2)
      // Emits: (PSRLQri:v2i64 VR128:v2i64:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 437) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::PSRLQri, MVT::v2i64);
        }
      }
    }
  }

  // Pattern: (intrinsic_wo_chain:v2i64 570:iPTR, VR128:v2i64:$src1, VR128:v2i64:$src2, (imm:i32):$src3)
  // Emits: (PALIGNR128rr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2, (imm:i32):$src3)
  // Pattern complexity = 11  cost = 1  size = 3
  if ((Subtarget->hasSSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 570) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::Constant) {
          return Emit_99(N, X86::PALIGNR128rr, MVT::v2i64);
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2i64 422:iPTR, VR128:v2i64:$src1, (imm:i32):$src2)
      // Emits: (PSLLDQri:v2i64 VR128:v16i8:$src1, (PSxLDQ_imm:i32 (imm:i32):$src2))
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 422) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_100(N, X86::PSLLDQri, MVT::v2i64);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i64 433:iPTR, VR128:v2i64:$src1, (imm:i32):$src2)
      // Emits: (PSRLDQri:v2i64 VR128:v16i8:$src1, (PSxLDQ_imm:i32 (imm:i32):$src2))
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 433) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_100(N, X86::PSRLDQri, MVT::v2i64);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2i64 419:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PMULUDQrr:v2i64 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 419) {
        return Emit_93(N, X86::PMULUDQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_wo_chain:v2i64 420:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PSADBWrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 420) {
        return Emit_93(N, X86::PSADBWrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_wo_chain:v2i64 423:iPTR, VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Emits: (PSLLQrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 423) {
        return Emit_93(N, X86::PSLLQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_wo_chain:v2i64 434:iPTR, VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Emits: (PSRLQrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 434) {
        return Emit_93(N, X86::PSRLQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_wo_chain:v2i64 443:iPTR, VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Emits: (PUNPCKHQDQrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 443) {
        return Emit_93(N, X86::PUNPCKHQDQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_wo_chain:v2i64 444:iPTR, VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Emits: (PUNPCKLQDQrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 444) {
        return Emit_93(N, X86::PUNPCKLQDQrr, MVT::v2i64);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_101(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand Chain2 = N2.getOperand(0);
  SDOperand N21 = N2.getOperand(1);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp3 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N3)->getValue()), MVT::i8);
  SDOperand LSI_N2 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N2)->getMemOperand());
  AddToISelQueue(Chain2);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp3, LSI_N2, Chain2 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 8);
  Chain2 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N2.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_102(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N2)->getValue()), MVT::i32);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_103(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPInChain, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3, SDOperand &Chain1) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N2)->getValue()), MVT::i32);
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(CPInChain.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_104(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPInChain, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3, SDOperand &Chain2) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain2);
  SDOperand Ops0[] = { N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain2 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(CPInChain.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_105(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPInChain, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3, SDOperand &Chain1) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 5);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(CPInChain.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_106(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N1);
  AddToISelQueue(N2);
  SDOperand Tmp3 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N3)->getValue()), MVT::i8);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N1, N2, Tmp3);
}
SDNode *Select_ISD_INTRINSIC_WO_CHAIN_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4f32 518:iPTR, VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>, (imm:i8):$cc)
      // Emits: (Int_CMPSSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src, (imm:i8):$cc)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 518) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N3 = N.getOperand(3);
              if (N3.getOpcode() == ISD::Constant) {
                return Emit_101(N, X86::Int_CMPSSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 517:iPTR, VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>, (imm:i8):$cc)
      // Emits: (CMPPSrmi:v4f32 VR128:v4f32:$src1, addr:iPTR:$src, (imm:i8):$cc)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 517) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N3 = N.getOperand(3);
              if (N3.getOpcode() == ISD::Constant) {
                return Emit_101(N, X86::CMPPSrmi, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (intrinsic_wo_chain:v4f32 363:iPTR, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src)<<P:Predicate_memop>>))
  // Emits: (Int_CVTDQ2PSrm:v4f32 addr:iPTR:$src)
  // Pattern complexity = 30  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 363) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_91(N, X86::Int_CVTDQ2PSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (intrinsic_wo_chain:v4f32 513:iPTR, (ld:v4f32 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i32):$src2)
  // Emits: (ROUNDPSm_Int:v4f32 addr:iPTR:$src1, (imm:i32):$src2)
  // Pattern complexity = 30  cost = 1  size = 3
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 513) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N2.getOpcode() == ISD::Constant) {
                return Emit_102(N, X86::ROUNDPSm_Int, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4f32 530:iPTR, VR128:v4f32:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (Int_CVTSI2SS64rm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 530) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::Int_CVTSI2SS64rm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 527:iPTR, VR128:v4f32:$src1, (ld:v2i32 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (Int_CVTPI2PSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 527) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::Int_CVTPI2PSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 529:iPTR, VR128:v4f32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (Int_CVTSI2SSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 529) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::Int_CVTSI2SSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 540:iPTR, VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (MAXPSrm_Int:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 540) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::MAXPSrm_Int, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 542:iPTR, VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (MINPSrm_Int:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 542) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::MINPSrm_Int, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 552:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (SQRTPSm_Int:v4f32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 552) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::SQRTPSm_Int, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 549:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (RSQRTPSm_Int:v4f32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 549) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::RSQRTPSm_Int, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 547:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (RCPPSm_Int:v4f32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 547) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::RCPPSm_Int, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4f32 365:iPTR, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTPD2PSrm:v4f32 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 365) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTPD2PSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 370:iPTR, VR128:v4f32:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (Int_CVTSD2SSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 370) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::Int_CVTSD2SSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4f32 461:iPTR, VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (ADDSUBPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 461) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::ADDSUBPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 463:iPTR, VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (HADDPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 463) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::HADDPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 465:iPTR, VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (HSUBPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 465) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::HSUBPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }

  // Pattern: (intrinsic_wo_chain:v4f32 515:iPTR, sse_load_f32:v4f32:$src1, (imm:i32):$src2)
  // Emits: (ROUNDSSm_Int:v4f32 sse_load_f32:v4f32:$src1, (imm:i32):$src2)
  // Pattern complexity = 26  cost = 1  size = 3
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 515) {
        SDOperand N1 = N.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain1;
        if (SelectScalarSSELoad(N, N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain1)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::Constant) {
            return Emit_103(N, X86::ROUNDSSm_Int, MVT::v4f32, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain1);
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4f32 516:iPTR, VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Emits: (ADDSSrm_Int:v4f32 VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 516) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::ADDSSrm_Int, MVT::v4f32, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 546:iPTR, VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Emits: (MULSSrm_Int:v4f32 VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 546) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::MULSSrm_Int, MVT::v4f32, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 556:iPTR, VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Emits: (SUBSSrm_Int:v4f32 VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 556) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::SUBSSrm_Int, MVT::v4f32, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 537:iPTR, VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Emits: (DIVSSrm_Int:v4f32 VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 537) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::DIVSSrm_Int, MVT::v4f32, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 541:iPTR, VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Emits: (MAXSSrm_Int:v4f32 VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 541) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::MAXSSrm_Int, MVT::v4f32, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 543:iPTR, VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Emits: (MINSSrm_Int:v4f32 VR128:v4f32:$src1, sse_load_f32:v4f32:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 543) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::MINSSrm_Int, MVT::v4f32, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 553:iPTR, sse_load_f32:v4f32:$src)
      // Emits: (SQRTSSm_Int:v4f32 sse_load_f32:v4f32:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 553) {
        SDOperand N1 = N.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain1;
        if (SelectScalarSSELoad(N, N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain1)) {
          return Emit_105(N, X86::SQRTSSm_Int, MVT::v4f32, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain1);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 550:iPTR, sse_load_f32:v4f32:$src)
      // Emits: (RSQRTSSm_Int:v4f32 sse_load_f32:v4f32:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 550) {
        SDOperand N1 = N.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain1;
        if (SelectScalarSSELoad(N, N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain1)) {
          return Emit_105(N, X86::RSQRTSSm_Int, MVT::v4f32, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain1);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 548:iPTR, sse_load_f32:v4f32:$src)
      // Emits: (RCPSSm_Int:v4f32 sse_load_f32:v4f32:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 548) {
        SDOperand N1 = N.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain1;
        if (SelectScalarSSELoad(N, N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain1)) {
          return Emit_105(N, X86::RCPSSm_Int, MVT::v4f32, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain1);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 518:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src, (imm:i8):$cc)
      // Emits: (Int_CMPSSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src, (imm:i8):$cc)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 518) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::Constant) {
          return Emit_106(N, X86::Int_CMPSSrr, MVT::v4f32);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 517:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src, (imm:i8):$cc)
      // Emits: (CMPPSrri:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src, (imm:i8):$cc)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 517) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::Constant) {
          return Emit_106(N, X86::CMPPSrri, MVT::v4f32);
        }
      }
    }
  }
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4f32 515:iPTR, VR128:v4f32:$src1, (imm:i32):$src2)
      // Emits: (ROUNDSSr_Int:v4f32 VR128:v4f32:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 515) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::ROUNDSSr_Int, MVT::v4f32);
        }
      }

      // Pattern: (intrinsic_wo_chain:v4f32 513:iPTR, VR128:v4f32:$src1, (imm:i32):$src2)
      // Emits: (ROUNDPSr_Int:v4f32 VR128:v4f32:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 513) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::ROUNDPSr_Int, MVT::v4f32);
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4f32 530:iPTR, VR128:v4f32:$src1, GR64:i64:$src2)
      // Emits: (Int_CVTSI2SS64rr:v4f32 VR128:v4f32:$src1, GR64:i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 530) {
        return Emit_93(N, X86::Int_CVTSI2SS64rr, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 527:iPTR, VR128:v4f32:$src1, VR64:v2i32:$src2)
      // Emits: (Int_CVTPI2PSrr:v4f32 VR128:v4f32:$src1, VR64:v2i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 527) {
        return Emit_93(N, X86::Int_CVTPI2PSrr, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 529:iPTR, VR128:v4f32:$src1, GR32:i32:$src2)
      // Emits: (Int_CVTSI2SSrr:v4f32 VR128:v4f32:$src1, GR32:i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 529) {
        return Emit_93(N, X86::Int_CVTSI2SSrr, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 516:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (ADDSSrr_Int:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 516) {
        return Emit_93(N, X86::ADDSSrr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 546:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (MULSSrr_Int:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 546) {
        return Emit_93(N, X86::MULSSrr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 556:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (SUBSSrr_Int:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 556) {
        return Emit_93(N, X86::SUBSSrr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 537:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (DIVSSrr_Int:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 537) {
        return Emit_93(N, X86::DIVSSrr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 541:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (MAXSSrr_Int:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 541) {
        return Emit_93(N, X86::MAXSSrr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 540:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (MAXPSrr_Int:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 540) {
        return Emit_93(N, X86::MAXPSrr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 543:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (MINSSrr_Int:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 543) {
        return Emit_93(N, X86::MINSSrr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 542:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (MINPSrr_Int:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 542) {
        return Emit_93(N, X86::MINPSrr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 553:iPTR, VR128:v4f32:$src)
      // Emits: (SQRTSSr_Int:v4f32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 553) {
        return Emit_90(N, X86::SQRTSSr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 552:iPTR, VR128:v4f32:$src)
      // Emits: (SQRTPSr_Int:v4f32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 552) {
        return Emit_90(N, X86::SQRTPSr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 550:iPTR, VR128:v4f32:$src)
      // Emits: (RSQRTSSr_Int:v4f32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 550) {
        return Emit_90(N, X86::RSQRTSSr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 549:iPTR, VR128:v4f32:$src)
      // Emits: (RSQRTPSr_Int:v4f32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 549) {
        return Emit_90(N, X86::RSQRTPSr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 548:iPTR, VR128:v4f32:$src)
      // Emits: (RCPSSr_Int:v4f32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 548) {
        return Emit_90(N, X86::RCPSSr_Int, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 547:iPTR, VR128:v4f32:$src)
      // Emits: (RCPPSr_Int:v4f32 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 547) {
        return Emit_90(N, X86::RCPPSr_Int, MVT::v4f32);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4f32 363:iPTR, VR128:v4i32:$src)
      // Emits: (Int_CVTDQ2PSrr:v4f32 VR128:v4i32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 363) {
        return Emit_90(N, X86::Int_CVTDQ2PSrr, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 365:iPTR, VR128:v2f64:$src)
      // Emits: (Int_CVTPD2PSrr:v4f32 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 365) {
        return Emit_90(N, X86::Int_CVTPD2PSrr, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 370:iPTR, VR128:v4f32:$src1, VR128:v2f64:$src2)
      // Emits: (Int_CVTSD2SSrr:v4f32 VR128:v4f32:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 370) {
        return Emit_93(N, X86::Int_CVTSD2SSrr, MVT::v4f32);
      }
    }
  }
  if ((Subtarget->hasSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v4f32 461:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (ADDSUBPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 461) {
        return Emit_93(N, X86::ADDSUBPSrr, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 463:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (HADDPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 463) {
        return Emit_93(N, X86::HADDPSrr, MVT::v4f32);
      }

      // Pattern: (intrinsic_wo_chain:v4f32 465:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Emits: (HSUBPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 465) {
        return Emit_93(N, X86::HSUBPSrr, MVT::v4f32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_INTRINSIC_WO_CHAIN_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2f64 355:iPTR, VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>, (imm:i8):$cc)
      // Emits: (Int_CMPSDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src, (imm:i8):$cc)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 355) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N3 = N.getOperand(3);
              if (N3.getOpcode() == ISD::Constant) {
                return Emit_101(N, X86::Int_CMPSDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 362:iPTR, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src)<<P:Predicate_memop>>))
      // Emits: (Int_CVTDQ2PDrm:v2f64 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 362) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_91(N, X86::Int_CVTDQ2PDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 354:iPTR, VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>, (imm:i8):$cc)
      // Emits: (CMPPDrmi:v2f64 VR128:v2f64:$src1, addr:iPTR:$src, (imm:i8):$cc)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 354) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N3 = N.getOperand(3);
              if (N3.getOpcode() == ISD::Constant) {
                return Emit_101(N, X86::CMPPDrmi, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (intrinsic_wo_chain:v2f64 512:iPTR, (ld:v2f64 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i32):$src2)
  // Emits: (ROUNDPDm_Int:v2f64 addr:iPTR:$src1, (imm:i32):$src2)
  // Pattern complexity = 30  cost = 1  size = 3
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 512) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N2.getOpcode() == ISD::Constant) {
                return Emit_102(N, X86::ROUNDPDm_Int, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2f64 445:iPTR, VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>, (imm:i32):$src3)
      // Emits: (SHUFPDrmi:v2f64 VR128:v16i8:$src1, addr:iPTR:$src2, (imm:i8):$src3)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 445) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N3 = N.getOperand(3);
              if (N3.getOpcode() == ISD::Constant) {
                return Emit_101(N, X86::SHUFPDrmi, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 372:iPTR, VR128:v2f64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (Int_CVTSI2SD64rm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 372) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::Int_CVTSI2SD64rm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 526:iPTR, (ld:v2i32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTPI2PDrm:v2f64 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 526) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTPI2PDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 385:iPTR, VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (MAXPDrm_Int:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 385) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::MAXPDrm_Int, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 388:iPTR, VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (MINPDrm_Int:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 388) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::MINPDrm_Int, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 367:iPTR, (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (Int_CVTPS2PDrm:v2f64 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 367) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::Int_CVTPS2PDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 371:iPTR, VR128:v2f64:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (Int_CVTSI2SDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 371) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::Int_CVTSI2SDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 373:iPTR, VR128:v2f64:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (Int_CVTSS2SDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 373) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::Int_CVTSS2SDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 446:iPTR, (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (SQRTPDm_Int:v2f64 addr:iPTR:$src)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 446) {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_89(N, X86::SQRTPDm_Int, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2f64 460:iPTR, VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (ADDSUBPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 460) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::ADDSUBPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 462:iPTR, VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (HADDPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 462) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::HADDPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 464:iPTR, VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (HSUBPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 464) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::HSUBPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2f64 458:iPTR, VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (UNPCKHPDrm:v2f64 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 458) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::UNPCKHPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 459:iPTR, VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (UNPCKLPDrm:v2f64 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 459) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::LOAD &&
            N2.hasOneUse() &&
            CanBeFoldedBy(N2.Val, N.Val, N.Val)) {
          SDOperand Chain2 = N2.getOperand(0);
          if (Predicate_load(N2.Val)) {
            SDOperand N21 = N2.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N21, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_98(N, X86::UNPCKLPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }

  // Pattern: (intrinsic_wo_chain:v2f64 514:iPTR, sse_load_f64:v2f64:$src1, (imm:i32):$src2)
  // Emits: (ROUNDSDm_Int:v2f64 sse_load_f64:v2f64:$src1, (imm:i32):$src2)
  // Pattern complexity = 26  cost = 1  size = 3
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 514) {
        SDOperand N1 = N.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain1;
        if (SelectScalarSSELoad(N, N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain1)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::Constant) {
            return Emit_103(N, X86::ROUNDSDm_Int, MVT::v2f64, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain1);
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2f64 352:iPTR, VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Emits: (ADDSDrm_Int:v2f64 VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 352) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::ADDSDrm_Int, MVT::v2f64, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 395:iPTR, VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Emits: (MULSDrm_Int:v2f64 VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 395) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::MULSDrm_Int, MVT::v2f64, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 451:iPTR, VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Emits: (SUBSDrm_Int:v2f64 VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 451) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::SUBSDrm_Int, MVT::v2f64, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 378:iPTR, VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Emits: (DIVSDrm_Int:v2f64 VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 378) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::DIVSDrm_Int, MVT::v2f64, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 386:iPTR, VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Emits: (MAXSDrm_Int:v2f64 VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 386) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::MAXSDrm_Int, MVT::v2f64, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 389:iPTR, VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Emits: (MINSDrm_Int:v2f64 VR128:v2f64:$src1, sse_load_f64:v2f64:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 389) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain2;
        if (SelectScalarSSELoad(N, N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain2)) {
          return Emit_104(N, X86::MINSDrm_Int, MVT::v2f64, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain2);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 447:iPTR, sse_load_f64:v2f64:$src)
      // Emits: (SQRTSDm_Int:v2f64 sse_load_f64:v2f64:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 447) {
        SDOperand N1 = N.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        SDOperand CPInChain;
        SDOperand Chain1;
        if (SelectScalarSSELoad(N, N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3, CPInChain, Chain1)) {
          return Emit_105(N, X86::SQRTSDm_Int, MVT::v2f64, CPInChain, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain1);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 355:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src, (imm:i8):$cc)
      // Emits: (Int_CMPSDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src, (imm:i8):$cc)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 355) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::Constant) {
          return Emit_106(N, X86::Int_CMPSDrr, MVT::v2f64);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 354:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src, (imm:i8):$cc)
      // Emits: (CMPPDrri:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src, (imm:i8):$cc)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 354) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::Constant) {
          return Emit_106(N, X86::CMPPDrri, MVT::v2f64);
        }
      }
    }
  }
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2f64 514:iPTR, VR128:v2f64:$src1, (imm:i32):$src2)
      // Emits: (ROUNDSDr_Int:v2f64 VR128:v2f64:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 514) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::ROUNDSDr_Int, MVT::v2f64);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 512:iPTR, VR128:v2f64:$src1, (imm:i32):$src2)
      // Emits: (ROUNDPDr_Int:v2f64 VR128:v2f64:$src1, (imm:i32):$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 512) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::Constant) {
          return Emit_94(N, X86::ROUNDPDr_Int, MVT::v2f64);
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2f64 445:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2, (imm:i32):$src3)
      // Emits: (SHUFPDrri:v2f64 VR128:v16i8:$src1, VR128:v16i8:$src2, (imm:i8):$src3)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 445) {
        SDOperand N1 = N.getOperand(1);
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::Constant) {
          return Emit_106(N, X86::SHUFPDrri, MVT::v2f64);
        }
      }

      // Pattern: (intrinsic_wo_chain:v2f64 372:iPTR, VR128:v2f64:$src1, GR64:i64:$src2)
      // Emits: (Int_CVTSI2SD64rr:v2f64 VR128:v2f64:$src1, GR64:i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 372) {
        return Emit_93(N, X86::Int_CVTSI2SD64rr, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 526:iPTR, VR64:v2i32:$src)
      // Emits: (Int_CVTPI2PDrr:v2f64 VR64:v2i32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 526) {
        return Emit_90(N, X86::Int_CVTPI2PDrr, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 352:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (ADDSDrr_Int:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 352) {
        return Emit_93(N, X86::ADDSDrr_Int, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 395:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (MULSDrr_Int:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 395) {
        return Emit_93(N, X86::MULSDrr_Int, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 451:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (SUBSDrr_Int:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 451) {
        return Emit_93(N, X86::SUBSDrr_Int, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 378:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (DIVSDrr_Int:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 378) {
        return Emit_93(N, X86::DIVSDrr_Int, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 386:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (MAXSDrr_Int:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 386) {
        return Emit_93(N, X86::MAXSDrr_Int, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 385:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (MAXPDrr_Int:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 385) {
        return Emit_93(N, X86::MAXPDrr_Int, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 389:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (MINSDrr_Int:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 389) {
        return Emit_93(N, X86::MINSDrr_Int, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 388:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (MINPDrr_Int:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 388) {
        return Emit_93(N, X86::MINPDrr_Int, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 362:iPTR, VR128:v4i32:$src)
      // Emits: (Int_CVTDQ2PDrr:v2f64 VR128:v4i32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 362) {
        return Emit_90(N, X86::Int_CVTDQ2PDrr, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 367:iPTR, VR128:v4f32:$src)
      // Emits: (Int_CVTPS2PDrr:v2f64 VR128:v4f32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 367) {
        return Emit_90(N, X86::Int_CVTPS2PDrr, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 371:iPTR, VR128:v2f64:$src1, GR32:i32:$src2)
      // Emits: (Int_CVTSI2SDrr:v2f64 VR128:v2f64:$src1, GR32:i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 371) {
        return Emit_93(N, X86::Int_CVTSI2SDrr, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 373:iPTR, VR128:v2f64:$src1, VR128:v4f32:$src2)
      // Emits: (Int_CVTSS2SDrr:v2f64 VR128:v2f64:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 373) {
        return Emit_93(N, X86::Int_CVTSS2SDrr, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 447:iPTR, VR128:v2f64:$src)
      // Emits: (SQRTSDr_Int:v2f64 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 447) {
        return Emit_90(N, X86::SQRTSDr_Int, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 446:iPTR, VR128:v2f64:$src)
      // Emits: (SQRTPDr_Int:v2f64 VR128:v2f64:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 446) {
        return Emit_90(N, X86::SQRTPDr_Int, MVT::v2f64);
      }
    }
  }
  if ((Subtarget->hasSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2f64 460:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (ADDSUBPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 460) {
        return Emit_93(N, X86::ADDSUBPDrr, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 462:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (HADDPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 462) {
        return Emit_93(N, X86::HADDPDrr, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 464:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (HSUBPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 464) {
        return Emit_93(N, X86::HSUBPDrr, MVT::v2f64);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

      // Pattern: (intrinsic_wo_chain:v2f64 394:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (MOVLPDrr:v2f64 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 394) {
        return Emit_93(N, X86::MOVLPDrr, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 458:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (UNPCKHPDrr:v2f64 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 458) {
        return Emit_93(N, X86::UNPCKHPDrr, MVT::v2f64);
      }

      // Pattern: (intrinsic_wo_chain:v2f64 459:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Emits: (UNPCKLPDrr:v2f64 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 459) {
        return Emit_93(N, X86::UNPCKLPDrr, MVT::v2f64);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_107(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  SDOperand N30 = N3.getOperand(0);
  SDOperand Chain30 = N30.getOperand(0);
  SDOperand N301 = N30.getOperand(1);
  SDOperand N4 = N.getOperand(4);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N30.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain30);
  InChains.push_back(Chain30);
  Chain30 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(N2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp3 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N4)->getValue()), MVT::i32);
  SDOperand LSI_N30 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N30)->getMemOperand());
  AddToISelQueue(Chain30);
  SDOperand Ops0[] = { N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp3, LSI_N30, Chain30 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 8);
  Chain30 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N30.Val, 1), SDOperand(ResNode, 1));
  ReplaceUses(SDOperand(N.Val, 1), SDOperand(Chain30.Val, Chain30.ResNo));
  return ResNode;
}
SDNode *Emit_108(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  SDOperand N30 = N3.getOperand(0);
  SDOperand Chain30 = N30.getOperand(0);
  SDOperand N301 = N30.getOperand(1);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N30.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain30);
  InChains.push_back(Chain30);
  Chain30 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(N2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N30 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N30)->getMemOperand());
  AddToISelQueue(Chain30);
  SDOperand Ops0[] = { N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N30, Chain30 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain30 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N30.Val, 1), SDOperand(ResNode, 1));
  ReplaceUses(SDOperand(N.Val, 1), SDOperand(Chain30.Val, Chain30.ResNo));
  return ResNode;
}
SDNode *Emit_109(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  SDOperand N30 = N3.getOperand(0);
  SDOperand Chain30 = N30.getOperand(0);
  SDOperand N301 = N30.getOperand(1);
  SDOperand N4 = N.getOperand(4);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N30.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain30);
  InChains.push_back(Chain30);
  Chain30 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(N2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N30 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N30)->getMemOperand());
  AddToISelQueue(Chain30);
  AddToISelQueue(N4);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain30, X86::XMM0, N4, InFlag).Val;
  Chain30 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N30, Chain30, InFlag };
  ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 8);
  Chain30 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N30.Val, 1), SDOperand(ResNode, 1));
  ReplaceUses(SDOperand(N.Val, 1), SDOperand(Chain30.Val, Chain30.ResNo));
  return ResNode;
}
SDNode *Emit_110(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 5);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Emit_111(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  SDOperand N4 = N.getOperand(4);
  AddToISelQueue(N2);
  AddToISelQueue(N3);
  SDOperand Tmp3 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N4)->getValue()), MVT::i32);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { N2, N3, Tmp3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 4);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Emit_112(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N2);
  AddToISelQueue(N3);
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, N2, N3, Chain);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Emit_113(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  SDOperand N4 = N.getOperand(4);
  AddToISelQueue(N2);
  AddToISelQueue(N3);
  AddToISelQueue(Chain);
  AddToISelQueue(N4);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::XMM0, N4, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { N2, N3, Chain, InFlag };
  ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 4);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Select_ISD_INTRINSIC_W_CHAIN_v16i8(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_w_chain:v16i8 478:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>), (imm:i32):$src3)
      // Emits: (MPSADBWrmi:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2, (imm:i32):$src3)
      // Pattern complexity = 33  cost = 1  size = 3
      if (CN0 == 478) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N4 = N.getOperand(4);
                if (N4.getOpcode() == ISD::Constant &&
                    N30.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_107(N, X86::MPSADBWrmi, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v16i8 494:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMINSBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 494) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PMINSBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v16i8 490:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMAXSBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 490) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PMAXSBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v16i8 480:iPTR, VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>), XMM0:v16i8)
      // Emits: (PBLENDVBrm0:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 480) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N4 = N.getOperand(4);
                if (N30.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_109(N, X86::PBLENDVBrm0, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (intrinsic_w_chain:v16i8 382:iPTR, addr:iPTR:$src)
  // Emits: (MOVDQUrm_Int:v16i8 addr:iPTR:$src)
  // Pattern complexity = 23  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
      if (CN0 == 382) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_110(N, X86::MOVDQUrm_Int, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (intrinsic_w_chain:v16i8 466:iPTR, addr:iPTR:$src)
  // Emits: (LDDQUrm:v16i8 addr:iPTR:$src)
  // Pattern complexity = 23  cost = 1  size = 3
  if ((Subtarget->hasSSE3())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
      if (CN0 == 466) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_110(N, X86::LDDQUrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_w_chain:v16i8 478:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2, (imm:i32):$src3)
      // Emits: (MPSADBWrri:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2, (imm:i32):$src3)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 478) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand N4 = N.getOperand(4);
        if (N4.getOpcode() == ISD::Constant) {
          return Emit_111(N, X86::MPSADBWrri, MVT::v16i8);
        }
      }

      // Pattern: (intrinsic_w_chain:v16i8 494:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PMINSBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 494) {
        return Emit_112(N, X86::PMINSBrr, MVT::v16i8);
      }

      // Pattern: (intrinsic_w_chain:v16i8 490:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Emits: (PMAXSBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 490) {
        return Emit_112(N, X86::PMAXSBrr, MVT::v16i8);
      }

      // Pattern: (intrinsic_w_chain:v16i8 480:iPTR, VR128:v16i8:$src1, VR128:v16i8:$src2, XMM0:v16i8)
      // Emits: (PBLENDVBrr0:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 480) {
        return Emit_113(N, X86::PBLENDVBrr0, MVT::v16i8);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_114(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N20 = N2.getOperand(0);
  SDOperand Chain20 = N20.getOperand(0);
  SDOperand N201 = N20.getOperand(1);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N20.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain20);
  InChains.push_back(Chain20);
  Chain20 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N20 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N20)->getMemOperand());
  AddToISelQueue(Chain20);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N20, Chain20 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain20 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N20.Val, 1), SDOperand(ResNode, 1));
  ReplaceUses(SDOperand(N.Val, 1), SDOperand(Chain20.Val, Chain20.ResNo));
  return ResNode;
}
SDNode *Emit_115(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N2);
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, N2, Chain);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Select_ISD_INTRINSIC_W_CHAIN_v8i16(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_w_chain:v8i16 481:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>), (imm:i32):$src3)
      // Emits: (PBLENDWrmi:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2, (imm:i32):$src3)
      // Pattern complexity = 33  cost = 1  size = 3
      if (CN0 == 481) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N4 = N.getOperand(4);
                if (N4.getOpcode() == ISD::Constant &&
                    N30.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_107(N, X86::PBLENDWrmi, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v8i16 486:iPTR, (bitconvert:v8i16 (ld:v8i16 addr:iPTR:$src)<<P:Predicate_memop64>>))
      // Emits: (PHMINPOSUWrm128:v8i16 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 486) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_memop64(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v8i16) {
                return Emit_114(N, X86::PHMINPOSUWrm128, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v8i16 479:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PACKUSDWrm:v8i16 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 479) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PACKUSDWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v8i16 497:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMINUWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 497) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PMINUWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v8i16 493:iPTR, VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMAXUWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 493) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PMAXUWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v8i16 500:iPTR, (bitconvert:v16i8 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVSXBWrm:v8i16 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 500) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVSXBWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v8i16 506:iPTR, (bitconvert:v16i8 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVZXBWrm:v8i16 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 506) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVZXBWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v8i16 481:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2, (imm:i32):$src3)
      // Emits: (PBLENDWrri:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2, (imm:i32):$src3)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 481) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand N4 = N.getOperand(4);
        if (N4.getOpcode() == ISD::Constant) {
          return Emit_111(N, X86::PBLENDWrri, MVT::v8i16);
        }
      }

      // Pattern: (intrinsic_w_chain:v8i16 486:iPTR, VR128:v8i16:$src)
      // Emits: (PHMINPOSUWrr128:v8i16 VR128:v8i16:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 486) {
        return Emit_115(N, X86::PHMINPOSUWrr128, MVT::v8i16);
      }

      // Pattern: (intrinsic_w_chain:v8i16 479:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PACKUSDWrr:v8i16 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 479) {
        return Emit_112(N, X86::PACKUSDWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_w_chain:v8i16 497:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PMINUWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 497) {
        return Emit_112(N, X86::PMINUWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_w_chain:v8i16 493:iPTR, VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Emits: (PMAXUWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 493) {
        return Emit_112(N, X86::PMAXUWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_w_chain:v8i16 500:iPTR, VR128:v16i8:$src)
      // Emits: (PMOVSXBWrr:v8i16 VR128:v16i8:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 500) {
        return Emit_115(N, X86::PMOVSXBWrr, MVT::v8i16);
      }

      // Pattern: (intrinsic_w_chain:v8i16 506:iPTR, VR128:v16i8:$src)
      // Emits: (PMOVZXBWrr:v8i16 VR128:v16i8:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 506) {
        return Emit_115(N, X86::PMOVZXBWrr, MVT::v8i16);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_116(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  SDOperand Chain3 = N3.getOperand(0);
  SDOperand N31 = N3.getOperand(1);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N3.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain3);
  InChains.push_back(Chain3);
  Chain3 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(N2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N3 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N3)->getMemOperand());
  AddToISelQueue(Chain3);
  SDOperand Ops0[] = { N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N3, Chain3 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain3 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N3.Val, 1), SDOperand(ResNode, 1));
  ReplaceUses(SDOperand(N.Val, 1), SDOperand(Chain3.Val, Chain3.ResNo));
  return ResNode;
}
SDNode *Select_ISD_INTRINSIC_W_CHAIN_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_w_chain:v4i32 495:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMINSDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 495) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PMINSDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4i32 496:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMINUDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 496) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PMINUDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4i32 491:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMAXSDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 491) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PMAXSDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4i32 492:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMAXUDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 492) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PMAXUDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4i32 502:iPTR, (bitconvert:v8i16 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVSXWDrm:v4i32 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 502) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVSXWDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4i32 508:iPTR, (bitconvert:v8i16 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVZXWDrm:v4i32 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 508) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVZXWDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4i32 498:iPTR, (bitconvert:v16i8 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVSXBDrm:v4i32 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 498) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVSXBDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4i32 504:iPTR, (bitconvert:v16i8 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVZXBDrm:v4i32 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 504) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVZXBDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4i32 511:iPTR, VR128:v4i32:$src1, (ld:v4i32 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (PMULLDrm_int:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 27  cost = 1  size = 3
      if (CN0 == 511) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::LOAD &&
            N3.hasOneUse() &&
            CanBeFoldedBy(N3.Val, N.Val, N.Val) &&
            (Chain.Val == N3.Val || IsChainCompatible(Chain.Val, N3.Val))) {
          SDOperand Chain3 = N3.getOperand(0);
          if (Predicate_memop(N3.Val)) {
            SDOperand N31 = N3.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N31, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_116(N, X86::PMULLDrm_int, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4i32 495:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PMINSDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 495) {
        return Emit_112(N, X86::PMINSDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_w_chain:v4i32 496:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PMINUDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 496) {
        return Emit_112(N, X86::PMINUDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_w_chain:v4i32 491:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PMAXSDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 491) {
        return Emit_112(N, X86::PMAXSDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_w_chain:v4i32 492:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PMAXUDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 492) {
        return Emit_112(N, X86::PMAXUDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_w_chain:v4i32 511:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PMULLDrr_int:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 511) {
        return Emit_112(N, X86::PMULLDrr_int, MVT::v4i32);
      }

      // Pattern: (intrinsic_w_chain:v4i32 502:iPTR, VR128:v8i16:$src)
      // Emits: (PMOVSXWDrr:v4i32 VR128:v8i16:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 502) {
        return Emit_115(N, X86::PMOVSXWDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_w_chain:v4i32 508:iPTR, VR128:v8i16:$src)
      // Emits: (PMOVZXWDrr:v4i32 VR128:v8i16:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 508) {
        return Emit_115(N, X86::PMOVZXWDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_w_chain:v4i32 498:iPTR, VR128:v16i8:$src)
      // Emits: (PMOVSXBDrr:v4i32 VR128:v16i8:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 498) {
        return Emit_115(N, X86::PMOVSXBDrr, MVT::v4i32);
      }

      // Pattern: (intrinsic_w_chain:v4i32 504:iPTR, VR128:v16i8:$src)
      // Emits: (PMOVZXBDrr:v4i32 VR128:v16i8:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 504) {
        return Emit_115(N, X86::PMOVZXBDrr, MVT::v4i32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_INTRINSIC_W_CHAIN_v2i64(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_w_chain:v2i64 482:iPTR, VR128:v2i64:$src1, (bitconvert:v2i64 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PCMPEQQrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 482) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PCMPEQQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v2i64 510:iPTR, VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMULDQrm:v2i64 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 510) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N30.Val->getValueType(0) == MVT::v16i8) {
                return Emit_108(N, X86::PMULDQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v2i64 501:iPTR, (bitconvert:v4i32 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVSXDQrm:v2i64 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 501) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVSXDQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v2i64 507:iPTR, (bitconvert:v4i32 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVZXDQrm:v2i64 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 507) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVZXDQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v2i64 503:iPTR, (bitconvert:v8i16 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVSXWQrm:v2i64 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 503) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVSXWQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v2i64 509:iPTR, (bitconvert:v8i16 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVZXWQrm:v2i64 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 509) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVZXWQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v2i64 499:iPTR, (bitconvert:v16i8 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVSXBQrm:v2i64 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 499) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVSXBQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v2i64 505:iPTR, (bitconvert:v16i8 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (PMOVZXBQrm:v2i64 addr:iPTR:$src)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 505) {
        SDOperand N2 = N.getOperand(2);
        if (N2.getOpcode() == ISD::BIT_CONVERT &&
            N2.hasOneUse()) {
          SDOperand N20 = N2.getOperand(0);
          if (N20.getOpcode() == ISD::LOAD &&
              N20.hasOneUse() &&
              CanBeFoldedBy(N20.Val, N2.Val, N.Val) &&
              (Chain.Val == N20.Val || IsChainCompatible(Chain.Val, N20.Val))) {
            SDOperand Chain20 = N20.getOperand(0);
            if (Predicate_load(N20.Val)) {
              SDOperand N201 = N20.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N201, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N20.Val->getValueType(0) == MVT::v4i32) {
                return Emit_114(N, X86::PMOVZXBQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v2i64 477:iPTR, addr:iPTR:$src)
      // Emits: (MOVNTDQArm:v2i64 addr:iPTR:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 477) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_110(N, X86::MOVNTDQArm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_w_chain:v2i64 482:iPTR, VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Emits: (PCMPEQQrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 482) {
        return Emit_112(N, X86::PCMPEQQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_w_chain:v2i64 510:iPTR, VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Emits: (PMULDQrr:v2i64 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 510) {
        return Emit_112(N, X86::PMULDQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_w_chain:v2i64 501:iPTR, VR128:v4i32:$src)
      // Emits: (PMOVSXDQrr:v2i64 VR128:v4i32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 501) {
        return Emit_115(N, X86::PMOVSXDQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_w_chain:v2i64 507:iPTR, VR128:v4i32:$src)
      // Emits: (PMOVZXDQrr:v2i64 VR128:v4i32:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 507) {
        return Emit_115(N, X86::PMOVZXDQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_w_chain:v2i64 503:iPTR, VR128:v8i16:$src)
      // Emits: (PMOVSXWQrr:v2i64 VR128:v8i16:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 503) {
        return Emit_115(N, X86::PMOVSXWQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_w_chain:v2i64 509:iPTR, VR128:v8i16:$src)
      // Emits: (PMOVZXWQrr:v2i64 VR128:v8i16:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 509) {
        return Emit_115(N, X86::PMOVZXWQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_w_chain:v2i64 499:iPTR, VR128:v16i8:$src)
      // Emits: (PMOVSXBQrr:v2i64 VR128:v16i8:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 499) {
        return Emit_115(N, X86::PMOVSXBQrr, MVT::v2i64);
      }

      // Pattern: (intrinsic_w_chain:v2i64 505:iPTR, VR128:v16i8:$src)
      // Emits: (PMOVZXBQrr:v2i64 VR128:v16i8:$src)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 505) {
        return Emit_115(N, X86::PMOVZXBQrr, MVT::v2i64);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_INTRINSIC_W_CHAIN_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_w_chain:v4f32 470:iPTR, VR128:v4f32:$src1, (bitconvert:v4f32 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>), (imm:i32):$src3)
      // Emits: (BLENDPSrmi:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2, (imm:i32):$src3)
      // Pattern complexity = 33  cost = 1  size = 3
      if (CN0 == 470) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N4 = N.getOperand(4);
                if (N4.getOpcode() == ISD::Constant &&
                    N30.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_107(N, X86::BLENDPSrmi, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4f32 474:iPTR, VR128:v4f32:$src1, (bitconvert:v4f32 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>), (imm:i32):$src3)
      // Emits: (DPPSrmi:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2, (imm:i32):$src3)
      // Pattern complexity = 33  cost = 1  size = 3
      if (CN0 == 474) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N4 = N.getOperand(4);
                if (N4.getOpcode() == ISD::Constant &&
                    N30.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_107(N, X86::DPPSrmi, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v4f32 472:iPTR, VR128:v4f32:$src1, (bitconvert:v4f32 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>), XMM0:v4f32)
      // Emits: (BLENDVPSrm0:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 472) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N4 = N.getOperand(4);
                if (N30.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_109(N, X86::BLENDVPSrm0, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (intrinsic_w_chain:v4f32 539:iPTR, addr:iPTR:$src)
  // Emits: (MOVUPSrm_Int:v4f32 addr:iPTR:$src)
  // Pattern complexity = 23  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
      if (CN0 == 539) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_110(N, X86::MOVUPSrm_Int, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_w_chain:v4f32 470:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2, (imm:i32):$src3)
      // Emits: (BLENDPSrri:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2, (imm:i32):$src3)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 470) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand N4 = N.getOperand(4);
        if (N4.getOpcode() == ISD::Constant) {
          return Emit_111(N, X86::BLENDPSrri, MVT::v4f32);
        }
      }

      // Pattern: (intrinsic_w_chain:v4f32 474:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2, (imm:i32):$src3)
      // Emits: (DPPSrri:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2, (imm:i32):$src3)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 474) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand N4 = N.getOperand(4);
        if (N4.getOpcode() == ISD::Constant) {
          return Emit_111(N, X86::DPPSrri, MVT::v4f32);
        }
      }

      // Pattern: (intrinsic_w_chain:v4f32 472:iPTR, VR128:v4f32:$src1, VR128:v4f32:$src2, XMM0:v4f32)
      // Emits: (BLENDVPSrr0:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 472) {
        return Emit_113(N, X86::BLENDVPSrr0, MVT::v4f32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_117(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Select_ISD_INTRINSIC_W_CHAIN_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_w_chain:v2f64 469:iPTR, VR128:v2f64:$src1, (bitconvert:v2f64 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>), (imm:i32):$src3)
      // Emits: (BLENDPDrmi:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2, (imm:i32):$src3)
      // Pattern complexity = 33  cost = 1  size = 3
      if (CN0 == 469) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N4 = N.getOperand(4);
                if (N4.getOpcode() == ISD::Constant &&
                    N30.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_107(N, X86::BLENDPDrmi, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v2f64 473:iPTR, VR128:v2f64:$src1, (bitconvert:v2f64 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>), (imm:i32):$src3)
      // Emits: (DPPDrmi:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2, (imm:i32):$src3)
      // Pattern complexity = 33  cost = 1  size = 3
      if (CN0 == 473) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N4 = N.getOperand(4);
                if (N4.getOpcode() == ISD::Constant &&
                    N30.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_107(N, X86::DPPDrmi, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (intrinsic_w_chain:v2f64 471:iPTR, VR128:v2f64:$src1, (bitconvert:v2f64 (ld:v16i8 addr:iPTR:$src2)<<P:Predicate_memop>>), XMM0:v2f64)
      // Emits: (BLENDVPDrm0:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 30  cost = 1  size = 3
      if (CN0 == 471) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        if (N3.getOpcode() == ISD::BIT_CONVERT &&
            N3.hasOneUse()) {
          SDOperand N30 = N3.getOperand(0);
          if (N30.getOpcode() == ISD::LOAD &&
              N30.hasOneUse() &&
              CanBeFoldedBy(N30.Val, N3.Val, N.Val) &&
              (Chain.Val == N30.Val || IsChainCompatible(Chain.Val, N30.Val))) {
            SDOperand Chain30 = N30.getOperand(0);
            if (Predicate_memop(N30.Val)) {
              SDOperand N301 = N30.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N301, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N4 = N.getOperand(4);
                if (N30.Val->getValueType(0) == MVT::v16i8) {
                  return Emit_109(N, X86::BLENDVPDrm0, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_w_chain:v2f64 383:iPTR, addr:iPTR:$src)
      // Emits: (MOVUPDrm_Int:v2f64 addr:iPTR:$src)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 383) {
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_110(N, X86::MOVUPDrm_Int, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_w_chain:v2f64 380:iPTR, VR128:v2f64:$src1, addr:iPTR:$src2)
      // Emits: (MOVHPDrm:v2f64 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 380) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N3, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_117(N, X86::MOVHPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }

      // Pattern: (intrinsic_w_chain:v2f64 381:iPTR, VR128:v2f64:$src1, addr:iPTR:$src2)
      // Emits: (MOVLPDrm:v2f64 VR128:v16i8:$src1, addr:iPTR:$src2)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 381) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N3, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_117(N, X86::MOVLPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

      // Pattern: (intrinsic_w_chain:v2f64 469:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2, (imm:i32):$src3)
      // Emits: (BLENDPDrri:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2, (imm:i32):$src3)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 469) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand N4 = N.getOperand(4);
        if (N4.getOpcode() == ISD::Constant) {
          return Emit_111(N, X86::BLENDPDrri, MVT::v2f64);
        }
      }

      // Pattern: (intrinsic_w_chain:v2f64 473:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2, (imm:i32):$src3)
      // Emits: (DPPDrri:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2, (imm:i32):$src3)
      // Pattern complexity = 11  cost = 1  size = 3
      if (CN0 == 473) {
        SDOperand N2 = N.getOperand(2);
        SDOperand N3 = N.getOperand(3);
        SDOperand N4 = N.getOperand(4);
        if (N4.getOpcode() == ISD::Constant) {
          return Emit_111(N, X86::DPPDrri, MVT::v2f64);
        }
      }

      // Pattern: (intrinsic_w_chain:v2f64 471:iPTR, VR128:v2f64:$src1, VR128:v2f64:$src2, XMM0:v2f64)
      // Emits: (BLENDVPDrr0:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 471) {
        return Emit_113(N, X86::BLENDVPDrr0, MVT::v2f64);
      }
    }
  }
  cerr << "Cannot yet select: ";
  unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
  cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_118(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Select_ISD_LOAD_i8(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);

  // Pattern: (ld:i8 addr:iPTR:$src)<<P:Predicate_load>>
  // Emits: (MOV8rm:i8 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_load(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOV8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i8 addr:iPTR:$src)<<P:Predicate_zextloadi1>>
  // Emits: (MOV8rm:i8 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_zextloadi1(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOV8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i8 addr:iPTR:$src)<<P:Predicate_extloadi1>>
  // Emits: (MOV8rm:i8 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_extloadi1(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOV8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_LOAD_i16(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);

  // Pattern: (ld:i16 addr:iPTR:$src)<<P:Predicate_load>>
  // Emits: (MOV16rm:i16 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_load(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOV16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i16 addr:iPTR:$src)<<P:Predicate_sextloadi8>>
  // Emits: (MOVSX16rm8:i16 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_sextloadi8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVSX16rm8, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i16 addr:iPTR:$src)<<P:Predicate_zextloadi8>>
  // Emits: (MOVZX16rm8:i16 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_zextloadi8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX16rm8, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i16 addr:iPTR:$src)<<P:Predicate_zextloadi1>>
  // Emits: (MOVZX16rm8:i16 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_zextloadi1(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX16rm8, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i16 addr:iPTR:$src)<<P:Predicate_extloadi1>>
  // Emits: (MOVZX16rm8:i16 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_extloadi1(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX16rm8, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i16 addr:iPTR:$src)<<P:Predicate_extloadi8>>
  // Emits: (MOVZX16rm8:i16 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_extloadi8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX16rm8, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_119(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N110 = N11.getOperand(0);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, N110, LSI_N, Chain);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Emit_120(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, N100, LSI_N, Chain);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Emit_121(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N11);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, N11, LSI_N, Chain);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Emit_122(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N10);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, N10, LSI_N, Chain);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Select_ISD_LOAD_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  if (Predicate_load(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::ADD) {
      SDOperand N10 = N1.getOperand(0);

      // Pattern: (ld:i32 (add:iPTR (X86TLStp:iPTR), (X86Wrapper:iPTR (tglobaltlsaddr:iPTR):$src)))<<P:Predicate_load>>
      // Emits: (TLS_gs_ri:i32 (tglobaltlsaddr:i32):$src)
      // Pattern complexity = 31  cost = 1  size = 3
      if (N10.getOpcode() == X86ISD::THREAD_POINTER) {
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == X86ISD::Wrapper) {
          SDOperand N110 = N11.getOperand(0);
          if (N110.getOpcode() == ISD::TargetGlobalTLSAddress) {
            return Emit_119(N, X86::TLS_gs_ri, MVT::i32);
          }
        }
      }

      // Pattern: (ld:i32 (add:iPTR (X86Wrapper:iPTR (tglobaltlsaddr:iPTR):$src), (X86TLStp:iPTR)))<<P:Predicate_load>>
      // Emits: (TLS_gs_ri:i32 (tglobaltlsaddr:i32):$src)
      // Pattern complexity = 31  cost = 1  size = 3
      if (N10.getOpcode() == X86ISD::Wrapper) {
        SDOperand N100 = N10.getOperand(0);
        if (N100.getOpcode() == ISD::TargetGlobalTLSAddress) {
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == X86ISD::THREAD_POINTER) {
            return Emit_120(N, X86::TLS_gs_ri, MVT::i32);
          }
        }
      }

      // Pattern: (ld:i32 (add:i32 (X86TLStp:i32), GR32:i32:$src))<<P:Predicate_load>>
      // Emits: (TLS_gs_rr:i32 GR32:i32:$src)
      // Pattern complexity = 20  cost = 1  size = 3
      if (N10.getOpcode() == X86ISD::THREAD_POINTER) {
        SDOperand N11 = N1.getOperand(1);
        if (N1.Val->getValueType(0) == MVT::i32) {
          return Emit_121(N, X86::TLS_gs_rr, MVT::i32);
        }
      }

      // Pattern: (ld:i32 (add:i32 GR32:i32:$src, (X86TLStp:i32)))<<P:Predicate_load>>
      // Emits: (TLS_gs_rr:i32 GR32:i32:$src)
      // Pattern complexity = 20  cost = 1  size = 3
      SDOperand N11 = N1.getOperand(1);
      if (N11.getOpcode() == X86ISD::THREAD_POINTER &&
          N1.Val->getValueType(0) == MVT::i32) {
        return Emit_122(N, X86::TLS_gs_rr, MVT::i32);
      }
    }

    // Pattern: (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>
    // Emits: (MOV32rm:i32 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOV32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i32 addr:iPTR:$src)<<P:Predicate_sextloadi8>>
  // Emits: (MOVSX32rm8:i32 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_sextloadi8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVSX32rm8, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i32 addr:iPTR:$src)<<P:Predicate_sextloadi16>>
  // Emits: (MOVSX32rm16:i32 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_sextloadi16(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVSX32rm16, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i32 addr:iPTR:$src)<<P:Predicate_zextloadi8>>
  // Emits: (MOVZX32rm8:i32 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_zextloadi8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX32rm8, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i32 addr:iPTR:$src)<<P:Predicate_zextloadi16>>
  // Emits: (MOVZX32rm16:i32 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_zextloadi16(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX32rm16, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i32 addr:iPTR:$src)<<P:Predicate_zextloadi1>>
  // Emits: (MOVZX32rm8:i32 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_zextloadi1(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX32rm8, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i32 addr:iPTR:$src)<<P:Predicate_extloadi1>>
  // Emits: (MOVZX32rm8:i32 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_extloadi1(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX32rm8, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i32 addr:iPTR:$src)<<P:Predicate_extloadi8>>
  // Emits: (MOVZX32rm8:i32 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_extloadi8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX32rm8, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i32 addr:iPTR:$src)<<P:Predicate_extloadi16>>
  // Emits: (MOVZX32rm16:i32 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_extloadi16(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX32rm16, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_123(const SDOperand &N, unsigned Opc0, unsigned Opc1, MVT::ValueType VT0, MVT::ValueType VT1, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Tmp0 = CurDAG->getTargetConstant(0x0ULL, MVT::i64);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N, Chain };
  SDOperand Tmp2(CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6), 0);
  Chain = SDOperand(Tmp2.Val, 1);
  SDOperand Tmp3 = CurDAG->getTargetConstant(0x3ULL, MVT::i32);
  AddToISelQueue(Chain);
  SDOperand Ops1[] = { Tmp0, Tmp2, Tmp3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc1, VT1, MVT::Other, Ops1, 4);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Emit_124(const SDOperand &N, unsigned Opc0, unsigned Opc1, unsigned Opc2, MVT::ValueType VT0, MVT::ValueType VT1, MVT::ValueType VT2, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Tmp0(CurDAG->getTargetNode(Opc0, VT0), 0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops1[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N, Chain };
  SDOperand Tmp2(CurDAG->getTargetNode(Opc1, VT1, MVT::Other, Ops1, 6), 0);
  Chain = SDOperand(Tmp2.Val, 1);
  SDOperand Tmp3 = CurDAG->getTargetConstant(0x3ULL, MVT::i32);
  AddToISelQueue(Chain);
  SDOperand Ops2[] = { Tmp0, Tmp2, Tmp3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc2, VT2, MVT::Other, Ops2, 4);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Select_ISD_LOAD_i64(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_load>>
  // Emits: (MOV64rm:i64 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_load(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOV64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_sextloadi8>>
  // Emits: (MOVSX64rm8:i64 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_sextloadi8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVSX64rm8, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_sextloadi16>>
  // Emits: (MOVSX64rm16:i64 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_sextloadi16(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVSX64rm16, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_sextloadi32>>
  // Emits: (MOVSX64rm32:i64 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_sextloadi32(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVSX64rm32, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_zextloadi8>>
  // Emits: (MOVZX64rm8:i64 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_zextloadi8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX64rm8, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_zextloadi16>>
  // Emits: (MOVZX64rm16:i64 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_zextloadi16(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX64rm16, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_zextloadi1>>
  // Emits: (MOVZX64rm8:i64 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_zextloadi1(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX64rm8, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_extloadi1>>
  // Emits: (MOVZX64rm8:i64 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_extloadi1(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX64rm8, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_extloadi8>>
  // Emits: (MOVZX64rm8:i64 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_extloadi8(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX64rm8, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_extloadi16>>
  // Emits: (MOVZX64rm16:i64 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if (Predicate_extloadi16(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::MOVZX64rm16, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_zextloadi32>>
  // Emits: (SUBREG_TO_REG:i64 0:i64, (MOV32rm:i32 addr:iPTR:$src), 3:i32)
  // Pattern complexity = 19  cost = 2  size = 3
  if (Predicate_zextloadi32(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_123(N, X86::MOV32rm, TargetInstrInfo::SUBREG_TO_REG, MVT::i32, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:i64 addr:iPTR:$src)<<P:Predicate_extloadi32>>
  // Emits: (INSERT_SUBREG:i64 (IMPLICIT_DEF:i64), (MOV32rm:i32 addr:iPTR:$src), 3:i32)
  // Pattern complexity = 19  cost = 3  size = 3
  if (Predicate_extloadi32(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_124(N, TargetInstrInfo::IMPLICIT_DEF, X86::MOV32rm, TargetInstrInfo::INSERT_SUBREG, MVT::i64, MVT::i32, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_LOAD_f32(const SDOperand &N) {

  // Pattern: (ld:f32 addr:iPTR:$src)<<P:Predicate_load>>
  // Emits: (LD_Fp32m:f32 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 0
  if ((!Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_load(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::LD_Fp32m, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (ld:f32 addr:iPTR:$src)<<P:Predicate_load>>
    // Emits: (MOVSSrm:f32 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_load(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MOVSSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (ld:f32 addr:iPTR:$src)<<P:Predicate_alignedload>>
    // Emits: (FsMOVAPSrm:f32 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_alignedload(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::FsMOVAPSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_LOAD_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (ld:f64 addr:iPTR:$src)<<P:Predicate_load>>
    // Emits: (LD_Fp64m:f64 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 0
    if (Predicate_load(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::LD_Fp64m, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (ld:f64 addr:iPTR:$src)<<P:Predicate_extloadf32>>
    // Emits: (LD_Fp32m64:f64 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 0
    if (Predicate_extloadf32(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::LD_Fp32m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (ld:f64 addr:iPTR:$src)<<P:Predicate_load>>
    // Emits: (MOVSDrm:f64 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_load(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MOVSDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (ld:f64 addr:iPTR:$src)<<P:Predicate_extloadf32>>
    // Emits: (CVTSS2SDrm:f64 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_extloadf32(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::CVTSS2SDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (ld:f64 addr:iPTR:$src)<<P:Predicate_alignedload>>
    // Emits: (FsMOVAPDrm:f64 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_alignedload(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::FsMOVAPDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_LOAD_f80(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);

  // Pattern: (ld:f80 addr:iPTR:$src)<<P:Predicate_load>>
  // Emits: (LD_Fp80m:f80 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 0
  if (Predicate_load(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::LD_Fp80m, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:f80 addr:iPTR:$src)<<P:Predicate_extloadf64>>
  // Emits: (LD_Fp64m80:f80 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 0
  if (Predicate_extloadf64(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::LD_Fp64m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: (ld:f80 addr:iPTR:$src)<<P:Predicate_extloadf32>>
  // Emits: (LD_Fp32m80:f80 addr:iPTR:$src)
  // Pattern complexity = 19  cost = 1  size = 0
  if (Predicate_extloadf32(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_118(N, X86::LD_Fp32m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_LOAD_v1i64(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_load(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MMX_MOVQ64rm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_LOAD_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (ld:v4i32 addr:iPTR:$src)<<P:Predicate_alignedload>>
    // Emits: (MOVAPSrm:v4i32 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_alignedload(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MOVAPSrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>
    // Emits: (MOVUPSrm:v4i32 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_load(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MOVUPSrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_LOAD_v2i64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (ld:v2i64 addr:iPTR:$src)<<P:Predicate_alignedload>>
    // Emits: (MOVAPSrm:v2i64 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_alignedload(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MOVAPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (ld:v2i64 addr:iPTR:$src)<<P:Predicate_load>>
    // Emits: (MOVUPSrm:v2i64 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_load(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MOVUPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_LOAD_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (ld:v4f32 addr:iPTR:$src)<<P:Predicate_alignedload>>
    // Emits: (MOVAPSrm:v4f32 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_alignedload(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MOVAPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>
    // Emits: (MOVUPSrm:v4f32 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_load(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MOVUPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_LOAD_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (ld:v2f64 addr:iPTR:$src)<<P:Predicate_alignedload>>
    // Emits: (MOVAPDrm:v2f64 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_alignedload(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MOVAPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>
    // Emits: (MOVUPDrm:v2f64 addr:iPTR:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_load(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        return Emit_118(N, X86::MOVUPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_125(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  SDOperand N4 = N.getOperand(4);
  SDOperand N5 = N.getOperand(5);
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Chain);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_ISD_MEMBARRIER(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();

    // Pattern: (membarrier:void 0:i8, 0:i8, 0:i8, 1:i8, 1:i8)
    // Emits: (SFENCE:void)
    // Pattern complexity = 28  cost = 1  size = 3
    if (CN0 == 0) {
      SDOperand N2 = N.getOperand(2);
      if (isa<ConstantSDNode>(N2)) {
        int64_t CN1 = cast<ConstantSDNode>(N2)->getSignExtended();
        if (CN1 == 0) {
          SDOperand N3 = N.getOperand(3);
          if (isa<ConstantSDNode>(N3)) {
            int64_t CN2 = cast<ConstantSDNode>(N3)->getSignExtended();
            if (CN2 == 0) {
              SDOperand N4 = N.getOperand(4);
              if (isa<ConstantSDNode>(N4)) {
                int64_t CN3 = cast<ConstantSDNode>(N4)->getSignExtended();
                if (CN3 == 1) {
                  SDOperand N5 = N.getOperand(5);
                  if (isa<ConstantSDNode>(N5)) {
                    int64_t CN4 = cast<ConstantSDNode>(N5)->getSignExtended();
                    if (CN4 == 1 &&
                        N1.Val->getValueType(0) == MVT::i8) {
                      return Emit_125(N, X86::SFENCE);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }

    // Pattern: (membarrier:void 1:i8, 0:i8, 0:i8, 0:i8, 1:i8)
    // Emits: (LFENCE:void)
    // Pattern complexity = 28  cost = 1  size = 3
    if (CN0 == 1) {
      SDOperand N2 = N.getOperand(2);
      if (isa<ConstantSDNode>(N2)) {
        int64_t CN1 = cast<ConstantSDNode>(N2)->getSignExtended();
        if (CN1 == 0) {
          SDOperand N3 = N.getOperand(3);
          if (isa<ConstantSDNode>(N3)) {
            int64_t CN2 = cast<ConstantSDNode>(N3)->getSignExtended();
            if (CN2 == 0) {
              SDOperand N4 = N.getOperand(4);
              if (isa<ConstantSDNode>(N4)) {
                int64_t CN3 = cast<ConstantSDNode>(N4)->getSignExtended();
                if (CN3 == 0) {
                  SDOperand N5 = N.getOperand(5);
                  if (isa<ConstantSDNode>(N5)) {
                    int64_t CN4 = cast<ConstantSDNode>(N5)->getSignExtended();
                    if (CN4 == 1 &&
                        N1.Val->getValueType(0) == MVT::i8) {
                      return Emit_125(N, X86::LFENCE);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  if (N1.getOpcode() == ISD::Constant) {
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant) {
      SDOperand N3 = N.getOperand(3);
      if (N3.getOpcode() == ISD::Constant) {
        SDOperand N4 = N.getOperand(4);
        if (N4.getOpcode() == ISD::Constant) {
          SDOperand N5 = N.getOperand(5);
          if (isa<ConstantSDNode>(N5)) {
            int64_t CN0 = cast<ConstantSDNode>(N5)->getSignExtended();

            // Pattern: (membarrier:void (imm:i8):$ll, (imm:i8):$ls, (imm:i8):$sl, (imm:i8):$ss, 0:i8)
            // Emits: (NOOP:void)
            // Pattern complexity = 20  cost = 1  size = 3
            if (CN0 == 0 &&
                N1.Val->getValueType(0) == MVT::i8) {
              return Emit_125(N, X86::NOOP);
            }

            // Pattern: (membarrier:void (imm:i8):$ll, (imm:i8):$ls, (imm:i8):$sl, (imm:i8):$ss, 1:i8)
            // Emits: (MFENCE:void)
            // Pattern complexity = 20  cost = 1  size = 3
            if (CN0 == 1 &&
                N1.Val->getValueType(0) == MVT::i8) {
              return Emit_125(N, X86::MFENCE);
            }
          }
        }
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_126(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  AddToISelQueue(N0);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain1, X86::AL, N0, InFlag).Val;
  Chain1 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1, InFlag };
  ResNode = CurDAG->getTargetNode(Opc0, MVT::i8, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_127(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N1);
  SDOperand Chain = CurDAG->getEntryNode();
  AddToISelQueue(N0);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::AL, N0, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ResNode = CurDAG->getTargetNode(Opc0, MVT::i8, N1, InFlag);
  return ResNode;
}
SDNode *Select_ISD_MUL_i8(const SDOperand &N) {

  // Pattern: (mul:i8 AL:i8, (ld:i8 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (MUL8m:void addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_126(N, X86::MUL8m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (mul:i8 AL:i8, GR8:i8:$src)
  // Emits: (MUL8r:void GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_127(N, X86::MUL8r);
}

SDNode *Emit_128(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N1)->getValue()), MVT::i16);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Select_ISD_MUL_i16(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::Constant) {

            // Pattern: (mul:i16 (ld:i16 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2)
            // Emits: (IMUL16rmi8:i16 addr:iPTR:$src1, (imm:i16):$src2)
            // Pattern complexity = 26  cost = 1  size = 3
            if (Predicate_i16immSExt8(N1.Val)) {
              return Emit_128(N, X86::IMUL16rmi8, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (mul:i16 (ld:i16 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i16):$src2)
            // Emits: (IMUL16rmi:i16 addr:iPTR:$src1, (imm:i16):$src2)
            // Pattern complexity = 25  cost = 1  size = 3
            return Emit_128(N, X86::IMUL16rmi, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (mul:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (IMUL16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::IMUL16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (mul:i16 (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, GR16:i16:$src1)
    // Emits: (IMUL16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::IMUL16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (mul:i16 GR16:i16:$src1, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2)
      // Emits: (IMUL16rri8:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i16immSExt8(N1.Val)) {
        return Emit_5(N, X86::IMUL16rri8, MVT::i16);
      }

      // Pattern: (mul:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Emits: (IMUL16rri:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_5(N, X86::IMUL16rri, MVT::i16);
    }
  }

  // Pattern: (mul:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Emits: (IMUL16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::IMUL16rr, MVT::i16);
}

SDNode *Emit_129(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Select_ISD_MUL_i32(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::Constant) {

            // Pattern: (mul:i32 (ld:i32 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
            // Emits: (IMUL32rmi8:i32 addr:iPTR:$src1, (imm:i32):$src2)
            // Pattern complexity = 26  cost = 1  size = 3
            if (Predicate_i32immSExt8(N1.Val)) {
              return Emit_129(N, X86::IMUL32rmi8, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (mul:i32 (ld:i32 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i32):$src2)
            // Emits: (IMUL32rmi:i32 addr:iPTR:$src1, (imm:i32):$src2)
            // Pattern complexity = 25  cost = 1  size = 3
            return Emit_129(N, X86::IMUL32rmi, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (mul:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (IMUL32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::IMUL32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (mul:i32 (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, GR32:i32:$src1)
    // Emits: (IMUL32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::IMUL32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: lea32addr:i32:$src
  // Emits: (LEA64_32r:i32 lea32addr:i32:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  if ((Subtarget->is64Bit())) {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA64_32r, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: lea32addr:i32:$src
  // Emits: (LEA32r:i32 lea32addr:i32:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  if ((!Subtarget->is64Bit())) {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA32r, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (mul:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (IMUL32rri8:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val)) {
        return Emit_11(N, X86::IMUL32rri8, MVT::i32);
      }

      // Pattern: (mul:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (IMUL32rri:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_11(N, X86::IMUL32rri, MVT::i32);
    }
  }

  // Pattern: (mul:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Emits: (IMUL32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::IMUL32rr, MVT::i32);
}

SDNode *Emit_130(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N1)->getValue()), MVT::i64);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Select_ISD_MUL_i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::Constant) {

            // Pattern: (mul:i64 (ld:i64 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
            // Emits: (IMUL64rmi32:i64 addr:iPTR:$src1, (imm:i64):$src2)
            // Pattern complexity = 26  cost = 1  size = 3
            if (Predicate_i64immSExt32(N1.Val)) {
              return Emit_130(N, X86::IMUL64rmi32, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (mul:i64 (ld:i64 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
            // Emits: (IMUL64rmi8:i64 addr:iPTR:$src1, (imm:i64):$src2)
            // Pattern complexity = 26  cost = 1  size = 3
            if (Predicate_i64immSExt8(N1.Val)) {
              return Emit_130(N, X86::IMUL64rmi8, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (mul:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (IMUL64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::IMUL64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (mul:i64 (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, GR64:i64:$src1)
    // Emits: (IMUL64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::IMUL64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: lea64addr:i64:$src
  // Emits: (LEA64r:i64 lea64addr:i64:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA64r, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (mul:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Emits: (IMUL64rri32:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt32(N1.Val)) {
        return Emit_12(N, X86::IMUL64rri32, MVT::i64);
      }

      // Pattern: (mul:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (IMUL64rri8:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val)) {
        return Emit_12(N, X86::IMUL64rri8, MVT::i64);
      }
    }
  }

  // Pattern: (mul:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Emits: (IMUL64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::IMUL64rr, MVT::i64);
}

SDNode *Select_ISD_MUL_v4i16(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (mul:v4i16 VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
      // Emits: (MMX_PMULLWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_load(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_13(N, X86::MMX_PMULLWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (mul:v4i16 (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), VR64:v4i16:$src1)
      // Emits: (MMX_PMULLWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      if (N0.getOpcode() == ISD::BIT_CONVERT &&
          N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_load(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N1 = N.getOperand(1);
              if (N00.Val->getValueType(0) == MVT::v1i64) {
                return Emit_14(N, X86::MMX_PMULLWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }

    // Pattern: (mul:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
    // Emits: (MMX_PMULLWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PMULLWrr, MVT::v4i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_MUL_v8i16(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (mul:v8i16 VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
      // Emits: (PMULLWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::BIT_CONVERT &&
            N1.hasOneUse()) {
          SDOperand N10 = N1.getOperand(0);
          if (N10.getOpcode() == ISD::LOAD &&
              N10.hasOneUse() &&
              CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
            SDOperand Chain10 = N10.getOperand(0);
            if (Predicate_memop(N10.Val)) {
              SDOperand N101 = N10.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_13(N, X86::PMULLWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (mul:v8i16 (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), VR128:v8i16:$src1)
      // Emits: (PMULLWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
      // Pattern complexity = 25  cost = 1  size = 3
      if (N0.getOpcode() == ISD::BIT_CONVERT &&
          N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_memop(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N1 = N.getOperand(1);
              if (N00.Val->getValueType(0) == MVT::v2i64) {
                return Emit_14(N, X86::PMULLWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }

    // Pattern: (mul:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
    // Emits: (PMULLWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::PMULLWrr, MVT::v8i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_MUL_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (mul:v4i32 VR128:v4i32:$src1, (ld:v4i32 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (PMULLDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::PMULLDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (mul:v4i32 (ld:v4i32 addr:iPTR:$src2)<<P:Predicate_memop>>, VR128:v4i32:$src1)
      // Emits: (PMULLDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::PMULLDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (mul:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
    // Emits: (PMULLDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::PMULLDrr, MVT::v4i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_OR_i8(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (or:i8 GR8:i8:$src1, (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (OR8rm:i8 GR8:i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::OR8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (or:i8 (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>, GR8:i8:$src1)
    // Emits: (OR8rm:i8 GR8:i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::OR8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (or:i8 GR8:i8:$src1, (imm:i8):$src2)
    // Emits: (OR8ri:i8 GR8:i8:$src1, (imm:i8):$src2)
    // Pattern complexity = 6  cost = 1  size = 3
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {
      return Emit_3(N, X86::OR8ri, MVT::i8);
    }
  }

  // Pattern: (or:i8 GR8:i8:$src1, GR8:i8:$src2)
  // Emits: (OR8rr:i8 GR8:i8:$src1, GR8:i8:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::OR8rr, MVT::i8);
}

SDNode *Emit_131(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N110 = N11.getOperand(0);
  SDOperand N111 = N11.getOperand(1);
  AddToISelQueue(N00);
  AddToISelQueue(N10);
  SDOperand Chain = CurDAG->getEntryNode();
  AddToISelQueue(N01);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::CL, N01, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N00, N10, InFlag);
}
SDNode *Emit_132(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N010 = N01.getOperand(0);
  SDOperand N011 = N01.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N10);
  AddToISelQueue(N00);
  SDOperand Chain = CurDAG->getEntryNode();
  AddToISelQueue(N011);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::CL, N011, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N10, N00, InFlag);
}
SDNode *Select_ISD_OR_i16(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (or:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (OR16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::OR16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (or:i16 (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, GR16:i16:$src1)
    // Emits: (OR16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::OR16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (or:i16 (srl:i16 GR16:i16:$src1, CL:i8:$amt), (shl:i16 GR16:i16:$src2, (sub:i8 16:i8, CL:i8:$amt)))
    // Emits: (SHRD16rrCL:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 17  cost = 1  size = 3
    if (N0.getOpcode() == ISD::SRL) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::SHL) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::SUB) {
          SDOperand N110 = N11.getOperand(0);
          if (isa<ConstantSDNode>(N110)) {
            int64_t CN0 = cast<ConstantSDNode>(N110)->getSignExtended();
            if (CN0 == 16) {
              SDOperand N111 = N11.getOperand(1);
              if (N01 == N111 &&
                  N01.Val->getValueType(0) == MVT::i8 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_131(N, X86::SHRD16rrCL, MVT::i16);
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::SHL) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);

      // Pattern: (or:i16 (shl:i16 GR16:i16:$src1, CL:i8:$amt), (srl:i16 GR16:i16:$src2, (sub:i8 16:i8, CL:i8:$amt)))
      // Emits: (SHLD16rrCL:i16 GR16:i16:$src1, GR16:i16:$src2)
      // Pattern complexity = 17  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::SRL) {
          SDOperand N10 = N1.getOperand(0);
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::SUB) {
            SDOperand N110 = N11.getOperand(0);
            if (isa<ConstantSDNode>(N110)) {
              int64_t CN0 = cast<ConstantSDNode>(N110)->getSignExtended();
              if (CN0 == 16) {
                SDOperand N111 = N11.getOperand(1);
                if (N01 == N111 &&
                    N01.Val->getValueType(0) == MVT::i8 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_131(N, X86::SHLD16rrCL, MVT::i16);
                }
              }
            }
          }
        }
      }

      // Pattern: (or:i16 (shl:i16 GR16:i16:$src2, (sub:i8 16:i8, CL:i8:$amt)), (srl:i16 GR16:i16:$src1, CL:i8:$amt))
      // Emits: (SHRD16rrCL:i16 GR16:i16:$src1, GR16:i16:$src2)
      // Pattern complexity = 17  cost = 1  size = 3
      if (N01.getOpcode() == ISD::SUB) {
        SDOperand N010 = N01.getOperand(0);
        if (isa<ConstantSDNode>(N010)) {
          int64_t CN0 = cast<ConstantSDNode>(N010)->getSignExtended();
          if (CN0 == 16) {
            SDOperand N011 = N01.getOperand(1);
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::SRL) {
              SDOperand N10 = N1.getOperand(0);
              SDOperand N11 = N1.getOperand(1);
              if (N011 == N11 &&
                  N01.Val->getValueType(0) == MVT::i8 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_132(N, X86::SHRD16rrCL, MVT::i16);
              }
            }
          }
        }
      }
    }

    // Pattern: (or:i16 (srl:i16 GR16:i16:$src2, (sub:i8 16:i8, CL:i8:$amt)), (shl:i16 GR16:i16:$src1, CL:i8:$amt))
    // Emits: (SHLD16rrCL:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 17  cost = 1  size = 3
    if (N0.getOpcode() == ISD::SRL) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      if (N01.getOpcode() == ISD::SUB) {
        SDOperand N010 = N01.getOperand(0);
        if (isa<ConstantSDNode>(N010)) {
          int64_t CN0 = cast<ConstantSDNode>(N010)->getSignExtended();
          if (CN0 == 16) {
            SDOperand N011 = N01.getOperand(1);
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::SHL) {
              SDOperand N10 = N1.getOperand(0);
              SDOperand N11 = N1.getOperand(1);
              if (N011 == N11 &&
                  N01.Val->getValueType(0) == MVT::i8 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_132(N, X86::SHLD16rrCL, MVT::i16);
              }
            }
          }
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (or:i16 GR16:i16:$src1, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2)
      // Emits: (OR16ri8:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i16immSExt8(N1.Val)) {
        return Emit_5(N, X86::OR16ri8, MVT::i16);
      }

      // Pattern: (or:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Emits: (OR16ri:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_5(N, X86::OR16ri, MVT::i16);
    }
  }

  // Pattern: (or:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Emits: (OR16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::OR16rr, MVT::i16);
}

SDNode *Select_ISD_OR_i32(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (or:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (OR32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::OR32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (or:i32 (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, GR32:i32:$src1)
    // Emits: (OR32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::OR32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (or:i32 (srl:i32 GR32:i32:$src1, CL:i8:$amt), (shl:i32 GR32:i32:$src2, (sub:i8 32:i8, CL:i8:$amt)))
    // Emits: (SHRD32rrCL:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 17  cost = 1  size = 3
    if (N0.getOpcode() == ISD::SRL) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::SHL) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::SUB) {
          SDOperand N110 = N11.getOperand(0);
          if (isa<ConstantSDNode>(N110)) {
            int64_t CN0 = cast<ConstantSDNode>(N110)->getSignExtended();
            if (CN0 == 32) {
              SDOperand N111 = N11.getOperand(1);
              if (N01 == N111 &&
                  N01.Val->getValueType(0) == MVT::i8 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_131(N, X86::SHRD32rrCL, MVT::i32);
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::SHL) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);

      // Pattern: (or:i32 (shl:i32 GR32:i32:$src1, CL:i8:$amt), (srl:i32 GR32:i32:$src2, (sub:i8 32:i8, CL:i8:$amt)))
      // Emits: (SHLD32rrCL:i32 GR32:i32:$src1, GR32:i32:$src2)
      // Pattern complexity = 17  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::SRL) {
          SDOperand N10 = N1.getOperand(0);
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::SUB) {
            SDOperand N110 = N11.getOperand(0);
            if (isa<ConstantSDNode>(N110)) {
              int64_t CN0 = cast<ConstantSDNode>(N110)->getSignExtended();
              if (CN0 == 32) {
                SDOperand N111 = N11.getOperand(1);
                if (N01 == N111 &&
                    N01.Val->getValueType(0) == MVT::i8 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_131(N, X86::SHLD32rrCL, MVT::i32);
                }
              }
            }
          }
        }
      }

      // Pattern: (or:i32 (shl:i32 GR32:i32:$src2, (sub:i8 32:i8, CL:i8:$amt)), (srl:i32 GR32:i32:$src1, CL:i8:$amt))
      // Emits: (SHRD32rrCL:i32 GR32:i32:$src1, GR32:i32:$src2)
      // Pattern complexity = 17  cost = 1  size = 3
      if (N01.getOpcode() == ISD::SUB) {
        SDOperand N010 = N01.getOperand(0);
        if (isa<ConstantSDNode>(N010)) {
          int64_t CN0 = cast<ConstantSDNode>(N010)->getSignExtended();
          if (CN0 == 32) {
            SDOperand N011 = N01.getOperand(1);
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::SRL) {
              SDOperand N10 = N1.getOperand(0);
              SDOperand N11 = N1.getOperand(1);
              if (N011 == N11 &&
                  N01.Val->getValueType(0) == MVT::i8 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_132(N, X86::SHRD32rrCL, MVT::i32);
              }
            }
          }
        }
      }
    }

    // Pattern: (or:i32 (srl:i32 GR32:i32:$src2, (sub:i8 32:i8, CL:i8:$amt)), (shl:i32 GR32:i32:$src1, CL:i8:$amt))
    // Emits: (SHLD32rrCL:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 17  cost = 1  size = 3
    if (N0.getOpcode() == ISD::SRL) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      if (N01.getOpcode() == ISD::SUB) {
        SDOperand N010 = N01.getOperand(0);
        if (isa<ConstantSDNode>(N010)) {
          int64_t CN0 = cast<ConstantSDNode>(N010)->getSignExtended();
          if (CN0 == 32) {
            SDOperand N011 = N01.getOperand(1);
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::SHL) {
              SDOperand N10 = N1.getOperand(0);
              SDOperand N11 = N1.getOperand(1);
              if (N011 == N11 &&
                  N01.Val->getValueType(0) == MVT::i8 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_132(N, X86::SHLD32rrCL, MVT::i32);
              }
            }
          }
        }
      }
    }
  }

  // Pattern: lea32addr:i32:$src
  // Emits: (LEA64_32r:i32 lea32addr:i32:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  if ((Subtarget->is64Bit())) {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA64_32r, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: lea32addr:i32:$src
  // Emits: (LEA32r:i32 lea32addr:i32:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  if ((!Subtarget->is64Bit())) {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA32r, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (or:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (OR32ri8:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val)) {
        return Emit_11(N, X86::OR32ri8, MVT::i32);
      }

      // Pattern: (or:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (OR32ri:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_11(N, X86::OR32ri, MVT::i32);
    }
  }

  // Pattern: (or:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Emits: (OR32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::OR32rr, MVT::i32);
}

SDNode *Select_ISD_OR_i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (or:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (OR64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::OR64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (or:i64 (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, GR64:i64:$src1)
    // Emits: (OR64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::OR64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (or:i64 (srl:i64 GR64:i64:$src1, CL:i8:$amt), (shl:i64 GR64:i64:$src2, (sub:i8 64:i8, CL:i8:$amt)))
    // Emits: (SHRD64rrCL:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 17  cost = 1  size = 3
    if (N0.getOpcode() == ISD::SRL) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::SHL) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::SUB) {
          SDOperand N110 = N11.getOperand(0);
          if (isa<ConstantSDNode>(N110)) {
            int64_t CN0 = cast<ConstantSDNode>(N110)->getSignExtended();
            if (CN0 == 64) {
              SDOperand N111 = N11.getOperand(1);
              if (N01 == N111 &&
                  N01.Val->getValueType(0) == MVT::i8 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_131(N, X86::SHRD64rrCL, MVT::i64);
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::SHL) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);

      // Pattern: (or:i64 (shl:i64 GR64:i64:$src1, CL:i8:$amt), (srl:i64 GR64:i64:$src2, (sub:i8 64:i8, CL:i8:$amt)))
      // Emits: (SHLD64rrCL:i64 GR64:i64:$src1, GR64:i64:$src2)
      // Pattern complexity = 17  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::SRL) {
          SDOperand N10 = N1.getOperand(0);
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::SUB) {
            SDOperand N110 = N11.getOperand(0);
            if (isa<ConstantSDNode>(N110)) {
              int64_t CN0 = cast<ConstantSDNode>(N110)->getSignExtended();
              if (CN0 == 64) {
                SDOperand N111 = N11.getOperand(1);
                if (N01 == N111 &&
                    N01.Val->getValueType(0) == MVT::i8 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_131(N, X86::SHLD64rrCL, MVT::i64);
                }
              }
            }
          }
        }
      }

      // Pattern: (or:i64 (shl:i64 GR64:i64:$src2, (sub:i8 64:i8, CL:i8:$amt)), (srl:i64 GR64:i64:$src1, CL:i8:$amt))
      // Emits: (SHRD64rrCL:i64 GR64:i64:$src1, GR64:i64:$src2)
      // Pattern complexity = 17  cost = 1  size = 3
      if (N01.getOpcode() == ISD::SUB) {
        SDOperand N010 = N01.getOperand(0);
        if (isa<ConstantSDNode>(N010)) {
          int64_t CN0 = cast<ConstantSDNode>(N010)->getSignExtended();
          if (CN0 == 64) {
            SDOperand N011 = N01.getOperand(1);
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::SRL) {
              SDOperand N10 = N1.getOperand(0);
              SDOperand N11 = N1.getOperand(1);
              if (N011 == N11 &&
                  N01.Val->getValueType(0) == MVT::i8 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_132(N, X86::SHRD64rrCL, MVT::i64);
              }
            }
          }
        }
      }
    }

    // Pattern: (or:i64 (srl:i64 GR64:i64:$src2, (sub:i8 64:i8, CL:i8:$amt)), (shl:i64 GR64:i64:$src1, CL:i8:$amt))
    // Emits: (SHLD64rrCL:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 17  cost = 1  size = 3
    if (N0.getOpcode() == ISD::SRL) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      if (N01.getOpcode() == ISD::SUB) {
        SDOperand N010 = N01.getOperand(0);
        if (isa<ConstantSDNode>(N010)) {
          int64_t CN0 = cast<ConstantSDNode>(N010)->getSignExtended();
          if (CN0 == 64) {
            SDOperand N011 = N01.getOperand(1);
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::SHL) {
              SDOperand N10 = N1.getOperand(0);
              SDOperand N11 = N1.getOperand(1);
              if (N011 == N11 &&
                  N01.Val->getValueType(0) == MVT::i8 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_132(N, X86::SHLD64rrCL, MVT::i64);
              }
            }
          }
        }
      }
    }
  }

  // Pattern: lea64addr:i64:$src
  // Emits: (LEA64r:i64 lea64addr:i64:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA64r, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (or:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Emits: (OR64ri32:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt32(N1.Val)) {
        return Emit_12(N, X86::OR64ri32, MVT::i64);
      }

      // Pattern: (or:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (OR64ri8:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val)) {
        return Emit_12(N, X86::OR64ri8, MVT::i64);
      }
    }
  }

  // Pattern: (or:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Emits: (OR64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::OR64rr, MVT::i64);
}

SDNode *Select_ISD_OR_v1i64(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (or:v1i64 VR64:v1i64:$src1, (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (MMX_PORrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::MMX_PORrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (or:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, VR64:v1i64:$src1)
      // Emits: (MMX_PORrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_load(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::MMX_PORrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (or:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
    // Emits: (MMX_PORrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PORrr, MVT::v1i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_OR_v2i64(const SDOperand &N) {

  // Pattern: (or:v2i64 (bitconvert:v2i64 VR128:v4f32:$src1), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
  // Emits: (ORPSrm:v2i64 VR128:v4f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 25  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::v4f32) {
            return Emit_51(N, X86::ORPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }

  // Pattern: (or:v2i64 (bitconvert:v2i64 VR128:v2f64:$src1), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
  // Emits: (ORPDrm:v2i64 VR128:v2f64:$src1, addr:iPTR:$src2)
  // Pattern complexity = 25  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::v2f64) {
            return Emit_51(N, X86::ORPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }

  // Pattern: (or:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (bitconvert:v2i64 VR128:v4f32:$src1))
  // Emits: (ORPSrm:v2i64 VR128:v4f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 25  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N10 = N1.getOperand(0);
            if (N10.Val->getValueType(0) == MVT::v4f32) {
              return Emit_52(N, X86::ORPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (or:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (bitconvert:v2i64 VR128:v2f64:$src1))
    // Emits: (ORPDrm:v2i64 VR128:v2f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 25  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N10 = N1.getOperand(0);
            if (N10.Val->getValueType(0) == MVT::v2f64) {
              return Emit_52(N, X86::ORPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (or:v2i64 VR128:v2i64:$src1, (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (PORrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::PORrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (or:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, VR128:v2i64:$src1)
    // Emits: (PORrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::PORrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (or:v2i64 (bitconvert:v2i64 VR128:v2f64:$src1), (bitconvert:v2i64 VR128:v2f64:$src2))
    // Emits: (ORPDrr:v2i64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Pattern complexity = 9  cost = 1  size = 3
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N10 = N1.getOperand(0);
        if (N00.Val->getValueType(0) == MVT::v2f64 &&
            N10.Val->getValueType(0) == MVT::v2f64) {
          return Emit_57(N, X86::ORPDrr, MVT::v2i64);
        }
      }
    }
  }

  // Pattern: (or:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Emits: (ORPSrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    return Emit_4(N, X86::ORPSrr, MVT::v2i64);
  }

  // Pattern: (or:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Emits: (PORrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    return Emit_4(N, X86::PORrr, MVT::v2i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_133(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 5);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_ISD_PREFETCH(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::Constant) {
        SDOperand N3 = N.getOperand(3);
        if (isa<ConstantSDNode>(N3)) {
          int64_t CN0 = cast<ConstantSDNode>(N3)->getSignExtended();

          // Pattern: (prefetch:void addr:iPTR:$src, (imm:i32), 3:i32)
          // Emits: (PREFETCHT0:void addr:iPTR:$src)
          // Pattern complexity = 26  cost = 1  size = 3
          if (CN0 == 3 &&
              N2.Val->getValueType(0) == MVT::i32) {
            return Emit_133(N, X86::PREFETCHT0, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (prefetch:void addr:iPTR:$src, (imm:i32), 2:i32)
          // Emits: (PREFETCHT1:void addr:iPTR:$src)
          // Pattern complexity = 26  cost = 1  size = 3
          if (CN0 == 2 &&
              N2.Val->getValueType(0) == MVT::i32) {
            return Emit_133(N, X86::PREFETCHT1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (prefetch:void addr:iPTR:$src, (imm:i32), 1:i32)
          // Emits: (PREFETCHT2:void addr:iPTR:$src)
          // Pattern complexity = 26  cost = 1  size = 3
          if (CN0 == 1 &&
              N2.Val->getValueType(0) == MVT::i32) {
            return Emit_133(N, X86::PREFETCHT2, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (prefetch:void addr:iPTR:$src, (imm:i32), 0:i32)
          // Emits: (PREFETCHNTA:void addr:iPTR:$src)
          // Pattern complexity = 26  cost = 1  size = 3
          if (CN0 == 0 &&
              N2.Val->getValueType(0) == MVT::i32) {
            return Emit_133(N, X86::PREFETCHNTA, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_134(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Chain = CurDAG->getEntryNode();
  AddToISelQueue(N1);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::CL, N1, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, InFlag);
}
SDNode *Select_ISD_ROTL_i8(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (rotl:i8 GR8:i8:$src1, 1:i8)
  // Emits: (ROL8r1:i8 GR8:i8:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::ROL8r1, MVT::i8);
    }
  }

  // Pattern: (rotl:i8 GR8:i8:$src1, (imm:i8):$src2)
  // Emits: (ROL8ri:i8 GR8:i8:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::ROL8ri, MVT::i8);
  }

  // Pattern: (rotl:i8 GR8:i8:$src, CL:i8)
  // Emits: (ROL8rCL:i8 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::ROL8rCL, MVT::i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ROTL_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (rotl:i16 GR16:i16:$src1, 1:i8)
  // Emits: (ROL16r1:i16 GR16:i16:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::ROL16r1, MVT::i16);
    }
  }

  // Pattern: (rotl:i16 GR16:i16:$src1, (imm:i8):$src2)
  // Emits: (ROL16ri:i16 GR16:i16:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::ROL16ri, MVT::i16);
  }

  // Pattern: (rotl:i16 GR16:i16:$src, CL:i8)
  // Emits: (ROL16rCL:i16 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::ROL16rCL, MVT::i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ROTL_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (rotl:i32 GR32:i32:$src1, 1:i8)
  // Emits: (ROL32r1:i32 GR32:i32:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::ROL32r1, MVT::i32);
    }
  }

  // Pattern: (rotl:i32 GR32:i32:$src1, (imm:i8):$src2)
  // Emits: (ROL32ri:i32 GR32:i32:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::ROL32ri, MVT::i32);
  }

  // Pattern: (rotl:i32 GR32:i32:$src, CL:i8)
  // Emits: (ROL32rCL:i32 GR32:i32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::ROL32rCL, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ROTL_i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (rotl:i64 GR64:i64:$src1, 1:i8)
  // Emits: (ROL64r1:i64 GR64:i64:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::ROL64r1, MVT::i64);
    }
  }

  // Pattern: (rotl:i64 GR64:i64:$src1, (imm:i8):$src2)
  // Emits: (ROL64ri:i64 GR64:i64:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::ROL64ri, MVT::i64);
  }

  // Pattern: (rotl:i64 GR64:i64:$src, CL:i8)
  // Emits: (ROL64rCL:i64 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::ROL64rCL, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ROTR_i8(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (rotr:i8 GR8:i8:$src1, 1:i8)
  // Emits: (ROR8r1:i8 GR8:i8:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::ROR8r1, MVT::i8);
    }
  }

  // Pattern: (rotr:i8 GR8:i8:$src1, (imm:i8):$src2)
  // Emits: (ROR8ri:i8 GR8:i8:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::ROR8ri, MVT::i8);
  }

  // Pattern: (rotr:i8 GR8:i8:$src, CL:i8)
  // Emits: (ROR8rCL:i8 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::ROR8rCL, MVT::i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ROTR_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (rotr:i16 GR16:i16:$src1, 1:i8)
  // Emits: (ROR16r1:i16 GR16:i16:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::ROR16r1, MVT::i16);
    }
  }

  // Pattern: (rotr:i16 GR16:i16:$src1, (imm:i8):$src2)
  // Emits: (ROR16ri:i16 GR16:i16:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::ROR16ri, MVT::i16);
  }

  // Pattern: (rotr:i16 GR16:i16:$src, CL:i8)
  // Emits: (ROR16rCL:i16 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::ROR16rCL, MVT::i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ROTR_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (rotr:i32 GR32:i32:$src1, 1:i8)
  // Emits: (ROR32r1:i32 GR32:i32:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::ROR32r1, MVT::i32);
    }
  }

  // Pattern: (rotr:i32 GR32:i32:$src1, (imm:i8):$src2)
  // Emits: (ROR32ri:i32 GR32:i32:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::ROR32ri, MVT::i32);
  }

  // Pattern: (rotr:i32 GR32:i32:$src, CL:i8)
  // Emits: (ROR32rCL:i32 GR32:i32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::ROR32rCL, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ROTR_i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (rotr:i64 GR64:i64:$src1, 1:i8)
  // Emits: (ROR64r1:i64 GR64:i64:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::ROR64r1, MVT::i64);
    }
  }

  // Pattern: (rotr:i64 GR64:i64:$src1, (imm:i8):$src2)
  // Emits: (ROR64ri:i64 GR64:i64:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::ROR64ri, MVT::i64);
  }

  // Pattern: (rotr:i64 GR64:i64:$src, CL:i8)
  // Emits: (ROR64rCL:i64 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::ROR64rCL, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SCALAR_TO_VECTOR_v2i32(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (scalar_to_vector:v2i32 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>)
    // Emits: (MMX_MOVD64rm:v2i32 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::i32) {
          return Emit_58(N, X86::MMX_MOVD64rm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (scalar_to_vector:v2i32 GR32:i32:$src)
    // Emits: (MMX_MOVD64rr:v2i32 GR32:i32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i32) {
      return Emit_59(N, X86::MMX_MOVD64rr, MVT::v2i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SCALAR_TO_VECTOR_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (scalar_to_vector:v4i32 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>)
    // Emits: (MOVDI2PDIrm:v4i32 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::i32) {
          return Emit_58(N, X86::MOVDI2PDIrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (scalar_to_vector:v4i32 GR32:i32:$src)
    // Emits: (MOVDI2PDIrr:v4i32 GR32:i32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i32) {
      return Emit_59(N, X86::MOVDI2PDIrr, MVT::v4i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SCALAR_TO_VECTOR_v2i64(const SDOperand &N) {

  // Pattern: (scalar_to_vector:v2i64 (ld:i64 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (MOVQI2PQIrm:v2i64 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::i64) {
          return Emit_58(N, X86::MOVQI2PQIrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (scalar_to_vector:v2i64 (bitconvert:i64 VR64:v8i8:$src))
  // Emits: (MMX_MOVQ2DQrr:v2i64 VR64:v8i8:$src)
  // Pattern complexity = 6  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      if (N0.Val->getValueType(0) == MVT::i64 &&
          N00.Val->getValueType(0) == MVT::v8i8) {
        return Emit_64(N, X86::MMX_MOVQ2DQrr, MVT::v2i64);
      }
    }
  }

  // Pattern: (scalar_to_vector:v2i64 GR64:i64:$src)
  // Emits: (MOV64toPQIrr:v2i64 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::i64) {
      return Emit_59(N, X86::MOV64toPQIrr, MVT::v2i64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SCALAR_TO_VECTOR_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (scalar_to_vector:v4f32 (ld:f32 addr:iPTR:$src)<<P:Predicate_load>>)
    // Emits: (MOVSS2PSrm:v4f32 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::f32) {
          return Emit_58(N, X86::MOVSS2PSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (scalar_to_vector:v4f32 FR32:f32:$src)
    // Emits: (MOVSS2PSrr:v4f32 FR32:f32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::f32) {
      return Emit_59(N, X86::MOVSS2PSrr, MVT::v4f32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SCALAR_TO_VECTOR_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (scalar_to_vector:v2f64 (ld:f64 addr:iPTR:$src)<<P:Predicate_load>>)
    // Emits: (MOVSD2PDrm:v2f64 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::f64) {
          return Emit_58(N, X86::MOVSD2PDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (scalar_to_vector:v2f64 FR64:f64:$src)
    // Emits: (MOVSD2PDrr:v2f64 FR64:f64:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::f64) {
      return Emit_59(N, X86::MOVSD2PDrr, MVT::v2f64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_135(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N0);
}
SDNode *Select_ISD_SHL_i8(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (shl:i8 GR8:i8:$src1, 1:i8)
  // Emits: (ADD8rr:i8 GR8:i8:$src1, GR8:i8:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_135(N, X86::ADD8rr, MVT::i8);
    }
  }

  // Pattern: (shl:i8 GR8:i8:$src1, (imm:i8):$src2)
  // Emits: (SHL8ri:i8 GR8:i8:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SHL8ri, MVT::i8);
  }

  // Pattern: (shl:i8 GR8:i8:$src, CL:i8)
  // Emits: (SHL8rCL:i8 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SHL8rCL, MVT::i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SHL_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (shl:i16 GR16:i16:$src1, 1:i8)
  // Emits: (ADD16rr:i16 GR16:i16:$src1, GR16:i16:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_135(N, X86::ADD16rr, MVT::i16);
    }
  }

  // Pattern: (shl:i16 GR16:i16:$src1, (imm:i8):$src2)
  // Emits: (SHL16ri:i16 GR16:i16:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SHL16ri, MVT::i16);
  }

  // Pattern: (shl:i16 GR16:i16:$src, CL:i8)
  // Emits: (SHL16rCL:i16 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SHL16rCL, MVT::i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SHL_i32(const SDOperand &N) {

  // Pattern: lea32addr:i32:$src
  // Emits: (LEA64_32r:i32 lea32addr:i32:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  if ((Subtarget->is64Bit())) {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA64_32r, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }

  // Pattern: lea32addr:i32:$src
  // Emits: (LEA32r:i32 lea32addr:i32:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  if ((!Subtarget->is64Bit())) {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA32r, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (shl:i32 GR32:i32:$src1, 1:i8)
  // Emits: (ADD32rr:i32 GR32:i32:$src1, GR32:i32:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_135(N, X86::ADD32rr, MVT::i32);
    }
  }

  // Pattern: (shl:i32 GR32:i32:$src1, (imm:i8):$src2)
  // Emits: (SHL32ri:i32 GR32:i32:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SHL32ri, MVT::i32);
  }

  // Pattern: (shl:i32 GR32:i32:$src, CL:i8)
  // Emits: (SHL32rCL:i32 GR32:i32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SHL32rCL, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SHL_i64(const SDOperand &N) {

  // Pattern: lea64addr:i64:$src
  // Emits: (LEA64r:i64 lea64addr:i64:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA64r, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (shl:i64 GR64:i64:$src1, 1:i8)
  // Emits: (ADD64rr:i64 GR64:i64:$src1, GR64:i64:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_135(N, X86::ADD64rr, MVT::i64);
    }
  }

  // Pattern: (shl:i64 GR64:i64:$src1, (imm:i8):$src2)
  // Emits: (SHL64ri:i64 GR64:i64:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SHL64ri, MVT::i64);
  }

  // Pattern: (shl:i64 GR64:i64:$src, CL:i8)
  // Emits: (SHL64rCL:i64 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SHL64rCL, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SIGN_EXTEND_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  if (N0.Val->getValueType(0) == MVT::i8) {
    return Emit_59(N, X86::MOVSX16rr8, MVT::i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SIGN_EXTEND_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);

  // Pattern: (sext:i32 GR8:i8:$src)
  // Emits: (MOVSX32rr8:i32 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i8) {
    return Emit_59(N, X86::MOVSX32rr8, MVT::i32);
  }

  // Pattern: (sext:i32 GR16:i16:$src)
  // Emits: (MOVSX32rr16:i32 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i16) {
    return Emit_59(N, X86::MOVSX32rr16, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SIGN_EXTEND_i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);

  // Pattern: (sext:i64 GR8:i8:$src)
  // Emits: (MOVSX64rr8:i64 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i8) {
    return Emit_59(N, X86::MOVSX64rr8, MVT::i64);
  }

  // Pattern: (sext:i64 GR16:i16:$src)
  // Emits: (MOVSX64rr16:i64 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i16) {
    return Emit_59(N, X86::MOVSX64rr16, MVT::i64);
  }

  // Pattern: (sext:i64 GR32:i32:$src)
  // Emits: (MOVSX64rr32:i64 GR32:i32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i32) {
    return Emit_59(N, X86::MOVSX64rr32, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SINT_TO_FP_f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

          // Pattern: (sint_to_fp:f32 (ld:i64 addr:iPTR:$src)<<P:Predicate_load>>)
          // Emits: (CVTSI2SS64rm:f32 addr:iPTR:$src)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N0.Val->getValueType(0) == MVT::i64) {
            return Emit_58(N, X86::CVTSI2SS64rm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (sint_to_fp:f32 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>)
          // Emits: (CVTSI2SSrm:f32 addr:iPTR:$src)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N0.Val->getValueType(0) == MVT::i32) {
            return Emit_58(N, X86::CVTSI2SSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (sint_to_fp:f32 GR64:i64:$src)
    // Emits: (CVTSI2SS64rr:f32 GR64:i64:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i64) {
      return Emit_59(N, X86::CVTSI2SS64rr, MVT::f32);
    }

    // Pattern: (sint_to_fp:f32 GR32:i32:$src)
    // Emits: (CVTSI2SSrr:f32 GR32:i32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i32) {
      return Emit_59(N, X86::CVTSI2SSrr, MVT::f32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SINT_TO_FP_f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

          // Pattern: (sint_to_fp:f64 (ld:i64 addr:iPTR:$src)<<P:Predicate_load>>)
          // Emits: (CVTSI2SD64rm:f64 addr:iPTR:$src)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N0.Val->getValueType(0) == MVT::i64) {
            return Emit_58(N, X86::CVTSI2SD64rm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (sint_to_fp:f64 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>)
          // Emits: (CVTSI2SDrm:f64 addr:iPTR:$src)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N0.Val->getValueType(0) == MVT::i32) {
            return Emit_58(N, X86::CVTSI2SDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (sint_to_fp:f64 GR64:i64:$src)
    // Emits: (CVTSI2SD64rr:f64 GR64:i64:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i64) {
      return Emit_59(N, X86::CVTSI2SD64rr, MVT::f64);
    }

    // Pattern: (sint_to_fp:f64 GR32:i32:$src)
    // Emits: (CVTSI2SDrr:f64 GR32:i32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i32) {
      return Emit_59(N, X86::CVTSI2SDrr, MVT::f64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SINT_TO_FP_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.Val->getValueType(0) == MVT::v4i32) {
      return Emit_59(N, X86::Int_CVTDQ2PSrr, MVT::v4f32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SRA_i8(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (sra:i8 GR8:i8:$src1, 1:i8)
  // Emits: (SAR8r1:i8 GR8:i8:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::SAR8r1, MVT::i8);
    }
  }

  // Pattern: (sra:i8 GR8:i8:$src1, (imm:i8):$src2)
  // Emits: (SAR8ri:i8 GR8:i8:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SAR8ri, MVT::i8);
  }

  // Pattern: (sra:i8 GR8:i8:$src, CL:i8)
  // Emits: (SAR8rCL:i8 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SAR8rCL, MVT::i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SRA_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (sra:i16 GR16:i16:$src1, 1:i8)
  // Emits: (SAR16r1:i16 GR16:i16:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::SAR16r1, MVT::i16);
    }
  }

  // Pattern: (sra:i16 GR16:i16:$src1, (imm:i8):$src2)
  // Emits: (SAR16ri:i16 GR16:i16:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SAR16ri, MVT::i16);
  }

  // Pattern: (sra:i16 GR16:i16:$src, CL:i8)
  // Emits: (SAR16rCL:i16 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SAR16rCL, MVT::i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SRA_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (sra:i32 GR32:i32:$src1, 1:i8)
  // Emits: (SAR32r1:i32 GR32:i32:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::SAR32r1, MVT::i32);
    }
  }

  // Pattern: (sra:i32 GR32:i32:$src1, (imm:i8):$src2)
  // Emits: (SAR32ri:i32 GR32:i32:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SAR32ri, MVT::i32);
  }

  // Pattern: (sra:i32 GR32:i32:$src, CL:i8)
  // Emits: (SAR32rCL:i32 GR32:i32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SAR32rCL, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SRA_i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (sra:i64 GR64:i64:$src1, 1:i8)
  // Emits: (SAR64r1:i64 GR64:i64:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::SAR64r1, MVT::i64);
    }
  }

  // Pattern: (sra:i64 GR64:i64:$src1, (imm:i8):$src2)
  // Emits: (SAR64ri:i64 GR64:i64:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SAR64ri, MVT::i64);
  }

  // Pattern: (sra:i64 GR64:i64:$src, CL:i8)
  // Emits: (SAR64rCL:i64 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SAR64rCL, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SRL_i8(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (srl:i8 GR8:i8:$src1, 1:i8)
  // Emits: (SHR8r1:i8 GR8:i8:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::SHR8r1, MVT::i8);
    }
  }

  // Pattern: (srl:i8 GR8:i8:$src1, (imm:i8):$src2)
  // Emits: (SHR8ri:i8 GR8:i8:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SHR8ri, MVT::i8);
  }

  // Pattern: (srl:i8 GR8:i8:$src, CL:i8)
  // Emits: (SHR8rCL:i8 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SHR8rCL, MVT::i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SRL_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (srl:i16 GR16:i16:$src1, 1:i8)
  // Emits: (SHR16r1:i16 GR16:i16:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::SHR16r1, MVT::i16);
    }
  }

  // Pattern: (srl:i16 GR16:i16:$src1, (imm:i8):$src2)
  // Emits: (SHR16ri:i16 GR16:i16:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SHR16ri, MVT::i16);
  }

  // Pattern: (srl:i16 GR16:i16:$src, CL:i8)
  // Emits: (SHR16rCL:i16 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SHR16rCL, MVT::i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SRL_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (srl:i32 GR32:i32:$src1, 1:i8)
  // Emits: (SHR32r1:i32 GR32:i32:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::SHR32r1, MVT::i32);
    }
  }

  // Pattern: (srl:i32 GR32:i32:$src1, (imm:i8):$src2)
  // Emits: (SHR32ri:i32 GR32:i32:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SHR32ri, MVT::i32);
  }

  // Pattern: (srl:i32 GR32:i32:$src, CL:i8)
  // Emits: (SHR32rCL:i32 GR32:i32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SHR32rCL, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SRL_i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (srl:i64 GR64:i64:$src1, 1:i8)
  // Emits: (SHR64r1:i64 GR64:i64:$src1)
  // Pattern complexity = 8  cost = 1  size = 3
  if (isa<ConstantSDNode>(N1)) {
    int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
    if (CN0 == 1 &&
        N1.Val->getValueType(0) == MVT::i8) {
      return Emit_2(N, X86::SHR64r1, MVT::i64);
    }
  }

  // Pattern: (srl:i64 GR64:i64:$src1, (imm:i8):$src2)
  // Emits: (SHR64ri:i64 GR64:i64:$src1, (imm:i8):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::Constant &&
      N1.Val->getValueType(0) == MVT::i8) {
    return Emit_3(N, X86::SHR64ri, MVT::i64);
  }

  // Pattern: (srl:i64 GR64:i64:$src, CL:i8)
  // Emits: (SHR64rCL:i64 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i8) {
    return Emit_134(N, X86::SHR64rCL, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_136(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand Chain100 = N100.getOperand(0);
  SDOperand N1001 = N100.getOperand(1);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N110 = N11.getOperand(0);
  SDOperand N111 = N11.getOperand(1);
  SDOperand N1110 = N111.getOperand(0);
  SDOperand N1111 = N111.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N100.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain100);
  InChains.push_back(Chain100);
  Chain100 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N110);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N100 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N100)->getMemOperand());
  AddToISelQueue(Chain100);
  AddToISelQueue(N101);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain100, X86::CL, N101, InFlag).Val;
  Chain100 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N110, LSI_N, LSI_N100, Chain100, InFlag };
  ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 9);
  Chain100 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N100.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain100.Val, Chain100.ResNo));
  return ResNode;
}
SDNode *Emit_137(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N1010 = N101.getOperand(0);
  SDOperand N1011 = N101.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N110 = N11.getOperand(0);
  SDOperand Chain110 = N110.getOperand(0);
  SDOperand N1101 = N110.getOperand(1);
  SDOperand N111 = N11.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N110.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain110);
  InChains.push_back(Chain110);
  Chain110 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N100);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N110 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N110)->getMemOperand());
  AddToISelQueue(Chain110);
  AddToISelQueue(N1011);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain110, X86::CL, N1011, InFlag).Val;
  Chain110 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N100, LSI_N, LSI_N110, Chain110, InFlag };
  ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 9);
  Chain110 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N110.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain110.Val, Chain110.ResNo));
  return ResNode;
}
SDNode *Emit_138(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand Chain11 = N11.getOperand(0);
  SDOperand N111 = N11.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N11.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain11);
  InChains.push_back(Chain11);
  Chain11 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N11 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N11)->getMemOperand());
  AddToISelQueue(Chain11);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N, LSI_N11, Chain11 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain11 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N11.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain11.Val, Chain11.ResNo));
  return ResNode;
}
SDNode *Emit_139(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain10 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_140(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N11)->getValue()), MVT::i16);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 8);
  Chain10 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_141(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N11)->getValue()), MVT::i32);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 8);
  Chain10 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_142(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N11)->getValue()), MVT::i32);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand InFlag = N1.getOperand(2);
  AddToISelQueue(InFlag);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, LSI_N10, Chain10, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Ops0, 9);
  Chain10 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N1.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_143(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N11)->getValue()), MVT::i64);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 8);
  Chain10 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_144(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N11)->getValue()), MVT::i64);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand InFlag = N1.getOperand(2);
  AddToISelQueue(InFlag);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, LSI_N10, Chain10, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Ops0, 9);
  Chain10 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N1.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_145(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N11)->getValue()), MVT::i8);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 8);
  Chain10 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_146(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N12 = N1.getOperand(2);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N11);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N12)->getValue()), MVT::i8);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N11, Tmp2, LSI_N, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 9);
  Chain10 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_147(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N11)->getValue()), MVT::i8);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand InFlag = N1.getOperand(2);
  AddToISelQueue(InFlag);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, LSI_N10, Chain10, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Ops0, 9);
  Chain10 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N1.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_148(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N11);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N11, LSI_N, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 8);
  Chain10 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_149(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  AddToISelQueue(N11);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain10, X86::CL, N11, InFlag).Val;
  Chain10 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N, LSI_N10, Chain10, InFlag };
  ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 8);
  Chain10 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_150(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N12 = N1.getOperand(2);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N11);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  AddToISelQueue(N12);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain10, X86::CL, N12, InFlag).Val;
  Chain10 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N11, LSI_N, LSI_N10, Chain10, InFlag };
  ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 9);
  Chain10 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_151(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N10.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain10);
  InChains.push_back(Chain10);
  Chain10 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N11);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand InFlag = N1.getOperand(2);
  AddToISelQueue(InFlag);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N11, LSI_N, LSI_N10, Chain10, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Ops0, 9);
  Chain10 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N1.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain10.Val, Chain10.ResNo));
  return ResNode;
}
SDNode *Emit_152(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand Chain11 = N11.getOperand(0);
  SDOperand N111 = N11.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N11.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain11);
  InChains.push_back(Chain11);
  Chain11 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N10);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N11 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N11)->getMemOperand());
  AddToISelQueue(Chain11);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N10, LSI_N, LSI_N11, Chain11 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 8);
  Chain11 = SDOperand(ResNode, 0);
  ReplaceUses(SDOperand(N11.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain11.Val, Chain11.ResNo));
  return ResNode;
}
SDNode *Emit_153(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand Chain11 = N11.getOperand(0);
  SDOperand N111 = N11.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SmallVector<SDOperand, 8> InChains;
  if (Chain.Val != N11.Val) {
    AddToISelQueue(Chain);
    InChains.push_back(Chain);
  }
  AddToISelQueue(Chain11);
  InChains.push_back(Chain11);
  Chain11 = CurDAG->getNode(ISD::TokenFactor, MVT::Other, &InChains[0], InChains.size());
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N10);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  SDOperand LSI_N11 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N11)->getMemOperand());
  AddToISelQueue(Chain11);
  SDOperand InFlag = N1.getOperand(2);
  AddToISelQueue(InFlag);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N10, LSI_N, LSI_N11, Chain11, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Ops0, 9);
  Chain11 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N11.Val, 1), SDOperand(ResNode, 0));
  ReplaceUses(SDOperand(N1.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain11.Val, Chain11.ResNo));
  return ResNode;
}
SDNode *Emit_154(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N1000 = N100.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N102 = N10.getOperand(2);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N1000);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N1000, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_155(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N102 = N10.getOperand(2);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N100);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N100, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_156(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N100);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N100, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_157(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N100);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N11)->getValue()), MVT::i32);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N100, Tmp2, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 8);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_158(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  AddToISelQueue(N11);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::EFLAGS, N11, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N, Chain, InFlag };
  ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_159(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N10);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N10, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_160(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N10);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N11)->getValue()), MVT::i32);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N10, Tmp2, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 8);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_161(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N10, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_162(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N10);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N10, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_163(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N1)->getValue()), MVT::i64);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_164(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N1)->getValue()), MVT::i8);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_165(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N1)->getValue()), MVT::i16);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_166(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_167(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N1);
  SDOperand LSI_N = CurDAG->getMemOperand(cast<LSBaseSDNode>(N)->getMemOperand());
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N1, LSI_N, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_ISD_STORE(const SDOperand &N) {
  {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::OR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (st:void (or:i64 (srl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt), (shl:i64 GR64:i64:$src2, (sub:i8 64:i8, CL:i8:$amt))), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (SHRD64mrCL:void addr:iPTR:$dst, GR64:i64:$src2)
        // Pattern complexity = 55  cost = 1  size = 3
        if (N10.getOpcode() == ISD::SRL &&
            N10.hasOneUse()) {
          SDOperand N100 = N10.getOperand(0);
          if (N100.getOpcode() == ISD::LOAD &&
              N100.hasOneUse() &&
              CanBeFoldedBy(N100.Val, N10.Val, N.Val) &&
              (Chain.Val == N100.Val || IsChainCompatible(Chain.Val, N100.Val))) {
            SDOperand Chain100 = N100.getOperand(0);
            if (Predicate_load(N100.Val)) {
              SDOperand N1001 = N100.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N1001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N101 = N10.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SHL) {
                  SDOperand N110 = N11.getOperand(0);
                  SDOperand N111 = N11.getOperand(1);
                  if (N111.getOpcode() == ISD::SUB) {
                    SDOperand N1110 = N111.getOperand(0);
                    if (isa<ConstantSDNode>(N1110)) {
                      int64_t CN0 = cast<ConstantSDNode>(N1110)->getSignExtended();
                      if (CN0 == 64) {
                        SDOperand N1111 = N111.getOperand(1);
                        if (N101 == N1111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1001 == N2 &&
                              N1.Val->getValueType(0) == MVT::i64 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_136(N, X86::SHRD64mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }

        // Pattern: (st:void (or:i64 (shl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt), (srl:i64 GR64:i64:$src2, (sub:i8 64:i8, CL:i8:$amt))), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (SHLD64mrCL:void addr:iPTR:$dst, GR64:i64:$src2)
        // Pattern complexity = 55  cost = 1  size = 3
        if (N10.getOpcode() == ISD::SHL &&
            N10.hasOneUse()) {
          SDOperand N100 = N10.getOperand(0);
          if (N100.getOpcode() == ISD::LOAD &&
              N100.hasOneUse() &&
              CanBeFoldedBy(N100.Val, N10.Val, N.Val) &&
              (Chain.Val == N100.Val || IsChainCompatible(Chain.Val, N100.Val))) {
            SDOperand Chain100 = N100.getOperand(0);
            if (Predicate_load(N100.Val)) {
              SDOperand N1001 = N100.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N1001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N101 = N10.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SRL) {
                  SDOperand N110 = N11.getOperand(0);
                  SDOperand N111 = N11.getOperand(1);
                  if (N111.getOpcode() == ISD::SUB) {
                    SDOperand N1110 = N111.getOperand(0);
                    if (isa<ConstantSDNode>(N1110)) {
                      int64_t CN0 = cast<ConstantSDNode>(N1110)->getSignExtended();
                      if (CN0 == 64) {
                        SDOperand N1111 = N111.getOperand(1);
                        if (N101 == N1111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1001 == N2 &&
                              N1.Val->getValueType(0) == MVT::i64 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_136(N, X86::SHLD64mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }

        // Pattern: (st:void (or:i32 (srl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt), (shl:i32 GR32:i32:$src2, (sub:i8 32:i8, CL:i8:$amt))), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (SHRD32mrCL:void addr:iPTR:$dst, GR32:i32:$src2)
        // Pattern complexity = 55  cost = 1  size = 3
        if (N10.getOpcode() == ISD::SRL &&
            N10.hasOneUse()) {
          SDOperand N100 = N10.getOperand(0);
          if (N100.getOpcode() == ISD::LOAD &&
              N100.hasOneUse() &&
              CanBeFoldedBy(N100.Val, N10.Val, N.Val) &&
              (Chain.Val == N100.Val || IsChainCompatible(Chain.Val, N100.Val))) {
            SDOperand Chain100 = N100.getOperand(0);
            if (Predicate_load(N100.Val)) {
              SDOperand N1001 = N100.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N1001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N101 = N10.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SHL) {
                  SDOperand N110 = N11.getOperand(0);
                  SDOperand N111 = N11.getOperand(1);
                  if (N111.getOpcode() == ISD::SUB) {
                    SDOperand N1110 = N111.getOperand(0);
                    if (isa<ConstantSDNode>(N1110)) {
                      int64_t CN0 = cast<ConstantSDNode>(N1110)->getSignExtended();
                      if (CN0 == 32) {
                        SDOperand N1111 = N111.getOperand(1);
                        if (N101 == N1111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1001 == N2 &&
                              N1.Val->getValueType(0) == MVT::i32 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_136(N, X86::SHRD32mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }

        // Pattern: (st:void (or:i32 (shl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt), (srl:i32 GR32:i32:$src2, (sub:i8 32:i8, CL:i8:$amt))), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (SHLD32mrCL:void addr:iPTR:$dst, GR32:i32:$src2)
        // Pattern complexity = 55  cost = 1  size = 3
        if (N10.getOpcode() == ISD::SHL &&
            N10.hasOneUse()) {
          SDOperand N100 = N10.getOperand(0);
          if (N100.getOpcode() == ISD::LOAD &&
              N100.hasOneUse() &&
              CanBeFoldedBy(N100.Val, N10.Val, N.Val) &&
              (Chain.Val == N100.Val || IsChainCompatible(Chain.Val, N100.Val))) {
            SDOperand Chain100 = N100.getOperand(0);
            if (Predicate_load(N100.Val)) {
              SDOperand N1001 = N100.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N1001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N101 = N10.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SRL) {
                  SDOperand N110 = N11.getOperand(0);
                  SDOperand N111 = N11.getOperand(1);
                  if (N111.getOpcode() == ISD::SUB) {
                    SDOperand N1110 = N111.getOperand(0);
                    if (isa<ConstantSDNode>(N1110)) {
                      int64_t CN0 = cast<ConstantSDNode>(N1110)->getSignExtended();
                      if (CN0 == 32) {
                        SDOperand N1111 = N111.getOperand(1);
                        if (N101 == N1111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1001 == N2 &&
                              N1.Val->getValueType(0) == MVT::i32 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_136(N, X86::SHLD32mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }

        // Pattern: (st:void (or:i16 (srl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt), (shl:i16 GR16:i16:$src2, (sub:i8 16:i8, CL:i8:$amt))), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (SHRD16mrCL:void addr:iPTR:$dst, GR16:i16:$src2)
        // Pattern complexity = 55  cost = 1  size = 3
        if (N10.getOpcode() == ISD::SRL &&
            N10.hasOneUse()) {
          SDOperand N100 = N10.getOperand(0);
          if (N100.getOpcode() == ISD::LOAD &&
              N100.hasOneUse() &&
              CanBeFoldedBy(N100.Val, N10.Val, N.Val) &&
              (Chain.Val == N100.Val || IsChainCompatible(Chain.Val, N100.Val))) {
            SDOperand Chain100 = N100.getOperand(0);
            if (Predicate_load(N100.Val)) {
              SDOperand N1001 = N100.getOperand(1);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N1001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                SDOperand N101 = N10.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SHL) {
                  SDOperand N110 = N11.getOperand(0);
                  SDOperand N111 = N11.getOperand(1);
                  if (N111.getOpcode() == ISD::SUB) {
                    SDOperand N1110 = N111.getOperand(0);
                    if (isa<ConstantSDNode>(N1110)) {
                      int64_t CN0 = cast<ConstantSDNode>(N1110)->getSignExtended();
                      if (CN0 == 16) {
                        SDOperand N1111 = N111.getOperand(1);
                        if (N101 == N1111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1001 == N2 &&
                              N1.Val->getValueType(0) == MVT::i16 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_136(N, X86::SHRD16mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
        if (N10.getOpcode() == ISD::SHL) {

          // Pattern: (st:void (or:i16 (shl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt), (srl:i16 GR16:i16:$src2, (sub:i8 16:i8, CL:i8:$amt))), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SHLD16mrCL:void addr:iPTR:$dst, GR16:i16:$src2)
          // Pattern complexity = 55  cost = 1  size = 3
          if (N10.hasOneUse()) {
            SDOperand N100 = N10.getOperand(0);
            if (N100.getOpcode() == ISD::LOAD &&
                N100.hasOneUse() &&
                CanBeFoldedBy(N100.Val, N10.Val, N.Val) &&
                (Chain.Val == N100.Val || IsChainCompatible(Chain.Val, N100.Val))) {
              SDOperand Chain100 = N100.getOperand(0);
              if (Predicate_load(N100.Val)) {
                SDOperand N1001 = N100.getOperand(1);
                SDOperand CPTmp0;
                SDOperand CPTmp1;
                SDOperand CPTmp2;
                SDOperand CPTmp3;
                if (SelectAddr(N, N1001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                  SDOperand N101 = N10.getOperand(1);
                  SDOperand N11 = N1.getOperand(1);
                  if (N11.getOpcode() == ISD::SRL) {
                    SDOperand N110 = N11.getOperand(0);
                    SDOperand N111 = N11.getOperand(1);
                    if (N111.getOpcode() == ISD::SUB) {
                      SDOperand N1110 = N111.getOperand(0);
                      if (isa<ConstantSDNode>(N1110)) {
                        int64_t CN0 = cast<ConstantSDNode>(N1110)->getSignExtended();
                        if (CN0 == 16) {
                          SDOperand N1111 = N111.getOperand(1);
                          if (N101 == N1111) {
                            SDOperand N2 = N.getOperand(2);
                            if (N1001 == N2 &&
                                N1.Val->getValueType(0) == MVT::i16 &&
                                N101.Val->getValueType(0) == MVT::i8 &&
                                N111.Val->getValueType(0) == MVT::i8) {
                              return Emit_136(N, X86::SHLD16mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }

          // Pattern: (st:void (or:i64 (shl:i64 GR64:i64:$src2, (sub:i8 64:i8, CL:i8:$amt)), (srl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt)), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SHRD64mrCL:void addr:iPTR:$dst, GR64:i64:$src2)
          // Pattern complexity = 55  cost = 1  size = 3
          SDOperand N100 = N10.getOperand(0);
          SDOperand N101 = N10.getOperand(1);
          if (N101.getOpcode() == ISD::SUB) {
            SDOperand N1010 = N101.getOperand(0);
            if (isa<ConstantSDNode>(N1010)) {
              int64_t CN0 = cast<ConstantSDNode>(N1010)->getSignExtended();
              if (CN0 == 64) {
                SDOperand N1011 = N101.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SRL &&
                    N11.hasOneUse()) {
                  SDOperand N110 = N11.getOperand(0);
                  if (N110.getOpcode() == ISD::LOAD &&
                      N110.hasOneUse() &&
                      CanBeFoldedBy(N110.Val, N11.Val, N.Val) &&
                      (Chain.Val == N110.Val || IsChainCompatible(Chain.Val, N110.Val))) {
                    SDOperand Chain110 = N110.getOperand(0);
                    if (Predicate_load(N110.Val)) {
                      SDOperand N1101 = N110.getOperand(1);
                      SDOperand CPTmp0;
                      SDOperand CPTmp1;
                      SDOperand CPTmp2;
                      SDOperand CPTmp3;
                      if (SelectAddr(N, N1101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                        SDOperand N111 = N11.getOperand(1);
                        if (N1011 == N111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1101 == N2 &&
                              N1.Val->getValueType(0) == MVT::i64 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_137(N, X86::SHRD64mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }

        // Pattern: (st:void (or:i64 (srl:i64 GR64:i64:$src2, (sub:i8 64:i8, CL:i8:$amt)), (shl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt)), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (SHLD64mrCL:void addr:iPTR:$dst, GR64:i64:$src2)
        // Pattern complexity = 55  cost = 1  size = 3
        if (N10.getOpcode() == ISD::SRL) {
          SDOperand N100 = N10.getOperand(0);
          SDOperand N101 = N10.getOperand(1);
          if (N101.getOpcode() == ISD::SUB) {
            SDOperand N1010 = N101.getOperand(0);
            if (isa<ConstantSDNode>(N1010)) {
              int64_t CN0 = cast<ConstantSDNode>(N1010)->getSignExtended();
              if (CN0 == 64) {
                SDOperand N1011 = N101.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SHL &&
                    N11.hasOneUse()) {
                  SDOperand N110 = N11.getOperand(0);
                  if (N110.getOpcode() == ISD::LOAD &&
                      N110.hasOneUse() &&
                      CanBeFoldedBy(N110.Val, N11.Val, N.Val) &&
                      (Chain.Val == N110.Val || IsChainCompatible(Chain.Val, N110.Val))) {
                    SDOperand Chain110 = N110.getOperand(0);
                    if (Predicate_load(N110.Val)) {
                      SDOperand N1101 = N110.getOperand(1);
                      SDOperand CPTmp0;
                      SDOperand CPTmp1;
                      SDOperand CPTmp2;
                      SDOperand CPTmp3;
                      if (SelectAddr(N, N1101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                        SDOperand N111 = N11.getOperand(1);
                        if (N1011 == N111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1101 == N2 &&
                              N1.Val->getValueType(0) == MVT::i64 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_137(N, X86::SHLD64mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }

        // Pattern: (st:void (or:i32 (shl:i32 GR32:i32:$src2, (sub:i8 32:i8, CL:i8:$amt)), (srl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt)), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (SHRD32mrCL:void addr:iPTR:$dst, GR32:i32:$src2)
        // Pattern complexity = 55  cost = 1  size = 3
        if (N10.getOpcode() == ISD::SHL) {
          SDOperand N100 = N10.getOperand(0);
          SDOperand N101 = N10.getOperand(1);
          if (N101.getOpcode() == ISD::SUB) {
            SDOperand N1010 = N101.getOperand(0);
            if (isa<ConstantSDNode>(N1010)) {
              int64_t CN0 = cast<ConstantSDNode>(N1010)->getSignExtended();
              if (CN0 == 32) {
                SDOperand N1011 = N101.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SRL &&
                    N11.hasOneUse()) {
                  SDOperand N110 = N11.getOperand(0);
                  if (N110.getOpcode() == ISD::LOAD &&
                      N110.hasOneUse() &&
                      CanBeFoldedBy(N110.Val, N11.Val, N.Val) &&
                      (Chain.Val == N110.Val || IsChainCompatible(Chain.Val, N110.Val))) {
                    SDOperand Chain110 = N110.getOperand(0);
                    if (Predicate_load(N110.Val)) {
                      SDOperand N1101 = N110.getOperand(1);
                      SDOperand CPTmp0;
                      SDOperand CPTmp1;
                      SDOperand CPTmp2;
                      SDOperand CPTmp3;
                      if (SelectAddr(N, N1101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                        SDOperand N111 = N11.getOperand(1);
                        if (N1011 == N111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1101 == N2 &&
                              N1.Val->getValueType(0) == MVT::i32 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_137(N, X86::SHRD32mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }

        // Pattern: (st:void (or:i32 (srl:i32 GR32:i32:$src2, (sub:i8 32:i8, CL:i8:$amt)), (shl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt)), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (SHLD32mrCL:void addr:iPTR:$dst, GR32:i32:$src2)
        // Pattern complexity = 55  cost = 1  size = 3
        if (N10.getOpcode() == ISD::SRL) {
          SDOperand N100 = N10.getOperand(0);
          SDOperand N101 = N10.getOperand(1);
          if (N101.getOpcode() == ISD::SUB) {
            SDOperand N1010 = N101.getOperand(0);
            if (isa<ConstantSDNode>(N1010)) {
              int64_t CN0 = cast<ConstantSDNode>(N1010)->getSignExtended();
              if (CN0 == 32) {
                SDOperand N1011 = N101.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SHL &&
                    N11.hasOneUse()) {
                  SDOperand N110 = N11.getOperand(0);
                  if (N110.getOpcode() == ISD::LOAD &&
                      N110.hasOneUse() &&
                      CanBeFoldedBy(N110.Val, N11.Val, N.Val) &&
                      (Chain.Val == N110.Val || IsChainCompatible(Chain.Val, N110.Val))) {
                    SDOperand Chain110 = N110.getOperand(0);
                    if (Predicate_load(N110.Val)) {
                      SDOperand N1101 = N110.getOperand(1);
                      SDOperand CPTmp0;
                      SDOperand CPTmp1;
                      SDOperand CPTmp2;
                      SDOperand CPTmp3;
                      if (SelectAddr(N, N1101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                        SDOperand N111 = N11.getOperand(1);
                        if (N1011 == N111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1101 == N2 &&
                              N1.Val->getValueType(0) == MVT::i32 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_137(N, X86::SHLD32mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }

        // Pattern: (st:void (or:i16 (shl:i16 GR16:i16:$src2, (sub:i8 16:i8, CL:i8:$amt)), (srl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt)), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (SHRD16mrCL:void addr:iPTR:$dst, GR16:i16:$src2)
        // Pattern complexity = 55  cost = 1  size = 3
        if (N10.getOpcode() == ISD::SHL) {
          SDOperand N100 = N10.getOperand(0);
          SDOperand N101 = N10.getOperand(1);
          if (N101.getOpcode() == ISD::SUB) {
            SDOperand N1010 = N101.getOperand(0);
            if (isa<ConstantSDNode>(N1010)) {
              int64_t CN0 = cast<ConstantSDNode>(N1010)->getSignExtended();
              if (CN0 == 16) {
                SDOperand N1011 = N101.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SRL &&
                    N11.hasOneUse()) {
                  SDOperand N110 = N11.getOperand(0);
                  if (N110.getOpcode() == ISD::LOAD &&
                      N110.hasOneUse() &&
                      CanBeFoldedBy(N110.Val, N11.Val, N.Val) &&
                      (Chain.Val == N110.Val || IsChainCompatible(Chain.Val, N110.Val))) {
                    SDOperand Chain110 = N110.getOperand(0);
                    if (Predicate_load(N110.Val)) {
                      SDOperand N1101 = N110.getOperand(1);
                      SDOperand CPTmp0;
                      SDOperand CPTmp1;
                      SDOperand CPTmp2;
                      SDOperand CPTmp3;
                      if (SelectAddr(N, N1101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                        SDOperand N111 = N11.getOperand(1);
                        if (N1011 == N111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1101 == N2 &&
                              N1.Val->getValueType(0) == MVT::i16 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_137(N, X86::SHRD16mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }

        // Pattern: (st:void (or:i16 (srl:i16 GR16:i16:$src2, (sub:i8 16:i8, CL:i8:$amt)), (shl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8:$amt)), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (SHLD16mrCL:void addr:iPTR:$dst, GR16:i16:$src2)
        // Pattern complexity = 55  cost = 1  size = 3
        if (N10.getOpcode() == ISD::SRL) {
          SDOperand N100 = N10.getOperand(0);
          SDOperand N101 = N10.getOperand(1);
          if (N101.getOpcode() == ISD::SUB) {
            SDOperand N1010 = N101.getOperand(0);
            if (isa<ConstantSDNode>(N1010)) {
              int64_t CN0 = cast<ConstantSDNode>(N1010)->getSignExtended();
              if (CN0 == 16) {
                SDOperand N1011 = N101.getOperand(1);
                SDOperand N11 = N1.getOperand(1);
                if (N11.getOpcode() == ISD::SHL &&
                    N11.hasOneUse()) {
                  SDOperand N110 = N11.getOperand(0);
                  if (N110.getOpcode() == ISD::LOAD &&
                      N110.hasOneUse() &&
                      CanBeFoldedBy(N110.Val, N11.Val, N.Val) &&
                      (Chain.Val == N110.Val || IsChainCompatible(Chain.Val, N110.Val))) {
                    SDOperand Chain110 = N110.getOperand(0);
                    if (Predicate_load(N110.Val)) {
                      SDOperand N1101 = N110.getOperand(1);
                      SDOperand CPTmp0;
                      SDOperand CPTmp1;
                      SDOperand CPTmp2;
                      SDOperand CPTmp3;
                      if (SelectAddr(N, N1101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                        SDOperand N111 = N11.getOperand(1);
                        if (N1011 == N111) {
                          SDOperand N2 = N.getOperand(2);
                          if (N1101 == N2 &&
                              N1.Val->getValueType(0) == MVT::i16 &&
                              N101.Val->getValueType(0) == MVT::i8 &&
                              N111.Val->getValueType(0) == MVT::i8) {
                            return Emit_137(N, X86::SHLD16mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SUB &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (isa<ConstantSDNode>(N10)) {
          int64_t CN0 = cast<ConstantSDNode>(N10)->getSignExtended();
          if (CN0 == 0) {
            SDOperand N11 = N1.getOperand(1);
            if (N11.getOpcode() == ISD::LOAD &&
                N11.hasOneUse() &&
                CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
                (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
              SDOperand Chain11 = N11.getOperand(0);
              if (Predicate_load(N11.Val)) {
                SDOperand N111 = N11.getOperand(1);
                SDOperand CPTmp0;
                SDOperand CPTmp1;
                SDOperand CPTmp2;
                SDOperand CPTmp3;
                if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N111 == N2) {

                    // Pattern: (st:void (sub:i8 0:i8, (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (NEG8m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i8) {
                      return Emit_138(N, X86::NEG8m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (sub:i16 0:i16, (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (NEG16m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i16) {
                      return Emit_138(N, X86::NEG16m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (sub:i32 0:i32, (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (NEG32m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i32) {
                      return Emit_138(N, X86::NEG32m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (add:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (INC8m:void addr:iPTR:$dst)
      // Pattern complexity = 46  cost = 1  size = 2
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i8) {
                    return Emit_139(N, X86::INC8m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  if ((!Subtarget->is64Bit())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2) {

                    // Pattern: (st:void (add:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i16), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (INC16m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i16) {
                      return Emit_139(N, X86::INC16m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (add:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i32), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (INC32m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i32) {
                      return Emit_139(N, X86::INC32m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (st:void (add:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, -1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
  // Emits: (DEC8m:void addr:iPTR:$dst)
  // Pattern complexity = 46  cost = 1  size = 2
  {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == -1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i8) {
                    return Emit_139(N, X86::DEC8m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  if ((!Subtarget->is64Bit())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == -1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2) {

                    // Pattern: (st:void (add:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, -1:i16), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (DEC16m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i16) {
                      return Emit_139(N, X86::DEC16m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (add:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, -1:i32), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (DEC32m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i32) {
                      return Emit_139(N, X86::DEC32m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);

      // Pattern: (st:void (sub:i64 0:i64, (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (NEG64m:void addr:iPTR:$dst)
      // Pattern complexity = 46  cost = 1  size = 2
      if (N1.getOpcode() == ISD::SUB &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (isa<ConstantSDNode>(N10)) {
          int64_t CN0 = cast<ConstantSDNode>(N10)->getSignExtended();
          if (CN0 == 0) {
            SDOperand N11 = N1.getOperand(1);
            if (N11.getOpcode() == ISD::LOAD &&
                N11.hasOneUse() &&
                CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
                (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
              SDOperand Chain11 = N11.getOperand(0);
              if (Predicate_load(N11.Val)) {
                SDOperand N111 = N11.getOperand(1);
                SDOperand CPTmp0;
                SDOperand CPTmp1;
                SDOperand CPTmp2;
                SDOperand CPTmp3;
                if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N111 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_138(N, X86::NEG64m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();

                // Pattern: (st:void (add:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i64), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (INC64m:void addr:iPTR:$dst)
                // Pattern complexity = 46  cost = 1  size = 2
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_139(N, X86::INC64m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (add:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, -1:i64), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (DEC64m:void addr:iPTR:$dst)
                // Pattern complexity = 46  cost = 1  size = 2
                if (CN0 == -1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_139(N, X86::DEC64m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->is64Bit())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2) {

                    // Pattern: (st:void (add:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i16), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (INC64_16m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i16) {
                      return Emit_139(N, X86::INC64_16m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (add:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i32), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (INC64_32m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i32) {
                      return Emit_139(N, X86::INC64_32m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
                if (CN0 == -1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2) {

                    // Pattern: (st:void (add:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, -1:i16), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (DEC64_16m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i16) {
                      return Emit_139(N, X86::DEC64_16m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (add:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, -1:i32), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (DEC64_32m:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 2
                    if (N1.Val->getValueType(0) == MVT::i32) {
                      return Emit_139(N, X86::DEC64_32m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::SHL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2) {

                    // Pattern: (st:void (shl:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (SHL8m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i8 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::SHL8m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (shl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (SHL16m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i16 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::SHL16m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (shl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (SHL32m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i32 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::SHL32m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SRL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2) {

                    // Pattern: (st:void (srl:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (SHR8m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i8 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::SHR8m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (srl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (SHR16m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i16 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::SHR16m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (srl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (SHR32m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i32 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::SHR32m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SRA &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2) {

                    // Pattern: (st:void (sra:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (SAR8m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i8 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::SAR8m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (sra:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (SAR16m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i16 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::SAR16m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (sra:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (SAR32m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i32 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::SAR32m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ROTL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2) {

                    // Pattern: (st:void (rotl:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (ROL8m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i8 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::ROL8m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (rotl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (ROL16m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i16 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::ROL16m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (rotl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (ROL32m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i32 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::ROL32m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ROTR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2) {

                    // Pattern: (st:void (rotr:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (ROR8m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i8 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::ROR8m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (rotr:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (ROR16m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i16 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::ROR16m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }

                    // Pattern: (st:void (rotr:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                    // Emits: (ROR32m1:void addr:iPTR:$dst)
                    // Pattern complexity = 46  cost = 1  size = 3
                    if (N1.Val->getValueType(0) == MVT::i32 &&
                        N11.Val->getValueType(0) == MVT::i8) {
                      return Emit_139(N, X86::ROR32m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (shl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHL64m1:void addr:iPTR:$dst)
      // Pattern complexity = 46  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SHL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_139(N, X86::SHL64m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (srl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHR64m1:void addr:iPTR:$dst)
      // Pattern complexity = 46  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SRL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_139(N, X86::SHR64m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (sra:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SAR64m1:void addr:iPTR:$dst)
      // Pattern complexity = 46  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SRA &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_139(N, X86::SAR64m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (rotl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ROL64m1:void addr:iPTR:$dst)
      // Pattern complexity = 46  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ROTL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_139(N, X86::ROL64m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (rotr:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, 1:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ROR64m1:void addr:iPTR:$dst)
      // Pattern complexity = 46  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ROTR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 1) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_139(N, X86::ROR64m1, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::XOR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant &&
                  Predicate_immAllOnes(N11.Val)) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (xor:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8)<<P:Predicate_immAllOnes>>), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (NOT8m:void addr:iPTR:$dst)
                  // Pattern complexity = 45  cost = 1  size = 2
                  if (N1.Val->getValueType(0) == MVT::i8) {
                    return Emit_139(N, X86::NOT8m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (xor:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16)<<P:Predicate_immAllOnes>>), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (NOT16m:void addr:iPTR:$dst)
                  // Pattern complexity = 45  cost = 1  size = 2
                  if (N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_139(N, X86::NOT16m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (xor:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32)<<P:Predicate_immAllOnes>>), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (NOT32m:void addr:iPTR:$dst)
                  // Pattern complexity = 45  cost = 1  size = 2
                  if (N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_139(N, X86::NOT32m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::AND &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (and:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16)<<P:Predicate_i16immSExt8>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (AND16mi8:void addr:iPTR:$dst, (imm:i16):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i16immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_140(N, X86::AND16mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (and:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32)<<P:Predicate_i32immSExt8>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (AND32mi8:void addr:iPTR:$dst, (imm:i32):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i32immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_141(N, X86::AND32mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::OR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (or:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16)<<P:Predicate_i16immSExt8>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (OR16mi8:void addr:iPTR:$dst, (imm:i16):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i16immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_140(N, X86::OR16mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (or:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32)<<P:Predicate_i32immSExt8>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (OR32mi8:void addr:iPTR:$dst, (imm:i32):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i32immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_141(N, X86::OR32mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::XOR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (xor:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16)<<P:Predicate_i16immSExt8>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (XOR16mi8:void addr:iPTR:$dst, (imm:i16):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i16immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_140(N, X86::XOR16mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (xor:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32)<<P:Predicate_i32immSExt8>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (XOR32mi8:void addr:iPTR:$dst, (imm:i32):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i32immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_141(N, X86::XOR32mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (add:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ADD16mi8:void addr:iPTR:$dst, (imm:i16):$src2)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i16immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_140(N, X86::ADD16mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (add:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ADD32mi8:void addr:iPTR:$dst, (imm:i32):$src2)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i32immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_141(N, X86::ADD32mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (adde:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ADC32mi8:void addr:iPTR:$dst, (imm:i32):$src2)
      // Pattern complexity = 45  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ADDE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant &&
                  Predicate_i32immSExt8(N11.Val)) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_142(N, X86::ADC32mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SUB &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (sub:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SUB16mi8:void addr:iPTR:$dst, (imm:i16):$src2)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i16immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_140(N, X86::SUB16mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (sub:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SUB32mi8:void addr:iPTR:$dst, (imm:i32):$src2)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i32immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_141(N, X86::SUB32mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (sube:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SBB32mi8:void addr:iPTR:$dst, (imm:i32):$src2)
      // Pattern complexity = 45  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SUBE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant &&
                  Predicate_i32immSExt8(N11.Val)) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_142(N, X86::SBB32mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (add:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ADD64mi32:void addr:iPTR:$dst, (imm:i64):$src2)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt32(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_143(N, X86::ADD64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (add:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ADD64mi8:void addr:iPTR:$dst, (imm:i64):$src2)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_143(N, X86::ADD64mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ADDE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant &&
                  Predicate_i64immSExt8(N11.Val)) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i64) {

                  // Pattern: (st:void (adde:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ADC64mi32:void addr:iPTR:$dst, (imm:i64):$src2)
                  // Pattern complexity = 45  cost = 1  size = 3
                  {
                    return Emit_144(N, X86::ADC64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (adde:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ADC64mi8:void addr:iPTR:$dst, (imm:i64):$src2)
                  // Pattern complexity = 45  cost = 1  size = 3
                  return Emit_144(N, X86::ADC64mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SUB &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (sub:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SUB64mi32:void addr:iPTR:$dst, (imm:i64):$src2)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt32(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_143(N, X86::SUB64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (sub:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SUB64mi8:void addr:iPTR:$dst, (imm:i64):$src2)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_143(N, X86::SUB64mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SUBE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (sube:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SBB64mi32:void addr:iPTR:$dst, (imm:i64):$src2)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt32(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_144(N, X86::SBB64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (sube:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SBB64mi8:void addr:iPTR:$dst, (imm:i64):$src2)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_144(N, X86::SBB64mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (xor:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_immAllOnes>>), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (NOT64m:void addr:iPTR:$dst)
      // Pattern complexity = 45  cost = 1  size = 3
      if (N1.getOpcode() == ISD::XOR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant &&
                  Predicate_immAllOnes(N11.Val)) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i64) {
                  return Emit_139(N, X86::NOT64m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::AND &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (and:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt32>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (AND64mi32:void addr:iPTR:$dst, (imm:i64):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt32(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_143(N, X86::AND64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (and:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt8>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (AND64mi8:void addr:iPTR:$dst, (imm:i64):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_143(N, X86::AND64mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::OR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (or:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt32>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (OR64mi32:void addr:iPTR:$dst, (imm:i64):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt32(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_143(N, X86::OR64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (or:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt8>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (OR64mi8:void addr:iPTR:$dst, (imm:i64):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_143(N, X86::OR64mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::XOR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {

                // Pattern: (st:void (xor:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt32>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (XOR64mi32:void addr:iPTR:$dst, (imm:i64):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt32(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_143(N, X86::XOR64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }

                // Pattern: (st:void (xor:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt8>>:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (XOR64mi8:void addr:iPTR:$dst, (imm:i64):$src)
                // Pattern complexity = 45  cost = 1  size = 3
                if (Predicate_i64immSExt8(N11.Val)) {
                  SDOperand N2 = N.getOperand(2);
                  if (N101 == N2 &&
                      N1.Val->getValueType(0) == MVT::i64) {
                    return Emit_143(N, X86::XOR64mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::AND &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (and:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (AND8mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::AND8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (and:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (AND16mi:void addr:iPTR:$dst, (imm:i16):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_140(N, X86::AND16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (and:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (AND32mi:void addr:iPTR:$dst, (imm:i32):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_141(N, X86::AND32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::OR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (or:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (OR8mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::OR8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (or:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (OR16mi:void addr:iPTR:$dst, (imm:i16):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_140(N, X86::OR16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (or:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (OR32mi:void addr:iPTR:$dst, (imm:i32):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_141(N, X86::OR32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::XOR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (xor:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (XOR8mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::XOR8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (xor:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (XOR16mi:void addr:iPTR:$dst, (imm:i16):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_140(N, X86::XOR16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (xor:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (XOR32mi:void addr:iPTR:$dst, (imm:i32):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_141(N, X86::XOR32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SHL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (shl:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SHL8mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::SHL8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (shl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SHL16mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i16 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::SHL16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (shl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SHL32mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::SHL32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SRL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (srl:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SHR8mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::SHR8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (srl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SHR16mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i16 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::SHR16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (srl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SHR32mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::SHR32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SRA &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (sra:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SAR8mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::SAR8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (sra:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SAR16mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i16 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::SAR16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (sra:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SAR32mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::SAR32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ROTL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (rotl:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ROL8mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::ROL8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (rotl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ROL16mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i16 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::ROL16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (rotl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ROL32mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::ROL32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ROTR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (rotr:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ROR8mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::ROR8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (rotr:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ROR16mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i16 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::ROR16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (rotr:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ROR32mi:void addr:iPTR:$dst, (imm:i8):$src)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32 &&
                      N11.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::ROR32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shld:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src2, (imm:i8):$src3), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHLD32mri8:void addr:iPTR:$dst, GR32:i32:$src2, (imm:i8):$src3)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHLD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              if (N12.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i32 &&
                    N12.Val->getValueType(0) == MVT::i8) {
                  return Emit_146(N, X86::SHLD32mri8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shrd:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src2, (imm:i8):$src3), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHRD32mri8:void addr:iPTR:$dst, GR32:i32:$src2, (imm:i8):$src3)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHRD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              if (N12.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i32 &&
                    N12.Val->getValueType(0) == MVT::i8) {
                  return Emit_146(N, X86::SHRD32mri8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shld:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, GR16:i16:$src2, (imm:i8):$src3), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHLD16mri8:void addr:iPTR:$dst, GR16:i16:$src2, (imm:i8):$src3)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHLD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              if (N12.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i16 &&
                    N12.Val->getValueType(0) == MVT::i8) {
                  return Emit_146(N, X86::SHLD16mri8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shrd:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, GR16:i16:$src2, (imm:i8):$src3), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHRD16mri8:void addr:iPTR:$dst, GR16:i16:$src2, (imm:i8):$src3)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHRD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              if (N12.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i16 &&
                    N12.Val->getValueType(0) == MVT::i8) {
                  return Emit_146(N, X86::SHRD16mri8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (add:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ADD8mi:void addr:iPTR:$dst, (imm:i8):$src2)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::ADD8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (add:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ADD16mi:void addr:iPTR:$dst, (imm:i16):$src2)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_140(N, X86::ADD16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (add:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (ADD32mi:void addr:iPTR:$dst, (imm:i32):$src2)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_141(N, X86::ADD32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (adde:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ADC32mi:void addr:iPTR:$dst, (imm:i32):$src2)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ADDE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_142(N, X86::ADC32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SUB &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (sub:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SUB8mi:void addr:iPTR:$dst, (imm:i8):$src2)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8) {
                    return Emit_145(N, X86::SUB8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (sub:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i16):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SUB16mi:void addr:iPTR:$dst, (imm:i16):$src2)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i16) {
                    return Emit_140(N, X86::SUB16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (sub:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SUB32mi:void addr:iPTR:$dst, (imm:i32):$src2)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_141(N, X86::SUB32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SUBE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2) {

                  // Pattern: (st:void (sube:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SBB8mi:void addr:iPTR:$dst, (imm:i8):$src2)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i8) {
                    return Emit_147(N, X86::SBB8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (st:void (sube:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i32):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                  // Emits: (SBB32mi:void addr:iPTR:$dst, (imm:i32):$src2)
                  // Pattern complexity = 44  cost = 1  size = 3
                  if (N1.Val->getValueType(0) == MVT::i32) {
                    return Emit_142(N, X86::SBB32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (shl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHL64mi:void addr:iPTR:$dst, (imm:i8):$src)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SHL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i64 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_145(N, X86::SHL64mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (srl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHR64mi:void addr:iPTR:$dst, (imm:i8):$src)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SRL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i64 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_145(N, X86::SHR64mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (sra:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SAR64mi:void addr:iPTR:$dst, (imm:i8):$src)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SRA &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i64 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_145(N, X86::SAR64mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (rotl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ROL64mi:void addr:iPTR:$dst, (imm:i8):$src)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ROTL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i64 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_145(N, X86::ROL64mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (rotr:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, (imm:i8):$src), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ROR64mi:void addr:iPTR:$dst, (imm:i8):$src)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ROTR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              if (N11.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i64 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_145(N, X86::ROR64mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shld:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src2, (imm:i8):$src3), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHLD64mri8:void addr:iPTR:$dst, GR64:i64:$src2, (imm:i8):$src3)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHLD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              if (N12.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i64 &&
                    N12.Val->getValueType(0) == MVT::i8) {
                  return Emit_146(N, X86::SHLD64mri8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shrd:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src2, (imm:i8):$src3), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHRD64mri8:void addr:iPTR:$dst, GR64:i64:$src2, (imm:i8):$src3)
      // Pattern complexity = 44  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHRD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              if (N12.getOpcode() == ISD::Constant) {
                SDOperand N2 = N.getOperand(2);
                if (N101 == N2 &&
                    N1.Val->getValueType(0) == MVT::i64 &&
                    N12.Val->getValueType(0) == MVT::i8) {
                  return Emit_146(N, X86::SHRD64mri8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::AND &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2) {

                // Pattern: (st:void (and:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, GR8:i8:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (AND8mr:void addr:iPTR:$dst, GR8:i8:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8) {
                  return Emit_148(N, X86::AND8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (and:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, GR16:i16:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (AND16mr:void addr:iPTR:$dst, GR16:i16:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16) {
                  return Emit_148(N, X86::AND16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (and:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (AND32mr:void addr:iPTR:$dst, GR32:i32:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_148(N, X86::AND32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::OR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2) {

                // Pattern: (st:void (or:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, GR8:i8:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (OR8mr:void addr:iPTR:$dst, GR8:i8:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8) {
                  return Emit_148(N, X86::OR8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (or:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, GR16:i16:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (OR16mr:void addr:iPTR:$dst, GR16:i16:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16) {
                  return Emit_148(N, X86::OR16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (or:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (OR32mr:void addr:iPTR:$dst, GR32:i32:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_148(N, X86::OR32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::XOR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2) {

                // Pattern: (st:void (xor:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, GR8:i8:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (XOR8mr:void addr:iPTR:$dst, GR8:i8:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8) {
                  return Emit_148(N, X86::XOR8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (xor:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, GR16:i16:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (XOR16mr:void addr:iPTR:$dst, GR16:i16:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16) {
                  return Emit_148(N, X86::XOR16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (xor:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (XOR32mr:void addr:iPTR:$dst, GR32:i32:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_148(N, X86::XOR32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SHL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2) {

                // Pattern: (st:void (shl:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SHL8mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::SHL8mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (shl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SHL16mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::SHL16mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (shl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SHL32mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::SHL32mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SRL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2) {

                // Pattern: (st:void (srl:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SHR8mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::SHR8mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (srl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SHR16mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::SHR16mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (srl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SHR32mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::SHR32mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SRA &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2) {

                // Pattern: (st:void (sra:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SAR8mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::SAR8mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (sra:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SAR16mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::SAR16mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (sra:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SAR32mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::SAR32mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ROTL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2) {

                // Pattern: (st:void (rotl:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ROL8mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::ROL8mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (rotl:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ROL16mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::ROL16mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (rotl:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ROL32mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::ROL32mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ROTR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2) {

                // Pattern: (st:void (rotr:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ROR8mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::ROR8mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (rotr:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ROR16mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::ROR16mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (rotr:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ROR32mCL:void addr:iPTR:$dst)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32 &&
                    N11.Val->getValueType(0) == MVT::i8) {
                  return Emit_149(N, X86::ROR32mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shld:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src2, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHLD32mrCL:void addr:iPTR:$dst, GR32:i32:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHLD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i32 &&
                  N12.Val->getValueType(0) == MVT::i8) {
                return Emit_150(N, X86::SHLD32mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shrd:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src2, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHRD32mrCL:void addr:iPTR:$dst, GR32:i32:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHRD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i32 &&
                  N12.Val->getValueType(0) == MVT::i8) {
                return Emit_150(N, X86::SHRD32mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shld:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, GR16:i16:$src2, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHLD16mrCL:void addr:iPTR:$dst, GR16:i16:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHLD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i16 &&
                  N12.Val->getValueType(0) == MVT::i8) {
                return Emit_150(N, X86::SHLD16mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shrd:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, GR16:i16:$src2, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHRD16mrCL:void addr:iPTR:$dst, GR16:i16:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHRD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i16 &&
                  N12.Val->getValueType(0) == MVT::i8) {
                return Emit_150(N, X86::SHRD16mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2) {

                // Pattern: (st:void (add:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, GR8:i8:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ADD8mr:void addr:iPTR:$dst, GR8:i8:$src2)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8) {
                  return Emit_148(N, X86::ADD8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (add:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, GR16:i16:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ADD16mr:void addr:iPTR:$dst, GR16:i16:$src2)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16) {
                  return Emit_148(N, X86::ADD16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (add:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ADD32mr:void addr:iPTR:$dst, GR32:i32:$src2)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_148(N, X86::ADD32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (adde:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ADC32mr:void addr:iPTR:$dst, GR32:i32:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ADDE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i32) {
                return Emit_151(N, X86::ADC32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::SUB &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2) {

                // Pattern: (st:void (sub:i8 (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>, GR8:i8:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SUB8mr:void addr:iPTR:$dst, GR8:i8:$src2)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8) {
                  return Emit_148(N, X86::SUB8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (sub:i16 (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>, GR16:i16:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SUB16mr:void addr:iPTR:$dst, GR16:i16:$src2)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16) {
                  return Emit_148(N, X86::SUB16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (sub:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (SUB32mr:void addr:iPTR:$dst, GR32:i32:$src2)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_148(N, X86::SUB32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (sube:i32 (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>, GR32:i32:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SBB32mr:void addr:iPTR:$dst, GR32:i32:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SUBE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i32) {
                return Emit_151(N, X86::SBB32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (add:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ADD64mr:void addr:iPTR:$dst, GR64:i64:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_148(N, X86::ADD64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (adde:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ADC64mr:void addr:iPTR:$dst, GR64:i64:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ADDE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_151(N, X86::ADC64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (sub:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SUB64mr:void addr:iPTR:$dst, GR64:i64:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SUB &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_148(N, X86::SUB64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (sube:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src2), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SBB64mr:void addr:iPTR:$dst, GR64:i64:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SUBE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_151(N, X86::SBB64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (shl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHL64mCL:void addr:iPTR:$dst)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SHL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_149(N, X86::SHL64mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (srl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHR64mCL:void addr:iPTR:$dst)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SRL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_149(N, X86::SHR64mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (sra:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SAR64mCL:void addr:iPTR:$dst)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::SRA &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_149(N, X86::SAR64mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (rotl:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ROL64mCL:void addr:iPTR:$dst)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ROTL &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_149(N, X86::ROL64mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (rotr:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ROR64mCL:void addr:iPTR:$dst)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ROTR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64 &&
                  N11.Val->getValueType(0) == MVT::i8) {
                return Emit_149(N, X86::ROR64mCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shld:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src2, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHLD64mrCL:void addr:iPTR:$dst, GR64:i64:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHLD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64 &&
                  N12.Val->getValueType(0) == MVT::i8) {
                return Emit_150(N, X86::SHLD64mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (X86shrd:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src2, CL:i8), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (SHRD64mrCL:void addr:iPTR:$dst, GR64:i64:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == X86ISD::SHRD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N12 = N1.getOperand(2);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64 &&
                  N12.Val->getValueType(0) == MVT::i8) {
                return Emit_150(N, X86::SHRD64mrCL, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (and:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (AND64mr:void addr:iPTR:$dst, GR64:i64:$src)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::AND &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_148(N, X86::AND64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (or:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (OR64mr:void addr:iPTR:$dst, GR64:i64:$src)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::OR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_148(N, X86::OR64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (xor:i64 (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>, GR64:i64:$src), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (XOR64mr:void addr:iPTR:$dst, GR64:i64:$src)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::XOR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val) &&
            (Chain.Val == N10.Val || IsChainCompatible(Chain.Val, N10.Val))) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N11 = N1.getOperand(1);
              SDOperand N2 = N.getOperand(2);
              if (N101 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_148(N, X86::XOR64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::AND &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::LOAD &&
            N11.hasOneUse() &&
            CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
            (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
          SDOperand Chain11 = N11.getOperand(0);
          if (Predicate_load(N11.Val)) {
            SDOperand N111 = N11.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N111 == N2) {

                // Pattern: (st:void (and:i8 GR8:i8:$src, (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (AND8mr:void addr:iPTR:$dst, GR8:i8:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8) {
                  return Emit_152(N, X86::AND8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (and:i16 GR16:i16:$src, (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (AND16mr:void addr:iPTR:$dst, GR16:i16:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16) {
                  return Emit_152(N, X86::AND16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (and:i32 GR32:i32:$src, (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (AND32mr:void addr:iPTR:$dst, GR32:i32:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_152(N, X86::AND32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::OR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::LOAD &&
            N11.hasOneUse() &&
            CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
            (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
          SDOperand Chain11 = N11.getOperand(0);
          if (Predicate_load(N11.Val)) {
            SDOperand N111 = N11.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N111 == N2) {

                // Pattern: (st:void (or:i8 GR8:i8:$src, (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (OR8mr:void addr:iPTR:$dst, GR8:i8:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8) {
                  return Emit_152(N, X86::OR8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (or:i16 GR16:i16:$src, (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (OR16mr:void addr:iPTR:$dst, GR16:i16:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16) {
                  return Emit_152(N, X86::OR16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (or:i32 GR32:i32:$src, (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (OR32mr:void addr:iPTR:$dst, GR32:i32:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_152(N, X86::OR32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::XOR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::LOAD &&
            N11.hasOneUse() &&
            CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
            (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
          SDOperand Chain11 = N11.getOperand(0);
          if (Predicate_load(N11.Val)) {
            SDOperand N111 = N11.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N111 == N2) {

                // Pattern: (st:void (xor:i8 GR8:i8:$src, (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (XOR8mr:void addr:iPTR:$dst, GR8:i8:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8) {
                  return Emit_152(N, X86::XOR8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (xor:i16 GR16:i16:$src, (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (XOR16mr:void addr:iPTR:$dst, GR16:i16:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16) {
                  return Emit_152(N, X86::XOR16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (xor:i32 GR32:i32:$src, (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (XOR32mr:void addr:iPTR:$dst, GR32:i32:$src)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_152(N, X86::XOR32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::LOAD &&
            N11.hasOneUse() &&
            CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
            (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
          SDOperand Chain11 = N11.getOperand(0);
          if (Predicate_load(N11.Val)) {
            SDOperand N111 = N11.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N111 == N2) {

                // Pattern: (st:void (add:i8 GR8:i8:$src2, (ld:i8 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ADD8mr:void addr:iPTR:$dst, GR8:i8:$src2)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i8) {
                  return Emit_152(N, X86::ADD8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (add:i16 GR16:i16:$src2, (ld:i16 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ADD16mr:void addr:iPTR:$dst, GR16:i16:$src2)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i16) {
                  return Emit_152(N, X86::ADD16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (st:void (add:i32 GR32:i32:$src2, (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
                // Emits: (ADD32mr:void addr:iPTR:$dst, GR32:i32:$src2)
                // Pattern complexity = 41  cost = 1  size = 3
                if (N1.Val->getValueType(0) == MVT::i32) {
                  return Emit_152(N, X86::ADD32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }

      // Pattern: (st:void (adde:i32 GR32:i32:$src2, (ld:i32 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ADC32mr:void addr:iPTR:$dst, GR32:i32:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ADDE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::LOAD &&
            N11.hasOneUse() &&
            CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
            (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
          SDOperand Chain11 = N11.getOperand(0);
          if (Predicate_load(N11.Val)) {
            SDOperand N111 = N11.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N111 == N2 &&
                  N1.Val->getValueType(0) == MVT::i32) {
                return Emit_153(N, X86::ADC32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (add:i64 GR64:i64:$src2, (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ADD64mr:void addr:iPTR:$dst, GR64:i64:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ADD &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::LOAD &&
            N11.hasOneUse() &&
            CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
            (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
          SDOperand Chain11 = N11.getOperand(0);
          if (Predicate_load(N11.Val)) {
            SDOperand N111 = N11.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N111 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_152(N, X86::ADD64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (adde:i64 GR64:i64:$src2, (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (ADC64mr:void addr:iPTR:$dst, GR64:i64:$src2)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::ADDE &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::LOAD &&
            N11.hasOneUse() &&
            CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
            (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
          SDOperand Chain11 = N11.getOperand(0);
          if (Predicate_load(N11.Val)) {
            SDOperand N111 = N11.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N111 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_153(N, X86::ADC64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (and:i64 GR64:i64:$src, (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (AND64mr:void addr:iPTR:$dst, GR64:i64:$src)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::AND &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::LOAD &&
            N11.hasOneUse() &&
            CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
            (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
          SDOperand Chain11 = N11.getOperand(0);
          if (Predicate_load(N11.Val)) {
            SDOperand N111 = N11.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N111 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_152(N, X86::AND64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (or:i64 GR64:i64:$src, (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (OR64mr:void addr:iPTR:$dst, GR64:i64:$src)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::OR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::LOAD &&
            N11.hasOneUse() &&
            CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
            (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
          SDOperand Chain11 = N11.getOperand(0);
          if (Predicate_load(N11.Val)) {
            SDOperand N111 = N11.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N111 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_152(N, X86::OR64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }

      // Pattern: (st:void (xor:i64 GR64:i64:$src, (ld:i64 addr:iPTR:$dst)<<P:Predicate_load>>), addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (XOR64mr:void addr:iPTR:$dst, GR64:i64:$src)
      // Pattern complexity = 41  cost = 1  size = 3
      if (N1.getOpcode() == ISD::XOR &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::LOAD &&
            N11.hasOneUse() &&
            CanBeFoldedBy(N11.Val, N1.Val, N.Val) &&
            (Chain.Val == N11.Val || IsChainCompatible(Chain.Val, N11.Val))) {
          SDOperand Chain11 = N11.getOperand(0);
          if (Predicate_load(N11.Val)) {
            SDOperand N111 = N11.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N111, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N111 == N2 &&
                  N1.Val->getValueType(0) == MVT::i64) {
                return Emit_152(N, X86::XOR64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (st:void (vector_extract:f64 (vector_shuffle:v2f64 (bitconvert:v2f64 VR128:v4f32:$src), (undef:v2f64), (build_vector:v2i32)<<P:Predicate_UNPCKH_shuffle_mask>>), 0:iPTR), addr:iPTR:$dst)<<P:Predicate_store>>
  // Emits: (MOVHPSmr:void addr:iPTR:$dst, VR128:v4f32:$src)
  // Pattern complexity = 40  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::VECTOR_SHUFFLE) {
          SDOperand N100 = N10.getOperand(0);
          if (N100.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N1000 = N100.getOperand(0);
            SDOperand N101 = N10.getOperand(1);
            if (N101.getOpcode() == ISD::UNDEF) {
              SDOperand N102 = N10.getOperand(2);
              if (N102.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_UNPCKH_shuffle_mask(N102.Val)) {
                SDOperand N11 = N1.getOperand(1);
                if (isa<ConstantSDNode>(N11)) {
                  int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                  if (CN0 == 0) {
                    SDOperand N2 = N.getOperand(2);
                    SDOperand CPTmp0;
                    SDOperand CPTmp1;
                    SDOperand CPTmp2;
                    SDOperand CPTmp3;
                    if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                        N1.Val->getValueType(0) == MVT::f64 &&
                        N10.Val->getValueType(0) == MVT::v2f64 &&
                        N1000.Val->getValueType(0) == MVT::v4f32) {
                      return Emit_154(N, X86::MOVHPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (st:void (vector_extract:f64 (vector_shuffle:v2f64 VR128:v2f64:$src, (undef:v2f64), (build_vector:v2i32)<<P:Predicate_UNPCKH_shuffle_mask>>), 0:iPTR), addr:iPTR:$dst)<<P:Predicate_store>>
  // Emits: (MOVHPDmr:void addr:iPTR:$dst, VR128:v2f64:$src)
  // Pattern complexity = 37  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::VECTOR_SHUFFLE) {
          SDOperand N100 = N10.getOperand(0);
          SDOperand N101 = N10.getOperand(1);
          if (N101.getOpcode() == ISD::UNDEF) {
            SDOperand N102 = N10.getOperand(2);
            if (N102.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_UNPCKH_shuffle_mask(N102.Val)) {
              SDOperand N11 = N1.getOperand(1);
              if (isa<ConstantSDNode>(N11)) {
                int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
                if (CN0 == 0) {
                  SDOperand N2 = N.getOperand(2);
                  SDOperand CPTmp0;
                  SDOperand CPTmp1;
                  SDOperand CPTmp2;
                  SDOperand CPTmp3;
                  if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                      N1.Val->getValueType(0) == MVT::f64 &&
                      N10.Val->getValueType(0) == MVT::v2f64) {
                    return Emit_155(N, X86::MOVHPDmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (st:void (vector_extract:f64 (bitconvert:v2f64 VR128:v4f32:$src), 0:iPTR), addr:iPTR:$dst)<<P:Predicate_store>>
  // Emits: (MOVLPSmr:void addr:iPTR:$dst, VR128:v4f32:$src)
  // Pattern complexity = 30  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N100 = N10.getOperand(0);
          SDOperand N11 = N1.getOperand(1);
          if (isa<ConstantSDNode>(N11)) {
            int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
            if (CN0 == 0) {
              SDOperand N2 = N.getOperand(2);
              SDOperand CPTmp0;
              SDOperand CPTmp1;
              SDOperand CPTmp2;
              SDOperand CPTmp3;
              if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                  N1.Val->getValueType(0) == MVT::f64 &&
                  N10.Val->getValueType(0) == MVT::v2f64 &&
                  N100.Val->getValueType(0) == MVT::v4f32) {
                return Emit_156(N, X86::MOVLPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (st:void (extractelt:i32 (bitconvert:v4i32 VR128:v4f32:$src1), (imm:iPTR):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
  // Emits: (EXTRACTPSmr:void addr:iPTR:$dst, VR128:v4f32:$src1, (imm:i32):$src2)
  // Pattern complexity = 28  cost = 1  size = 3
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N100 = N10.getOperand(0);
          SDOperand N11 = N1.getOperand(1);
          if (N11.getOpcode() == ISD::Constant) {
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N1.Val->getValueType(0) == MVT::i32 &&
                N10.Val->getValueType(0) == MVT::v4i32 &&
                N100.Val->getValueType(0) == MVT::v4f32) {
              return Emit_157(N, X86::EXTRACTPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == X86ISD::SETCC) {
        SDOperand N10 = N1.getOperand(0);
        if (isa<ConstantSDNode>(N10)) {
          int64_t CN0 = cast<ConstantSDNode>(N10)->getSignExtended();

          // Pattern: (st:void (X86setcc:i8 4:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETEm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 4) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETEm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 9:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETNEm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 9) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETNEm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 7:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETLm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 7) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETLm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 6:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETGEm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 6) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETGEm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 8:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETLEm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 8) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETLEm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 5:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETGm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 5) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETGm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 2:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETBm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 2) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETBm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 1:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETAEm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 1) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETAEm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 3:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETBEm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 3) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETBEm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 0:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETAm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 0) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETAm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 15:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETSm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 15) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETSm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 12:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETNSm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 12) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETNSm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 14:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETPm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 14) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETPm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (st:void (X86setcc:i8 11:i8, EFLAGS:i32), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (SETNPm:void addr:iPTR:$dst)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 11) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_158(N, X86::SETNPm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }

  // Pattern: (st:void (vector_extract:f32 VR128:v4f32:$src, 0:iPTR), addr:iPTR:$dst)<<P:Predicate_store>>
  // Emits: (MOVPS2SSmr:void addr:iPTR:$dst, VR128:v4f32:$src)
  // Pattern complexity = 27  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (isa<ConstantSDNode>(N11)) {
          int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
          if (CN0 == 0) {
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N1.Val->getValueType(0) == MVT::f32 &&
                N10.Val->getValueType(0) == MVT::v4f32) {
              return Emit_159(N, X86::MOVPS2SSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (isa<ConstantSDNode>(N11)) {
          int64_t CN0 = cast<ConstantSDNode>(N11)->getSignExtended();
          if (CN0 == 0) {
            SDOperand N2 = N.getOperand(2);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

              // Pattern: (st:void (vector_extract:f64 VR128:v2f64:$src, 0:iPTR), addr:iPTR:$dst)<<P:Predicate_store>>
              // Emits: (MOVLPDmr:void addr:iPTR:$dst, VR128:v2f64:$src)
              // Pattern complexity = 27  cost = 1  size = 3
              if (N1.Val->getValueType(0) == MVT::f64 &&
                  N10.Val->getValueType(0) == MVT::v2f64) {
                return Emit_159(N, X86::MOVLPDmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (st:void (vector_extract:i64 VR128:v2i64:$src, 0:iPTR), addr:iPTR:$dst)<<P:Predicate_store>>
              // Emits: (MOVPQI2QImr:void addr:iPTR:$dst, VR128:v2i64:$src)
              // Pattern complexity = 27  cost = 1  size = 3
              if (N1.Val->getValueType(0) == MVT::i64 &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_159(N, X86::MOVPQI2QImr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (st:void (vector_extract:f64 VR128:v2f64:$src, 0:iPTR), addr:iPTR:$dst)<<P:Predicate_store>>
              // Emits: (MOVPD2SDmr:void addr:iPTR:$dst, VR128:v2f64:$src)
              // Pattern complexity = 27  cost = 1  size = 3
              if (N1.Val->getValueType(0) == MVT::f64 &&
                  N10.Val->getValueType(0) == MVT::v2f64) {
                return Emit_159(N, X86::MOVPD2SDmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (st:void (vector_extract:i32 VR128:v4i32:$src, 0:iPTR), addr:iPTR:$dst)<<P:Predicate_store>>
              // Emits: (MOVPDI2DImr:void addr:iPTR:$dst, VR128:v4i32:$src)
              // Pattern complexity = 27  cost = 1  size = 3
              if (N1.Val->getValueType(0) == MVT::i32 &&
                  N10.Val->getValueType(0) == MVT::v4i32) {
                return Emit_159(N, X86::MOVPDI2DImr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE41())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::EXTRACT_VECTOR_ELT) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N11 = N1.getOperand(1);
        if (N11.getOpcode() == ISD::Constant) {
          SDOperand N2 = N.getOperand(2);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

            // Pattern: (st:void (extractelt:i64 VR128:v2i64:$src1, (imm:iPTR):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
            // Emits: (PEXTRQmr:void addr:iPTR:$dst, VR128:v2i64:$src1, (imm:i32):$src2)
            // Pattern complexity = 25  cost = 1  size = 3
            if (N1.Val->getValueType(0) == MVT::i64 &&
                N10.Val->getValueType(0) == MVT::v2i64) {
              return Emit_160(N, X86::PEXTRQmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (st:void (extractelt:i32 VR128:v4i32:$src1, (imm:iPTR):$src2), addr:iPTR:$dst)<<P:Predicate_store>>
            // Emits: (PEXTRDmr:void addr:iPTR:$dst, VR128:v4i32:$src1, (imm:i32):$src2)
            // Pattern complexity = 25  cost = 1  size = 3
            if (N1.Val->getValueType(0) == MVT::i32 &&
                N10.Val->getValueType(0) == MVT::v4i32) {
              return Emit_160(N, X86::PEXTRDmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((TM.getCodeModel() == CodeModel::Small) && (TM.getRelocationModel() == Reloc::Static)) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == X86ISD::Wrapper) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (st:void (X86Wrapper:i64 (tconstpool:i64):$src), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV64mi32:void addr:iPTR:$dst, (tconstpool:i64):$src)
        // Pattern complexity = 25  cost = 1  size = 3
        if (N10.getOpcode() == ISD::TargetConstantPool) {
          SDOperand N2 = N.getOperand(2);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N1.Val->getValueType(0) == MVT::i64) {
            return Emit_161(N, X86::MOV64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }

        // Pattern: (st:void (X86Wrapper:i64 (tjumptable:i64):$src), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV64mi32:void addr:iPTR:$dst, (tjumptable:i64):$src)
        // Pattern complexity = 25  cost = 1  size = 3
        if (N10.getOpcode() == ISD::TargetJumpTable) {
          SDOperand N2 = N.getOperand(2);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N1.Val->getValueType(0) == MVT::i64) {
            return Emit_162(N, X86::MOV64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }

        // Pattern: (st:void (X86Wrapper:i64 (tglobaladdr:i64):$src), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV64mi32:void addr:iPTR:$dst, (tglobaladdr:i64):$src)
        // Pattern complexity = 25  cost = 1  size = 3
        if (N10.getOpcode() == ISD::TargetGlobalAddress) {
          SDOperand N2 = N.getOperand(2);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N1.Val->getValueType(0) == MVT::i64) {
            return Emit_161(N, X86::MOV64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }

        // Pattern: (st:void (X86Wrapper:i64 (texternalsym:i64):$src), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV64mi32:void addr:iPTR:$dst, (texternalsym:i64):$src)
        // Pattern complexity = 25  cost = 1  size = 3
        if (N10.getOpcode() == ISD::TargetExternalSymbol) {
          SDOperand N2 = N.getOperand(2);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N1.Val->getValueType(0) == MVT::i64) {
            return Emit_161(N, X86::MOV64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }
  {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == X86ISD::Wrapper) {
        SDOperand N10 = N1.getOperand(0);

        // Pattern: (st:void (X86Wrapper:i32 (tglobaladdr:i32):$src), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV32mi:void addr:iPTR:$dst, (tglobaladdr:i32):$src)
        // Pattern complexity = 25  cost = 1  size = 3
        if (N10.getOpcode() == ISD::TargetGlobalAddress) {
          SDOperand N2 = N.getOperand(2);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N1.Val->getValueType(0) == MVT::i32) {
            return Emit_161(N, X86::MOV32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }

        // Pattern: (st:void (X86Wrapper:i32 (texternalsym:i32):$src), addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV32mi:void addr:iPTR:$dst, (texternalsym:i32):$src)
        // Pattern complexity = 25  cost = 1  size = 3
        if (N10.getOpcode() == ISD::TargetExternalSymbol) {
          SDOperand N2 = N.getOperand(2);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N1.Val->getValueType(0) == MVT::i32) {
            return Emit_161(N, X86::MOV32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
      if (N1.getOpcode() == ISD::Constant) {

        // Pattern: (st:void (imm:i64)<<P:Predicate_i64immSExt32>>:$src, addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV64mi32:void addr:iPTR:$dst, (imm:i64):$src)
        // Pattern complexity = 23  cost = 1  size = 3
        if (Predicate_i64immSExt32(N1.Val)) {
          SDOperand N2 = N.getOperand(2);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N1.Val->getValueType(0) == MVT::i64) {
            return Emit_163(N, X86::MOV64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

          // Pattern: (st:void (imm:i8):$src, addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (MOV8mi:void addr:iPTR:$dst, (imm:i8):$src)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N1.Val->getValueType(0) == MVT::i8) {
            return Emit_164(N, X86::MOV8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (st:void (imm:i16):$src, addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (MOV16mi:void addr:iPTR:$dst, (imm:i16):$src)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N1.Val->getValueType(0) == MVT::i16) {
            return Emit_165(N, X86::MOV16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (st:void (imm:i32):$src, addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (MOV32mi:void addr:iPTR:$dst, (imm:i32):$src)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N1.Val->getValueType(0) == MVT::i32) {
            return Emit_166(N, X86::MOV32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N10 = N1.getOperand(0);
        SDOperand N2 = N.getOperand(2);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

          // Pattern: (st:void (bitconvert:i64 FR64:f64:$src), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (MOVSDto64mr:void addr:iPTR:$dst, FR64:f64:$src)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N1.Val->getValueType(0) == MVT::i64 &&
              N10.Val->getValueType(0) == MVT::f64) {
            return Emit_162(N, X86::MOVSDto64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (st:void (bitconvert:i32 FR32:f32:$src), addr:iPTR:$dst)<<P:Predicate_store>>
          // Emits: (MOVSS2DImr:void addr:iPTR:$dst, FR32:f32:$src)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N1.Val->getValueType(0) == MVT::i32 &&
              N10.Val->getValueType(0) == MVT::f32) {
            return Emit_162(N, X86::MOVSS2DImr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }

  // Pattern: (st:void RFP32:f32:$src, addr:iPTR:$op)<<P:Predicate_store>>
  // Emits: (ST_Fp32m:void addr:iPTR:$op, RFP32:f32:$src)
  // Pattern complexity = 19  cost = 1  size = 0
  if ((!Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::f32) {
        return Emit_167(N, X86::ST_Fp32m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  if ((!Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (st:void RFP64:f64:$src, addr:iPTR:$op)<<P:Predicate_truncstoref32>>
    // Emits: (ST_Fp64m32:void addr:iPTR:$op, RFP64:f64:$src)
    // Pattern complexity = 19  cost = 1  size = 0
    if (Predicate_truncstoref32(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::f64) {
        return Emit_167(N, X86::ST_Fp64m32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (st:void RFP64:f64:$src, addr:iPTR:$op)<<P:Predicate_store>>
    // Emits: (ST_Fp64m:void addr:iPTR:$op, RFP64:f64:$src)
    // Pattern complexity = 19  cost = 1  size = 0
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::f64) {
        return Emit_167(N, X86::ST_Fp64m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (st:void RFP80:f80:$src, addr:iPTR:$op)<<P:Predicate_truncstoref32>>
    // Emits: (ST_Fp80m32:void addr:iPTR:$op, RFP80:f80:$src)
    // Pattern complexity = 19  cost = 1  size = 0
    if (Predicate_truncstoref32(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::f80) {
        return Emit_167(N, X86::ST_Fp80m32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (st:void RFP80:f80:$src, addr:iPTR:$op)<<P:Predicate_truncstoref64>>
    // Emits: (ST_Fp80m64:void addr:iPTR:$op, RFP80:f80:$src)
    // Pattern complexity = 19  cost = 1  size = 0
    if (Predicate_truncstoref64(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::f80) {
        return Emit_167(N, X86::ST_Fp80m64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

        // Pattern: (st:void RFP80:f80:$src, addr:iPTR:$op)<<P:Predicate_store>>
        // Emits: (ST_FpP80m:void addr:iPTR:$op, RFP80:f80:$src)
        // Pattern complexity = 19  cost = 1  size = 0
        if (N1.Val->getValueType(0) == MVT::f80) {
          return Emit_167(N, X86::ST_FpP80m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (st:void GR8:i8:$src, addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV8mr:void addr:iPTR:$dst, GR8:i8:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::i8) {
          return Emit_167(N, X86::MOV8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (st:void GR16:i16:$src, addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV16mr:void addr:iPTR:$dst, GR16:i16:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::i16) {
          return Emit_167(N, X86::MOV16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (st:void GR32:i32:$src, addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV32mr:void addr:iPTR:$dst, GR32:i32:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::i32) {
          return Emit_167(N, X86::MOV32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (st:void GR64:i64:$src, addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOV64mr:void addr:iPTR:$dst, GR64:i64:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::i64) {
          return Emit_167(N, X86::MOV64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (st:void FR32:f32:$src, addr:iPTR:$dst)<<P:Predicate_store>>
    // Emits: (MOVSSmr:void addr:iPTR:$dst, FR32:f32:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::f32) {
        return Emit_167(N, X86::MOVSSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (st:void VR128:v4f32:$src, addr:iPTR:$dst)<<P:Predicate_alignedstore>>
    // Emits: (MOVAPSmr:void addr:iPTR:$dst, VR128:v4f32:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_alignedstore(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::v4f32) {
        return Emit_167(N, X86::MOVAPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (st:void VR128:v4f32:$src, addr:iPTR:$dst)<<P:Predicate_store>>
    // Emits: (MOVUPSmr:void addr:iPTR:$dst, VR128:v4f32:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::v4f32) {
        return Emit_167(N, X86::MOVUPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);

    // Pattern: (st:void FR64:f64:$src, addr:iPTR:$dst)<<P:Predicate_store>>
    // Emits: (MOVSDmr:void addr:iPTR:$dst, FR64:f64:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::f64) {
        return Emit_167(N, X86::MOVSDmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (st:void VR128:v2f64:$src, addr:iPTR:$dst)<<P:Predicate_alignedstore>>
    // Emits: (MOVAPDmr:void addr:iPTR:$dst, VR128:v2f64:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_alignedstore(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::v2f64) {
        return Emit_167(N, X86::MOVAPDmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (st:void VR128:v2f64:$src, addr:iPTR:$dst)<<P:Predicate_store>>
    // Emits: (MOVUPDmr:void addr:iPTR:$dst, VR128:v2f64:$src)
    // Pattern complexity = 19  cost = 1  size = 3
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::v2f64) {
        return Emit_167(N, X86::MOVUPDmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }

  // Pattern: (st:void VR64:v1i64:$src, addr:iPTR:$dst)<<P:Predicate_store>>
  // Emits: (MMX_MOVQ64mr:void addr:iPTR:$dst, VR64:v1i64:$src)
  // Pattern complexity = 19  cost = 1  size = 3
  if ((Subtarget->hasMMX())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
          N1.Val->getValueType(0) == MVT::v1i64) {
        return Emit_167(N, X86::MMX_MOVQ64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);
    if (Predicate_alignedstore(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

        // Pattern: (st:void VR128:v2i64:$src, addr:iPTR:$dst)<<P:Predicate_alignedstore>>
        // Emits: (MOVAPSmr:void addr:iPTR:$dst, VR128:v16i8:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::v2i64) {
          return Emit_167(N, X86::MOVAPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (st:void VR128:v4i32:$src, addr:iPTR:$dst)<<P:Predicate_alignedstore>>
        // Emits: (MOVAPSmr:void addr:iPTR:$dst, VR128:v16i8:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::v4i32) {
          return Emit_167(N, X86::MOVAPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (st:void VR128:v8i16:$src, addr:iPTR:$dst)<<P:Predicate_alignedstore>>
        // Emits: (MOVAPSmr:void addr:iPTR:$dst, VR128:v16i8:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::v8i16) {
          return Emit_167(N, X86::MOVAPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (st:void VR128:v16i8:$src, addr:iPTR:$dst)<<P:Predicate_alignedstore>>
        // Emits: (MOVAPSmr:void addr:iPTR:$dst, VR128:v16i8:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::v16i8) {
          return Emit_167(N, X86::MOVAPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    if (Predicate_store(N.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

        // Pattern: (st:void VR128:v2i64:$src, addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOVUPSmr:void addr:iPTR:$dst, VR128:v16i8:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::v2i64) {
          return Emit_167(N, X86::MOVUPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (st:void VR128:v4i32:$src, addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOVUPSmr:void addr:iPTR:$dst, VR128:v16i8:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::v4i32) {
          return Emit_167(N, X86::MOVUPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (st:void VR128:v8i16:$src, addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOVUPSmr:void addr:iPTR:$dst, VR128:v16i8:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::v8i16) {
          return Emit_167(N, X86::MOVUPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }

        // Pattern: (st:void VR128:v16i8:$src, addr:iPTR:$dst)<<P:Predicate_store>>
        // Emits: (MOVUPSmr:void addr:iPTR:$dst, VR128:v16i8:$src)
        // Pattern complexity = 19  cost = 1  size = 3
        if (N1.Val->getValueType(0) == MVT::v16i8) {
          return Emit_167(N, X86::MOVUPSmr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  SDOperand Chain = N.getOperand(0);
  if (Predicate_store(N.Val)) {
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

      // Pattern: (st:void VR64:v8i8:$src, addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (MMX_MOVQ64mr:void addr:iPTR:$dst, VR64:v8i8:$src)
      // Pattern complexity = 19  cost = 1  size = 3
      if (N1.Val->getValueType(0) == MVT::v8i8) {
        return Emit_167(N, X86::MMX_MOVQ64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (st:void VR64:v4i16:$src, addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (MMX_MOVQ64mr:void addr:iPTR:$dst, VR64:v8i8:$src)
      // Pattern complexity = 19  cost = 1  size = 3
      if (N1.Val->getValueType(0) == MVT::v4i16) {
        return Emit_167(N, X86::MMX_MOVQ64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (st:void VR64:v2i32:$src, addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (MMX_MOVQ64mr:void addr:iPTR:$dst, VR64:v8i8:$src)
      // Pattern complexity = 19  cost = 1  size = 3
      if (N1.Val->getValueType(0) == MVT::v2i32) {
        return Emit_167(N, X86::MMX_MOVQ64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (st:void VR64:v1i64:$src, addr:iPTR:$dst)<<P:Predicate_store>>
      // Emits: (MMX_MOVQ64mr:void addr:iPTR:$dst, VR64:v8i8:$src)
      // Pattern complexity = 19  cost = 1  size = 3
      if (N1.Val->getValueType(0) == MVT::v1i64) {
        return Emit_167(N, X86::MMX_MOVQ64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SUB_i8(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (sub:i8 GR8:i8:$src1, (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (SUB8rm:i8 GR8:i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::SUB8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (sub:i8 0:i8, GR8:i8:$src)
    // Emits: (NEG8r:i8 GR8:i8:$src)
    // Pattern complexity = 8  cost = 1  size = 2
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 0) {
        return Emit_90(N, X86::NEG8r, MVT::i8);
      }
    }

    // Pattern: (sub:i8 GR8:i8:$src1, (imm:i8):$src2)
    // Emits: (SUB8ri:i8 GR8:i8:$src1, (imm:i8):$src2)
    // Pattern complexity = 6  cost = 1  size = 3
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {
      return Emit_3(N, X86::SUB8ri, MVT::i8);
    }
  }

  // Pattern: (sub:i8 GR8:i8:$src1, GR8:i8:$src2)
  // Emits: (SUB8rr:i8 GR8:i8:$src1, GR8:i8:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::SUB8rr, MVT::i8);
}

SDNode *Select_ISD_SUB_i16(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (sub:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (SUB16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::SUB16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (sub:i16 0:i16, GR16:i16:$src)
    // Emits: (NEG16r:i16 GR16:i16:$src)
    // Pattern complexity = 8  cost = 1  size = 2
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 0) {
        return Emit_90(N, X86::NEG16r, MVT::i16);
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (sub:i16 GR16:i16:$src1, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2)
      // Emits: (SUB16ri8:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i16immSExt8(N1.Val)) {
        return Emit_5(N, X86::SUB16ri8, MVT::i16);
      }

      // Pattern: (sub:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Emits: (SUB16ri:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_5(N, X86::SUB16ri, MVT::i16);
    }
  }

  // Pattern: (sub:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Emits: (SUB16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::SUB16rr, MVT::i16);
}

SDNode *Select_ISD_SUB_i32(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (sub:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (SUB32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::SUB32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (sub:i32 0:i32, GR32:i32:$src)
    // Emits: (NEG32r:i32 GR32:i32:$src)
    // Pattern complexity = 8  cost = 1  size = 2
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 0) {
        return Emit_90(N, X86::NEG32r, MVT::i32);
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (sub:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (SUB32ri8:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val)) {
        return Emit_11(N, X86::SUB32ri8, MVT::i32);
      }

      // Pattern: (sub:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (SUB32ri:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_11(N, X86::SUB32ri, MVT::i32);
    }
  }

  // Pattern: (sub:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Emits: (SUB32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::SUB32rr, MVT::i32);
}

SDNode *Select_ISD_SUB_i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (sub:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (SUB64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::SUB64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (sub:i64 0:i64, GR64:i64:$src)
    // Emits: (NEG64r:i64 GR64:i64:$src)
    // Pattern complexity = 8  cost = 1  size = 2
    if (isa<ConstantSDNode>(N0)) {
      int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();
      if (CN0 == 0) {
        return Emit_90(N, X86::NEG64r, MVT::i64);
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (sub:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Emits: (SUB64ri32:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt32(N1.Val)) {
        return Emit_12(N, X86::SUB64ri32, MVT::i64);
      }

      // Pattern: (sub:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (SUB64ri8:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val)) {
        return Emit_12(N, X86::SUB64ri8, MVT::i64);
      }
    }
  }

  // Pattern: (sub:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Emits: (SUB64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::SUB64rr, MVT::i64);
}

SDNode *Select_ISD_SUB_v8i8(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {

    // Pattern: (sub:v8i8 VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
    // Emits: (MMX_PSUBBrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 25  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N10.Val->getValueType(0) == MVT::v1i64) {
              return Emit_13(N, X86::MMX_PSUBBrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (sub:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
    // Emits: (MMX_PSUBBrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PSUBBrr, MVT::v8i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SUB_v4i16(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {

    // Pattern: (sub:v4i16 VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
    // Emits: (MMX_PSUBWrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 25  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N10.Val->getValueType(0) == MVT::v1i64) {
              return Emit_13(N, X86::MMX_PSUBWrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (sub:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
    // Emits: (MMX_PSUBWrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PSUBWrr, MVT::v4i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SUB_v2i32(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {

    // Pattern: (sub:v2i32 VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
    // Emits: (MMX_PSUBDrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 25  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N10.Val->getValueType(0) == MVT::v1i64) {
              return Emit_13(N, X86::MMX_PSUBDrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (sub:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
    // Emits: (MMX_PSUBDrr:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PSUBDrr, MVT::v2i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SUB_v1i64(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {

    // Pattern: (sub:v1i64 VR64:v1i64:$src1, (bitconvert:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>))
    // Emits: (MMX_PSUBQrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 25  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_load(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N10.Val->getValueType(0) == MVT::v1i64) {
              return Emit_13(N, X86::MMX_PSUBQrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (sub:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
    // Emits: (MMX_PSUBQrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PSUBQrr, MVT::v1i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SUB_v16i8(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (sub:v16i8 VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
    // Emits: (PSUBBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 25  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_memop(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N10.Val->getValueType(0) == MVT::v2i64) {
              return Emit_13(N, X86::PSUBBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (sub:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
    // Emits: (PSUBBrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::PSUBBrr, MVT::v16i8);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SUB_v8i16(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (sub:v8i16 VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
    // Emits: (PSUBWrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 25  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_memop(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N10.Val->getValueType(0) == MVT::v2i64) {
              return Emit_13(N, X86::PSUBWrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (sub:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
    // Emits: (PSUBWrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::PSUBWrr, MVT::v8i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SUB_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (sub:v4i32 VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>))
    // Emits: (PSUBDrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 25  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT &&
          N1.hasOneUse()) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::LOAD &&
            N10.hasOneUse() &&
            CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
          SDOperand Chain10 = N10.getOperand(0);
          if (Predicate_memop(N10.Val)) {
            SDOperand N101 = N10.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N10.Val->getValueType(0) == MVT::v2i64) {
              return Emit_13(N, X86::PSUBDrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (sub:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
    // Emits: (PSUBDrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::PSUBDrr, MVT::v4i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SUB_v2i64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (sub:v2i64 VR128:v2i64:$src1, (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (PSUBQrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::PSUBQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (sub:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
    // Emits: (PSUBQrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::PSUBQrr, MVT::v2i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_SUBC_i32(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (subc:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (SUB32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_15(N, X86::SUB32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (subc:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (SUB32ri8:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val)) {
        return Emit_17(N, X86::SUB32ri8, MVT::i32);
      }

      // Pattern: (subc:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (SUB32ri:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_17(N, X86::SUB32ri, MVT::i32);
    }
  }

  // Pattern: (subc:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Emits: (SUB32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_18(N, X86::SUB32rr, MVT::i32);
}

SDNode *Select_ISD_SUBC_i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (subc:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (SUB64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_15(N, X86::SUB64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (subc:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (SUB64ri8:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val)) {
        return Emit_19(N, X86::SUB64ri8, MVT::i64);
      }

      // Pattern: (subc:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Emits: (SUB64ri32:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_19(N, X86::SUB64ri32, MVT::i64);
    }
  }

  // Pattern: (subc:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Emits: (SUB64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_18(N, X86::SUB64rr, MVT::i64);
}

SDNode *Select_ISD_SUBE_i32(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (sube:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (SBB32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_20(N, X86::SBB32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (sube:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (SBB32ri8:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val)) {
        return Emit_22(N, X86::SBB32ri8, MVT::i32);
      }

      // Pattern: (sube:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (SBB32ri:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_22(N, X86::SBB32ri, MVT::i32);
    }
  }

  // Pattern: (sube:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Emits: (SBB32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_23(N, X86::SBB32rr, MVT::i32);
}

SDNode *Select_ISD_SUBE_i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (sube:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (SBB64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_20(N, X86::SBB64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (sube:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Emits: (SBB64ri32:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt32(N1.Val)) {
        return Emit_24(N, X86::SBB64ri32, MVT::i64);
      }

      // Pattern: (sube:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (SBB64ri8:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val)) {
        return Emit_24(N, X86::SBB64ri8, MVT::i64);
      }
    }
  }

  // Pattern: (sube:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Emits: (SBB64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_23(N, X86::SBB64rr, MVT::i64);
}

SDNode *Emit_168(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Chain);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_ISD_TRAP(const SDOperand &N) {
  return Emit_168(N, X86::TRAP);
}

SDNode *Emit_169(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain10 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_170(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N0);
}
SDNode *Emit_171(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N1);
}
SDNode *Select_ISD_VECTOR_SHUFFLE_v8i8(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::BIT_CONVERT &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::LOAD &&
          N10.hasOneUse() &&
          CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
        SDOperand Chain10 = N10.getOperand(0);
        if (Predicate_load(N10.Val)) {
          SDOperand N101 = N10.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR) {

              // Pattern: (vector_shuffle:v8i8 VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), (build_vector:v8i8)<<P:Predicate_MMX_UNPCKH_shuffle_mask>>)
              // Emits: (MMX_PUNPCKHBWrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_MMX_UNPCKH_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_169(N, X86::MMX_PUNPCKHBWrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (vector_shuffle:v8i8 VR64:v8i8:$src1, (bitconvert:v8i8 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), (build_vector:v8i8)<<P:Predicate_MMX_UNPCKL_shuffle_mask>>)
              // Emits: (MMX_PUNPCKLBWrm:v8i8 VR64:v8i8:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_MMX_UNPCKL_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_169(N, X86::MMX_PUNPCKLBWrm, MVT::v8i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v8i8 VR64:v8i8:$src, (undef:v8i8), (build_vector:v8i8)<<P:Predicate_MMX_UNPCKL_v_undef_shuffle_mask>>)
        // Emits: (MMX_PUNPCKLBWrr:v8i8 VR64:v8i8:$src, VR64:v8i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_MMX_UNPCKL_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::MMX_PUNPCKLBWrr, MVT::v8i8);
        }

        // Pattern: (vector_shuffle:v8i8 VR64:v8i8:$src, (undef:v8i8), (build_vector:v8i8)<<P:Predicate_MMX_UNPCKH_v_undef_shuffle_mask>>)
        // Emits: (MMX_PUNPCKHBWrr:v8i8 VR64:v8i8:$src, VR64:v8i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_MMX_UNPCKH_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::MMX_PUNPCKHBWrr, MVT::v8i8);
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2, (build_vector:v8i8)<<P:Predicate_MMX_UNPCKH_shuffle_mask>>)
      // Emits: (MMX_PUNPCKHBWrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_MMX_UNPCKH_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MMX_PUNPCKHBWrr, MVT::v8i8);
      }

      // Pattern: (vector_shuffle:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2, (build_vector:v8i8)<<P:Predicate_MMX_UNPCKL_shuffle_mask>>)
      // Emits: (MMX_PUNPCKLBWrr:v8i8 VR64:v8i8:$src1, VR64:v8i8:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_MMX_UNPCKL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MMX_PUNPCKLBWrr, MVT::v8i8);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_172(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = Transform_MMX_SHUFFLE_get_shuf_imm(N2.Val);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_173(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  SDOperand Tmp2 = Transform_MMX_SHUFFLE_get_shuf_imm(N2.Val);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, Tmp2);
}
SDNode *Select_ISD_VECTOR_SHUFFLE_v4i16(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (vector_shuffle:v4i16 (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src1)<<P:Predicate_load>>), (undef:v4i16), (build_vector:v4i16)<<P:Predicate_MMX_PSHUFW_shuffle_mask>><<X:MMX_SHUFFLE_get_shuf_imm>>:$src2)
    // Emits: (MMX_PSHUFWmi:v4i16 addr:iPTR:$src1, (MMX_SHUFFLE_get_shuf_imm:i8 (build_vector:v4i16):$src2))
    // Pattern complexity = 32  cost = 1  size = 3
    if (N0.getOpcode() == ISD::BIT_CONVERT &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_load(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::UNDEF) {
              SDOperand N2 = N.getOperand(2);
              if (N2.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_MMX_PSHUFW_shuffle_mask(N2.Val) &&
                  N00.Val->getValueType(0) == MVT::v1i64) {
                return Emit_172(N, X86::MMX_PSHUFWmi, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::BIT_CONVERT &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::LOAD &&
          N10.hasOneUse() &&
          CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
        SDOperand Chain10 = N10.getOperand(0);
        if (Predicate_load(N10.Val)) {
          SDOperand N101 = N10.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR) {

              // Pattern: (vector_shuffle:v4i16 VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), (build_vector:v4i16)<<P:Predicate_MMX_UNPCKH_shuffle_mask>>)
              // Emits: (MMX_PUNPCKHWDrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_MMX_UNPCKH_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_169(N, X86::MMX_PUNPCKHWDrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (vector_shuffle:v4i16 VR64:v4i16:$src1, (bitconvert:v4i16 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), (build_vector:v4i16)<<P:Predicate_MMX_UNPCKL_shuffle_mask>>)
              // Emits: (MMX_PUNPCKLWDrm:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_MMX_UNPCKL_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_169(N, X86::MMX_PUNPCKLWDrm, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v4i16 VR64:v4i16:$src, (undef:v4i16), (build_vector:v4i16)<<P:Predicate_MMX_UNPCKL_v_undef_shuffle_mask>>)
        // Emits: (MMX_PUNPCKLWDrr:v4i16 VR64:v8i8:$src, VR64:v8i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_MMX_UNPCKL_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::MMX_PUNPCKLWDrr, MVT::v4i16);
        }

        // Pattern: (vector_shuffle:v4i16 VR64:v4i16:$src, (undef:v4i16), (build_vector:v4i16)<<P:Predicate_MMX_UNPCKH_v_undef_shuffle_mask>>)
        // Emits: (MMX_PUNPCKHWDrr:v4i16 VR64:v8i8:$src, VR64:v8i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_MMX_UNPCKH_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::MMX_PUNPCKHWDrr, MVT::v4i16);
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (vector_shuffle:v4i16 VR64:v4i16:$src1, (undef:v4i16), (build_vector:v4i16)<<P:Predicate_MMX_PSHUFW_shuffle_mask>><<X:MMX_SHUFFLE_get_shuf_imm>>:$src2)
    // Emits: (MMX_PSHUFWri:v4i16 VR64:v4i16:$src1, (MMX_SHUFFLE_get_shuf_imm:i8 (build_vector:v4i16):$src2))
    // Pattern complexity = 10  cost = 1  size = 3
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_MMX_PSHUFW_shuffle_mask(N2.Val)) {
        return Emit_173(N, X86::MMX_PSHUFWri, MVT::v4i16);
      }
    }
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2, (build_vector:v4i16)<<P:Predicate_MMX_UNPCKH_shuffle_mask>>)
      // Emits: (MMX_PUNPCKHWDrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_MMX_UNPCKH_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MMX_PUNPCKHWDrr, MVT::v4i16);
      }

      // Pattern: (vector_shuffle:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2, (build_vector:v4i16)<<P:Predicate_MMX_UNPCKL_shuffle_mask>>)
      // Emits: (MMX_PUNPCKLWDrr:v4i16 VR64:v4i16:$src1, VR64:v4i16:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_MMX_UNPCKL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MMX_PUNPCKLWDrr, MVT::v4i16);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_VECTOR_SHUFFLE_v2i32(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::BIT_CONVERT &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::LOAD &&
          N10.hasOneUse() &&
          CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
        SDOperand Chain10 = N10.getOperand(0);
        if (Predicate_load(N10.Val)) {
          SDOperand N101 = N10.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR) {

              // Pattern: (vector_shuffle:v2i32 VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), (build_vector:v2i32)<<P:Predicate_MMX_UNPCKH_shuffle_mask>>)
              // Emits: (MMX_PUNPCKHDQrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_MMX_UNPCKH_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_169(N, X86::MMX_PUNPCKHDQrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (vector_shuffle:v2i32 VR64:v2i32:$src1, (bitconvert:v2i32 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>), (build_vector:v2i32)<<P:Predicate_MMX_UNPCKL_shuffle_mask>>)
              // Emits: (MMX_PUNPCKLDQrm:v2i32 VR64:v2i32:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_MMX_UNPCKL_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v1i64) {
                return Emit_169(N, X86::MMX_PUNPCKLDQrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v2i32 VR64:v2i32:$src, (undef:v2i32), (build_vector:v2i32)<<P:Predicate_MMX_UNPCKL_v_undef_shuffle_mask>>)
        // Emits: (MMX_PUNPCKLDQrr:v2i32 VR64:v8i8:$src, VR64:v8i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_MMX_UNPCKL_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::MMX_PUNPCKLDQrr, MVT::v2i32);
        }

        // Pattern: (vector_shuffle:v2i32 VR64:v2i32:$src, (undef:v2i32), (build_vector:v2i32)<<P:Predicate_MMX_UNPCKH_v_undef_shuffle_mask>>)
        // Emits: (MMX_PUNPCKHDQrr:v2i32 VR64:v8i8:$src, VR64:v8i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_MMX_UNPCKH_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::MMX_PUNPCKHDQrr, MVT::v2i32);
        }
      }
    }
  }
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2, (build_vector:v2i32)<<P:Predicate_MMX_UNPCKH_shuffle_mask>>)
      // Emits: (MMX_PUNPCKHDQrr:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_MMX_UNPCKH_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MMX_PUNPCKHDQrr, MVT::v2i32);
      }

      // Pattern: (vector_shuffle:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2, (build_vector:v2i32)<<P:Predicate_MMX_UNPCKL_shuffle_mask>>)
      // Emits: (MMX_PUNPCKLDQrr:v2i32 VR64:v2i32:$src1, VR64:v2i32:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_MMX_UNPCKL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MMX_PUNPCKLDQrr, MVT::v2i32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_VECTOR_SHUFFLE_v16i8(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::BIT_CONVERT &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::LOAD &&
          N10.hasOneUse() &&
          CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
        SDOperand Chain10 = N10.getOperand(0);
        if (Predicate_memop(N10.Val)) {
          SDOperand N101 = N10.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR) {

              // Pattern: (vector_shuffle:v16i8 VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), (build_vector:v16i8)<<P:Predicate_UNPCKL_shuffle_mask>>)
              // Emits: (PUNPCKLBWrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_UNPCKL_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_169(N, X86::PUNPCKLBWrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (vector_shuffle:v16i8 VR128:v16i8:$src1, (bitconvert:v16i8 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), (build_vector:v16i8)<<P:Predicate_UNPCKH_shuffle_mask>>)
              // Emits: (PUNPCKHBWrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_UNPCKH_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_169(N, X86::PUNPCKHBWrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v16i8 VR128:v16i8:$src, (undef:v16i8), (build_vector:v16i8)<<P:Predicate_UNPCKL_v_undef_shuffle_mask>>)
        // Emits: (PUNPCKLBWrr:v16i8 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_UNPCKL_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::PUNPCKLBWrr, MVT::v16i8);
        }

        // Pattern: (vector_shuffle:v16i8 VR128:v16i8:$src, (undef:v16i8), (build_vector:v16i8)<<P:Predicate_UNPCKH_v_undef_shuffle_mask>>)
        // Emits: (PUNPCKHBWrr:v16i8 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_UNPCKH_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::PUNPCKHBWrr, MVT::v16i8);
        }
      }
    }
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2, (build_vector:v16i8)<<P:Predicate_UNPCKL_shuffle_mask>>)
      // Emits: (PUNPCKLBWrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_UNPCKL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::PUNPCKLBWrr, MVT::v16i8);
      }

      // Pattern: (vector_shuffle:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2, (build_vector:v16i8)<<P:Predicate_UNPCKH_shuffle_mask>>)
      // Emits: (PUNPCKHBWrr:v16i8 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::PUNPCKHBWrr, MVT::v16i8);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_174(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = Transform_SHUFFLE_get_pshufhw_imm(N2.Val);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_175(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = Transform_SHUFFLE_get_pshuflw_imm(N2.Val);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_176(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  SDOperand Tmp2 = Transform_SHUFFLE_get_pshufhw_imm(N2.Val);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, Tmp2);
}
SDNode *Emit_177(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  SDOperand Tmp2 = Transform_SHUFFLE_get_pshuflw_imm(N2.Val);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, Tmp2);
}
SDNode *Select_ISD_VECTOR_SHUFFLE_v8i16(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_memop(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::UNDEF) {
              SDOperand N2 = N.getOperand(2);
              if (N2.getOpcode() == ISD::BUILD_VECTOR) {

                // Pattern: (vector_shuffle:v8i16 (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src1)<<P:Predicate_memop>>), (undef:v8i16), (build_vector:v8i8)<<P:Predicate_PSHUFHW_shuffle_mask>><<X:SHUFFLE_get_pshufhw_imm>>:$src2)
                // Emits: (PSHUFHWmi:v8i16 addr:iPTR:$src1, (SHUFFLE_get_pshufhw_imm:i8 (build_vector:v8i8):$src2))
                // Pattern complexity = 32  cost = 1  size = 3
                if (Predicate_PSHUFHW_shuffle_mask(N2.Val) &&
                    N00.Val->getValueType(0) == MVT::v2i64) {
                  return Emit_174(N, X86::PSHUFHWmi, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (vector_shuffle:v8i16 (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src1)<<P:Predicate_memop>>), (undef:v8i16), (build_vector:v8i8)<<P:Predicate_PSHUFLW_shuffle_mask>><<X:SHUFFLE_get_pshuflw_imm>>:$src2)
                // Emits: (PSHUFLWmi:v8i16 addr:iPTR:$src1, (SHUFFLE_get_pshuflw_imm:i32 (build_vector:v8i8):$src2))
                // Pattern complexity = 32  cost = 1  size = 3
                if (Predicate_PSHUFLW_shuffle_mask(N2.Val) &&
                    N00.Val->getValueType(0) == MVT::v2i64) {
                  return Emit_175(N, X86::PSHUFLWmi, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::BIT_CONVERT &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::LOAD &&
          N10.hasOneUse() &&
          CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
        SDOperand Chain10 = N10.getOperand(0);
        if (Predicate_memop(N10.Val)) {
          SDOperand N101 = N10.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR) {

              // Pattern: (vector_shuffle:v8i16 VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), (build_vector:v8i8)<<P:Predicate_UNPCKL_shuffle_mask>>)
              // Emits: (PUNPCKLWDrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_UNPCKL_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_169(N, X86::PUNPCKLWDrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (vector_shuffle:v8i16 VR128:v8i16:$src1, (bitconvert:v8i16 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), (build_vector:v8i8)<<P:Predicate_UNPCKH_shuffle_mask>>)
              // Emits: (PUNPCKHWDrm:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_UNPCKH_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_169(N, X86::PUNPCKHWDrm, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v8i16 VR128:v8i16:$src, (undef:v8i16), (build_vector:v8i8)<<P:Predicate_UNPCKL_v_undef_shuffle_mask>>)
        // Emits: (PUNPCKLWDrr:v8i16 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_UNPCKL_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::PUNPCKLWDrr, MVT::v8i16);
        }

        // Pattern: (vector_shuffle:v8i16 VR128:v8i16:$src, (undef:v8i16), (build_vector:v8i8)<<P:Predicate_UNPCKH_v_undef_shuffle_mask>>)
        // Emits: (PUNPCKHWDrr:v8i16 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_UNPCKH_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::PUNPCKHWDrr, MVT::v8i16);
        }

        // Pattern: (vector_shuffle:v8i16 VR128:v8i16:$src1, (undef:v8i16), (build_vector:v8i8)<<P:Predicate_PSHUFHW_shuffle_mask>><<X:SHUFFLE_get_pshufhw_imm>>:$src2)
        // Emits: (PSHUFHWri:v8i16 VR128:v8i16:$src1, (SHUFFLE_get_pshufhw_imm:i8 (build_vector:v8i8):$src2))
        // Pattern complexity = 10  cost = 1  size = 3
        if (Predicate_PSHUFHW_shuffle_mask(N2.Val)) {
          return Emit_176(N, X86::PSHUFHWri, MVT::v8i16);
        }

        // Pattern: (vector_shuffle:v8i16 VR128:v8i16:$src1, (undef:v8i16), (build_vector:v8i8)<<P:Predicate_PSHUFLW_shuffle_mask>><<X:SHUFFLE_get_pshuflw_imm>>:$src2)
        // Emits: (PSHUFLWri:v8i16 VR128:v8i16:$src1, (SHUFFLE_get_pshuflw_imm:i32 (build_vector:v8i8):$src2))
        // Pattern complexity = 10  cost = 1  size = 3
        if (Predicate_PSHUFLW_shuffle_mask(N2.Val)) {
          return Emit_177(N, X86::PSHUFLWri, MVT::v8i16);
        }
      }
    }
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2, (build_vector:v8i8)<<P:Predicate_UNPCKL_shuffle_mask>>)
      // Emits: (PUNPCKLWDrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_UNPCKL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::PUNPCKLWDrr, MVT::v8i16);
      }

      // Pattern: (vector_shuffle:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2, (build_vector:v8i8)<<P:Predicate_UNPCKH_shuffle_mask>>)
      // Emits: (PUNPCKHWDrr:v8i16 VR128:v8i16:$src1, VR128:v8i16:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::PUNPCKHWDrr, MVT::v8i16);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_178(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_179(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = Transform_SHUFFLE_get_shuf_imm(N2.Val);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_180(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp3 = Transform_SHUFFLE_get_shuf_imm(N2.Val);
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp3, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 8);
  Chain10 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_181(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0);
}
SDNode *Emit_182(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  SDOperand Tmp2 = Transform_SHUFFLE_get_shuf_imm(N2.Val);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, Tmp2);
}
SDNode *Emit_183(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  SDOperand Tmp3 = Transform_SHUFFLE_get_shuf_imm(N2.Val);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N1, Tmp3);
}
SDNode *Select_ISD_VECTOR_SHUFFLE_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_memop(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::UNDEF) {
              SDOperand N2 = N.getOperand(2);
              if (N2.getOpcode() == ISD::BUILD_VECTOR) {

                // Pattern: (vector_shuffle:v4i32 (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src)<<P:Predicate_memop>>), (undef:v4i32), (build_vector:v4i16)<<P:Predicate_MOVSHDUP_shuffle_mask>>)
                // Emits: (MOVSHDUPrm:v4i32 addr:iPTR:$src)
                // Pattern complexity = 52  cost = 1  size = 3
                if (Predicate_MOVSHDUP_shuffle_mask(N2.Val) &&
                    N00.Val->getValueType(0) == MVT::v2i64) {
                  return Emit_178(N, X86::MOVSHDUPrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (vector_shuffle:v4i32 (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src)<<P:Predicate_memop>>), (undef:v4i32), (build_vector:v4i16)<<P:Predicate_MOVSLDUP_shuffle_mask>>)
                // Emits: (MOVSLDUPrm:v4i32 addr:iPTR:$src)
                // Pattern complexity = 52  cost = 1  size = 3
                if (Predicate_MOVSLDUP_shuffle_mask(N2.Val) &&
                    N00.Val->getValueType(0) == MVT::v2i64) {
                  return Emit_178(N, X86::MOVSLDUPrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), (build_vector:v4i16)<<P:Predicate_MOVLP_shuffle_mask>>)
  // Emits: (MOVLPSrm:v4i32 VR128:v16i8:$src1, addr:iPTR:$src2)
  // Pattern complexity = 49  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::BIT_CONVERT &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::LOAD &&
          N10.hasOneUse() &&
          CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
        SDOperand Chain10 = N10.getOperand(0);
        if (Predicate_memop(N10.Val)) {
          SDOperand N101 = N10.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_MOVLP_shuffle_mask(N2.Val) &&
                N10.Val->getValueType(0) == MVT::v2i64) {
              return Emit_169(N, X86::MOVLPSrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }

  // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), (build_vector:v4i16)<<P:Predicate_MOVHP_shuffle_mask>>)
  // Emits: (MOVHPSrm:v4i32 VR128:v16i8:$src1, addr:iPTR:$src2)
  // Pattern complexity = 49  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::BIT_CONVERT &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::LOAD &&
          N10.hasOneUse() &&
          CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
        SDOperand Chain10 = N10.getOperand(0);
        if (Predicate_memop(N10.Val)) {
          SDOperand N101 = N10.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_MOVHP_shuffle_mask(N2.Val) &&
                N10.Val->getValueType(0) == MVT::v2i64) {
              return Emit_169(N, X86::MOVHPSrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_memop(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::UNDEF) {
              SDOperand N2 = N.getOperand(2);
              if (N2.getOpcode() == ISD::BUILD_VECTOR) {

                // Pattern: (vector_shuffle:v4i32 (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src1)<<P:Predicate_memop>>), (undef:v4i32), (build_vector:v4i16)<<P:Predicate_PSHUFD_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$src2)
                // Emits: (PSHUFDmi:v4i32 addr:iPTR:$src1, (SHUFFLE_get_shuf_imm:i8 (build_vector:v4i16):$src2))
                // Pattern complexity = 32  cost = 1  size = 3
                if (Predicate_PSHUFD_shuffle_mask(N2.Val) &&
                    N00.Val->getValueType(0) == MVT::v2i64) {
                  return Emit_179(N, X86::PSHUFDmi, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (vector_shuffle:v4i32 (bitconvert:v4i32 (ld:v4f32 addr:iPTR:$src1)<<P:Predicate_memop>>), (undef:v4i32), (build_vector:v4i16)<<P:Predicate_SHUFP_unary_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$sm)
                // Emits: (PSHUFDmi:v4i32 addr:iPTR:$src1, (SHUFFLE_get_shuf_imm:i8 (build_vector:i8)<<P:Predicate_SHUFP_unary_shuffle_mask>>:$sm))
                // Pattern complexity = 32  cost = 1  size = 3
                if (Predicate_SHUFP_unary_shuffle_mask(N2.Val) &&
                    N00.Val->getValueType(0) == MVT::v4f32) {
                  return Emit_179(N, X86::PSHUFDmi, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::BIT_CONVERT &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::LOAD &&
          N10.hasOneUse() &&
          CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
        SDOperand Chain10 = N10.getOperand(0);
        if (Predicate_memop(N10.Val)) {
          SDOperand N101 = N10.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR) {

              // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), (build_vector:v4i16)<<P:Predicate_UNPCKL_shuffle_mask>>)
              // Emits: (PUNPCKLDQrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_UNPCKL_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_169(N, X86::PUNPCKLDQrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), (build_vector:v4i16)<<P:Predicate_UNPCKH_shuffle_mask>>)
              // Emits: (PUNPCKHDQrm:v4i32 VR128:v4i32:$src1, addr:iPTR:$src2)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_UNPCKH_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_169(N, X86::PUNPCKHDQrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, (bitconvert:v4i32 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>), (build_vector:v4i16)<<P:Predicate_PSHUFD_binary_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$sm)
              // Emits: (SHUFPSrmi:v4i32 VR128:v16i8:$src1, addr:iPTR:$src2, (SHUFFLE_get_shuf_imm:i32 (build_vector:i32)<<P:Predicate_PSHUFD_binary_shuffle_mask>>:$sm))
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_PSHUFD_binary_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::v2i64) {
                return Emit_180(N, X86::SHUFPSrmi, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE3())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src, (undef:v4i32), (build_vector:v4i16)<<P:Predicate_MOVSHDUP_shuffle_mask>>)
        // Emits: (MOVSHDUPrr:v4i32 VR128:v16i8:$src)
        // Pattern complexity = 25  cost = 1  size = 3
        if (Predicate_MOVSHDUP_shuffle_mask(N2.Val)) {
          return Emit_181(N, X86::MOVSHDUPrr, MVT::v4i32);
        }

        // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src, (undef:v4i32), (build_vector:v4i16)<<P:Predicate_MOVSLDUP_shuffle_mask>>)
        // Emits: (MOVSLDUPrr:v4i32 VR128:v16i8:$src)
        // Pattern complexity = 25  cost = 1  size = 3
        if (Predicate_MOVSLDUP_shuffle_mask(N2.Val)) {
          return Emit_181(N, X86::MOVSLDUPrr, MVT::v4i32);
        }
      }
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, (undef:v4i32), (build_vector:v4i16)<<P:Predicate_MOVHLPS_v_undef_shuffle_mask>>)
    // Emits: (MOVHLPSrr:v4i32 VR128:v16i8:$src1, VR128:v16i8:$src1)
    // Pattern complexity = 25  cost = 1  size = 3
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_MOVHLPS_v_undef_shuffle_mask(N2.Val)) {
        return Emit_170(N, X86::MOVHLPSrr, MVT::v4i32);
      }
    }
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2, (build_vector:v4i16)<<P:Predicate_MOVHP_shuffle_mask>>)
      // Emits: (MOVLHPSrr:v4i32 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (Predicate_MOVHP_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MOVLHPSrr, MVT::v4i32);
      }

      // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2, (build_vector:v4i16)<<P:Predicate_MOVHLPS_shuffle_mask>>)
      // Emits: (MOVHLPSrr:v4i32 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (Predicate_MOVHLPS_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MOVHLPSrr, MVT::v4i32);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2, (build_vector:v4i16)<<P:Predicate_MOVL_shuffle_mask>>)
      // Emits: (MOVLPSrr:v4i32 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (Predicate_MOVL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MOVLPSrr, MVT::v4i32);
      }

      // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2, (build_vector:v4i16)<<P:Predicate_MOVLP_shuffle_mask>>)
      // Emits: (MOVLPDrr:v4i32 VR128:v16i8:$src1, VR128:v16i8:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (Predicate_MOVLP_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MOVLPDrr, MVT::v4i32);
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src, (undef:v4i32), (build_vector:v4i16)<<P:Predicate_UNPCKL_v_undef_shuffle_mask>>)
        // Emits: (PUNPCKLDQrr:v4i32 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_UNPCKL_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::PUNPCKLDQrr, MVT::v4i32);
        }

        // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src, (undef:v4i32), (build_vector:v4i16)<<P:Predicate_UNPCKH_v_undef_shuffle_mask>>)
        // Emits: (PUNPCKHDQrr:v4i32 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_UNPCKH_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::PUNPCKHDQrr, MVT::v4i32);
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, (undef:v4i32), (build_vector:v4i16)<<P:Predicate_PSHUFD_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$src2)
    // Emits: (PSHUFDri:v4i32 VR128:v4i32:$src1, (SHUFFLE_get_shuf_imm:i8 (build_vector:v4i16):$src2))
    // Pattern complexity = 10  cost = 1  size = 3
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_PSHUFD_shuffle_mask(N2.Val)) {
        return Emit_182(N, X86::PSHUFDri, MVT::v4i32);
      }
    }
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2, (build_vector:v4i16)<<P:Predicate_UNPCKL_shuffle_mask>>)
      // Emits: (PUNPCKLDQrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_UNPCKL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::PUNPCKLDQrr, MVT::v4i32);
      }

      // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2, (build_vector:v4i16)<<P:Predicate_UNPCKH_shuffle_mask>>)
      // Emits: (PUNPCKHDQrr:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::PUNPCKHDQrr, MVT::v4i32);
      }

      // Pattern: (vector_shuffle:v4i32 VR128:v4i32:$src1, VR128:v4i32:$src2, (build_vector:v4i16)<<P:Predicate_PSHUFD_binary_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$sm)
      // Emits: (SHUFPSrri:v4i32 VR128:v16i8:$src1, VR128:v16i8:$src2, (SHUFFLE_get_shuf_imm:i32 (build_vector:i32)<<P:Predicate_PSHUFD_binary_shuffle_mask>>:$sm))
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_PSHUFD_binary_shuffle_mask(N2.Val)) {
        return Emit_183(N, X86::SHUFPSrri, MVT::v4i32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_184(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_185(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N100);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N100);
}
SDNode *Emit_186(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(N0);
  SDOperand Tmp3 = Transform_SHUFFLE_get_shuf_imm(N2.Val);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N0, Tmp3);
}
SDNode *Select_ISD_VECTOR_SHUFFLE_v2i64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_memop(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::BUILD_VECTOR) {
            if (Predicate_MOVLP_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::MOVLPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (vector_shuffle:v2i64 VR128:v2i64:$src1, (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v2i32)<<P:Predicate_UNPCKL_shuffle_mask>>)
            // Emits: (PUNPCKLQDQrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 26  cost = 1  size = 3
            if (Predicate_UNPCKL_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::PUNPCKLQDQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (vector_shuffle:v2i64 VR128:v2i64:$src1, (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v2i32)<<P:Predicate_UNPCKH_shuffle_mask>>)
            // Emits: (PUNPCKHQDQrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 26  cost = 1  size = 3
            if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::PUNPCKHQDQrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (vector_shuffle:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2, (build_vector:v2i32)<<P:Predicate_MOVL_shuffle_mask>>)
    // Emits: (MOVLPDrr:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_MOVL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MOVLPDrr, MVT::v2i64);
      }
    }
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v2i64 VR128:v2i64:$src, (undef:v2i64), (build_vector:v2i32)<<P:Predicate_SSE_splat_lo_mask>>:$sm)
        // Emits: (PUNPCKLQDQrr:v2i64 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_SSE_splat_lo_mask(N2.Val)) {
          return Emit_170(N, X86::PUNPCKLQDQrr, MVT::v2i64);
        }

        // Pattern: (vector_shuffle:v2i64 VR128:v2i64:$src, (undef:v2i64), (build_vector:v2i32)<<P:Predicate_UNPCKH_shuffle_mask>>:$sm)
        // Emits: (PUNPCKHQDQrr:v2i64 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::PUNPCKHQDQrr, MVT::v2i64);
        }
      }
    }
  }

  // Pattern: (vector_shuffle:v2i64 (build_vector:v2i64)<<P:Predicate_immAllZerosV>>, (scalar_to_vector:v2i64 (bitconvert:i64 VR64:v8i8:$src)), (build_vector:v2i32)<<P:Predicate_MOVL_shuffle_mask>>)
  // Emits: (MMX_MOVQ2DQrr:v2i64 VR64:v8i8:$src)
  // Pattern complexity = 17  cost = 1  size = 3
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BUILD_VECTOR &&
        Predicate_immAllZerosV(N0.Val)) {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::SCALAR_TO_VECTOR) {
        SDOperand N10 = N1.getOperand(0);
        if (N10.getOpcode() == ISD::BIT_CONVERT) {
          SDOperand N100 = N10.getOperand(0);
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_MOVL_shuffle_mask(N2.Val) &&
              N10.Val->getValueType(0) == MVT::i64 &&
              N100.Val->getValueType(0) == MVT::v8i8) {
            return Emit_185(N, X86::MMX_MOVQ2DQrr, MVT::v2i64);
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (vector_shuffle:v2i64 VR128:v2i64:$src1, (undef:v2i64), (build_vector:v2i32)<<P:Predicate_SHUFP_unary_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$sm)
    // Emits: (SHUFPDrri:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src1, (SHUFFLE_get_shuf_imm:i8 (build_vector:i8)<<P:Predicate_SHUFP_unary_shuffle_mask>>:$sm))
    // Pattern complexity = 10  cost = 1  size = 3
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_SHUFP_unary_shuffle_mask(N2.Val)) {
        return Emit_186(N, X86::SHUFPDrri, MVT::v2i64);
      }
    }
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2, (build_vector:v2i32)<<P:Predicate_UNPCKL_shuffle_mask>>)
      // Emits: (PUNPCKLQDQrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_UNPCKL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::PUNPCKLQDQrr, MVT::v2i64);
      }

      // Pattern: (vector_shuffle:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2, (build_vector:v2i32)<<P:Predicate_UNPCKH_shuffle_mask>>)
      // Emits: (PUNPCKHQDQrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::PUNPCKHQDQrr, MVT::v2i64);
      }

      // Pattern: (vector_shuffle:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2, (build_vector:v2i32)<<P:Predicate_SHUFP_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$sm)
      // Emits: (SHUFPDrri:v2i64 VR128:v16i8:$src1, VR128:v16i8:$src2, (SHUFFLE_get_shuf_imm:i8 (build_vector:i8)<<P:Predicate_SHUFP_shuffle_mask>>:$sm))
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_SHUFP_shuffle_mask(N2.Val)) {
        return Emit_183(N, X86::SHUFPDrri, MVT::v2i64);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_187(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand N100 = N10.getOperand(0);
  SDOperand Chain100 = N100.getOperand(0);
  SDOperand N1001 = N100.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N100 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N100)->getMemOperand());
  AddToISelQueue(Chain100);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N100, Chain100 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 7);
  Chain100 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N100.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_188(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_189(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp3 = Transform_SHUFFLE_get_shuf_imm(N2.Val);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 8);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Select_ISD_VECTOR_SHUFFLE_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::BIT_CONVERT &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::SCALAR_TO_VECTOR &&
          N10.hasOneUse()) {
        SDOperand N100 = N10.getOperand(0);
        if (N100.getOpcode() == ISD::LOAD &&
            N100.hasOneUse() &&
            CanBeFoldedBy(N100.Val, N10.Val, N.Val)) {
          SDOperand Chain100 = N100.getOperand(0);
          if (Predicate_load(N100.Val)) {
            SDOperand N1001 = N100.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N1001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N2 = N.getOperand(2);
              if (N2.getOpcode() == ISD::BUILD_VECTOR) {

                // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, (bitconvert:v4f32 (scalar_to_vector:v2f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)), (build_vector:v4i16)<<P:Predicate_MOVLP_shuffle_mask>>)
                // Emits: (MOVLPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
                // Pattern complexity = 52  cost = 1  size = 3
                if (Predicate_MOVLP_shuffle_mask(N2.Val) &&
                    N10.Val->getValueType(0) == MVT::v2f64 &&
                    N100.Val->getValueType(0) == MVT::f64) {
                  return Emit_187(N, X86::MOVLPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, (bitconvert:v4f32 (scalar_to_vector:v2f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)), (build_vector:v4i16)<<P:Predicate_MOVHP_shuffle_mask>>)
                // Emits: (MOVHPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
                // Pattern complexity = 52  cost = 1  size = 3
                if (Predicate_MOVHP_shuffle_mask(N2.Val) &&
                    N10.Val->getValueType(0) == MVT::v2f64 &&
                    N100.Val->getValueType(0) == MVT::f64) {
                  return Emit_187(N, X86::MOVHPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_memop(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::BUILD_VECTOR) {

            // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v4i16)<<P:Predicate_MOVLP_shuffle_mask>>)
            // Emits: (MOVLPSrm:v4f32 VR128:v16i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 46  cost = 1  size = 3
            if (Predicate_MOVLP_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::MOVLPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v4i16)<<P:Predicate_MOVHP_shuffle_mask>>)
            // Emits: (MOVHPSrm:v4f32 VR128:v16i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 46  cost = 1  size = 3
            if (Predicate_MOVHP_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::MOVHPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v4i16)<<P:Predicate_UNPCKH_shuffle_mask>>)
            // Emits: (UNPCKHPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
            // Pattern complexity = 36  cost = 1  size = 3
            if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::UNPCKHPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v4i16)<<P:Predicate_UNPCKL_shuffle_mask>>)
            // Emits: (UNPCKLPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
            // Pattern complexity = 36  cost = 1  size = 3
            if (Predicate_UNPCKL_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::UNPCKLPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::UNDEF) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR) {

              // Pattern: (vector_shuffle:v4f32 (ld:v4f32 addr:iPTR:$src)<<P:Predicate_memop>>, (undef:v4f32), (build_vector:v4i16)<<P:Predicate_MOVSHDUP_shuffle_mask>>)
              // Emits: (MOVSHDUPrm:v4f32 addr:iPTR:$src)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_MOVSHDUP_shuffle_mask(N2.Val)) {
                return Emit_188(N, X86::MOVSHDUPrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (vector_shuffle:v4f32 (ld:v4f32 addr:iPTR:$src)<<P:Predicate_memop>>, (undef:v4f32), (build_vector:v4i16)<<P:Predicate_MOVSLDUP_shuffle_mask>>)
              // Emits: (MOVSLDUPrm:v4f32 addr:iPTR:$src)
              // Pattern complexity = 29  cost = 1  size = 3
              if (Predicate_MOVSLDUP_shuffle_mask(N2.Val)) {
                return Emit_188(N, X86::MOVSLDUPrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }

  // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v4i16)<<P:Predicate_SHUFP_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$src3)
  // Emits: (SHUFPSrmi:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2, (SHUFFLE_get_shuf_imm:i32 (build_vector:v4i16):$src3))
  // Pattern complexity = 26  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_memop(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::BUILD_VECTOR &&
              Predicate_SHUFP_shuffle_mask(N2.Val)) {
            return Emit_189(N, X86::SHUFPSrmi, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }

  // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, (undef:v4f32), (build_vector:v4i16)<<P:Predicate_MOVHLPS_v_undef_shuffle_mask>>)
  // Emits: (MOVHLPSrr:v4f32 VR128:v16i8:$src1, VR128:v16i8:$src1)
  // Pattern complexity = 25  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_MOVHLPS_v_undef_shuffle_mask(N2.Val)) {
        return Emit_170(N, X86::MOVHLPSrr, MVT::v4f32);
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2, (build_vector:v4i16)<<P:Predicate_MOVHP_shuffle_mask>>)
      // Emits: (MOVLHPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (Predicate_MOVHP_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MOVLHPSrr, MVT::v4f32);
      }

      // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2, (build_vector:v4i16)<<P:Predicate_MOVHLPS_shuffle_mask>>)
      // Emits: (MOVHLPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (Predicate_MOVHLPS_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MOVHLPSrr, MVT::v4f32);
      }

      // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2, (build_vector:v4i16)<<P:Predicate_MOVL_shuffle_mask>>)
      // Emits: (MOVLPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (Predicate_MOVL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MOVLPSrr, MVT::v4f32);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2, (build_vector:v4i16)<<P:Predicate_MOVLP_shuffle_mask>>)
    // Emits: (MOVLPDrr:v4f32 VR128:v16i8:$src1, VR128:v16i8:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_MOVLP_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MOVLPDrr, MVT::v4f32);
      }
    }
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src, (undef:v4f32), (build_vector:v4i16)<<P:Predicate_UNPCKL_v_undef_shuffle_mask>>)
        // Emits: (UNPCKLPSrr:v4f32 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_UNPCKL_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::UNPCKLPSrr, MVT::v4f32);
        }

        // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src, (undef:v4f32), (build_vector:v4i16)<<P:Predicate_UNPCKH_v_undef_shuffle_mask>>)
        // Emits: (UNPCKHPSrr:v4f32 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_UNPCKH_v_undef_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::UNPCKHPSrr, MVT::v4f32);
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2, (build_vector:v4i16)<<P:Predicate_UNPCKH_shuffle_mask>>)
      // Emits: (UNPCKHPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 17  cost = 1  size = 3
      if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::UNPCKHPSrr, MVT::v4f32);
      }

      // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2, (build_vector:v4i16)<<P:Predicate_UNPCKL_shuffle_mask>>)
      // Emits: (UNPCKLPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
      // Pattern complexity = 17  cost = 1  size = 3
      if (Predicate_UNPCKL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::UNPCKLPSrr, MVT::v4f32);
      }
    }
  }
  if ((Subtarget->hasSSE3())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src, (undef:v4f32), (build_vector:v4i16)<<P:Predicate_MOVSHDUP_shuffle_mask>>)
        // Emits: (MOVSHDUPrr:v4f32 VR128:v4f32:$src)
        // Pattern complexity = 10  cost = 1  size = 3
        if (Predicate_MOVSHDUP_shuffle_mask(N2.Val)) {
          return Emit_181(N, X86::MOVSHDUPrr, MVT::v4f32);
        }

        // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src, (undef:v4f32), (build_vector:v4i16)<<P:Predicate_MOVSLDUP_shuffle_mask>>)
        // Emits: (MOVSLDUPrr:v4f32 VR128:v4f32:$src)
        // Pattern complexity = 10  cost = 1  size = 3
        if (Predicate_MOVSLDUP_shuffle_mask(N2.Val)) {
          return Emit_181(N, X86::MOVSLDUPrr, MVT::v4f32);
        }
      }
    }
  }
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, (undef:v4f32), (build_vector:v4i16)<<P:Predicate_SHUFP_unary_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$sm)
    // Emits: (SHUFPSrri:v4f32 VR128:v16i8:$src1, VR128:v16i8:$src1, (SHUFFLE_get_shuf_imm:i32 (build_vector:i32)<<P:Predicate_SHUFP_unary_shuffle_mask>>:$sm))
    // Pattern complexity = 10  cost = 1  size = 3
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_SHUFP_unary_shuffle_mask(N2.Val)) {
        return Emit_186(N, X86::SHUFPSrri, MVT::v4f32);
      }
    }

    // Pattern: (vector_shuffle:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2, (build_vector:v4i16)<<P:Predicate_SHUFP_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$src3)
    // Emits: (SHUFPSrri:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2, (SHUFFLE_get_shuf_imm:i32 (build_vector:v4i16):$src3))
    // Pattern complexity = 7  cost = 1  size = 3
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR &&
        Predicate_SHUFP_shuffle_mask(N2.Val)) {
      return Emit_183(N, X86::SHUFPSrri, MVT::v4f32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_190(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N1);
}
SDNode *Select_ISD_VECTOR_SHUFFLE_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::SCALAR_TO_VECTOR &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::LOAD &&
          N10.hasOneUse() &&
          CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
        SDOperand Chain10 = N10.getOperand(0);
        if (Predicate_load(N10.Val)) {
          SDOperand N101 = N10.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR) {

              // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, (scalar_to_vector:v2f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>), (build_vector:v2i32)<<P:Predicate_MOVLP_shuffle_mask>>)
              // Emits: (MOVLPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
              // Pattern complexity = 49  cost = 1  size = 3
              if (Predicate_MOVLP_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::f64) {
                return Emit_169(N, X86::MOVLPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }

              // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, (scalar_to_vector:v2f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>), (build_vector:v2i32)<<P:Predicate_MOVHP_shuffle_mask>>)
              // Emits: (MOVHPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
              // Pattern complexity = 49  cost = 1  size = 3
              if (Predicate_MOVHP_shuffle_mask(N2.Val) &&
                  N10.Val->getValueType(0) == MVT::f64) {
                return Emit_169(N, X86::MOVHPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_memop(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::BUILD_VECTOR) {

            // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v2i32)<<P:Predicate_MOVLP_shuffle_mask>>)
            // Emits: (MOVLPDrm:v2f64 VR128:v16i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 46  cost = 1  size = 3
            if (Predicate_MOVLP_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::MOVLPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v2i32)<<P:Predicate_MOVHP_shuffle_mask>>)
            // Emits: (MOVHPDrm:v2f64 VR128:v16i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 46  cost = 1  size = 3
            if (Predicate_MOVHP_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::MOVHPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v2i32)<<P:Predicate_UNPCKH_shuffle_mask>>)
            // Emits: (UNPCKHPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 36  cost = 1  size = 3
            if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::UNPCKHPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v2i32)<<P:Predicate_UNPCKL_shuffle_mask>>)
            // Emits: (UNPCKLPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
            // Pattern complexity = 36  cost = 1  size = 3
            if (Predicate_UNPCKL_shuffle_mask(N2.Val)) {
              return Emit_184(N, X86::UNPCKLPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }

  // Pattern: (vector_shuffle:v2f64 (scalar_to_vector:v2f64 (ld:f64 addr:iPTR:$src)<<P:Predicate_load>>), (undef:v2f64), (build_vector:v2i32)<<P:Predicate_SSE_splat_lo_mask>>)
  // Emits: (MOVDDUPrm:v2f64 addr:iPTR:$src)
  // Pattern complexity = 32  cost = 1  size = 3
  if ((Subtarget->hasSSE3())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_load(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N1 = N.getOperand(1);
            if (N1.getOpcode() == ISD::UNDEF) {
              SDOperand N2 = N.getOperand(2);
              if (N2.getOpcode() == ISD::BUILD_VECTOR &&
                  Predicate_SSE_splat_lo_mask(N2.Val) &&
                  N00.Val->getValueType(0) == MVT::f64) {
                return Emit_178(N, X86::MOVDDUPrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
              }
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>, (build_vector:v2i32)<<P:Predicate_SHUFP_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$src3)
    // Emits: (SHUFPDrmi:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2, (SHUFFLE_get_shuf_imm:i8 (build_vector:v2i32):$src3))
    // Pattern complexity = 26  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::BUILD_VECTOR &&
                Predicate_SHUFP_shuffle_mask(N2.Val)) {
              return Emit_189(N, X86::SHUFPDrmi, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (vector_shuffle:v2f64 (bitconvert:v2f64)<<P:Predicate_immAllZerosV_bc>>, VR128:v2f64:$src, (build_vector:v2i32)<<P:Predicate_MOVL_shuffle_mask>>)
    // Emits: (MOVZPQILo2PQIrr:v2f64 VR128:v16i8:$src)
    // Pattern complexity = 26  cost = 1  size = 3
    if (N0.getOpcode() == ISD::BIT_CONVERT &&
        Predicate_immAllZerosV_bc(N0.Val)) {
      SDOperand N1 = N.getOperand(1);
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_MOVL_shuffle_mask(N2.Val)) {
        return Emit_190(N, X86::MOVZPQILo2PQIrr, MVT::v2f64);
      }
    }
    SDOperand N1 = N.getOperand(1);

    // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2, (build_vector:v2i32)<<P:Predicate_MOVL_shuffle_mask>>)
    // Emits: (MOVLPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_MOVL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::MOVLPDrr, MVT::v2f64);
      }
    }
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR) {

        // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src, (undef:v2f64), (build_vector:v2i32)<<P:Predicate_SSE_splat_lo_mask>>:$sm)
        // Emits: (UNPCKLPDrr:v2f64 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_SSE_splat_lo_mask(N2.Val)) {
          return Emit_170(N, X86::UNPCKLPDrr, MVT::v2f64);
        }

        // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src, (undef:v2f64), (build_vector:v2i32)<<P:Predicate_UNPCKH_shuffle_mask>>:$sm)
        // Emits: (UNPCKHPDrr:v2f64 VR128:v16i8:$src, VR128:v16i8:$src)
        // Pattern complexity = 20  cost = 1  size = 3
        if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
          return Emit_170(N, X86::UNPCKHPDrr, MVT::v2f64);
        }
      }
    }
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR) {

      // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2, (build_vector:v2i32)<<P:Predicate_UNPCKH_shuffle_mask>>)
      // Emits: (UNPCKHPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 17  cost = 1  size = 3
      if (Predicate_UNPCKH_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::UNPCKHPDrr, MVT::v2f64);
      }

      // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2, (build_vector:v2i32)<<P:Predicate_UNPCKL_shuffle_mask>>)
      // Emits: (UNPCKLPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
      // Pattern complexity = 17  cost = 1  size = 3
      if (Predicate_UNPCKL_shuffle_mask(N2.Val)) {
        return Emit_171(N, X86::UNPCKLPDrr, MVT::v2f64);
      }
    }
  }

  // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src, (undef:v2f64), (build_vector:v2i32)<<P:Predicate_SSE_splat_lo_mask>>)
  // Emits: (MOVDDUPrr:v2f64 VR128:v2f64:$src)
  // Pattern complexity = 10  cost = 1  size = 3
  if ((Subtarget->hasSSE3())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_SSE_splat_lo_mask(N2.Val)) {
        return Emit_181(N, X86::MOVDDUPrr, MVT::v2f64);
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, (undef:v2f64), (build_vector:v2i32)<<P:Predicate_SHUFP_unary_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$sm)
    // Emits: (SHUFPDrri:v2f64 VR128:v16i8:$src1, VR128:v16i8:$src1, (SHUFFLE_get_shuf_imm:i8 (build_vector:i8)<<P:Predicate_SHUFP_unary_shuffle_mask>>:$sm))
    // Pattern complexity = 10  cost = 1  size = 3
    if (N1.getOpcode() == ISD::UNDEF) {
      SDOperand N2 = N.getOperand(2);
      if (N2.getOpcode() == ISD::BUILD_VECTOR &&
          Predicate_SHUFP_unary_shuffle_mask(N2.Val)) {
        return Emit_186(N, X86::SHUFPDrri, MVT::v2f64);
      }
    }

    // Pattern: (vector_shuffle:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2, (build_vector:v2i32)<<P:Predicate_SHUFP_shuffle_mask>><<X:SHUFFLE_get_shuf_imm>>:$src3)
    // Emits: (SHUFPDrri:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2, (SHUFFLE_get_shuf_imm:i8 (build_vector:v2i32):$src3))
    // Pattern complexity = 7  cost = 1  size = 3
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::BUILD_VECTOR &&
        Predicate_SHUFP_shuffle_mask(N2.Val)) {
      return Emit_183(N, X86::SHUFPDrri, MVT::v2f64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_XOR_i8(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (xor:i8 GR8:i8:$src1, (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (XOR8rm:i8 GR8:i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::XOR8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (xor:i8 (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>, GR8:i8:$src1)
    // Emits: (XOR8rm:i8 GR8:i8:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::XOR8rm, MVT::i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (xor:i8 GR8:i8:$src, (imm:i8)<<P:Predicate_immAllOnes>>)
      // Emits: (NOT8r:i8 GR8:i8:$src)
      // Pattern complexity = 7  cost = 1  size = 2
      if (Predicate_immAllOnes(N1.Val)) {
        return Emit_2(N, X86::NOT8r, MVT::i8);
      }

      // Pattern: (xor:i8 GR8:i8:$src1, (imm:i8):$src2)
      // Emits: (XOR8ri:i8 GR8:i8:$src1, (imm:i8):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_3(N, X86::XOR8ri, MVT::i8);
    }
  }

  // Pattern: (xor:i8 GR8:i8:$src1, GR8:i8:$src2)
  // Emits: (XOR8rr:i8 GR8:i8:$src1, GR8:i8:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::XOR8rr, MVT::i8);
}

SDNode *Select_ISD_XOR_i16(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (xor:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (XOR16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::XOR16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (xor:i16 (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, GR16:i16:$src1)
    // Emits: (XOR16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::XOR16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (xor:i16 GR16:i16:$src, (imm:i16)<<P:Predicate_immAllOnes>>)
      // Emits: (NOT16r:i16 GR16:i16:$src)
      // Pattern complexity = 7  cost = 1  size = 2
      if (Predicate_immAllOnes(N1.Val)) {
        return Emit_2(N, X86::NOT16r, MVT::i16);
      }

      // Pattern: (xor:i16 GR16:i16:$src1, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2)
      // Emits: (XOR16ri8:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i16immSExt8(N1.Val)) {
        return Emit_5(N, X86::XOR16ri8, MVT::i16);
      }

      // Pattern: (xor:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Emits: (XOR16ri:i16 GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_5(N, X86::XOR16ri, MVT::i16);
    }
  }

  // Pattern: (xor:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Emits: (XOR16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::XOR16rr, MVT::i16);
}

SDNode *Select_ISD_XOR_i32(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (xor:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (XOR32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::XOR32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (xor:i32 (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, GR32:i32:$src1)
    // Emits: (XOR32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::XOR32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (xor:i32 GR32:i32:$src, (imm:i32)<<P:Predicate_immAllOnes>>)
      // Emits: (NOT32r:i32 GR32:i32:$src)
      // Pattern complexity = 7  cost = 1  size = 2
      if (Predicate_immAllOnes(N1.Val)) {
        return Emit_2(N, X86::NOT32r, MVT::i32);
      }

      // Pattern: (xor:i32 GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (XOR32ri8:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val)) {
        return Emit_11(N, X86::XOR32ri8, MVT::i32);
      }

      // Pattern: (xor:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (XOR32ri:i32 GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      return Emit_11(N, X86::XOR32ri, MVT::i32);
    }
  }

  // Pattern: (xor:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Emits: (XOR32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::XOR32rr, MVT::i32);
}

SDNode *Select_ISD_XOR_i64(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (xor:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (XOR64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::XOR64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (xor:i64 (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, GR64:i64:$src1)
    // Emits: (XOR64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::XOR64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (xor:i64 GR64:i64:$src, (imm:i64)<<P:Predicate_immAllOnes>>)
      // Emits: (NOT64r:i64 GR64:i64:$src)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_immAllOnes(N1.Val)) {
        return Emit_2(N, X86::NOT64r, MVT::i64);
      }

      // Pattern: (xor:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Emits: (XOR64ri32:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt32(N1.Val)) {
        return Emit_12(N, X86::XOR64ri32, MVT::i64);
      }

      // Pattern: (xor:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (XOR64ri8:i64 GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val)) {
        return Emit_12(N, X86::XOR64ri8, MVT::i64);
      }
    }
  }

  // Pattern: (xor:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Emits: (XOR64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_4(N, X86::XOR64rr, MVT::i64);
}

SDNode *Select_ISD_XOR_v1i64(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (xor:v1i64 VR64:v1i64:$src1, (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>)
      // Emits: (MMX_PXORrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_load(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::MMX_PXORrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (xor:v1i64 (ld:v1i64 addr:iPTR:$src2)<<P:Predicate_load>>, VR64:v1i64:$src1)
      // Emits: (MMX_PXORrm:v1i64 VR64:v1i64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_load(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::MMX_PXORrm, MVT::v1i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (xor:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
    // Emits: (MMX_PXORrr:v1i64 VR64:v1i64:$src1, VR64:v1i64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MMX_PXORrr, MVT::v1i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_XOR_v2i64(const SDOperand &N) {

  // Pattern: (xor:v2i64 (bitconvert:v2i64 VR128:v4f32:$src1), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
  // Emits: (XORPSrm:v2i64 VR128:v4f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 25  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::v4f32) {
            return Emit_51(N, X86::XORPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }

  // Pattern: (xor:v2i64 (bitconvert:v2i64 VR128:v2f64:$src1), (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
  // Emits: (XORPDrm:v2i64 VR128:v2f64:$src1, addr:iPTR:$src2)
  // Pattern complexity = 25  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::v2f64) {
            return Emit_51(N, X86::XORPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }

  // Pattern: (xor:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (bitconvert:v2i64 VR128:v4f32:$src1))
  // Emits: (XORPSrm:v2i64 VR128:v4f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 25  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N10 = N1.getOperand(0);
            if (N10.Val->getValueType(0) == MVT::v4f32) {
              return Emit_52(N, X86::XORPSrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (xor:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, (bitconvert:v2i64 VR128:v2f64:$src1))
    // Emits: (XORPDrm:v2i64 VR128:v2f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 25  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::BIT_CONVERT) {
            SDOperand N10 = N1.getOperand(0);
            if (N10.Val->getValueType(0) == MVT::v2f64) {
              return Emit_52(N, X86::XORPDrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (xor:v2i64 VR128:v2i64:$src1, (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (PXORrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::PXORrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (xor:v2i64 (ld:v2i64 addr:iPTR:$src2)<<P:Predicate_memop>>, VR128:v2i64:$src1)
    // Emits: (PXORrm:v2i64 VR128:v2i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_memop(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_1(N, X86::PXORrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }

    // Pattern: (xor:v2i64 (bitconvert:v2i64 VR128:v2f64:$src1), (bitconvert:v2i64 VR128:v2f64:$src2))
    // Emits: (XORPDrr:v2i64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Pattern complexity = 9  cost = 1  size = 3
    if (N0.getOpcode() == ISD::BIT_CONVERT) {
      SDOperand N00 = N0.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::BIT_CONVERT) {
        SDOperand N10 = N1.getOperand(0);
        if (N00.Val->getValueType(0) == MVT::v2f64 &&
            N10.Val->getValueType(0) == MVT::v2f64) {
          return Emit_57(N, X86::XORPDrr, MVT::v2i64);
        }
      }
    }
  }

  // Pattern: (xor:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Emits: (XORPSrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    return Emit_4(N, X86::XORPSrr, MVT::v2i64);
  }

  // Pattern: (xor:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Emits: (PXORrr:v2i64 VR128:v2i64:$src1, VR128:v2i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    return Emit_4(N, X86::PXORrr, MVT::v2i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ZERO_EXTEND_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  if (N0.Val->getValueType(0) == MVT::i8) {
    return Emit_59(N, X86::MOVZX16rr8, MVT::i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_ISD_ZERO_EXTEND_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);

  // Pattern: (zext:i32 GR8:i8:$src)
  // Emits: (MOVZX32rr8:i32 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i8) {
    return Emit_59(N, X86::MOVZX32rr8, MVT::i32);
  }

  // Pattern: (zext:i32 GR16:i16:$src)
  // Emits: (MOVZX32rr16:i32 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i16) {
    return Emit_59(N, X86::MOVZX32rr16, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_191(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Tmp0 = CurDAG->getTargetConstant(0x0ULL, MVT::i64);
  AddToISelQueue(N0);
  SDOperand Tmp2 = CurDAG->getTargetConstant(0x3ULL, MVT::i32);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, Tmp0, N0, Tmp2);
}
SDNode *Select_ISD_ZERO_EXTEND_i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);

  // Pattern: (zext:i64 GR32:i32:$src)
  // Emits: (SUBREG_TO_REG:i64 0:i64, GR32:i32:$src, 3:i32)
  // Pattern complexity = 3  cost = 1  size = 0
  if (N0.Val->getValueType(0) == MVT::i32) {
    return Emit_191(N, TargetInstrInfo::SUBREG_TO_REG, MVT::i64);
  }

  // Pattern: (zext:i64 GR8:i8:$src)
  // Emits: (MOVZX64rr8:i64 GR8:i8:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i8) {
    return Emit_59(N, X86::MOVZX64rr8, MVT::i64);
  }

  // Pattern: (zext:i64 GR16:i16:$src)
  // Emits: (MOVZX64rr16:i64 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N0.Val->getValueType(0) == MVT::i16) {
    return Emit_59(N, X86::MOVZX64rr16, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_192(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N1);
  AddToISelQueue(Chain);
  AddToISelQueue(N3);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::EFLAGS, N3, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, N1, Chain, InFlag);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_X86ISD_BRCOND(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::BasicBlock) {
    SDOperand N2 = N.getOperand(2);
    if (isa<ConstantSDNode>(N2)) {
      int64_t CN0 = cast<ConstantSDNode>(N2)->getSignExtended();

      // Pattern: (X86brcond:void (bb:Other):$dst, 4:i8, EFLAGS:i32)
      // Emits: (JE:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 4) {
        return Emit_192(N, X86::JE);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 9:i8, EFLAGS:i32)
      // Emits: (JNE:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 9) {
        return Emit_192(N, X86::JNE);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 7:i8, EFLAGS:i32)
      // Emits: (JL:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 7) {
        return Emit_192(N, X86::JL);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 8:i8, EFLAGS:i32)
      // Emits: (JLE:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 8) {
        return Emit_192(N, X86::JLE);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 5:i8, EFLAGS:i32)
      // Emits: (JG:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 5) {
        return Emit_192(N, X86::JG);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 6:i8, EFLAGS:i32)
      // Emits: (JGE:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 6) {
        return Emit_192(N, X86::JGE);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 2:i8, EFLAGS:i32)
      // Emits: (JB:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 2) {
        return Emit_192(N, X86::JB);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 3:i8, EFLAGS:i32)
      // Emits: (JBE:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 3) {
        return Emit_192(N, X86::JBE);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 0:i8, EFLAGS:i32)
      // Emits: (JA:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 0) {
        return Emit_192(N, X86::JA);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 1:i8, EFLAGS:i32)
      // Emits: (JAE:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 1) {
        return Emit_192(N, X86::JAE);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 15:i8, EFLAGS:i32)
      // Emits: (JS:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 15) {
        return Emit_192(N, X86::JS);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 12:i8, EFLAGS:i32)
      // Emits: (JNS:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 12) {
        return Emit_192(N, X86::JNS);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 14:i8, EFLAGS:i32)
      // Emits: (JP:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 14) {
        return Emit_192(N, X86::JP);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 11:i8, EFLAGS:i32)
      // Emits: (JNP:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 11) {
        return Emit_192(N, X86::JNP);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 13:i8, EFLAGS:i32)
      // Emits: (JO:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 13) {
        return Emit_192(N, X86::JO);
      }

      // Pattern: (X86brcond:void (bb:Other):$dst, 10:i8, EFLAGS:i32)
      // Emits: (JNO:void (bb:Other):$dst)
      // Pattern complexity = 8  cost = 1  size = 3
      if (CN0 == 10) {
        return Emit_192(N, X86::JNO);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_193(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::i32, MVT::Other, Ops0, 6);
  Chain0 = SDOperand(ResNode, 2);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 2));
  return ResNode;
}
SDNode *Emit_194(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  AddToISelQueue(N0);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::i32, N0);
  return ResNode;
}
SDNode *Select_X86ISD_BSF_i16(const SDOperand &N) {

  // Pattern: (X86bsf:i16 (ld:i16 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (BSF16rm:i16 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_193(N, X86::BSF16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86bsf:i16 GR16:i16:$src)
  // Emits: (BSF16rr:i16 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_194(N, X86::BSF16rr, MVT::i16);
}

SDNode *Select_X86ISD_BSF_i32(const SDOperand &N) {

  // Pattern: (X86bsf:i32 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (BSF32rm:i32 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_193(N, X86::BSF32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86bsf:i32 GR32:i32:$src)
  // Emits: (BSF32rr:i32 GR32:i32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_194(N, X86::BSF32rr, MVT::i32);
}

SDNode *Select_X86ISD_BSF_i64(const SDOperand &N) {

  // Pattern: (X86bsf:i64 (ld:i64 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (BSF64rm:i64 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_193(N, X86::BSF64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86bsf:i64 GR64:i64:$src)
  // Emits: (BSF64rr:i64 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_194(N, X86::BSF64rr, MVT::i64);
}

SDNode *Select_X86ISD_BSR_i16(const SDOperand &N) {

  // Pattern: (X86bsr:i16 (ld:i16 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (BSR16rm:i16 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_193(N, X86::BSR16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86bsr:i16 GR16:i16:$src)
  // Emits: (BSR16rr:i16 GR16:i16:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_194(N, X86::BSR16rr, MVT::i16);
}

SDNode *Select_X86ISD_BSR_i32(const SDOperand &N) {

  // Pattern: (X86bsr:i32 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (BSR32rm:i32 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_193(N, X86::BSR32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86bsr:i32 GR32:i32:$src)
  // Emits: (BSR32rr:i32 GR32:i32:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_194(N, X86::BSR32rr, MVT::i32);
}

SDNode *Select_X86ISD_BSR_i64(const SDOperand &N) {

  // Pattern: (X86bsr:i64 (ld:i64 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (BSR64rm:i64 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_193(N, X86::BSR64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86bsr:i64 GR64:i64:$src)
  // Emits: (BSR64rr:i64 GR64:i64:$src)
  // Pattern complexity = 3  cost = 1  size = 3
  return Emit_194(N, X86::BSR64rr, MVT::i64);
}

SDNode *Emit_195(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  bool HasInFlag = (N.getOperand(N.getNumOperands()-1).getValueType() == MVT::Flag);
  SmallVector<SDOperand, 8> Ops0;
  AddToISelQueue(Chain);
  SDOperand InFlag(0, 0);
  if (HasInFlag) {
    InFlag = N.getOperand(N.getNumOperands()-1);
    AddToISelQueue(InFlag);
  }
  Ops0.push_back(N1);
  for (unsigned i = 2, e = N.getNumOperands()-(HasInFlag?1:0); i != e; ++i) {
    AddToISelQueue(N.getOperand(i));
    Ops0.push_back(N.getOperand(i));
  }
  Ops0.push_back(Chain);
  if (HasInFlag)
    Ops0.push_back(InFlag);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, &Ops0[0], Ops0.size());
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Emit_196(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  bool HasInFlag = (N.getOperand(N.getNumOperands()-1).getValueType() == MVT::Flag);
  SmallVector<SDOperand, 8> Ops0;
  AddToISelQueue(N1);
  AddToISelQueue(Chain);
  SDOperand InFlag(0, 0);
  if (HasInFlag) {
    InFlag = N.getOperand(N.getNumOperands()-1);
    AddToISelQueue(InFlag);
  }
  Ops0.push_back(N1);
  for (unsigned i = 2, e = N.getNumOperands()-(HasInFlag?1:0); i != e; ++i) {
    AddToISelQueue(N.getOperand(i));
    Ops0.push_back(N.getOperand(i));
  }
  Ops0.push_back(Chain);
  if (HasInFlag)
    Ops0.push_back(InFlag);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, &Ops0[0], Ops0.size());
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Select_X86ISD_CALL(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (X86call:void (tglobaladdr:i64):$dst)
  // Emits: (CALL64pcrel32:void (tglobaladdr:i64):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetGlobalAddress &&
      N1.Val->getValueType(0) == MVT::i64) {
    return Emit_195(N, X86::CALL64pcrel32);
  }

  // Pattern: (X86call:void (texternalsym:i64):$dst)
  // Emits: (CALL64pcrel32:void (texternalsym:i64):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetExternalSymbol &&
      N1.Val->getValueType(0) == MVT::i64) {
    return Emit_195(N, X86::CALL64pcrel32);
  }

  // Pattern: (X86call:void (tglobaladdr:i32):$dst)
  // Emits: (CALLpcrel32:void (tglobaladdr:i32):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetGlobalAddress &&
      N1.Val->getValueType(0) == MVT::i32) {
    return Emit_195(N, X86::CALLpcrel32);
  }

  // Pattern: (X86call:void (texternalsym:i32):$dst)
  // Emits: (CALLpcrel32:void (texternalsym:i32):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetExternalSymbol &&
      N1.Val->getValueType(0) == MVT::i32) {
    return Emit_195(N, X86::CALLpcrel32);
  }

  // Pattern: (X86call:void GR32:i32:$dst)
  // Emits: (CALL32r:void GR32:i32:$dst)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i32) {
    return Emit_196(N, X86::CALL32r);
  }

  // Pattern: (X86call:void GR64:i64:$dst)
  // Emits: (CALL64r:void GR64:i64:$dst)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i64) {
    return Emit_196(N, X86::CALL64r);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_197(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  AddToISelQueue(N3);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain1, X86::EFLAGS, N3, InFlag).Val;
  Chain1 = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1, InFlag };
  ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 8);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_198(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  SDOperand Chain = CurDAG->getEntryNode();
  AddToISelQueue(N3);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::EFLAGS, N3, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N1, InFlag);
}
SDNode *Select_X86ISD_CMOV_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::LOAD &&
      N1.hasOneUse() &&
      CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
    SDOperand Chain1 = N1.getOperand(0);
    if (Predicate_load(N1.Val)) {
      SDOperand N11 = N1.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        SDOperand N2 = N.getOperand(2);
        if (isa<ConstantSDNode>(N2)) {
          int64_t CN0 = cast<ConstantSDNode>(N2)->getSignExtended();

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 2:i8, EFLAGS:i32)
          // Emits: (CMOVB16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 2) {
            return Emit_197(N, X86::CMOVB16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 1:i8, EFLAGS:i32)
          // Emits: (CMOVAE16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 1) {
            return Emit_197(N, X86::CMOVAE16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 4:i8, EFLAGS:i32)
          // Emits: (CMOVE16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 4) {
            return Emit_197(N, X86::CMOVE16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 9:i8, EFLAGS:i32)
          // Emits: (CMOVNE16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 9) {
            return Emit_197(N, X86::CMOVNE16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 3:i8, EFLAGS:i32)
          // Emits: (CMOVBE16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 3) {
            return Emit_197(N, X86::CMOVBE16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 0:i8, EFLAGS:i32)
          // Emits: (CMOVA16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 0) {
            return Emit_197(N, X86::CMOVA16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 7:i8, EFLAGS:i32)
          // Emits: (CMOVL16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 7) {
            return Emit_197(N, X86::CMOVL16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 6:i8, EFLAGS:i32)
          // Emits: (CMOVGE16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 6) {
            return Emit_197(N, X86::CMOVGE16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 8:i8, EFLAGS:i32)
          // Emits: (CMOVLE16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 8) {
            return Emit_197(N, X86::CMOVLE16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 5:i8, EFLAGS:i32)
          // Emits: (CMOVG16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 5) {
            return Emit_197(N, X86::CMOVG16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 15:i8, EFLAGS:i32)
          // Emits: (CMOVS16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 15) {
            return Emit_197(N, X86::CMOVS16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 12:i8, EFLAGS:i32)
          // Emits: (CMOVNS16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 12) {
            return Emit_197(N, X86::CMOVNS16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 14:i8, EFLAGS:i32)
          // Emits: (CMOVP16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 14) {
            return Emit_197(N, X86::CMOVP16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, 11:i8, EFLAGS:i32)
          // Emits: (CMOVNP16rm:i16 GR16:i16:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 11) {
            return Emit_197(N, X86::CMOVNP16rm, MVT::i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }
  SDOperand N2 = N.getOperand(2);
  if (isa<ConstantSDNode>(N2)) {
    int64_t CN0 = cast<ConstantSDNode>(N2)->getSignExtended();

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 2:i8, EFLAGS:i32)
    // Emits: (CMOVB16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 2) {
      return Emit_198(N, X86::CMOVB16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 1:i8, EFLAGS:i32)
    // Emits: (CMOVAE16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 1) {
      return Emit_198(N, X86::CMOVAE16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 4:i8, EFLAGS:i32)
    // Emits: (CMOVE16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 4) {
      return Emit_198(N, X86::CMOVE16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 9:i8, EFLAGS:i32)
    // Emits: (CMOVNE16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 9) {
      return Emit_198(N, X86::CMOVNE16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 3:i8, EFLAGS:i32)
    // Emits: (CMOVBE16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 3) {
      return Emit_198(N, X86::CMOVBE16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 0:i8, EFLAGS:i32)
    // Emits: (CMOVA16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 0) {
      return Emit_198(N, X86::CMOVA16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 7:i8, EFLAGS:i32)
    // Emits: (CMOVL16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 7) {
      return Emit_198(N, X86::CMOVL16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 6:i8, EFLAGS:i32)
    // Emits: (CMOVGE16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 6) {
      return Emit_198(N, X86::CMOVGE16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 8:i8, EFLAGS:i32)
    // Emits: (CMOVLE16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 8) {
      return Emit_198(N, X86::CMOVLE16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 5:i8, EFLAGS:i32)
    // Emits: (CMOVG16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 5) {
      return Emit_198(N, X86::CMOVG16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 15:i8, EFLAGS:i32)
    // Emits: (CMOVS16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 15) {
      return Emit_198(N, X86::CMOVS16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 12:i8, EFLAGS:i32)
    // Emits: (CMOVNS16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 12) {
      return Emit_198(N, X86::CMOVNS16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 14:i8, EFLAGS:i32)
    // Emits: (CMOVP16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 14) {
      return Emit_198(N, X86::CMOVP16rr, MVT::i16);
    }

    // Pattern: (X86cmov:i16 GR16:i16:$src1, GR16:i16:$src2, 11:i8, EFLAGS:i32)
    // Emits: (CMOVNP16rr:i16 GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 11) {
      return Emit_198(N, X86::CMOVNP16rr, MVT::i16);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_CMOV_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::LOAD &&
      N1.hasOneUse() &&
      CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
    SDOperand Chain1 = N1.getOperand(0);
    if (Predicate_load(N1.Val)) {
      SDOperand N11 = N1.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        SDOperand N2 = N.getOperand(2);
        if (isa<ConstantSDNode>(N2)) {
          int64_t CN0 = cast<ConstantSDNode>(N2)->getSignExtended();

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 11:i8, EFLAGS:i32)
          // Emits: (CMOVNP32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 11) {
            return Emit_197(N, X86::CMOVNP32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 2:i8, EFLAGS:i32)
          // Emits: (CMOVB32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 2) {
            return Emit_197(N, X86::CMOVB32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 1:i8, EFLAGS:i32)
          // Emits: (CMOVAE32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 1) {
            return Emit_197(N, X86::CMOVAE32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 4:i8, EFLAGS:i32)
          // Emits: (CMOVE32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 4) {
            return Emit_197(N, X86::CMOVE32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 9:i8, EFLAGS:i32)
          // Emits: (CMOVNE32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 9) {
            return Emit_197(N, X86::CMOVNE32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 3:i8, EFLAGS:i32)
          // Emits: (CMOVBE32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 3) {
            return Emit_197(N, X86::CMOVBE32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 0:i8, EFLAGS:i32)
          // Emits: (CMOVA32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 0) {
            return Emit_197(N, X86::CMOVA32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 7:i8, EFLAGS:i32)
          // Emits: (CMOVL32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 7) {
            return Emit_197(N, X86::CMOVL32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 6:i8, EFLAGS:i32)
          // Emits: (CMOVGE32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 6) {
            return Emit_197(N, X86::CMOVGE32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 8:i8, EFLAGS:i32)
          // Emits: (CMOVLE32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 8) {
            return Emit_197(N, X86::CMOVLE32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 5:i8, EFLAGS:i32)
          // Emits: (CMOVG32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 5) {
            return Emit_197(N, X86::CMOVG32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 15:i8, EFLAGS:i32)
          // Emits: (CMOVS32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 15) {
            return Emit_197(N, X86::CMOVS32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 12:i8, EFLAGS:i32)
          // Emits: (CMOVNS32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 12) {
            return Emit_197(N, X86::CMOVNS32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, 14:i8, EFLAGS:i32)
          // Emits: (CMOVP32rm:i32 GR32:i32:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 14) {
            return Emit_197(N, X86::CMOVP32rm, MVT::i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }
  SDOperand N2 = N.getOperand(2);
  if (isa<ConstantSDNode>(N2)) {
    int64_t CN0 = cast<ConstantSDNode>(N2)->getSignExtended();

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 2:i8, EFLAGS:i32)
    // Emits: (CMOVB32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 2) {
      return Emit_198(N, X86::CMOVB32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 1:i8, EFLAGS:i32)
    // Emits: (CMOVAE32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 1) {
      return Emit_198(N, X86::CMOVAE32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 4:i8, EFLAGS:i32)
    // Emits: (CMOVE32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 4) {
      return Emit_198(N, X86::CMOVE32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 9:i8, EFLAGS:i32)
    // Emits: (CMOVNE32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 9) {
      return Emit_198(N, X86::CMOVNE32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 3:i8, EFLAGS:i32)
    // Emits: (CMOVBE32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 3) {
      return Emit_198(N, X86::CMOVBE32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 0:i8, EFLAGS:i32)
    // Emits: (CMOVA32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 0) {
      return Emit_198(N, X86::CMOVA32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 7:i8, EFLAGS:i32)
    // Emits: (CMOVL32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 7) {
      return Emit_198(N, X86::CMOVL32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 6:i8, EFLAGS:i32)
    // Emits: (CMOVGE32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 6) {
      return Emit_198(N, X86::CMOVGE32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 8:i8, EFLAGS:i32)
    // Emits: (CMOVLE32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 8) {
      return Emit_198(N, X86::CMOVLE32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 5:i8, EFLAGS:i32)
    // Emits: (CMOVG32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 5) {
      return Emit_198(N, X86::CMOVG32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 15:i8, EFLAGS:i32)
    // Emits: (CMOVS32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 15) {
      return Emit_198(N, X86::CMOVS32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 12:i8, EFLAGS:i32)
    // Emits: (CMOVNS32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 12) {
      return Emit_198(N, X86::CMOVNS32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 14:i8, EFLAGS:i32)
    // Emits: (CMOVP32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 14) {
      return Emit_198(N, X86::CMOVP32rr, MVT::i32);
    }

    // Pattern: (X86cmov:i32 GR32:i32:$src1, GR32:i32:$src2, 11:i8, EFLAGS:i32)
    // Emits: (CMOVNP32rr:i32 GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 11) {
      return Emit_198(N, X86::CMOVNP32rr, MVT::i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_CMOV_i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::LOAD &&
      N1.hasOneUse() &&
      CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
    SDOperand Chain1 = N1.getOperand(0);
    if (Predicate_load(N1.Val)) {
      SDOperand N11 = N1.getOperand(1);
      SDOperand CPTmp0;
      SDOperand CPTmp1;
      SDOperand CPTmp2;
      SDOperand CPTmp3;
      if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
        SDOperand N2 = N.getOperand(2);
        if (isa<ConstantSDNode>(N2)) {
          int64_t CN0 = cast<ConstantSDNode>(N2)->getSignExtended();

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 2:i8, EFLAGS:i32)
          // Emits: (CMOVB64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 2) {
            return Emit_197(N, X86::CMOVB64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 1:i8, EFLAGS:i32)
          // Emits: (CMOVAE64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 1) {
            return Emit_197(N, X86::CMOVAE64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 4:i8, EFLAGS:i32)
          // Emits: (CMOVE64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 4) {
            return Emit_197(N, X86::CMOVE64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 9:i8, EFLAGS:i32)
          // Emits: (CMOVNE64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 9) {
            return Emit_197(N, X86::CMOVNE64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 3:i8, EFLAGS:i32)
          // Emits: (CMOVBE64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 3) {
            return Emit_197(N, X86::CMOVBE64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 0:i8, EFLAGS:i32)
          // Emits: (CMOVA64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 0) {
            return Emit_197(N, X86::CMOVA64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 7:i8, EFLAGS:i32)
          // Emits: (CMOVL64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 7) {
            return Emit_197(N, X86::CMOVL64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 6:i8, EFLAGS:i32)
          // Emits: (CMOVGE64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 6) {
            return Emit_197(N, X86::CMOVGE64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 8:i8, EFLAGS:i32)
          // Emits: (CMOVLE64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 8) {
            return Emit_197(N, X86::CMOVLE64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 5:i8, EFLAGS:i32)
          // Emits: (CMOVG64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 5) {
            return Emit_197(N, X86::CMOVG64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 15:i8, EFLAGS:i32)
          // Emits: (CMOVS64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 15) {
            return Emit_197(N, X86::CMOVS64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 12:i8, EFLAGS:i32)
          // Emits: (CMOVNS64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 12) {
            return Emit_197(N, X86::CMOVNS64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 14:i8, EFLAGS:i32)
          // Emits: (CMOVP64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 14) {
            return Emit_197(N, X86::CMOVP64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmov:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, 11:i8, EFLAGS:i32)
          // Emits: (CMOVNP64rm:i64 GR64:i64:$src1, addr:iPTR:$src2)
          // Pattern complexity = 27  cost = 1  size = 3
          if (CN0 == 11) {
            return Emit_197(N, X86::CMOVNP64rm, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }
  SDOperand N2 = N.getOperand(2);
  if (isa<ConstantSDNode>(N2)) {
    int64_t CN0 = cast<ConstantSDNode>(N2)->getSignExtended();

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 2:i8, EFLAGS:i32)
    // Emits: (CMOVB64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 2) {
      return Emit_198(N, X86::CMOVB64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 1:i8, EFLAGS:i32)
    // Emits: (CMOVAE64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 1) {
      return Emit_198(N, X86::CMOVAE64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 4:i8, EFLAGS:i32)
    // Emits: (CMOVE64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 4) {
      return Emit_198(N, X86::CMOVE64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 9:i8, EFLAGS:i32)
    // Emits: (CMOVNE64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 9) {
      return Emit_198(N, X86::CMOVNE64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 3:i8, EFLAGS:i32)
    // Emits: (CMOVBE64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 3) {
      return Emit_198(N, X86::CMOVBE64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 0:i8, EFLAGS:i32)
    // Emits: (CMOVA64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 0) {
      return Emit_198(N, X86::CMOVA64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 7:i8, EFLAGS:i32)
    // Emits: (CMOVL64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 7) {
      return Emit_198(N, X86::CMOVL64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 6:i8, EFLAGS:i32)
    // Emits: (CMOVGE64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 6) {
      return Emit_198(N, X86::CMOVGE64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 8:i8, EFLAGS:i32)
    // Emits: (CMOVLE64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 8) {
      return Emit_198(N, X86::CMOVLE64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 5:i8, EFLAGS:i32)
    // Emits: (CMOVG64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 5) {
      return Emit_198(N, X86::CMOVG64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 15:i8, EFLAGS:i32)
    // Emits: (CMOVS64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 15) {
      return Emit_198(N, X86::CMOVS64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 12:i8, EFLAGS:i32)
    // Emits: (CMOVNS64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 12) {
      return Emit_198(N, X86::CMOVNS64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 14:i8, EFLAGS:i32)
    // Emits: (CMOVP64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 14) {
      return Emit_198(N, X86::CMOVP64rr, MVT::i64);
    }

    // Pattern: (X86cmov:i64 GR64:i64:$src1, GR64:i64:$src2, 11:i8, EFLAGS:i32)
    // Emits: (CMOVNP64rr:i64 GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 11) {
      return Emit_198(N, X86::CMOVNP64rr, MVT::i64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_199(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N2)->getValue()), MVT::i8);
  SDOperand Chain = CurDAG->getEntryNode();
  AddToISelQueue(N3);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::EFLAGS, N3, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  SDOperand Ops0[] = { N0, N1, Tmp2, InFlag };
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, Ops0, 4);
}
SDNode *Select_X86ISD_CMOV_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    if (isa<ConstantSDNode>(N2)) {
      int64_t CN0 = cast<ConstantSDNode>(N2)->getSignExtended();

      // Pattern: (X86cmov:f32 RFP32:f32:$src1, RFP32:f32:$src2, 2:i8, EFLAGS:i32)
      // Emits: (CMOVB_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 2) {
        return Emit_198(N, X86::CMOVB_Fp32, MVT::f32);
      }

      // Pattern: (X86cmov:f32 RFP32:f32:$src1, RFP32:f32:$src2, 3:i8, EFLAGS:i32)
      // Emits: (CMOVBE_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 3) {
        return Emit_198(N, X86::CMOVBE_Fp32, MVT::f32);
      }

      // Pattern: (X86cmov:f32 RFP32:f32:$src1, RFP32:f32:$src2, 4:i8, EFLAGS:i32)
      // Emits: (CMOVE_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 4) {
        return Emit_198(N, X86::CMOVE_Fp32, MVT::f32);
      }

      // Pattern: (X86cmov:f32 RFP32:f32:$src1, RFP32:f32:$src2, 14:i8, EFLAGS:i32)
      // Emits: (CMOVP_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 14) {
        return Emit_198(N, X86::CMOVP_Fp32, MVT::f32);
      }

      // Pattern: (X86cmov:f32 RFP32:f32:$src1, RFP32:f32:$src2, 1:i8, EFLAGS:i32)
      // Emits: (CMOVNB_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 1) {
        return Emit_198(N, X86::CMOVNB_Fp32, MVT::f32);
      }

      // Pattern: (X86cmov:f32 RFP32:f32:$src1, RFP32:f32:$src2, 0:i8, EFLAGS:i32)
      // Emits: (CMOVNBE_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 0) {
        return Emit_198(N, X86::CMOVNBE_Fp32, MVT::f32);
      }

      // Pattern: (X86cmov:f32 RFP32:f32:$src1, RFP32:f32:$src2, 9:i8, EFLAGS:i32)
      // Emits: (CMOVNE_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 9) {
        return Emit_198(N, X86::CMOVNE_Fp32, MVT::f32);
      }

      // Pattern: (X86cmov:f32 RFP32:f32:$src1, RFP32:f32:$src2, 11:i8, EFLAGS:i32)
      // Emits: (CMOVNP_Fp32:f32 RFP32:f32:$src1, RFP32:f32:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 11) {
        return Emit_198(N, X86::CMOVNP_Fp32, MVT::f32);
      }
    }
  }

  // Pattern: (X86cmov:f32 FR32:f32:$t, FR32:f32:$f, (imm:i8):$cond, EFLAGS:i32)
  // Emits: (CMOV_FR32:f32 FR32:f32:$t, FR32:f32:$f, (imm:i8):$cond)
  // Pattern complexity = 6  cost = 11  size = 3
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  if (N2.getOpcode() == ISD::Constant) {
    return Emit_199(N, X86::CMOV_FR32, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_CMOV_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    if (isa<ConstantSDNode>(N2)) {
      int64_t CN0 = cast<ConstantSDNode>(N2)->getSignExtended();

      // Pattern: (X86cmov:f64 RFP64:f64:$src1, RFP64:f64:$src2, 2:i8, EFLAGS:i32)
      // Emits: (CMOVB_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 2) {
        return Emit_198(N, X86::CMOVB_Fp64, MVT::f64);
      }

      // Pattern: (X86cmov:f64 RFP64:f64:$src1, RFP64:f64:$src2, 3:i8, EFLAGS:i32)
      // Emits: (CMOVBE_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 3) {
        return Emit_198(N, X86::CMOVBE_Fp64, MVT::f64);
      }

      // Pattern: (X86cmov:f64 RFP64:f64:$src1, RFP64:f64:$src2, 4:i8, EFLAGS:i32)
      // Emits: (CMOVE_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 4) {
        return Emit_198(N, X86::CMOVE_Fp64, MVT::f64);
      }

      // Pattern: (X86cmov:f64 RFP64:f64:$src1, RFP64:f64:$src2, 14:i8, EFLAGS:i32)
      // Emits: (CMOVP_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 14) {
        return Emit_198(N, X86::CMOVP_Fp64, MVT::f64);
      }

      // Pattern: (X86cmov:f64 RFP64:f64:$src1, RFP64:f64:$src2, 1:i8, EFLAGS:i32)
      // Emits: (CMOVNB_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 1) {
        return Emit_198(N, X86::CMOVNB_Fp64, MVT::f64);
      }

      // Pattern: (X86cmov:f64 RFP64:f64:$src1, RFP64:f64:$src2, 0:i8, EFLAGS:i32)
      // Emits: (CMOVNBE_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 0) {
        return Emit_198(N, X86::CMOVNBE_Fp64, MVT::f64);
      }

      // Pattern: (X86cmov:f64 RFP64:f64:$src1, RFP64:f64:$src2, 9:i8, EFLAGS:i32)
      // Emits: (CMOVNE_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 9) {
        return Emit_198(N, X86::CMOVNE_Fp64, MVT::f64);
      }

      // Pattern: (X86cmov:f64 RFP64:f64:$src1, RFP64:f64:$src2, 11:i8, EFLAGS:i32)
      // Emits: (CMOVNP_Fp64:f64 RFP64:f64:$src1, RFP64:f64:$src2)
      // Pattern complexity = 8  cost = 1  size = 0
      if (CN0 == 11) {
        return Emit_198(N, X86::CMOVNP_Fp64, MVT::f64);
      }
    }
  }

  // Pattern: (X86cmov:f64 FR64:f64:$t, FR64:f64:$f, (imm:i8):$cond, EFLAGS:i32)
  // Emits: (CMOV_FR64:f64 FR64:f64:$t, FR64:f64:$f, (imm:i8):$cond)
  // Pattern complexity = 6  cost = 11  size = 3
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  if (N2.getOpcode() == ISD::Constant) {
    return Emit_199(N, X86::CMOV_FR64, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_CMOV_f80(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  if (isa<ConstantSDNode>(N2)) {
    int64_t CN0 = cast<ConstantSDNode>(N2)->getSignExtended();

    // Pattern: (X86cmov:f80 RFP80:f80:$src1, RFP80:f80:$src2, 2:i8, EFLAGS:i32)
    // Emits: (CMOVB_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
    // Pattern complexity = 8  cost = 1  size = 0
    if (CN0 == 2) {
      return Emit_198(N, X86::CMOVB_Fp80, MVT::f80);
    }

    // Pattern: (X86cmov:f80 RFP80:f80:$src1, RFP80:f80:$src2, 3:i8, EFLAGS:i32)
    // Emits: (CMOVBE_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
    // Pattern complexity = 8  cost = 1  size = 0
    if (CN0 == 3) {
      return Emit_198(N, X86::CMOVBE_Fp80, MVT::f80);
    }

    // Pattern: (X86cmov:f80 RFP80:f80:$src1, RFP80:f80:$src2, 4:i8, EFLAGS:i32)
    // Emits: (CMOVE_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
    // Pattern complexity = 8  cost = 1  size = 0
    if (CN0 == 4) {
      return Emit_198(N, X86::CMOVE_Fp80, MVT::f80);
    }

    // Pattern: (X86cmov:f80 RFP80:f80:$src1, RFP80:f80:$src2, 14:i8, EFLAGS:i32)
    // Emits: (CMOVP_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
    // Pattern complexity = 8  cost = 1  size = 0
    if (CN0 == 14) {
      return Emit_198(N, X86::CMOVP_Fp80, MVT::f80);
    }

    // Pattern: (X86cmov:f80 RFP80:f80:$src1, RFP80:f80:$src2, 1:i8, EFLAGS:i32)
    // Emits: (CMOVNB_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
    // Pattern complexity = 8  cost = 1  size = 0
    if (CN0 == 1) {
      return Emit_198(N, X86::CMOVNB_Fp80, MVT::f80);
    }

    // Pattern: (X86cmov:f80 RFP80:f80:$src1, RFP80:f80:$src2, 0:i8, EFLAGS:i32)
    // Emits: (CMOVNBE_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
    // Pattern complexity = 8  cost = 1  size = 0
    if (CN0 == 0) {
      return Emit_198(N, X86::CMOVNBE_Fp80, MVT::f80);
    }

    // Pattern: (X86cmov:f80 RFP80:f80:$src1, RFP80:f80:$src2, 9:i8, EFLAGS:i32)
    // Emits: (CMOVNE_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
    // Pattern complexity = 8  cost = 1  size = 0
    if (CN0 == 9) {
      return Emit_198(N, X86::CMOVNE_Fp80, MVT::f80);
    }

    // Pattern: (X86cmov:f80 RFP80:f80:$src1, RFP80:f80:$src2, 11:i8, EFLAGS:i32)
    // Emits: (CMOVNP_Fp80:f80 RFP80:f80:$src1, RFP80:f80:$src2)
    // Pattern complexity = 8  cost = 1  size = 0
    if (CN0 == 11) {
      return Emit_198(N, X86::CMOVNP_Fp80, MVT::f80);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_CMOV_v2i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  if (N2.getOpcode() == ISD::Constant) {
    return Emit_199(N, X86::CMOV_V2I64, MVT::v2i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_CMOV_v4f32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  if (N2.getOpcode() == ISD::Constant) {
    return Emit_199(N, X86::CMOV_V4F32, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_CMOV_v2f64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  if (N2.getOpcode() == ISD::Constant) {
    return Emit_199(N, X86::CMOV_V2F64, MVT::v2f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_200(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N01)->getValue()), MVT::i64);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_201(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N01)->getValue()), MVT::i8);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_202(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N01)->getValue()), MVT::i16);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_203(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N01)->getValue()), MVT::i32);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_204(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand Chain01 = N01.getOperand(0);
  SDOperand N011 = N01.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N00);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N01 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N01)->getMemOperand());
  AddToISelQueue(Chain01);
  SDOperand Ops0[] = { N00, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N01, Chain01 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain01 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N01.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_205(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N01);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_206(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N1)->getValue()), MVT::i16);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_207(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_208(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N1)->getValue()), MVT::i64);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_209(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N1)->getValue()), MVT::i8);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp1, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_210(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Chain0 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N1);
  SDOperand LSI_N0 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N0)->getMemOperand());
  AddToISelQueue(Chain0);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N1, LSI_N0, Chain0 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain0 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N0.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_211(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain1 = N1.getOperand(0);
  SDOperand N11 = N1.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N1 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N1)->getMemOperand());
  AddToISelQueue(Chain1);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N1, Chain1 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, MVT::Other, Ops0, 7);
  Chain1 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N1.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_212(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N00);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N01)->getValue()), MVT::i8);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N00, Tmp2);
  return ResNode;
}
SDNode *Emit_213(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N00);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N01)->getValue()), MVT::i16);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N00, Tmp2);
  return ResNode;
}
SDNode *Emit_214(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N00);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N01)->getValue()), MVT::i32);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N00, Tmp2);
  return ResNode;
}
SDNode *Emit_215(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N00);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N01)->getValue()), MVT::i64);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N00, Tmp2);
  return ResNode;
}
SDNode *Emit_216(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N01 = N0.getOperand(1);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N00);
  AddToISelQueue(N01);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N00, N01);
  return ResNode;
}
SDNode *Emit_217(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(N0);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N0, N0);
  return ResNode;
}
SDNode *Emit_218(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N1)->getValue()), MVT::i16);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N0, Tmp1);
  return ResNode;
}
SDNode *Emit_219(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N0, Tmp1);
  return ResNode;
}
SDNode *Emit_220(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((uint64_t) cast<ConstantSDNode>(N1)->getValue()), MVT::i64);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N0, Tmp1);
  return ResNode;
}
SDNode *Emit_221(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N1)->getValue()), MVT::i8);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N0, Tmp1);
  return ResNode;
}
SDNode *Emit_222(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::i32, N0, N1);
  return ResNode;
}
SDNode *Select_X86ISD_CMP(const SDOperand &N) {
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::AND &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_load(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N01 = N0.getOperand(1);
            if (N01.getOpcode() == ISD::Constant) {

              // Pattern: (X86cmp:void (and:i64 (ld:i64 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2), 0:i64)
              // Emits: (TEST64mi32:void addr:iPTR:$src1, (imm:i64):$src2)
              // Pattern complexity = 34  cost = 1  size = 3
              if (Predicate_i64immSExt32(N01.Val)) {
                SDOperand N1 = N.getOperand(1);
                if (isa<ConstantSDNode>(N1)) {
                  int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
                  if (CN0 == 0 &&
                      N0.Val->getValueType(0) == MVT::i64) {
                    return Emit_200(N, X86::TEST64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
              SDOperand N1 = N.getOperand(1);
              if (isa<ConstantSDNode>(N1)) {
                int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
                if (CN0 == 0) {

                  // Pattern: (X86cmp:void (and:i8 (ld:i8 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i8):$src2), 0:i8)
                  // Emits: (TEST8mi:void addr:iPTR:$src1, (imm:i8):$src2)
                  // Pattern complexity = 33  cost = 1  size = 3
                  if (N0.Val->getValueType(0) == MVT::i8) {
                    return Emit_201(N, X86::TEST8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (X86cmp:void (and:i16 (ld:i16 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i16):$src2), 0:i16)
                  // Emits: (TEST16mi:void addr:iPTR:$src1, (imm:i16):$src2)
                  // Pattern complexity = 33  cost = 1  size = 3
                  if (N0.Val->getValueType(0) == MVT::i16) {
                    return Emit_202(N, X86::TEST16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (X86cmp:void (and:i32 (ld:i32 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i32):$src2), 0:i32)
                  // Emits: (TEST32mi:void addr:iPTR:$src1, (imm:i32):$src2)
                  // Pattern complexity = 33  cost = 1  size = 3
                  if (N0.Val->getValueType(0) == MVT::i32) {
                    return Emit_203(N, X86::TEST32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      {
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::LOAD &&
            N01.hasOneUse() &&
            CanBeFoldedBy(N01.Val, N0.Val, N.Val)) {
          SDOperand Chain01 = N01.getOperand(0);
          if (Predicate_load(N01.Val)) {
            SDOperand N011 = N01.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N011, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              SDOperand N1 = N.getOperand(1);
              if (isa<ConstantSDNode>(N1)) {
                int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
                if (CN0 == 0) {

                  // Pattern: (X86cmp:void (and:i8 GR8:i8:$src1, (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>), 0:i8)
                  // Emits: (TEST8rm:void GR8:i8:$src1, addr:iPTR:$src2)
                  // Pattern complexity = 30  cost = 1  size = 3
                  if (N0.Val->getValueType(0) == MVT::i8) {
                    return Emit_204(N, X86::TEST8rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (X86cmp:void (and:i16 GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>), 0:i16)
                  // Emits: (TEST16rm:void GR16:i16:$src1, addr:iPTR:$src2)
                  // Pattern complexity = 30  cost = 1  size = 3
                  if (N0.Val->getValueType(0) == MVT::i16) {
                    return Emit_204(N, X86::TEST16rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (X86cmp:void (and:i32 GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>), 0:i32)
                  // Emits: (TEST32rm:void GR32:i32:$src1, addr:iPTR:$src2)
                  // Pattern complexity = 30  cost = 1  size = 3
                  if (N0.Val->getValueType(0) == MVT::i32) {
                    return Emit_204(N, X86::TEST32rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }

                  // Pattern: (X86cmp:void (and:i64 GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>), 0:i64)
                  // Emits: (TEST64rm:void GR64:i64:$src1, addr:iPTR:$src2)
                  // Pattern complexity = 30  cost = 1  size = 3
                  if (N0.Val->getValueType(0) == MVT::i64) {
                    return Emit_204(N, X86::TEST64rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                  }
                }
              }
            }
          }
        }
      }
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_load(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N01 = N0.getOperand(1);
            SDOperand N1 = N.getOperand(1);
            if (isa<ConstantSDNode>(N1)) {
              int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
              if (CN0 == 0) {

                // Pattern: (X86cmp:void (and:i8 (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>, GR8:i8:$src1), 0:i8)
                // Emits: (TEST8rm:void GR8:i8:$src1, addr:iPTR:$src2)
                // Pattern complexity = 30  cost = 1  size = 3
                if (N0.Val->getValueType(0) == MVT::i8) {
                  return Emit_205(N, X86::TEST8rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (X86cmp:void (and:i16 (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>, GR16:i16:$src1), 0:i16)
                // Emits: (TEST16rm:void GR16:i16:$src1, addr:iPTR:$src2)
                // Pattern complexity = 30  cost = 1  size = 3
                if (N0.Val->getValueType(0) == MVT::i16) {
                  return Emit_205(N, X86::TEST16rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (X86cmp:void (and:i32 (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>, GR32:i32:$src1), 0:i32)
                // Emits: (TEST32rm:void GR32:i32:$src1, addr:iPTR:$src2)
                // Pattern complexity = 30  cost = 1  size = 3
                if (N0.Val->getValueType(0) == MVT::i32) {
                  return Emit_205(N, X86::TEST32rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }

                // Pattern: (X86cmp:void (and:i64 (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>, GR64:i64:$src1), 0:i64)
                // Emits: (TEST64rm:void GR64:i64:$src1, addr:iPTR:$src2)
                // Pattern complexity = 30  cost = 1  size = 3
                if (N0.Val->getValueType(0) == MVT::i64) {
                  return Emit_205(N, X86::TEST64rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
                }
              }
            }
          }
        }
      }
    }
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N1.getOpcode() == ISD::Constant) {

            // Pattern: (X86cmp:void (ld:i16 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2)
            // Emits: (CMP16mi8:void addr:iPTR:$src1, (imm:i16):$src2)
            // Pattern complexity = 26  cost = 1  size = 3
            if (Predicate_i16immSExt8(N1.Val) &&
                N0.Val->getValueType(0) == MVT::i16) {
              return Emit_206(N, X86::CMP16mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (X86cmp:void (ld:i32 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
            // Emits: (CMP32mi8:void addr:iPTR:$src1, (imm:i32):$src2)
            // Pattern complexity = 26  cost = 1  size = 3
            if (Predicate_i32immSExt8(N1.Val) &&
                N0.Val->getValueType(0) == MVT::i32) {
              return Emit_207(N, X86::CMP32mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (X86cmp:void (ld:i64 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
            // Emits: (CMP64mi32:void addr:iPTR:$src1, (imm:i64):$src2)
            // Pattern complexity = 26  cost = 1  size = 3
            if (Predicate_i64immSExt32(N1.Val) &&
                N0.Val->getValueType(0) == MVT::i64) {
              return Emit_208(N, X86::CMP64mi32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (X86cmp:void (ld:i64 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
            // Emits: (CMP64mi8:void addr:iPTR:$src1, (imm:i64):$src2)
            // Pattern complexity = 26  cost = 1  size = 3
            if (Predicate_i64immSExt8(N1.Val) &&
                N0.Val->getValueType(0) == MVT::i64) {
              return Emit_208(N, X86::CMP64mi8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (X86cmp:void (ld:i8 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i8):$src2)
            // Emits: (CMP8mi:void addr:iPTR:$src1, (imm:i8):$src2)
            // Pattern complexity = 25  cost = 1  size = 3
            if (N0.Val->getValueType(0) == MVT::i8) {
              return Emit_209(N, X86::CMP8mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (X86cmp:void (ld:i16 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i16):$src2)
            // Emits: (CMP16mi:void addr:iPTR:$src1, (imm:i16):$src2)
            // Pattern complexity = 25  cost = 1  size = 3
            if (N0.Val->getValueType(0) == MVT::i16) {
              return Emit_206(N, X86::CMP16mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (X86cmp:void (ld:i32 addr:iPTR:$src1)<<P:Predicate_load>>, (imm:i32):$src2)
            // Emits: (CMP32mi:void addr:iPTR:$src1, (imm:i32):$src2)
            // Pattern complexity = 25  cost = 1  size = 3
            if (N0.Val->getValueType(0) == MVT::i32) {
              return Emit_207(N, X86::CMP32mi, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }

          // Pattern: (X86cmp:void (ld:i8 addr:iPTR:$src1)<<P:Predicate_load>>, GR8:i8:$src2)
          // Emits: (CMP8mr:void addr:iPTR:$src1, GR8:i8:$src2)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N0.Val->getValueType(0) == MVT::i8) {
            return Emit_210(N, X86::CMP8mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmp:void (ld:i16 addr:iPTR:$src1)<<P:Predicate_load>>, GR16:i16:$src2)
          // Emits: (CMP16mr:void addr:iPTR:$src1, GR16:i16:$src2)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N0.Val->getValueType(0) == MVT::i16) {
            return Emit_210(N, X86::CMP16mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }

          // Pattern: (X86cmp:void (ld:i32 addr:iPTR:$src1)<<P:Predicate_load>>, GR32:i32:$src2)
          // Emits: (CMP32mr:void addr:iPTR:$src1, GR32:i32:$src2)
          // Pattern complexity = 22  cost = 1  size = 3
          if (N0.Val->getValueType(0) == MVT::i32) {
            return Emit_210(N, X86::CMP32mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
    {
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

            // Pattern: (X86cmp:void GR8:i8:$src1, (ld:i8 addr:iPTR:$src2)<<P:Predicate_load>>)
            // Emits: (CMP8rm:void GR8:i8:$src1, addr:iPTR:$src2)
            // Pattern complexity = 22  cost = 1  size = 3
            if (N0.Val->getValueType(0) == MVT::i8) {
              return Emit_211(N, X86::CMP8rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (X86cmp:void GR16:i16:$src1, (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>)
            // Emits: (CMP16rm:void GR16:i16:$src1, addr:iPTR:$src2)
            // Pattern complexity = 22  cost = 1  size = 3
            if (N0.Val->getValueType(0) == MVT::i16) {
              return Emit_211(N, X86::CMP16rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }

            // Pattern: (X86cmp:void GR32:i32:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_load>>)
            // Emits: (CMP32rm:void GR32:i32:$src1, addr:iPTR:$src2)
            // Pattern complexity = 22  cost = 1  size = 3
            if (N0.Val->getValueType(0) == MVT::i32) {
              return Emit_211(N, X86::CMP32rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (X86cmp:void (ld:i64 addr:iPTR:$src1)<<P:Predicate_load>>, GR64:i64:$src2)
    // Emits: (CMP64mr:void addr:iPTR:$src1, GR64:i64:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse() &&
        CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N1 = N.getOperand(1);
          if (N0.Val->getValueType(0) == MVT::i64) {
            return Emit_210(N, X86::CMP64mr, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86cmp:void GR64:i64:$src1, (ld:i64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (CMP64rm:void GR64:i64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::i64) {
          return Emit_211(N, X86::CMP64rm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86cmp:void FR32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
  // Emits: (UCOMISSrm:void FR32:f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::f32) {
          return Emit_211(N, X86::UCOMISSrm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86cmp:void FR64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
  // Emits: (UCOMISDrm:void FR64:f64:$src1, addr:iPTR:$src2)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::f64) {
          return Emit_211(N, X86::UCOMISDrm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::AND) {
      if (Predicate_and_su(N0.Val)) {
        SDOperand N00 = N0.getOperand(0);
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::Constant) {
          SDOperand N1 = N.getOperand(1);
          if (isa<ConstantSDNode>(N1)) {
            int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
            if (CN0 == 0) {

              // Pattern: (X86cmp:void (and:i8 GR8:i8:$src1, (imm:i8):$src2)<<P:Predicate_and_su>>, 0:i8)
              // Emits: (TEST8ri:void GR8:i8:$src1, (imm:i8):$src2)
              // Pattern complexity = 15  cost = 1  size = 3
              if (N0.Val->getValueType(0) == MVT::i8) {
                return Emit_212(N, X86::TEST8ri);
              }

              // Pattern: (X86cmp:void (and:i16 GR16:i16:$src1, (imm:i16):$src2)<<P:Predicate_and_su>>, 0:i16)
              // Emits: (TEST16ri:void GR16:i16:$src1, (imm:i16):$src2)
              // Pattern complexity = 15  cost = 1  size = 3
              if (N0.Val->getValueType(0) == MVT::i16) {
                return Emit_213(N, X86::TEST16ri);
              }

              // Pattern: (X86cmp:void (and:i32 GR32:i32:$src1, (imm:i32):$src2)<<P:Predicate_and_su>>, 0:i32)
              // Emits: (TEST32ri:void GR32:i32:$src1, (imm:i32):$src2)
              // Pattern complexity = 15  cost = 1  size = 3
              if (N0.Val->getValueType(0) == MVT::i32) {
                return Emit_214(N, X86::TEST32ri);
              }
            }
          }
        }
      }

      // Pattern: (X86cmp:void (and:i64 GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2), 0:i64)
      // Emits: (TEST64ri32:void GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 15  cost = 1  size = 3
      {
        SDOperand N00 = N0.getOperand(0);
        SDOperand N01 = N0.getOperand(1);
        if (N01.getOpcode() == ISD::Constant &&
            Predicate_i64immSExt32(N01.Val)) {
          SDOperand N1 = N.getOperand(1);
          if (isa<ConstantSDNode>(N1)) {
            int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
            if (CN0 == 0 &&
                N0.Val->getValueType(0) == MVT::i64) {
              return Emit_215(N, X86::TEST64ri32);
            }
          }
        }
      }
      if (Predicate_and_su(N0.Val)) {
        SDOperand N00 = N0.getOperand(0);
        SDOperand N01 = N0.getOperand(1);
        SDOperand N1 = N.getOperand(1);
        if (isa<ConstantSDNode>(N1)) {
          int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
          if (CN0 == 0) {

            // Pattern: (X86cmp:void (and:i8 GR8:i8:$src1, GR8:i8:$src2)<<P:Predicate_and_su>>, 0:i8)
            // Emits: (TEST8rr:void GR8:i8:$src1, GR8:i8:$src2)
            // Pattern complexity = 12  cost = 1  size = 3
            if (N0.Val->getValueType(0) == MVT::i8) {
              return Emit_216(N, X86::TEST8rr);
            }

            // Pattern: (X86cmp:void (and:i16 GR16:i16:$src1, GR16:i16:$src2)<<P:Predicate_and_su>>, 0:i16)
            // Emits: (TEST16rr:void GR16:i16:$src1, GR16:i16:$src2)
            // Pattern complexity = 12  cost = 1  size = 3
            if (N0.Val->getValueType(0) == MVT::i16) {
              return Emit_216(N, X86::TEST16rr);
            }

            // Pattern: (X86cmp:void (and:i32 GR32:i32:$src1, GR32:i32:$src2)<<P:Predicate_and_su>>, 0:i32)
            // Emits: (TEST32rr:void GR32:i32:$src1, GR32:i32:$src2)
            // Pattern complexity = 12  cost = 1  size = 3
            if (N0.Val->getValueType(0) == MVT::i32) {
              return Emit_216(N, X86::TEST32rr);
            }
          }
        }
      }

      // Pattern: (X86cmp:void (and:i64 GR64:i64:$src1, GR64:i64:$src2), 0:i64)
      // Emits: (TEST64rr:void GR64:i64:$src1, GR64:i64:$src2)
      // Pattern complexity = 11  cost = 1  size = 3
      SDOperand N00 = N0.getOperand(0);
      SDOperand N01 = N0.getOperand(1);
      SDOperand N1 = N.getOperand(1);
      if (isa<ConstantSDNode>(N1)) {
        int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
        if (CN0 == 0 &&
            N0.Val->getValueType(0) == MVT::i64) {
          return Emit_216(N, X86::TEST64rr);
        }
      }
    }
    SDOperand N1 = N.getOperand(1);
    if (isa<ConstantSDNode>(N1)) {
      int64_t CN0 = cast<ConstantSDNode>(N1)->getSignExtended();
      if (CN0 == 0) {

        // Pattern: (X86cmp:void GR64:i64:$src1, 0:i64)
        // Emits: (TEST64rr:void GR64:i64:$src1, GR64:i64:$src1)
        // Pattern complexity = 8  cost = 1  size = 3
        if (N0.Val->getValueType(0) == MVT::i64) {
          return Emit_217(N, X86::TEST64rr);
        }

        // Pattern: (X86cmp:void GR8:i8:$src1, 0:i8)
        // Emits: (TEST8rr:void GR8:i8:$src1, GR8:i8:$src1)
        // Pattern complexity = 8  cost = 1  size = 3
        if (N0.Val->getValueType(0) == MVT::i8) {
          return Emit_217(N, X86::TEST8rr);
        }

        // Pattern: (X86cmp:void GR16:i16:$src1, 0:i16)
        // Emits: (TEST16rr:void GR16:i16:$src1, GR16:i16:$src1)
        // Pattern complexity = 8  cost = 1  size = 3
        if (N0.Val->getValueType(0) == MVT::i16) {
          return Emit_217(N, X86::TEST16rr);
        }

        // Pattern: (X86cmp:void GR32:i32:$src1, 0:i32)
        // Emits: (TEST32rr:void GR32:i32:$src1, GR32:i32:$src1)
        // Pattern complexity = 8  cost = 1  size = 3
        if (N0.Val->getValueType(0) == MVT::i32) {
          return Emit_217(N, X86::TEST32rr);
        }
      }
    }
    if (N1.getOpcode() == ISD::Constant) {

      // Pattern: (X86cmp:void GR16:i16:$src1, (imm:i16)<<P:Predicate_i16immSExt8>>:$src2)
      // Emits: (CMP16ri8:void GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i16immSExt8(N1.Val) &&
          N0.Val->getValueType(0) == MVT::i16) {
        return Emit_218(N, X86::CMP16ri8);
      }

      // Pattern: (X86cmp:void GR32:i32:$src1, (imm:i32)<<P:Predicate_i32immSExt8>>:$src2)
      // Emits: (CMP32ri8:void GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i32immSExt8(N1.Val) &&
          N0.Val->getValueType(0) == MVT::i32) {
        return Emit_219(N, X86::CMP32ri8);
      }

      // Pattern: (X86cmp:void GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt32>>:$src2)
      // Emits: (CMP64ri32:void GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt32(N1.Val) &&
          N0.Val->getValueType(0) == MVT::i64) {
        return Emit_220(N, X86::CMP64ri32);
      }

      // Pattern: (X86cmp:void GR64:i64:$src1, (imm:i64)<<P:Predicate_i64immSExt8>>:$src2)
      // Emits: (CMP64ri8:void GR64:i64:$src1, (imm:i64):$src2)
      // Pattern complexity = 7  cost = 1  size = 3
      if (Predicate_i64immSExt8(N1.Val) &&
          N0.Val->getValueType(0) == MVT::i64) {
        return Emit_220(N, X86::CMP64ri8);
      }

      // Pattern: (X86cmp:void GR8:i8:$src1, (imm:i8):$src2)
      // Emits: (CMP8ri:void GR8:i8:$src1, (imm:i8):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      if (N0.Val->getValueType(0) == MVT::i8) {
        return Emit_221(N, X86::CMP8ri);
      }

      // Pattern: (X86cmp:void GR16:i16:$src1, (imm:i16):$src2)
      // Emits: (CMP16ri:void GR16:i16:$src1, (imm:i16):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      if (N0.Val->getValueType(0) == MVT::i16) {
        return Emit_218(N, X86::CMP16ri);
      }

      // Pattern: (X86cmp:void GR32:i32:$src1, (imm:i32):$src2)
      // Emits: (CMP32ri:void GR32:i32:$src1, (imm:i32):$src2)
      // Pattern complexity = 6  cost = 1  size = 3
      if (N0.Val->getValueType(0) == MVT::i32) {
        return Emit_219(N, X86::CMP32ri);
      }
    }
  }

  // Pattern: (X86cmp:void RFP32:f32:$lhs, RFP32:f32:$rhs)
  // Emits: (UCOM_FpIr32:void RFP32:f32:$lhs, RFP32:f32:$rhs)
  // Pattern complexity = 3  cost = 1  size = 0
  if ((!Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N0.Val->getValueType(0) == MVT::f32) {
      return Emit_222(N, X86::UCOM_FpIr32);
    }
  }

  // Pattern: (X86cmp:void RFP64:f64:$lhs, RFP64:f64:$rhs)
  // Emits: (UCOM_FpIr64:void RFP64:f64:$lhs, RFP64:f64:$rhs)
  // Pattern complexity = 3  cost = 1  size = 0
  if ((!Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N0.Val->getValueType(0) == MVT::f64) {
      return Emit_222(N, X86::UCOM_FpIr64);
    }
  }
  {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (X86cmp:void RFP80:f80:$lhs, RFP80:f80:$rhs)
    // Emits: (UCOM_FpIr80:void RFP80:f80:$lhs, RFP80:f80:$rhs)
    // Pattern complexity = 3  cost = 1  size = 0
    if (N0.Val->getValueType(0) == MVT::f80) {
      return Emit_222(N, X86::UCOM_FpIr80);
    }

    // Pattern: (X86cmp:void GR8:i8:$src1, GR8:i8:$src2)
    // Emits: (CMP8rr:void GR8:i8:$src1, GR8:i8:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i8) {
      return Emit_222(N, X86::CMP8rr);
    }

    // Pattern: (X86cmp:void GR16:i16:$src1, GR16:i16:$src2)
    // Emits: (CMP16rr:void GR16:i16:$src1, GR16:i16:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i16) {
      return Emit_222(N, X86::CMP16rr);
    }

    // Pattern: (X86cmp:void GR32:i32:$src1, GR32:i32:$src2)
    // Emits: (CMP32rr:void GR32:i32:$src1, GR32:i32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i32) {
      return Emit_222(N, X86::CMP32rr);
    }

    // Pattern: (X86cmp:void GR64:i64:$src1, GR64:i64:$src2)
    // Emits: (CMP64rr:void GR64:i64:$src1, GR64:i64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    if (N0.Val->getValueType(0) == MVT::i64) {
      return Emit_222(N, X86::CMP64rr);
    }
  }

  // Pattern: (X86cmp:void FR32:f32:$src1, FR32:f32:$src2)
  // Emits: (UCOMISSrr:void FR32:f32:$src1, FR32:f32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N0.Val->getValueType(0) == MVT::f32) {
      return Emit_222(N, X86::UCOMISSrr);
    }
  }

  // Pattern: (X86cmp:void FR64:f64:$src1, FR64:f64:$src2)
  // Emits: (UCOMISDrr:void FR64:f64:$src1, FR64:f64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N0.Val->getValueType(0) == MVT::f64) {
      return Emit_222(N, X86::UCOMISDrr);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_COMI(const SDOperand &N) {

  // Pattern: (X86comi:void VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_load>>)
  // Emits: (Int_COMISSrm:void VR128:v4f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::v4f32) {
          return Emit_211(N, X86::Int_COMISSrm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86comi:void VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>)
  // Emits: (Int_COMISDrm:void VR128:v2f64:$src1, addr:iPTR:$src2)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::v2f64) {
          return Emit_211(N, X86::Int_COMISDrm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86comi:void VR128:v4f32:$src1, VR128:v4f32:$src2)
  // Emits: (Int_COMISSrr:void VR128:v4f32:$src1, VR128:v4f32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N0.Val->getValueType(0) == MVT::v4f32) {
      return Emit_222(N, X86::Int_COMISSrr);
    }
  }

  // Pattern: (X86comi:void VR128:v2f64:$src1, VR128:v2f64:$src2)
  // Emits: (Int_COMISDrr:void VR128:v2f64:$src1, VR128:v2f64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N0.Val->getValueType(0) == MVT::v2f64) {
      return Emit_222(N, X86::Int_COMISDrr);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_EH_RETURN(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.Val->getValueType(0) == MVT::i32) {
    return Emit_67(N, X86::EH_RETURN);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FAND_f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (X86fand:f32 FR32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (FsANDPSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::FsANDPSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (X86fand:f32 (ld:f32 addr:iPTR:$src2)<<P:Predicate_memop>>, FR32:f32:$src1)
      // Emits: (FsANDPSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::FsANDPSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fand:f32 FR32:f32:$src1, FR32:f32:$src2)
    // Emits: (FsANDPSrr:f32 FR32:f32:$src1, FR32:f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::FsANDPSrr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FAND_f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (X86fand:f64 FR64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (FsANDPDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::FsANDPDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (X86fand:f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_memop>>, FR64:f64:$src1)
      // Emits: (FsANDPDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::FsANDPDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fand:f64 FR64:f64:$src1, FR64:f64:$src2)
    // Emits: (FsANDPDrr:f64 FR64:f64:$src1, FR64:f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::FsANDPDrr, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FILD_f32(const SDOperand &N) {
  if ((!Subtarget->hasSSE1())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      SDOperand N2 = N.getOperand(2);

      // Pattern: (X86fild:f32 addr:iPTR:$src, i16:Other)
      // Emits: (ILD_Fp16m32:f32 addr:iPTR:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (cast<VTSDNode>(N2)->getVT() == MVT::i16) {
        return Emit_110(N, X86::ILD_Fp16m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fild:f32 addr:iPTR:$src, i32:Other)
      // Emits: (ILD_Fp32m32:f32 addr:iPTR:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (cast<VTSDNode>(N2)->getVT() == MVT::i32) {
        return Emit_110(N, X86::ILD_Fp32m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fild:f32 addr:iPTR:$src, i64:Other)
      // Emits: (ILD_Fp64m32:f32 addr:iPTR:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (cast<VTSDNode>(N2)->getVT() == MVT::i64) {
        return Emit_110(N, X86::ILD_Fp64m32, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FILD_f64(const SDOperand &N) {
  if ((!Subtarget->hasSSE2())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      SDOperand N2 = N.getOperand(2);

      // Pattern: (X86fild:f64 addr:iPTR:$src, i16:Other)
      // Emits: (ILD_Fp16m64:f64 addr:iPTR:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (cast<VTSDNode>(N2)->getVT() == MVT::i16) {
        return Emit_110(N, X86::ILD_Fp16m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fild:f64 addr:iPTR:$src, i32:Other)
      // Emits: (ILD_Fp32m64:f64 addr:iPTR:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (cast<VTSDNode>(N2)->getVT() == MVT::i32) {
        return Emit_110(N, X86::ILD_Fp32m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fild:f64 addr:iPTR:$src, i64:Other)
      // Emits: (ILD_Fp64m64:f64 addr:iPTR:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (cast<VTSDNode>(N2)->getVT() == MVT::i64) {
        return Emit_110(N, X86::ILD_Fp64m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FILD_f80(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    SDOperand N2 = N.getOperand(2);

    // Pattern: (X86fild:f80 addr:iPTR:$src, i16:Other)
    // Emits: (ILD_Fp16m80:f80 addr:iPTR:$src)
    // Pattern complexity = 18  cost = 1  size = 0
    if (cast<VTSDNode>(N2)->getVT() == MVT::i16) {
      return Emit_110(N, X86::ILD_Fp16m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fild:f80 addr:iPTR:$src, i32:Other)
    // Emits: (ILD_Fp32m80:f80 addr:iPTR:$src)
    // Pattern complexity = 18  cost = 1  size = 0
    if (cast<VTSDNode>(N2)->getVT() == MVT::i32) {
      return Emit_110(N, X86::ILD_Fp32m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fild:f80 addr:iPTR:$src, i64:Other)
    // Emits: (ILD_Fp64m80:f80 addr:iPTR:$src)
    // Pattern complexity = 18  cost = 1  size = 0
    if (cast<VTSDNode>(N2)->getVT() == MVT::i64) {
      return Emit_110(N, X86::ILD_Fp64m80, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_223(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, MVT::Flag, Ops0, 5);
  Chain = SDOperand(ResNode, 1);
  SDOperand InFlag(ResNode, 2);
  ReplaceUses(SDOperand(N.Val, 2), InFlag);
  ReplaceUses(SDOperand(N.Val, 1), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Select_X86ISD_FILD_FLAG_f64(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    SDOperand N2 = N.getOperand(2);
    if (cast<VTSDNode>(N2)->getVT() == MVT::i64) {
      return Emit_223(N, X86::ILD_Fp64m64, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FLD_f32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    SDOperand N2 = N.getOperand(2);
    if (cast<VTSDNode>(N2)->getVT() == MVT::f32) {
      return Emit_110(N, X86::LD_Fp32m, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FLD_f64(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    SDOperand N2 = N.getOperand(2);
    if (cast<VTSDNode>(N2)->getVT() == MVT::f64) {
      return Emit_110(N, X86::LD_Fp64m, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FLD_f80(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    SDOperand N2 = N.getOperand(2);
    if (cast<VTSDNode>(N2)->getVT() == MVT::f80) {
      return Emit_110(N, X86::LD_Fp80m, MVT::f80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FMAX_f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (X86fmax:f32 FR32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (MAXSSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MAXSSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fmax:f32 FR32:f32:$src1, FR32:f32:$src2)
    // Emits: (MAXSSrr:f32 FR32:f32:$src1, FR32:f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MAXSSrr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FMAX_f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (X86fmax:f64 FR64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (MAXSDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MAXSDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fmax:f64 FR64:f64:$src1, FR64:f64:$src2)
    // Emits: (MAXSDrr:f64 FR64:f64:$src1, FR64:f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MAXSDrr, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FMAX_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (X86fmax:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (MAXPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MAXPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fmax:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Emits: (MAXPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MAXPSrr, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FMAX_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (X86fmax:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (MAXPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MAXPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fmax:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Emits: (MAXPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MAXPDrr, MVT::v2f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FMIN_f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (X86fmin:f32 FR32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (MINSSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MINSSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fmin:f32 FR32:f32:$src1, FR32:f32:$src2)
    // Emits: (MINSSrr:f32 FR32:f32:$src1, FR32:f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MINSSrr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FMIN_f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (X86fmin:f64 FR64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_load>>)
    // Emits: (MINSDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_load(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MINSDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fmin:f64 FR64:f64:$src1, FR64:f64:$src2)
    // Emits: (MINSDrr:f64 FR64:f64:$src1, FR64:f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MINSDrr, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FMIN_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (X86fmin:v4f32 VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (MINPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MINPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fmin:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Emits: (MINPSrr:v4f32 VR128:v4f32:$src1, VR128:v4f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MINPSrr, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FMIN_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {

    // Pattern: (X86fmin:v2f64 VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_memop>>)
    // Emits: (MINPDrm:v2f64 VR128:v2f64:$src1, addr:iPTR:$src2)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      SDOperand N1 = N.getOperand(1);
      if (N1.getOpcode() == ISD::LOAD &&
          N1.hasOneUse() &&
          CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
        SDOperand Chain1 = N1.getOperand(0);
        if (Predicate_memop(N1.Val)) {
          SDOperand N11 = N1.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_0(N, X86::MINPDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fmin:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Emits: (MINPDrr:v2f64 VR128:v2f64:$src1, VR128:v2f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::MINPDrr, MVT::v2f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_224(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 5);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_X86ISD_FNSTCW16m(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_224(N, X86::FNSTCW16m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FOR_f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (X86for:f32 FR32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (FsORPSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::FsORPSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (X86for:f32 (ld:f32 addr:iPTR:$src2)<<P:Predicate_memop>>, FR32:f32:$src1)
      // Emits: (FsORPSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::FsORPSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86for:f32 FR32:f32:$src1, FR32:f32:$src2)
    // Emits: (FsORPSrr:f32 FR32:f32:$src1, FR32:f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::FsORPSrr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FOR_f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (X86for:f64 FR64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (FsORPDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::FsORPDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (X86for:f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_memop>>, FR64:f64:$src1)
      // Emits: (FsORPDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::FsORPDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86for:f64 FR64:f64:$src1, FR64:f64:$src2)
    // Emits: (FsORPDrr:f64 FR64:f64:$src1, FR64:f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::FsORPDrr, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_225(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N1);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N1, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 6);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_X86ISD_FP_TO_INT16_IN_MEM(const SDOperand &N) {
  if ((Subtarget->hasSSE3())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

      // Pattern: (X86fp_to_i16mem:void RFP32:f32:$src, addr:iPTR:$op)
      // Emits: (ISTT_Fp16m32:void addr:iPTR:$op, RFP32:f32:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f32) {
        return Emit_225(N, X86::ISTT_Fp16m32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fp_to_i16mem:void RFP64:f64:$src, addr:iPTR:$op)
      // Emits: (ISTT_Fp16m64:void addr:iPTR:$op, RFP64:f64:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f64) {
        return Emit_225(N, X86::ISTT_Fp16m64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fp_to_i16mem:void RFP80:f80:$src, addr:iPTR:$op)
      // Emits: (ISTT_Fp16m80:void addr:iPTR:$op, RFP80:f80:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f80) {
        return Emit_225(N, X86::ISTT_Fp16m80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

    // Pattern: (X86fp_to_i16mem:void RFP32:f32:$src, addr:iPTR:$dst)
    // Emits: (FP32_TO_INT16_IN_MEM:void addr:iPTR:$dst, RFP32:f32:$src)
    // Pattern complexity = 18  cost = 11  size = 3
    if (N1.Val->getValueType(0) == MVT::f32) {
      return Emit_225(N, X86::FP32_TO_INT16_IN_MEM, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fp_to_i16mem:void RFP64:f64:$src, addr:iPTR:$dst)
    // Emits: (FP64_TO_INT16_IN_MEM:void addr:iPTR:$dst, RFP64:f64:$src)
    // Pattern complexity = 18  cost = 11  size = 3
    if (N1.Val->getValueType(0) == MVT::f64) {
      return Emit_225(N, X86::FP64_TO_INT16_IN_MEM, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fp_to_i16mem:void RFP80:f80:$src, addr:iPTR:$dst)
    // Emits: (FP80_TO_INT16_IN_MEM:void addr:iPTR:$dst, RFP80:f80:$src)
    // Pattern complexity = 18  cost = 11  size = 3
    if (N1.Val->getValueType(0) == MVT::f80) {
      return Emit_225(N, X86::FP80_TO_INT16_IN_MEM, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FP_TO_INT32_IN_MEM(const SDOperand &N) {
  if ((Subtarget->hasSSE3())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

      // Pattern: (X86fp_to_i32mem:void RFP32:f32:$src, addr:iPTR:$op)
      // Emits: (ISTT_Fp32m32:void addr:iPTR:$op, RFP32:f32:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f32) {
        return Emit_225(N, X86::ISTT_Fp32m32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fp_to_i32mem:void RFP64:f64:$src, addr:iPTR:$op)
      // Emits: (ISTT_Fp32m64:void addr:iPTR:$op, RFP64:f64:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f64) {
        return Emit_225(N, X86::ISTT_Fp32m64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fp_to_i32mem:void RFP80:f80:$src, addr:iPTR:$op)
      // Emits: (ISTT_Fp32m80:void addr:iPTR:$op, RFP80:f80:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f80) {
        return Emit_225(N, X86::ISTT_Fp32m80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

    // Pattern: (X86fp_to_i32mem:void RFP32:f32:$src, addr:iPTR:$dst)
    // Emits: (FP32_TO_INT32_IN_MEM:void addr:iPTR:$dst, RFP32:f32:$src)
    // Pattern complexity = 18  cost = 11  size = 3
    if (N1.Val->getValueType(0) == MVT::f32) {
      return Emit_225(N, X86::FP32_TO_INT32_IN_MEM, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fp_to_i32mem:void RFP64:f64:$src, addr:iPTR:$dst)
    // Emits: (FP64_TO_INT32_IN_MEM:void addr:iPTR:$dst, RFP64:f64:$src)
    // Pattern complexity = 18  cost = 11  size = 3
    if (N1.Val->getValueType(0) == MVT::f64) {
      return Emit_225(N, X86::FP64_TO_INT32_IN_MEM, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fp_to_i32mem:void RFP80:f80:$src, addr:iPTR:$dst)
    // Emits: (FP80_TO_INT32_IN_MEM:void addr:iPTR:$dst, RFP80:f80:$src)
    // Pattern complexity = 18  cost = 11  size = 3
    if (N1.Val->getValueType(0) == MVT::f80) {
      return Emit_225(N, X86::FP80_TO_INT32_IN_MEM, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FP_TO_INT64_IN_MEM(const SDOperand &N) {
  if ((Subtarget->hasSSE3())) {
    SDOperand Chain = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    SDOperand N2 = N.getOperand(2);
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

      // Pattern: (X86fp_to_i64mem:void RFP32:f32:$src, addr:iPTR:$op)
      // Emits: (ISTT_Fp64m32:void addr:iPTR:$op, RFP32:f32:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f32) {
        return Emit_225(N, X86::ISTT_Fp64m32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fp_to_i64mem:void RFP64:f64:$src, addr:iPTR:$op)
      // Emits: (ISTT_Fp64m64:void addr:iPTR:$op, RFP64:f64:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f64) {
        return Emit_225(N, X86::ISTT_Fp64m64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fp_to_i64mem:void RFP80:f80:$src, addr:iPTR:$op)
      // Emits: (ISTT_Fp64m80:void addr:iPTR:$op, RFP80:f80:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f80) {
        return Emit_225(N, X86::ISTT_Fp64m80, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {

    // Pattern: (X86fp_to_i64mem:void RFP32:f32:$src, addr:iPTR:$dst)
    // Emits: (FP32_TO_INT64_IN_MEM:void addr:iPTR:$dst, RFP32:f32:$src)
    // Pattern complexity = 18  cost = 11  size = 3
    if (N1.Val->getValueType(0) == MVT::f32) {
      return Emit_225(N, X86::FP32_TO_INT64_IN_MEM, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fp_to_i64mem:void RFP64:f64:$src, addr:iPTR:$dst)
    // Emits: (FP64_TO_INT64_IN_MEM:void addr:iPTR:$dst, RFP64:f64:$src)
    // Pattern complexity = 18  cost = 11  size = 3
    if (N1.Val->getValueType(0) == MVT::f64) {
      return Emit_225(N, X86::FP64_TO_INT64_IN_MEM, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fp_to_i64mem:void RFP80:f80:$src, addr:iPTR:$dst)
    // Emits: (FP80_TO_INT64_IN_MEM:void addr:iPTR:$dst, RFP80:f80:$src)
    // Pattern complexity = 18  cost = 11  size = 3
    if (N1.Val->getValueType(0) == MVT::f80) {
      return Emit_225(N, X86::FP80_TO_INT64_IN_MEM, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FRCP_f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (X86frcp:f32 (ld:f32 addr:iPTR:$src)<<P:Predicate_load>>)
    // Emits: (RCPSSm:f32 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse()) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_load(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_58(N, X86::RCPSSm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86frcp:f32 FR32:f32:$src)
    // Emits: (RCPSSr:f32 FR32:f32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_59(N, X86::RCPSSr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FRCP_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (X86frcp:v4f32 (ld:v4f32 addr:iPTR:$src)<<P:Predicate_memop>>)
    // Emits: (RCPPSm:v4f32 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse()) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_58(N, X86::RCPPSm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86frcp:v4f32 VR128:v4f32:$src)
    // Emits: (RCPPSr:v4f32 VR128:v4f32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_59(N, X86::RCPPSr, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FRSQRT_f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (X86frsqrt:f32 (ld:f32 addr:iPTR:$src)<<P:Predicate_load>>)
    // Emits: (RSQRTSSm:f32 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse()) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_load(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_58(N, X86::RSQRTSSm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86frsqrt:f32 FR32:f32:$src)
    // Emits: (RSQRTSSr:f32 FR32:f32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_59(N, X86::RSQRTSSr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FRSQRT_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {

    // Pattern: (X86frsqrt:v4f32 (ld:v4f32 addr:iPTR:$src)<<P:Predicate_memop>>)
    // Emits: (RSQRTPSm:v4f32 addr:iPTR:$src)
    // Pattern complexity = 22  cost = 1  size = 3
    {
      SDOperand N0 = N.getOperand(0);
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse()) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_58(N, X86::RSQRTPSm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86frsqrt:v4f32 VR128:v4f32:$src)
    // Emits: (RSQRTPSr:v4f32 VR128:v4f32:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_59(N, X86::RSQRTPSr, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_226(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(N0);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N1)->getValue()), MVT::i32);
  SDOperand Tmp2 = Transform_PSxLDQ_imm(Tmp1.Val);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, Tmp2);
}
SDNode *Select_X86ISD_FSRL_v2f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant &&
        Predicate_i32immSExt8(N1.Val) &&
        N1.Val->getValueType(0) == MVT::i32) {
      return Emit_226(N, X86::PSRLDQri, MVT::v2f64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_227(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N1);
  AddToISelQueue(Chain);
  SDOperand InFlag = N.getOperand(4);
  AddToISelQueue(InFlag);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N1, Chain, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_X86ISD_FST(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N2, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    SDOperand N3 = N.getOperand(3);
    if (cast<VTSDNode>(N3)->getVT() == MVT::f32) {

      // Pattern: (X86fst:void RFP32:f32:$src, addr:iPTR:$op, f32:Other)
      // Emits: (ST_Fp32m:void addr:iPTR:$op, RFP32:f32:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f32) {
        return Emit_227(N, X86::ST_Fp32m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86fst:void RFP64:f64:$src, addr:iPTR:$op, f32:Other)
      // Emits: (ST_Fp64m32:void addr:iPTR:$op, RFP64:f64:$src)
      // Pattern complexity = 18  cost = 1  size = 0
      if (N1.Val->getValueType(0) == MVT::f64) {
        return Emit_227(N, X86::ST_Fp64m32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }

    // Pattern: (X86fst:void RFP64:f64:$src, addr:iPTR:$op, f64:Other)
    // Emits: (ST_Fp64m:void addr:iPTR:$op, RFP64:f64:$src)
    // Pattern complexity = 18  cost = 1  size = 0
    if (cast<VTSDNode>(N3)->getVT() == MVT::f64 &&
        N1.Val->getValueType(0) == MVT::f64) {
      return Emit_227(N, X86::ST_Fp64m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fst:void RFP80:f80:$src, addr:iPTR:$op, f32:Other)
    // Emits: (ST_Fp80m32:void addr:iPTR:$op, RFP80:f80:$src)
    // Pattern complexity = 18  cost = 1  size = 0
    if (cast<VTSDNode>(N3)->getVT() == MVT::f32 &&
        N1.Val->getValueType(0) == MVT::f80) {
      return Emit_227(N, X86::ST_Fp80m32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fst:void RFP80:f80:$src, addr:iPTR:$op, f64:Other)
    // Emits: (ST_Fp80m64:void addr:iPTR:$op, RFP80:f80:$src)
    // Pattern complexity = 18  cost = 1  size = 0
    if (cast<VTSDNode>(N3)->getVT() == MVT::f64 &&
        N1.Val->getValueType(0) == MVT::f80) {
      return Emit_227(N, X86::ST_Fp80m64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }

    // Pattern: (X86fst:void RFP80:f80:$src, addr:iPTR:$op, f80:Other)
    // Emits: (ST_FpP80m:void addr:iPTR:$op, RFP80:f80:$src)
    // Pattern complexity = 18  cost = 1  size = 0
    if (cast<VTSDNode>(N3)->getVT() == MVT::f80 &&
        N1.Val->getValueType(0) == MVT::f80) {
      return Emit_227(N, X86::ST_FpP80m, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FXOR_f32(const SDOperand &N) {
  if ((Subtarget->hasSSE1())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (X86fxor:f32 FR32:f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (FsXORPSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::FsXORPSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (X86fxor:f32 (ld:f32 addr:iPTR:$src2)<<P:Predicate_memop>>, FR32:f32:$src1)
      // Emits: (FsXORPSrm:f32 FR32:f32:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::FsXORPSrm, MVT::f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fxor:f32 FR32:f32:$src1, FR32:f32:$src2)
    // Emits: (FsXORPSrr:f32 FR32:f32:$src1, FR32:f32:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::FsXORPSrr, MVT::f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_FXOR_f64(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (X86fxor:f64 FR64:f64:$src1, (ld:f64 addr:iPTR:$src2)<<P:Predicate_memop>>)
      // Emits: (FsXORPDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      {
        SDOperand N1 = N.getOperand(1);
        if (N1.getOpcode() == ISD::LOAD &&
            N1.hasOneUse() &&
            CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
          SDOperand Chain1 = N1.getOperand(0);
          if (Predicate_memop(N1.Val)) {
            SDOperand N11 = N1.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
              return Emit_0(N, X86::FsXORPDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (X86fxor:f64 (ld:f64 addr:iPTR:$src2)<<P:Predicate_memop>>, FR64:f64:$src1)
      // Emits: (FsXORPDrm:f64 FR64:f64:$src1, addr:iPTR:$src2)
      // Pattern complexity = 22  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse() &&
          CanBeFoldedBy(N0.Val, N.Val, N.Val)) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_memop(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_1(N, X86::FsXORPDrm, MVT::f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86fxor:f64 FR64:f64:$src1, FR64:f64:$src2)
    // Emits: (FsXORPDrr:f64 FR64:f64:$src1, FR64:f64:$src2)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_4(N, X86::FsXORPDrr, MVT::f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_INSERTPS_v4f32(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (X86insrtps:v4f32 VR128:v4f32:$src1, (ld:f32 addr:iPTR:$src2)<<P:Predicate_load>>, (imm:iPTR):$src3)
    // Emits: (INSERTPSrm:v4f32 VR128:v4f32:$src1, addr:iPTR:$src2, (imm:i32):$src3)
    // Pattern complexity = 25  cost = 1  size = 3
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::Constant) {
            return Emit_81(N, X86::INSERTPSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86insrtps:v4f32 VR128:v4f32:$src1, FR32:f32:$src2, (imm:iPTR):$src3)
    // Emits: (INSERTPSrr:v4f32 VR128:v4f32:$src1, FR32:f32:$src2, (imm:i32):$src3)
    // Pattern complexity = 6  cost = 1  size = 3
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant) {
      return Emit_82(N, X86::INSERTPSrr, MVT::v4f32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_228(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain);
  SDOperand InFlag = N.getOperand(2);
  AddToISelQueue(InFlag);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Ops0, 6);
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Select_X86ISD_LCMPXCHG8_DAG(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_228(N, X86::LCMPXCHG8B, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_229(const SDOperand &N, unsigned Opc0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  SDOperand N3 = N.getOperand(3);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(N2);
  AddToISelQueue(Chain);
  SDOperand InFlag = N.getOperand(4);
  AddToISelQueue(InFlag);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, N2, Chain, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Ops0, 7);
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Select_X86ISD_LCMPXCHG_DAG(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    SDOperand N2 = N.getOperand(2);
    SDOperand N3 = N.getOperand(3);
    if (isa<ConstantSDNode>(N3)) {
      int64_t CN0 = cast<ConstantSDNode>(N3)->getSignExtended();

      // Pattern: (X86cas:void addr:iPTR:$ptr, GR32:i32:$swap, 4:i8)
      // Emits: (LCMPXCHG32:void addr:iPTR:$ptr, GR32:i32:$swap)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 4 &&
          N2.Val->getValueType(0) == MVT::i32) {
        return Emit_229(N, X86::LCMPXCHG32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86cas:void addr:iPTR:$ptr, GR16:i16:$swap, 2:i8)
      // Emits: (LCMPXCHG16:void addr:iPTR:$ptr, GR16:i16:$swap)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 2 &&
          N2.Val->getValueType(0) == MVT::i16) {
        return Emit_229(N, X86::LCMPXCHG16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86cas:void addr:iPTR:$ptr, GR8:i8:$swap, 1:i8)
      // Emits: (LCMPXCHG8:void addr:iPTR:$ptr, GR8:i8:$swap)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 1 &&
          N2.Val->getValueType(0) == MVT::i8) {
        return Emit_229(N, X86::LCMPXCHG8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }

      // Pattern: (X86cas:void addr:iPTR:$ptr, GR64:i64:$swap, 8:i8)
      // Emits: (LCMPXCHG64:void addr:iPTR:$ptr, GR64:i64:$swap)
      // Pattern complexity = 23  cost = 1  size = 3
      if (CN0 == 8 &&
          N2.Val->getValueType(0) == MVT::i64) {
        return Emit_229(N, X86::LCMPXCHG64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_PEXTRB_i32(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant &&
        N0.Val->getValueType(0) == MVT::v16i8) {
      return Emit_11(N, X86::PEXTRBrr, MVT::i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_PEXTRW_i32(const SDOperand &N) {

  // Pattern: (X86pextrw:i32 VR128:v8i16:$src1, (imm:iPTR):$src2)
  // Emits: (PEXTRWri:i32 VR128:v8i16:$src1, (imm:i32):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant &&
        N0.Val->getValueType(0) == MVT::v8i16) {
      return Emit_11(N, X86::PEXTRWri, MVT::i32);
    }
  }

  // Pattern: (MMX_X86pextrw:i32 VR64:v4i16:$src1, (imm:iPTR):$src2)
  // Emits: (MMX_PEXTRWri:i32 VR64:v4i16:$src1, (imm:i16):$src2)
  // Pattern complexity = 6  cost = 1  size = 3
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::Constant &&
        N0.Val->getValueType(0) == MVT::v4i16 &&
        N1.Val->getValueType(0) == TLI.getPointerTy()) {
      return Emit_5(N, X86::MMX_PEXTRWri, MVT::i32);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_PINSRB_v16i8(const SDOperand &N) {
  if ((Subtarget->hasSSE41())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (X86pinsrb:v16i8 VR128:v16i8:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_extloadi8>>, (imm:iPTR):$src3)
    // Emits: (PINSRBrm:v16i8 VR128:v16i8:$src1, addr:iPTR:$src2, (imm:i32):$src3)
    // Pattern complexity = 25  cost = 1  size = 3
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_extloadi8(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::Constant) {
            return Emit_81(N, X86::PINSRBrm, MVT::v16i8, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86pinsrb:v16i8 VR128:v16i8:$src1, GR32:i32:$src2, (imm:iPTR):$src3)
    // Emits: (PINSRBrr:v16i8 VR128:v16i8:$src1, GR32:i32:$src2, (imm:i32):$src3)
    // Pattern complexity = 6  cost = 1  size = 3
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant) {
      return Emit_82(N, X86::PINSRBrr, MVT::v16i8);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_230(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N10 = N1.getOperand(0);
  SDOperand Chain10 = N10.getOperand(0);
  SDOperand N101 = N10.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N2)->getValue()), MVT::i16);
  SDOperand LSI_N10 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N10)->getMemOperand());
  AddToISelQueue(Chain10);
  SDOperand Ops0[] = { N0, CPTmp0, CPTmp1, CPTmp2, CPTmp3, Tmp2, LSI_N10, Chain10 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 8);
  Chain10 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N10.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Emit_231(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N2)->getValue()), MVT::i16);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N1, Tmp2);
}
SDNode *Select_X86ISD_PINSRW_v4i16(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (MMX_X86pinsrw:v4i16 VR64:v4i16:$src1, (anyext:i32 (ld:i16 addr:iPTR:$src2)<<P:Predicate_load>>), (imm:iPTR):$src3)
    // Emits: (MMX_PINSRWrmi:v4i16 VR64:v4i16:$src1, addr:iPTR:$src2, (imm:i16):$src3)
    // Pattern complexity = 28  cost = 1  size = 3
    if (N1.getOpcode() == ISD::ANY_EXTEND &&
        N1.hasOneUse()) {
      SDOperand N10 = N1.getOperand(0);
      if (N10.getOpcode() == ISD::LOAD &&
          N10.hasOneUse() &&
          CanBeFoldedBy(N10.Val, N1.Val, N.Val)) {
        SDOperand Chain10 = N10.getOperand(0);
        if (Predicate_load(N10.Val)) {
          SDOperand N101 = N10.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N101, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            SDOperand N2 = N.getOperand(2);
            if (N2.getOpcode() == ISD::Constant &&
                N0.Val->getValueType(0) == MVT::v4i16 &&
                N1.Val->getValueType(0) == MVT::i32 &&
                N10.Val->getValueType(0) == MVT::i16 &&
                N2.Val->getValueType(0) == TLI.getPointerTy()) {
              return Emit_230(N, X86::MMX_PINSRWrmi, MVT::v4i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }
    }

    // Pattern: (MMX_X86pinsrw:v4i16 VR64:v4i16:$src1, GR32:i32:$src2, (imm:iPTR):$src3)
    // Emits: (MMX_PINSRWrri:v4i16 VR64:v4i16:$src1, GR32:i32:$src2, (imm:i16):$src3)
    // Pattern complexity = 6  cost = 1  size = 3
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant &&
        N0.Val->getValueType(0) == MVT::v4i16 &&
        N1.Val->getValueType(0) == MVT::i32 &&
        N2.Val->getValueType(0) == TLI.getPointerTy()) {
      return Emit_231(N, X86::MMX_PINSRWrri, MVT::v4i16);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_PINSRW_v8i16(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);

    // Pattern: (X86pinsrw:v8i16 VR128:v8i16:$src1, (ld:i32 addr:iPTR:$src2)<<P:Predicate_extloadi16>>, (imm:iPTR):$src3)
    // Emits: (PINSRWrmi:v8i16 VR128:v8i16:$src1, addr:iPTR:$src2, (imm:i32):$src3)
    // Pattern complexity = 25  cost = 1  size = 3
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_extloadi16(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          SDOperand N2 = N.getOperand(2);
          if (N2.getOpcode() == ISD::Constant) {
            return Emit_81(N, X86::PINSRWrmi, MVT::v8i16, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86pinsrw:v8i16 VR128:v8i16:$src1, GR32:i32:$src2, (imm:iPTR):$src3)
    // Emits: (PINSRWrri:v8i16 VR128:v8i16:$src1, GR32:i32:$src2, (imm:i32):$src3)
    // Pattern complexity = 6  cost = 1  size = 3
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant) {
      return Emit_82(N, X86::PINSRWrri, MVT::v8i16);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_232(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  AddToISelQueue(Chain);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Chain);
  Chain = SDOperand(ResNode, 0);
  SDOperand InFlag(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Select_X86ISD_RDTSC_DAG(const SDOperand &N) {
  return Emit_232(N, X86::RDTSC);
}

SDNode *Emit_233(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(Chain);
  SDOperand InFlag = N.getOperand(2);
  AddToISelQueue(InFlag);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Chain, InFlag);
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Select_X86ISD_REP_MOVS(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (X86rep_movs:void i8:Other)
  // Emits: (REP_MOVSB:void)
  // Pattern complexity = 3  cost = 1  size = 3
  if (cast<VTSDNode>(N1)->getVT() == MVT::i8) {
    return Emit_233(N, X86::REP_MOVSB);
  }

  // Pattern: (X86rep_movs:void i16:Other)
  // Emits: (REP_MOVSW:void)
  // Pattern complexity = 3  cost = 1  size = 3
  if (cast<VTSDNode>(N1)->getVT() == MVT::i16) {
    return Emit_233(N, X86::REP_MOVSW);
  }

  // Pattern: (X86rep_movs:void i32:Other)
  // Emits: (REP_MOVSD:void)
  // Pattern complexity = 3  cost = 1  size = 3
  if (cast<VTSDNode>(N1)->getVT() == MVT::i32) {
    return Emit_233(N, X86::REP_MOVSD);
  }

  // Pattern: (X86rep_movs:void i64:Other)
  // Emits: (REP_MOVSQ:void)
  // Pattern complexity = 3  cost = 1  size = 3
  if (cast<VTSDNode>(N1)->getVT() == MVT::i64) {
    return Emit_233(N, X86::REP_MOVSQ);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_REP_STOS(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (X86rep_stos:void i8:Other)
  // Emits: (REP_STOSB:void)
  // Pattern complexity = 3  cost = 1  size = 3
  if (cast<VTSDNode>(N1)->getVT() == MVT::i8) {
    return Emit_233(N, X86::REP_STOSB);
  }

  // Pattern: (X86rep_stos:void i16:Other)
  // Emits: (REP_STOSW:void)
  // Pattern complexity = 3  cost = 1  size = 3
  if (cast<VTSDNode>(N1)->getVT() == MVT::i16) {
    return Emit_233(N, X86::REP_STOSW);
  }

  // Pattern: (X86rep_stos:void i32:Other)
  // Emits: (REP_STOSD:void)
  // Pattern complexity = 3  cost = 1  size = 3
  if (cast<VTSDNode>(N1)->getVT() == MVT::i32) {
    return Emit_233(N, X86::REP_STOSD);
  }

  // Pattern: (X86rep_stos:void i64:Other)
  // Emits: (REP_STOSQ:void)
  // Pattern complexity = 3  cost = 1  size = 3
  if (cast<VTSDNode>(N1)->getVT() == MVT::i64) {
    return Emit_233(N, X86::REP_STOSQ);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_234(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  bool HasInFlag = (N.getOperand(N.getNumOperands()-1).getValueType() == MVT::Flag);
  SmallVector<SDOperand, 8> Ops0;
  SDOperand Tmp0 = CurDAG->getTargetConstant(((unsigned short) cast<ConstantSDNode>(N1)->getValue()), MVT::i16);
  AddToISelQueue(Chain);
  SDOperand InFlag(0, 0);
  if (HasInFlag) {
    InFlag = N.getOperand(N.getNumOperands()-1);
    AddToISelQueue(InFlag);
  }
  Ops0.push_back(Tmp0);
  for (unsigned i = 2, e = N.getNumOperands()-(HasInFlag?1:0); i != e; ++i) {
    AddToISelQueue(N.getOperand(i));
    Ops0.push_back(N.getOperand(i));
  }
  Ops0.push_back(Chain);
  if (HasInFlag)
    Ops0.push_back(InFlag);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, &Ops0[0], Ops0.size());
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_X86ISD_RET_FLAG(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::Constant) {
    return Emit_234(N, X86::RETI);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_235(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand Chain = CurDAG->getEntryNode();
  AddToISelQueue(N1);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::EFLAGS, N1, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, InFlag);
}
SDNode *Select_X86ISD_SETCC_i8(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  if (isa<ConstantSDNode>(N0)) {
    int64_t CN0 = cast<ConstantSDNode>(N0)->getSignExtended();

    // Pattern: (X86setcc:i8 4:i8, EFLAGS:i32)
    // Emits: (SETEr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 4) {
      return Emit_235(N, X86::SETEr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 9:i8, EFLAGS:i32)
    // Emits: (SETNEr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 9) {
      return Emit_235(N, X86::SETNEr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 7:i8, EFLAGS:i32)
    // Emits: (SETLr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 7) {
      return Emit_235(N, X86::SETLr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 6:i8, EFLAGS:i32)
    // Emits: (SETGEr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 6) {
      return Emit_235(N, X86::SETGEr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 8:i8, EFLAGS:i32)
    // Emits: (SETLEr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 8) {
      return Emit_235(N, X86::SETLEr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 5:i8, EFLAGS:i32)
    // Emits: (SETGr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 5) {
      return Emit_235(N, X86::SETGr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 2:i8, EFLAGS:i32)
    // Emits: (SETBr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 2) {
      return Emit_235(N, X86::SETBr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 1:i8, EFLAGS:i32)
    // Emits: (SETAEr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 1) {
      return Emit_235(N, X86::SETAEr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 3:i8, EFLAGS:i32)
    // Emits: (SETBEr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 3) {
      return Emit_235(N, X86::SETBEr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 0:i8, EFLAGS:i32)
    // Emits: (SETAr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 0) {
      return Emit_235(N, X86::SETAr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 15:i8, EFLAGS:i32)
    // Emits: (SETSr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 15) {
      return Emit_235(N, X86::SETSr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 12:i8, EFLAGS:i32)
    // Emits: (SETNSr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 12) {
      return Emit_235(N, X86::SETNSr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 14:i8, EFLAGS:i32)
    // Emits: (SETPr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 14) {
      return Emit_235(N, X86::SETPr, MVT::i8);
    }

    // Pattern: (X86setcc:i8 11:i8, EFLAGS:i32)
    // Emits: (SETNPr:i8)
    // Pattern complexity = 8  cost = 1  size = 3
    if (CN0 == 11) {
      return Emit_235(N, X86::SETNPr, MVT::i8);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_236(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  SDOperand Tmp2 = CurDAG->getTargetConstant(((unsigned char) cast<ConstantSDNode>(N2)->getValue()), MVT::i8);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N1, Tmp2);
}
SDNode *Emit_237(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  AddToISelQueue(N0);
  AddToISelQueue(N1);
  SDOperand Chain = CurDAG->getEntryNode();
  AddToISelQueue(N2);
  SDOperand InFlag(0, 0);
  SDNode *ResNode = CurDAG->getCopyToReg(Chain, X86::CL, N2, InFlag).Val;
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0, N1, InFlag);
}
SDNode *Select_X86ISD_SHLD_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);

  // Pattern: (X86shld:i16 GR16:i16:$src1, GR16:i16:$src2, (imm:i8):$src3)
  // Emits: (SHLD16rri8:i16 GR16:i16:$src1, GR16:i16:$src2, (imm:i8):$src3)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N2.getOpcode() == ISD::Constant &&
      N2.Val->getValueType(0) == MVT::i8) {
    return Emit_236(N, X86::SHLD16rri8, MVT::i16);
  }

  // Pattern: (X86shld:i16 GR16:i16:$src1, GR16:i16:$src2, CL:i8)
  // Emits: (SHLD16rrCL:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N2.Val->getValueType(0) == MVT::i8) {
    return Emit_237(N, X86::SHLD16rrCL, MVT::i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_SHLD_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);

  // Pattern: (X86shld:i32 GR32:i32:$src1, GR32:i32:$src2, (imm:i8):$src3)
  // Emits: (SHLD32rri8:i32 GR32:i32:$src1, GR32:i32:$src2, (imm:i8):$src3)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N2.getOpcode() == ISD::Constant &&
      N2.Val->getValueType(0) == MVT::i8) {
    return Emit_236(N, X86::SHLD32rri8, MVT::i32);
  }

  // Pattern: (X86shld:i32 GR32:i32:$src1, GR32:i32:$src2, CL:i8)
  // Emits: (SHLD32rrCL:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N2.Val->getValueType(0) == MVT::i8) {
    return Emit_237(N, X86::SHLD32rrCL, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_SHLD_i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);

  // Pattern: (X86shld:i64 GR64:i64:$src1, GR64:i64:$src2, (imm:i8):$src3)
  // Emits: (SHLD64rri8:i64 GR64:i64:$src1, GR64:i64:$src2, (imm:i8):$src3)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N2.getOpcode() == ISD::Constant &&
      N2.Val->getValueType(0) == MVT::i8) {
    return Emit_236(N, X86::SHLD64rri8, MVT::i64);
  }

  // Pattern: (X86shld:i64 GR64:i64:$src1, GR64:i64:$src2, CL:i8)
  // Emits: (SHLD64rrCL:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N2.Val->getValueType(0) == MVT::i8) {
    return Emit_237(N, X86::SHLD64rrCL, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_SHRD_i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);

  // Pattern: (X86shrd:i16 GR16:i16:$src1, GR16:i16:$src2, (imm:i8):$src3)
  // Emits: (SHRD16rri8:i16 GR16:i16:$src1, GR16:i16:$src2, (imm:i8):$src3)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N2.getOpcode() == ISD::Constant &&
      N2.Val->getValueType(0) == MVT::i8) {
    return Emit_236(N, X86::SHRD16rri8, MVT::i16);
  }

  // Pattern: (X86shrd:i16 GR16:i16:$src1, GR16:i16:$src2, CL:i8)
  // Emits: (SHRD16rrCL:i16 GR16:i16:$src1, GR16:i16:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N2.Val->getValueType(0) == MVT::i8) {
    return Emit_237(N, X86::SHRD16rrCL, MVT::i16);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_SHRD_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);

  // Pattern: (X86shrd:i32 GR32:i32:$src1, GR32:i32:$src2, (imm:i8):$src3)
  // Emits: (SHRD32rri8:i32 GR32:i32:$src1, GR32:i32:$src2, (imm:i8):$src3)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N2.getOpcode() == ISD::Constant &&
      N2.Val->getValueType(0) == MVT::i8) {
    return Emit_236(N, X86::SHRD32rri8, MVT::i32);
  }

  // Pattern: (X86shrd:i32 GR32:i32:$src1, GR32:i32:$src2, CL:i8)
  // Emits: (SHRD32rrCL:i32 GR32:i32:$src1, GR32:i32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N2.Val->getValueType(0) == MVT::i8) {
    return Emit_237(N, X86::SHRD32rrCL, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_SHRD_i64(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);

  // Pattern: (X86shrd:i64 GR64:i64:$src1, GR64:i64:$src2, (imm:i8):$src3)
  // Emits: (SHRD64rri8:i64 GR64:i64:$src1, GR64:i64:$src2, (imm:i8):$src3)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N2.getOpcode() == ISD::Constant &&
      N2.Val->getValueType(0) == MVT::i8) {
    return Emit_236(N, X86::SHRD64rri8, MVT::i64);
  }

  // Pattern: (X86shrd:i64 GR64:i64:$src1, GR64:i64:$src2, CL:i8)
  // Emits: (SHRD64rrCL:i64 GR64:i64:$src1, GR64:i64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N2.Val->getValueType(0) == MVT::i8) {
    return Emit_237(N, X86::SHRD64rrCL, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_238(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  bool HasInFlag = (N.getOperand(N.getNumOperands()-1).getValueType() == MVT::Flag);
  AddToISelQueue(Chain);
  SDOperand InFlag(0, 0);
  if (HasInFlag) {
    InFlag = N.getOperand(N.getNumOperands()-1);
    AddToISelQueue(InFlag);
  }
  SDOperand Ops0[] = { Chain, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, MVT::Flag, Ops0, HasInFlag ? 2 : 1);
  Chain = SDOperand(ResNode, 0);
  InFlag = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N.Val, 1), InFlag);
  ReplaceUses(SDOperand(N.Val, 0), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Select_X86ISD_TAILCALL(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (X86tailcall:void (tglobaladdr:i64):$dst)
  // Emits: (CALL64pcrel32:void (tglobaladdr:i64):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetGlobalAddress &&
      N1.Val->getValueType(0) == MVT::i64) {
    return Emit_195(N, X86::CALL64pcrel32);
  }

  // Pattern: (X86tailcall:void (texternalsym:i64):$dst)
  // Emits: (CALL64pcrel32:void (texternalsym:i64):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetExternalSymbol &&
      N1.Val->getValueType(0) == MVT::i64) {
    return Emit_195(N, X86::CALL64pcrel32);
  }

  // Pattern: (X86tailcall:void (tglobaladdr:i64):$dst)
  // Emits: (TAILCALL:void)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetGlobalAddress &&
      N1.Val->getValueType(0) == MVT::i64) {
    return Emit_238(N, X86::TAILCALL);
  }

  // Pattern: (X86tailcall:void (texternalsym:i64):$dst)
  // Emits: (TAILCALL:void)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetExternalSymbol &&
      N1.Val->getValueType(0) == MVT::i64) {
    return Emit_238(N, X86::TAILCALL);
  }

  // Pattern: (X86tailcall:void (tglobaladdr:i32):$dst)
  // Emits: (TAILCALL:void)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetGlobalAddress &&
      N1.Val->getValueType(0) == MVT::i32) {
    return Emit_238(N, X86::TAILCALL);
  }

  // Pattern: (X86tailcall:void (texternalsym:i32):$dst)
  // Emits: (TAILCALL:void)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetExternalSymbol &&
      N1.Val->getValueType(0) == MVT::i32) {
    return Emit_238(N, X86::TAILCALL);
  }

  // Pattern: (X86tailcall:void GR64:i64:$dst)
  // Emits: (CALL64r:void GR64:i64:$dst)
  // Pattern complexity = 3  cost = 1  size = 3
  if (N1.Val->getValueType(0) == MVT::i64) {
    return Emit_196(N, X86::CALL64r);
  }
  if (N1.Val->getValueType(0) == MVT::i32) {
    return Emit_238(N, X86::TAILCALL);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_239(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  bool HasInFlag = (N.getOperand(N.getNumOperands()-1).getValueType() == MVT::Flag);
  SmallVector<SDOperand, 8> Ops0;
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N2)->getValue()), MVT::i32);
  AddToISelQueue(Chain);
  SDOperand InFlag(0, 0);
  if (HasInFlag) {
    InFlag = N.getOperand(N.getNumOperands()-1);
    AddToISelQueue(InFlag);
  }
  Ops0.push_back(N1);
  Ops0.push_back(Tmp1);
  for (unsigned i = 3, e = N.getNumOperands()-(HasInFlag?1:0); i != e; ++i) {
    AddToISelQueue(N.getOperand(i));
    Ops0.push_back(N.getOperand(i));
  }
  Ops0.push_back(Chain);
  if (HasInFlag)
    Ops0.push_back(InFlag);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, &Ops0[0], Ops0.size());
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Emit_240(const SDOperand &N, unsigned Opc0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  bool HasInFlag = (N.getOperand(N.getNumOperands()-1).getValueType() == MVT::Flag);
  SmallVector<SDOperand, 8> Ops0;
  AddToISelQueue(N1);
  SDOperand Tmp1 = CurDAG->getTargetConstant(((unsigned) cast<ConstantSDNode>(N2)->getValue()), MVT::i32);
  AddToISelQueue(Chain);
  SDOperand InFlag(0, 0);
  if (HasInFlag) {
    InFlag = N.getOperand(N.getNumOperands()-1);
    AddToISelQueue(InFlag);
  }
  Ops0.push_back(N1);
  Ops0.push_back(Tmp1);
  for (unsigned i = 3, e = N.getNumOperands()-(HasInFlag?1:0); i != e; ++i) {
    AddToISelQueue(N.getOperand(i));
    Ops0.push_back(N.getOperand(i));
  }
  Ops0.push_back(Chain);
  if (HasInFlag)
    Ops0.push_back(InFlag);
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, MVT::Other, &Ops0[0], Ops0.size());
  Chain = SDOperand(ResNode, 0);
  return ResNode;
}
SDNode *Select_X86ISD_TC_RETURN(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);

  // Pattern: (X86tcret:void (tglobaladdr:i64):$dst, (imm:i32):$off)
  // Emits: (TCRETURNdi64:void (texternalsym:i64):$dst, (imm:i32):$off)
  // Pattern complexity = 9  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetGlobalAddress) {
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant &&
        N1.Val->getValueType(0) == MVT::i64) {
      return Emit_239(N, X86::TCRETURNdi64);
    }
  }

  // Pattern: (X86tcret:void (texternalsym:i64):$dst, (imm:i32):$off)
  // Emits: (TCRETURNdi64:void (texternalsym:i64):$dst, (imm:i32):$off)
  // Pattern complexity = 9  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetExternalSymbol) {
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant &&
        N1.Val->getValueType(0) == MVT::i64) {
      return Emit_239(N, X86::TCRETURNdi64);
    }
  }

  // Pattern: (X86tcret:void (tglobaladdr:i32):$dst, (imm:i32):$off)
  // Emits: (TCRETURNdi:void (texternalsym:i32):$dst, (imm:i32):$off)
  // Pattern complexity = 9  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetGlobalAddress) {
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant &&
        N1.Val->getValueType(0) == MVT::i32) {
      return Emit_239(N, X86::TCRETURNdi);
    }
  }

  // Pattern: (X86tcret:void (texternalsym:i32):$dst, (imm:i32):$off)
  // Emits: (TCRETURNdi:void (texternalsym:i32):$dst, (imm:i32):$off)
  // Pattern complexity = 9  cost = 1  size = 3
  if (N1.getOpcode() == ISD::TargetExternalSymbol) {
    SDOperand N2 = N.getOperand(2);
    if (N2.getOpcode() == ISD::Constant &&
        N1.Val->getValueType(0) == MVT::i32) {
      return Emit_239(N, X86::TCRETURNdi);
    }
  }
  SDOperand N2 = N.getOperand(2);
  if (N2.getOpcode() == ISD::Constant) {

    // Pattern: (X86tcret:void GR64:i64:$dst, (imm:i32):$off)
    // Emits: (TCRETURNri64:void GR64:i64:$dst, (imm:i32):$off)
    // Pattern complexity = 6  cost = 1  size = 3
    if (N1.Val->getValueType(0) == MVT::i64) {
      return Emit_240(N, X86::TCRETURNri64);
    }

    // Pattern: (X86tcret:void GR32:i32:$dst, (imm:i32):$off)
    // Emits: (TCRETURNri:void GR32:i32:$dst, (imm:i32):$off)
    // Pattern complexity = 6  cost = 1  size = 3
    if (N1.Val->getValueType(0) == MVT::i32) {
      return Emit_240(N, X86::TCRETURNri);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_THREAD_POINTER_i32(const SDOperand &N) {
  return Emit_69(N, X86::TLS_tp, MVT::i32);
}

SDNode *Emit_241(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  bool HasInFlag = (N.getOperand(N.getNumOperands()-1).getValueType() == MVT::Flag);
  AddToISelQueue(Chain);
  SDOperand InFlag(0, 0);
  if (HasInFlag) {
    InFlag = N.getOperand(N.getNumOperands()-1);
    AddToISelQueue(InFlag);
  }
  SDOperand Ops0[] = { N1, Chain, InFlag };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, MVT::Flag, Ops0, HasInFlag ? 3 : 2);
  Chain = SDOperand(ResNode, 1);
  InFlag = SDOperand(ResNode, 2);
  ReplaceUses(SDOperand(N.Val, 2), InFlag);
  ReplaceUses(SDOperand(N.Val, 1), SDOperand(Chain.Val, Chain.ResNo));
  return ResNode;
}
SDNode *Select_X86ISD_TLSADDR_i32(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::TargetGlobalTLSAddress) {
    return Emit_241(N, X86::TLS_addr32, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_TLSADDR_i64(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  if (N1.getOpcode() == ISD::TargetGlobalTLSAddress) {
    return Emit_241(N, X86::TLS_addr64, MVT::i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_UCOMI(const SDOperand &N) {

  // Pattern: (X86ucomi:void VR128:v4f32:$src1, (ld:v4f32 addr:iPTR:$src2)<<P:Predicate_load>>)
  // Emits: (Int_UCOMISSrm:void VR128:v4f32:$src1, addr:iPTR:$src2)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::v4f32) {
          return Emit_211(N, X86::Int_UCOMISSrm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86ucomi:void VR128:v2f64:$src1, (ld:v2f64 addr:iPTR:$src2)<<P:Predicate_load>>)
  // Emits: (Int_UCOMISDrm:void VR128:v2f64:$src1, addr:iPTR:$src2)
  // Pattern complexity = 22  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N1.getOpcode() == ISD::LOAD &&
        N1.hasOneUse() &&
        CanBeFoldedBy(N1.Val, N.Val, N.Val)) {
      SDOperand Chain1 = N1.getOperand(0);
      if (Predicate_load(N1.Val)) {
        SDOperand N11 = N1.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N11, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
            N0.Val->getValueType(0) == MVT::v2f64) {
          return Emit_211(N, X86::Int_UCOMISDrm, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }

  // Pattern: (X86ucomi:void VR128:v4f32:$src1, VR128:v4f32:$src2)
  // Emits: (Int_UCOMISSrr:void VR128:v4f32:$src1, VR128:v4f32:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N0.Val->getValueType(0) == MVT::v4f32) {
      return Emit_222(N, X86::Int_UCOMISSrr);
    }
  }

  // Pattern: (X86ucomi:void VR128:v2f64:$src1, VR128:v2f64:$src2)
  // Emits: (Int_UCOMISDrr:void VR128:v2f64:$src1, VR128:v2f64:$src2)
  // Pattern complexity = 3  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    SDOperand N1 = N.getOperand(1);
    if (N0.Val->getValueType(0) == MVT::v2f64) {
      return Emit_222(N, X86::Int_UCOMISDrr);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_242(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  AddToISelQueue(Chain);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, Chain };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 5);
  Chain = SDOperand(ResNode, 1);
  return ResNode;
}
SDNode *Select_X86ISD_VZEXT_LOAD_v2i64(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_242(N, X86::MOVZQI2PQIrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_VZEXT_LOAD_v2f64(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand CPTmp0;
  SDOperand CPTmp1;
  SDOperand CPTmp2;
  SDOperand CPTmp3;
  if (SelectAddr(N, N1, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
    return Emit_242(N, X86::MOVZSD2PDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_243(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand N000 = N00.getOperand(0);
  AddToISelQueue(N000);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N000);
}
SDNode *Select_X86ISD_VZEXT_MOVL_v8i8(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  if (N0.getOpcode() == ISD::BIT_CONVERT) {
    SDOperand N00 = N0.getOperand(0);
    if (N00.getOpcode() == ISD::SCALAR_TO_VECTOR) {
      SDOperand N000 = N00.getOperand(0);
      if (N00.Val->getValueType(0) == MVT::v2i32 &&
          N000.Val->getValueType(0) == MVT::i32) {
        return Emit_243(N, X86::MMX_MOVZDI2PDIrr, MVT::v8i8);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_VZEXT_MOVL_v4i16(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  if (N0.getOpcode() == ISD::BIT_CONVERT) {
    SDOperand N00 = N0.getOperand(0);
    if (N00.getOpcode() == ISD::SCALAR_TO_VECTOR) {
      SDOperand N000 = N00.getOperand(0);
      if (N00.Val->getValueType(0) == MVT::v2i32 &&
          N000.Val->getValueType(0) == MVT::i32) {
        return Emit_243(N, X86::MMX_MOVZDI2PDIrr, MVT::v4i16);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_244(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0, SDOperand &CPTmp0, SDOperand &CPTmp1, SDOperand &CPTmp2, SDOperand &CPTmp3) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Chain00 = N00.getOperand(0);
  SDOperand N001 = N00.getOperand(1);
  AddToISelQueue(CPTmp0);
  AddToISelQueue(CPTmp1);
  AddToISelQueue(CPTmp2);
  AddToISelQueue(CPTmp3);
  SDOperand LSI_N00 = CurDAG->getMemOperand(cast<LSBaseSDNode>(N00)->getMemOperand());
  AddToISelQueue(Chain00);
  SDOperand Ops0[] = { CPTmp0, CPTmp1, CPTmp2, CPTmp3, LSI_N00, Chain00 };
  SDNode *ResNode = CurDAG->getTargetNode(Opc0, VT0, MVT::Other, Ops0, 6);
  Chain00 = SDOperand(ResNode, 1);
  ReplaceUses(SDOperand(N00.Val, 1), SDOperand(ResNode, 1));
  return ResNode;
}
SDNode *Select_X86ISD_VZEXT_MOVL_v2i32(const SDOperand &N) {
  if ((Subtarget->hasMMX())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR) {

      // Pattern: (X86vzmovl:v2i32 (scalar_to_vector:v2i32 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (MMX_MOVZDI2PDIrm:v2i32 addr:iPTR:$src)
      // Pattern complexity = 45  cost = 1  size = 3
      if (N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_load(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N00.Val->getValueType(0) == MVT::i32) {
              return Emit_244(N, X86::MMX_MOVZDI2PDIrm, MVT::v2i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (X86vzmovl:v2i32 (scalar_to_vector:v2i32 GR32:i32:$src))
      // Emits: (MMX_MOVZDI2PDIrr:v2i32 GR32:i32:$src)
      // Pattern complexity = 21  cost = 1  size = 3
      SDOperand N00 = N0.getOperand(0);
      if (N00.Val->getValueType(0) == MVT::i32) {
        return Emit_64(N, X86::MMX_MOVZDI2PDIrr, MVT::v2i32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_VZEXT_MOVL_v4i32(const SDOperand &N) {
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR) {

      // Pattern: (X86vzmovl:v4i32 (scalar_to_vector:v4i32 (ld:i32 addr:iPTR:$src)<<P:Predicate_load>>))
      // Emits: (MOVZDI2PDIrm:v4i32 addr:iPTR:$src)
      // Pattern complexity = 45  cost = 1  size = 3
      if (N0.hasOneUse()) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.getOpcode() == ISD::LOAD &&
            N00.hasOneUse() &&
            CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
          SDOperand Chain00 = N00.getOperand(0);
          if (Predicate_load(N00.Val)) {
            SDOperand N001 = N00.getOperand(1);
            SDOperand CPTmp0;
            SDOperand CPTmp1;
            SDOperand CPTmp2;
            SDOperand CPTmp3;
            if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
                N00.Val->getValueType(0) == MVT::i32) {
              return Emit_244(N, X86::MOVZDI2PDIrm, MVT::v4i32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
            }
          }
        }
      }

      // Pattern: (X86vzmovl:v4i32 (scalar_to_vector:v4i32 GR32:i32:$src))
      // Emits: (MOVZDI2PDIrr:v4i32 GR32:i32:$src)
      // Pattern complexity = 21  cost = 1  size = 3
      SDOperand N00 = N0.getOperand(0);
      if (N00.Val->getValueType(0) == MVT::i32) {
        return Emit_64(N, X86::MOVZDI2PDIrr, MVT::v4i32);
      }
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_VZEXT_MOVL_v2i64(const SDOperand &N) {

  // Pattern: (X86vzmovl:v2i64 (scalar_to_vector:v2i64 (ld:i64 addr:iPTR:$src)<<P:Predicate_load>>))
  // Emits: (MOVZQI2PQIrm:v2i64 addr:iPTR:$src)
  // Pattern complexity = 45  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_load(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::i64) {
            return Emit_244(N, X86::MOVZQI2PQIrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }

  // Pattern: (X86vzmovl:v2i64 (bitconvert:v2i64 (ld:v4i32 addr:iPTR:$src)<<P:Predicate_load>>))
  // Emits: (MOVZPQILo2PQIrm:v2i64 addr:iPTR:$src)
  // Pattern complexity = 45  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::BIT_CONVERT &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_load(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::v4i32) {
            return Emit_244(N, X86::MOVZPQILo2PQIrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {
    {
      SDOperand N0 = N.getOperand(0);

      // Pattern: (X86vzmovl:v2i64 (ld:v2i64 addr:iPTR:$src)<<P:Predicate_load>>)
      // Emits: (MOVZPQILo2PQIrm:v2i64 addr:iPTR:$src)
      // Pattern complexity = 42  cost = 1  size = 3
      if (N0.getOpcode() == ISD::LOAD &&
          N0.hasOneUse()) {
        SDOperand Chain0 = N0.getOperand(0);
        if (Predicate_load(N0.Val)) {
          SDOperand N01 = N0.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
            return Emit_58(N, X86::MOVZPQILo2PQIrm, MVT::v2i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }

      // Pattern: (X86vzmovl:v2i64 (scalar_to_vector:v2i64 GR64:i64:$src))
      // Emits: (MOVZQI2PQIrr:v2i64 GR64:i64:$src)
      // Pattern complexity = 21  cost = 1  size = 3
      if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.Val->getValueType(0) == MVT::i64) {
          return Emit_64(N, X86::MOVZQI2PQIrr, MVT::v2i64);
        }
      }
    }

    // Pattern: (X86vzmovl:v2i64 VR128:v2i64:$src)
    // Emits: (MOVZPQILo2PQIrr:v2i64 VR128:v2i64:$src)
    // Pattern complexity = 18  cost = 1  size = 3
    return Emit_59(N, X86::MOVZPQILo2PQIrr, MVT::v2i64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_245(const SDOperand &N, unsigned Opc0, unsigned Opc1, MVT::ValueType VT0, MVT::ValueType VT1) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand N00 = N0.getOperand(0);
  SDOperand Tmp0(CurDAG->getTargetNode(Opc0, VT0), 0);
  AddToISelQueue(N00);
  return CurDAG->SelectNodeTo(N.Val, Opc1, VT1, Tmp0, N00);
}
SDNode *Emit_246(const SDOperand &N, unsigned Opc0, unsigned Opc1, MVT::ValueType VT0, MVT::ValueType VT1) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  SDOperand Tmp0(CurDAG->getTargetNode(Opc0, VT0), 0);
  AddToISelQueue(N0);
  return CurDAG->SelectNodeTo(N.Val, Opc1, VT1, Tmp0, N0);
}
SDNode *Select_X86ISD_VZEXT_MOVL_v4f32(const SDOperand &N) {

  // Pattern: (X86vzmovl:v4f32 (scalar_to_vector:v4f32 (ld:f32 addr:iPTR:$src)<<P:Predicate_load>>))
  // Emits: (MOVZSS2PSrm:v4f32 addr:iPTR:$src)
  // Pattern complexity = 45  cost = 1  size = 3
  if ((Subtarget->hasSSE1())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_load(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::f32) {
            return Emit_244(N, X86::MOVZSS2PSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }

  // Pattern: (X86vzmovl:v4f32 (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>)
  // Emits: (MOVZSS2PSrm:v4f32 addr:iPTR:$src)
  // Pattern complexity = 22  cost = 1  size = 3
  {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_58(N, X86::MOVZSS2PSrm, MVT::v4f32, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {

    // Pattern: (X86vzmovl:v4f32 (scalar_to_vector:v4f32 FR32:f32:$src))
    // Emits: (MOVLSS2PSrr:v4f32 (V_SET0:v16i8), FR32:f32:$src)
    // Pattern complexity = 21  cost = 2  size = 6
    {
      SDOperand N0 = N.getOperand(0);
      if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.Val->getValueType(0) == MVT::f32) {
          return Emit_245(N, X86::V_SET0, X86::MOVLSS2PSrr, MVT::v16i8, MVT::v4f32);
        }
      }
    }

    // Pattern: (X86vzmovl:v4f32 VR128:v4f32:$src)
    // Emits: (MOVLPSrr:v4f32 (V_SET0:v16i8), VR128:v16i8:$src)
    // Pattern complexity = 18  cost = 2  size = 6
    return Emit_246(N, X86::V_SET0, X86::MOVLPSrr, MVT::v16i8, MVT::v4f32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_VZEXT_MOVL_v2f64(const SDOperand &N) {

  // Pattern: (X86vzmovl:v2f64 (scalar_to_vector:v2f64 (ld:f64 addr:iPTR:$src)<<P:Predicate_load>>))
  // Emits: (MOVZSD2PDrm:v2f64 addr:iPTR:$src)
  // Pattern complexity = 45  cost = 1  size = 3
  if ((Subtarget->hasSSE2())) {
    SDOperand N0 = N.getOperand(0);
    if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_load(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::f64) {
            return Emit_244(N, X86::MOVZSD2PDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }
  }
  {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (X86vzmovl:v2f64 (bitconvert:v2f64 (ld:v4f32 addr:iPTR:$src)<<P:Predicate_load>>))
    // Emits: (MOVZSD2PDrm:v2f64 addr:iPTR:$src)
    // Pattern complexity = 45  cost = 1  size = 3
    if (N0.getOpcode() == ISD::BIT_CONVERT &&
        N0.hasOneUse()) {
      SDOperand N00 = N0.getOperand(0);
      if (N00.getOpcode() == ISD::LOAD &&
          N00.hasOneUse() &&
          CanBeFoldedBy(N00.Val, N0.Val, N.Val)) {
        SDOperand Chain00 = N00.getOperand(0);
        if (Predicate_load(N00.Val)) {
          SDOperand N001 = N00.getOperand(1);
          SDOperand CPTmp0;
          SDOperand CPTmp1;
          SDOperand CPTmp2;
          SDOperand CPTmp3;
          if (SelectAddr(N, N001, CPTmp0, CPTmp1, CPTmp2, CPTmp3) &&
              N00.Val->getValueType(0) == MVT::v4f32) {
            return Emit_244(N, X86::MOVZSD2PDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
          }
        }
      }
    }

    // Pattern: (X86vzmovl:v2f64 (ld:v2f64 addr:iPTR:$src)<<P:Predicate_load>>)
    // Emits: (MOVZSD2PDrm:v2f64 addr:iPTR:$src)
    // Pattern complexity = 42  cost = 1  size = 3
    if (N0.getOpcode() == ISD::LOAD &&
        N0.hasOneUse()) {
      SDOperand Chain0 = N0.getOperand(0);
      if (Predicate_load(N0.Val)) {
        SDOperand N01 = N0.getOperand(1);
        SDOperand CPTmp0;
        SDOperand CPTmp1;
        SDOperand CPTmp2;
        SDOperand CPTmp3;
        if (SelectAddr(N, N01, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
          return Emit_58(N, X86::MOVZSD2PDrm, MVT::v2f64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
        }
      }
    }
  }
  if ((Subtarget->hasSSE2())) {

    // Pattern: (X86vzmovl:v2f64 (scalar_to_vector:v2f64 FR64:f64:$src))
    // Emits: (MOVLSD2PDrr:v2f64 (V_SET0:v16i8), FR64:f64:$src)
    // Pattern complexity = 21  cost = 2  size = 6
    {
      SDOperand N0 = N.getOperand(0);
      if (N0.getOpcode() == ISD::SCALAR_TO_VECTOR) {
        SDOperand N00 = N0.getOperand(0);
        if (N00.Val->getValueType(0) == MVT::f64) {
          return Emit_245(N, X86::V_SET0, X86::MOVLSD2PDrr, MVT::v16i8, MVT::v2f64);
        }
      }
    }

    // Pattern: (X86vzmovl:v2f64 VR128:v2f64:$src)
    // Emits: (MOVZPQILo2PQIrr:v2f64 VR128:v16i8:$src)
    // Pattern complexity = 3  cost = 1  size = 3
    return Emit_59(N, X86::MOVZPQILo2PQIrr, MVT::v2f64);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Emit_247(const SDOperand &N, unsigned Opc0, MVT::ValueType VT0) DISABLE_INLINE {
  SDOperand N0 = N.getOperand(0);
  return CurDAG->SelectNodeTo(N.Val, Opc0, VT0, N0);
}
SDNode *Select_X86ISD_Wrapper_i32(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);

  // Pattern: (X86Wrapper:i32 (tconstpool:i32):$dst)
  // Emits: (MOV32ri:i32 (tconstpool:i32):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N0.getOpcode() == ISD::TargetConstantPool) {
    return Emit_247(N, X86::MOV32ri, MVT::i32);
  }

  // Pattern: (X86Wrapper:i32 (tjumptable:i32):$dst)
  // Emits: (MOV32ri:i32 (tjumptable:i32):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N0.getOpcode() == ISD::TargetJumpTable) {
    return Emit_59(N, X86::MOV32ri, MVT::i32);
  }

  // Pattern: (X86Wrapper:i32 (tglobaltlsaddr:i32):$dst)
  // Emits: (MOV32ri:i32 (tglobaltlsaddr:i32):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N0.getOpcode() == ISD::TargetGlobalTLSAddress) {
    return Emit_247(N, X86::MOV32ri, MVT::i32);
  }

  // Pattern: (X86Wrapper:i32 (tglobaladdr:i32):$dst)
  // Emits: (MOV32ri:i32 (tglobaladdr:i32):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N0.getOpcode() == ISD::TargetGlobalAddress) {
    return Emit_247(N, X86::MOV32ri, MVT::i32);
  }

  // Pattern: (X86Wrapper:i32 (texternalsym:i32):$dst)
  // Emits: (MOV32ri:i32 (texternalsym:i32):$dst)
  // Pattern complexity = 6  cost = 1  size = 3
  if (N0.getOpcode() == ISD::TargetExternalSymbol) {
    return Emit_247(N, X86::MOV32ri, MVT::i32);
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_X86ISD_Wrapper_i64(const SDOperand &N) {

  // Pattern: lea64addr:i64:$src
  // Emits: (LEA64r:i64 lea64addr:i64:$src)
  // Pattern complexity = 15  cost = 1  size = 3
  {
    SDOperand CPTmp0;
    SDOperand CPTmp1;
    SDOperand CPTmp2;
    SDOperand CPTmp3;
    if (SelectLEAAddr(N, N, CPTmp0, CPTmp1, CPTmp2, CPTmp3)) {
      return Emit_6(N, X86::LEA64r, MVT::i64, CPTmp0, CPTmp1, CPTmp2, CPTmp3);
    }
  }
  if ((TM.getCodeModel() != CodeModel::Small)) {
    SDOperand N0 = N.getOperand(0);

    // Pattern: (X86Wrapper:i64 (tconstpool:i64):$dst)
    // Emits: (MOV64ri:i64 (tconstpool:i64):$dst)
    // Pattern complexity = 6  cost = 1  size = 3
    if (N0.getOpcode() == ISD::TargetConstantPool) {
      return Emit_247(N, X86::MOV64ri, MVT::i64);
    }

    // Pattern: (X86Wrapper:i64 (tjumptable:i64):$dst)
    // Emits: (MOV64ri:i64 (tjumptable:i64):$dst)
    // Pattern complexity = 6  cost = 1  size = 3
    if (N0.getOpcode() == ISD::TargetJumpTable) {
      return Emit_59(N, X86::MOV64ri, MVT::i64);
    }

    // Pattern: (X86Wrapper:i64 (tglobaladdr:i64):$dst)
    // Emits: (MOV64ri:i64 (tglobaladdr:i64):$dst)
    // Pattern complexity = 6  cost = 1  size = 3
    if (N0.getOpcode() == ISD::TargetGlobalAddress) {
      return Emit_247(N, X86::MOV64ri, MVT::i64);
    }

    // Pattern: (X86Wrapper:i64 (texternalsym:i64):$dst)
    // Emits: (MOV64ri:i64 (texternalsym:i64):$dst)
    // Pattern complexity = 6  cost = 1  size = 3
    if (N0.getOpcode() == ISD::TargetExternalSymbol) {
      return Emit_247(N, X86::MOV64ri, MVT::i64);
    }
  }
  cerr << "Cannot yet select: ";
  N.Val->dump(CurDAG);
  cerr << '\n';
  abort();
  return NULL;
}

SDNode *Select_INLINEASM(SDOperand N) {
  std::vector<SDOperand> Ops(N.Val->op_begin(), N.Val->op_end());
  SelectInlineAsmMemoryOperands(Ops, *CurDAG);

  // Ensure that the asm operands are themselves selected.
  for (unsigned j = 0, e = Ops.size(); j != e; ++j)
    AddToISelQueue(Ops[j]);

  std::vector<MVT::ValueType> VTs;
  VTs.push_back(MVT::Other);
  VTs.push_back(MVT::Flag);
  SDOperand New = CurDAG->getNode(ISD::INLINEASM, VTs, &Ops[0], Ops.size());
  return New.Val;
}

SDNode *Select_UNDEF(const SDOperand &N) {
  return CurDAG->getTargetNode(TargetInstrInfo::IMPLICIT_DEF,
                               N.getValueType());
}

SDNode *Select_LABEL(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  unsigned C1 = cast<ConstantSDNode>(N1)->getValue();
  unsigned C2 = cast<ConstantSDNode>(N2)->getValue();
  SDOperand Tmp1 = CurDAG->getTargetConstant(C1, MVT::i32);
  SDOperand Tmp2 = CurDAG->getTargetConstant(C2, MVT::i32);
  AddToISelQueue(Chain);
  SDOperand Ops[] = { Tmp1, Tmp2, Chain };
  return CurDAG->getTargetNode(TargetInstrInfo::LABEL,
                               MVT::Other, Ops, 3);
}

SDNode *Select_DECLARE(const SDOperand &N) {
  SDOperand Chain = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  if (!isa<FrameIndexSDNode>(N1) || !isa<GlobalAddressSDNode>(N2)) {
    cerr << "Cannot yet select llvm.dbg.declare: ";
    N.Val->dump(CurDAG);
    abort();
  }
  int FI = cast<FrameIndexSDNode>(N1)->getIndex();
  GlobalValue *GV = cast<GlobalAddressSDNode>(N2)->getGlobal();
  SDOperand Tmp1 = CurDAG->getTargetFrameIndex(FI, TLI.getPointerTy());
  SDOperand Tmp2 = CurDAG->getTargetGlobalAddress(GV, TLI.getPointerTy());
  AddToISelQueue(Chain);
  SDOperand Ops[] = { Tmp1, Tmp2, Chain };
  return CurDAG->getTargetNode(TargetInstrInfo::DECLARE,
                               MVT::Other, Ops, 3);
}

SDNode *Select_EXTRACT_SUBREG(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  unsigned C = cast<ConstantSDNode>(N1)->getValue();
  SDOperand Tmp = CurDAG->getTargetConstant(C, MVT::i32);
  AddToISelQueue(N0);
  SDOperand Ops[] = { N0, Tmp };
  return CurDAG->getTargetNode(TargetInstrInfo::EXTRACT_SUBREG,
                               N.getValueType(), Ops, 2);
}

SDNode *Select_INSERT_SUBREG(const SDOperand &N) {
  SDOperand N0 = N.getOperand(0);
  SDOperand N1 = N.getOperand(1);
  SDOperand N2 = N.getOperand(2);
  unsigned C = cast<ConstantSDNode>(N2)->getValue();
  SDOperand Tmp = CurDAG->getTargetConstant(C, MVT::i32);
  AddToISelQueue(N1);
  SDOperand Ops[] = { N0, N1, Tmp };
  AddToISelQueue(N0);
  return CurDAG->getTargetNode(TargetInstrInfo::INSERT_SUBREG,
                               N.getValueType(), Ops, 3);
}

// The main instruction selector code.
SDNode *SelectCode(SDOperand N) {
  if (N.getOpcode() >= ISD::BUILTIN_OP_END &&
      N.getOpcode() < (ISD::BUILTIN_OP_END+X86::INSTRUCTION_LIST_END)) {
    return NULL;   // Already selected.
  }

  MVT::ValueType NVT = N.Val->getValueType(0);
  switch (N.getOpcode()) {
  default: break;
  case ISD::EntryToken:       // These leaves remain the same.
  case ISD::BasicBlock:
  case ISD::Register:
  case ISD::HANDLENODE:
  case ISD::TargetConstant:
  case ISD::TargetConstantFP:
  case ISD::TargetConstantPool:
  case ISD::TargetFrameIndex:
  case ISD::TargetExternalSymbol:
  case ISD::TargetJumpTable:
  case ISD::TargetGlobalTLSAddress:
  case ISD::TargetGlobalAddress: {
    return NULL;
  }
  case ISD::AssertSext:
  case ISD::AssertZext: {
    AddToISelQueue(N.getOperand(0));
    ReplaceUses(N, N.getOperand(0));
    return NULL;
  }
  case ISD::TokenFactor:
  case ISD::CopyFromReg:
  case ISD::CopyToReg: {
    for (unsigned i = 0, e = N.getNumOperands(); i != e; ++i)
      AddToISelQueue(N.getOperand(i));
    return NULL;
  }
  case ISD::INLINEASM: return Select_INLINEASM(N);
  case ISD::LABEL: return Select_LABEL(N);
  case ISD::DECLARE: return Select_DECLARE(N);
  case ISD::EXTRACT_SUBREG: return Select_EXTRACT_SUBREG(N);
  case ISD::INSERT_SUBREG: return Select_INSERT_SUBREG(N);
  case ISD::UNDEF: return Select_UNDEF(N);
  case ISD::ADD: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_ADD_i8(N);
    case MVT::i16:
      return Select_ISD_ADD_i16(N);
    case MVT::i32:
      return Select_ISD_ADD_i32(N);
    case MVT::i64:
      return Select_ISD_ADD_i64(N);
    case MVT::v8i8:
      return Select_ISD_ADD_v8i8(N);
    case MVT::v4i16:
      return Select_ISD_ADD_v4i16(N);
    case MVT::v2i32:
      return Select_ISD_ADD_v2i32(N);
    case MVT::v1i64:
      return Select_ISD_ADD_v1i64(N);
    case MVT::v16i8:
      return Select_ISD_ADD_v16i8(N);
    case MVT::v8i16:
      return Select_ISD_ADD_v8i16(N);
    case MVT::v4i32:
      return Select_ISD_ADD_v4i32(N);
    case MVT::v2i64:
      return Select_ISD_ADD_v2i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::ADDC: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_ADDC_i32(N);
    case MVT::i64:
      return Select_ISD_ADDC_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::ADDE: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_ADDE_i32(N);
    case MVT::i64:
      return Select_ISD_ADDE_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::AND: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_AND_i8(N);
    case MVT::i16:
      return Select_ISD_AND_i16(N);
    case MVT::i32:
      return Select_ISD_AND_i32(N);
    case MVT::i64:
      return Select_ISD_AND_i64(N);
    case MVT::v1i64:
      return Select_ISD_AND_v1i64(N);
    case MVT::v2i64:
      return Select_ISD_AND_v2i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::ANY_EXTEND: {
    switch (NVT) {
    case MVT::i16:
      return Select_ISD_ANY_EXTEND_i16(N);
    case MVT::i32:
      return Select_ISD_ANY_EXTEND_i32(N);
    case MVT::i64:
      return Select_ISD_ANY_EXTEND_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::ATOMIC_LAS: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_ATOMIC_LAS_i8(N);
    case MVT::i16:
      return Select_ISD_ATOMIC_LAS_i16(N);
    case MVT::i32:
      return Select_ISD_ATOMIC_LAS_i32(N);
    case MVT::i64:
      return Select_ISD_ATOMIC_LAS_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::ATOMIC_LOAD_AND: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_ATOMIC_LOAD_AND_i32(N);
    default:
      break;
    }
    break;
  }
  case ISD::ATOMIC_LOAD_MAX: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_ATOMIC_LOAD_MAX_i32(N);
    default:
      break;
    }
    break;
  }
  case ISD::ATOMIC_LOAD_MIN: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_ATOMIC_LOAD_MIN_i32(N);
    default:
      break;
    }
    break;
  }
  case ISD::ATOMIC_LOAD_OR: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_ATOMIC_LOAD_OR_i32(N);
    default:
      break;
    }
    break;
  }
  case ISD::ATOMIC_LOAD_UMAX: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_ATOMIC_LOAD_UMAX_i32(N);
    default:
      break;
    }
    break;
  }
  case ISD::ATOMIC_LOAD_UMIN: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_ATOMIC_LOAD_UMIN_i32(N);
    default:
      break;
    }
    break;
  }
  case ISD::ATOMIC_LOAD_XOR: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_ATOMIC_LOAD_XOR_i32(N);
    default:
      break;
    }
    break;
  }
  case ISD::ATOMIC_SWAP: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_ATOMIC_SWAP_i8(N);
    case MVT::i16:
      return Select_ISD_ATOMIC_SWAP_i16(N);
    case MVT::i32:
      return Select_ISD_ATOMIC_SWAP_i32(N);
    case MVT::i64:
      return Select_ISD_ATOMIC_SWAP_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::BIT_CONVERT: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_BIT_CONVERT_i32(N);
    case MVT::i64:
      return Select_ISD_BIT_CONVERT_i64(N);
    case MVT::f32:
      return Select_ISD_BIT_CONVERT_f32(N);
    case MVT::f64:
      return Select_ISD_BIT_CONVERT_f64(N);
    case MVT::v8i8:
      return Select_ISD_BIT_CONVERT_v8i8(N);
    case MVT::v4i16:
      return Select_ISD_BIT_CONVERT_v4i16(N);
    case MVT::v2i32:
      return Select_ISD_BIT_CONVERT_v2i32(N);
    case MVT::v1i64:
      return Select_ISD_BIT_CONVERT_v1i64(N);
    case MVT::v16i8:
      return Select_ISD_BIT_CONVERT_v16i8(N);
    case MVT::v8i16:
      return Select_ISD_BIT_CONVERT_v8i16(N);
    case MVT::v4i32:
      return Select_ISD_BIT_CONVERT_v4i32(N);
    case MVT::v2i64:
      return Select_ISD_BIT_CONVERT_v2i64(N);
    case MVT::v4f32:
      return Select_ISD_BIT_CONVERT_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_BIT_CONVERT_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::BR: {
    switch (NVT) {
    default:
      return Select_ISD_BR(N);
      break;
    }
    break;
  }
  case ISD::BRIND: {
    switch (NVT) {
    default:
      return Select_ISD_BRIND(N);
      break;
    }
    break;
  }
  case ISD::BSWAP: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_BSWAP_i32(N);
    case MVT::i64:
      return Select_ISD_BSWAP_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::BUILD_VECTOR: {
    switch (NVT) {
    case MVT::v8i8:
      return Select_ISD_BUILD_VECTOR_v8i8(N);
    case MVT::v4i16:
      return Select_ISD_BUILD_VECTOR_v4i16(N);
    case MVT::v2i32:
      return Select_ISD_BUILD_VECTOR_v2i32(N);
    case MVT::v1i64:
      return Select_ISD_BUILD_VECTOR_v1i64(N);
    case MVT::v16i8:
      return Select_ISD_BUILD_VECTOR_v16i8(N);
    case MVT::v8i16:
      return Select_ISD_BUILD_VECTOR_v8i16(N);
    case MVT::v4i32:
      return Select_ISD_BUILD_VECTOR_v4i32(N);
    case MVT::v2i64:
      return Select_ISD_BUILD_VECTOR_v2i64(N);
    case MVT::v4f32:
      return Select_ISD_BUILD_VECTOR_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_BUILD_VECTOR_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::CALLSEQ_END: {
    switch (NVT) {
    default:
      return Select_ISD_CALLSEQ_END(N);
      break;
    }
    break;
  }
  case ISD::CALLSEQ_START: {
    switch (NVT) {
    default:
      return Select_ISD_CALLSEQ_START(N);
      break;
    }
    break;
  }
  case ISD::Constant: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_Constant_i8(N);
    case MVT::i16:
      return Select_ISD_Constant_i16(N);
    case MVT::i32:
      return Select_ISD_Constant_i32(N);
    case MVT::i64:
      return Select_ISD_Constant_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::ConstantFP: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_ConstantFP_f32(N);
    case MVT::f64:
      return Select_ISD_ConstantFP_f64(N);
    case MVT::f80:
      return Select_ISD_ConstantFP_f80(N);
    default:
      break;
    }
    break;
  }
  case ISD::DEBUG_LOC: {
    switch (NVT) {
    default:
      return Select_ISD_DEBUG_LOC(N);
      break;
    }
    break;
  }
  case ISD::EXTRACT_VECTOR_ELT: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_EXTRACT_VECTOR_ELT_i32(N);
    case MVT::i64:
      return Select_ISD_EXTRACT_VECTOR_ELT_i64(N);
    case MVT::f32:
      return Select_ISD_EXTRACT_VECTOR_ELT_f32(N);
    case MVT::f64:
      return Select_ISD_EXTRACT_VECTOR_ELT_f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::FABS: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_FABS_f32(N);
    case MVT::f64:
      return Select_ISD_FABS_f64(N);
    case MVT::f80:
      return Select_ISD_FABS_f80(N);
    default:
      break;
    }
    break;
  }
  case ISD::FADD: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_FADD_f32(N);
    case MVT::f64:
      return Select_ISD_FADD_f64(N);
    case MVT::f80:
      return Select_ISD_FADD_f80(N);
    case MVT::v4f32:
      return Select_ISD_FADD_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_FADD_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::FCOS: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_FCOS_f32(N);
    case MVT::f64:
      return Select_ISD_FCOS_f64(N);
    case MVT::f80:
      return Select_ISD_FCOS_f80(N);
    default:
      break;
    }
    break;
  }
  case ISD::FDIV: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_FDIV_f32(N);
    case MVT::f64:
      return Select_ISD_FDIV_f64(N);
    case MVT::f80:
      return Select_ISD_FDIV_f80(N);
    case MVT::v4f32:
      return Select_ISD_FDIV_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_FDIV_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::FMUL: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_FMUL_f32(N);
    case MVT::f64:
      return Select_ISD_FMUL_f64(N);
    case MVT::f80:
      return Select_ISD_FMUL_f80(N);
    case MVT::v4f32:
      return Select_ISD_FMUL_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_FMUL_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::FNEG: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_FNEG_f32(N);
    case MVT::f64:
      return Select_ISD_FNEG_f64(N);
    case MVT::f80:
      return Select_ISD_FNEG_f80(N);
    default:
      break;
    }
    break;
  }
  case ISD::FP_EXTEND: {
    switch (NVT) {
    case MVT::f64:
      return Select_ISD_FP_EXTEND_f64(N);
    case MVT::f80:
      return Select_ISD_FP_EXTEND_f80(N);
    default:
      break;
    }
    break;
  }
  case ISD::FP_ROUND: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_FP_ROUND_f32(N);
    case MVT::f64:
      return Select_ISD_FP_ROUND_f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::FP_TO_SINT: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_FP_TO_SINT_i32(N);
    case MVT::i64:
      return Select_ISD_FP_TO_SINT_i64(N);
    case MVT::v4i32:
      return Select_ISD_FP_TO_SINT_v4i32(N);
    default:
      break;
    }
    break;
  }
  case ISD::FSIN: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_FSIN_f32(N);
    case MVT::f64:
      return Select_ISD_FSIN_f64(N);
    case MVT::f80:
      return Select_ISD_FSIN_f80(N);
    default:
      break;
    }
    break;
  }
  case ISD::FSQRT: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_FSQRT_f32(N);
    case MVT::f64:
      return Select_ISD_FSQRT_f64(N);
    case MVT::f80:
      return Select_ISD_FSQRT_f80(N);
    case MVT::v4f32:
      return Select_ISD_FSQRT_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_FSQRT_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::FSUB: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_FSUB_f32(N);
    case MVT::f64:
      return Select_ISD_FSUB_f64(N);
    case MVT::f80:
      return Select_ISD_FSUB_f80(N);
    case MVT::v4f32:
      return Select_ISD_FSUB_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_FSUB_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::FrameIndex: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_FrameIndex_i32(N);
    case MVT::i64:
      return Select_ISD_FrameIndex_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::INSERT_VECTOR_ELT: {
    switch (NVT) {
    case MVT::v4i32:
      return Select_ISD_INSERT_VECTOR_ELT_v4i32(N);
    case MVT::v2i64:
      return Select_ISD_INSERT_VECTOR_ELT_v2i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::INTRINSIC_VOID: {
    switch (NVT) {
    default:
      return Select_ISD_INTRINSIC_VOID(N);
      break;
    }
    break;
  }
  case ISD::INTRINSIC_WO_CHAIN: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_INTRINSIC_WO_CHAIN_i32(N);
    case MVT::i64:
      return Select_ISD_INTRINSIC_WO_CHAIN_i64(N);
    case MVT::v8i8:
      return Select_ISD_INTRINSIC_WO_CHAIN_v8i8(N);
    case MVT::v4i16:
      return Select_ISD_INTRINSIC_WO_CHAIN_v4i16(N);
    case MVT::v2i32:
      return Select_ISD_INTRINSIC_WO_CHAIN_v2i32(N);
    case MVT::v1i64:
      return Select_ISD_INTRINSIC_WO_CHAIN_v1i64(N);
    case MVT::v16i8:
      return Select_ISD_INTRINSIC_WO_CHAIN_v16i8(N);
    case MVT::v8i16:
      return Select_ISD_INTRINSIC_WO_CHAIN_v8i16(N);
    case MVT::v4i32:
      return Select_ISD_INTRINSIC_WO_CHAIN_v4i32(N);
    case MVT::v2i64:
      return Select_ISD_INTRINSIC_WO_CHAIN_v2i64(N);
    case MVT::v4f32:
      return Select_ISD_INTRINSIC_WO_CHAIN_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_INTRINSIC_WO_CHAIN_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::INTRINSIC_W_CHAIN: {
    switch (NVT) {
    case MVT::v16i8:
      return Select_ISD_INTRINSIC_W_CHAIN_v16i8(N);
    case MVT::v8i16:
      return Select_ISD_INTRINSIC_W_CHAIN_v8i16(N);
    case MVT::v4i32:
      return Select_ISD_INTRINSIC_W_CHAIN_v4i32(N);
    case MVT::v2i64:
      return Select_ISD_INTRINSIC_W_CHAIN_v2i64(N);
    case MVT::v4f32:
      return Select_ISD_INTRINSIC_W_CHAIN_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_INTRINSIC_W_CHAIN_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::LOAD: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_LOAD_i8(N);
    case MVT::i16:
      return Select_ISD_LOAD_i16(N);
    case MVT::i32:
      return Select_ISD_LOAD_i32(N);
    case MVT::i64:
      return Select_ISD_LOAD_i64(N);
    case MVT::f32:
      return Select_ISD_LOAD_f32(N);
    case MVT::f64:
      return Select_ISD_LOAD_f64(N);
    case MVT::f80:
      return Select_ISD_LOAD_f80(N);
    case MVT::v1i64:
      return Select_ISD_LOAD_v1i64(N);
    case MVT::v4i32:
      return Select_ISD_LOAD_v4i32(N);
    case MVT::v2i64:
      return Select_ISD_LOAD_v2i64(N);
    case MVT::v4f32:
      return Select_ISD_LOAD_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_LOAD_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::MEMBARRIER: {
    switch (NVT) {
    default:
      return Select_ISD_MEMBARRIER(N);
      break;
    }
    break;
  }
  case ISD::MUL: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_MUL_i8(N);
    case MVT::i16:
      return Select_ISD_MUL_i16(N);
    case MVT::i32:
      return Select_ISD_MUL_i32(N);
    case MVT::i64:
      return Select_ISD_MUL_i64(N);
    case MVT::v4i16:
      return Select_ISD_MUL_v4i16(N);
    case MVT::v8i16:
      return Select_ISD_MUL_v8i16(N);
    case MVT::v4i32:
      return Select_ISD_MUL_v4i32(N);
    default:
      break;
    }
    break;
  }
  case ISD::OR: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_OR_i8(N);
    case MVT::i16:
      return Select_ISD_OR_i16(N);
    case MVT::i32:
      return Select_ISD_OR_i32(N);
    case MVT::i64:
      return Select_ISD_OR_i64(N);
    case MVT::v1i64:
      return Select_ISD_OR_v1i64(N);
    case MVT::v2i64:
      return Select_ISD_OR_v2i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::PREFETCH: {
    switch (NVT) {
    default:
      return Select_ISD_PREFETCH(N);
      break;
    }
    break;
  }
  case ISD::ROTL: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_ROTL_i8(N);
    case MVT::i16:
      return Select_ISD_ROTL_i16(N);
    case MVT::i32:
      return Select_ISD_ROTL_i32(N);
    case MVT::i64:
      return Select_ISD_ROTL_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::ROTR: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_ROTR_i8(N);
    case MVT::i16:
      return Select_ISD_ROTR_i16(N);
    case MVT::i32:
      return Select_ISD_ROTR_i32(N);
    case MVT::i64:
      return Select_ISD_ROTR_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::SCALAR_TO_VECTOR: {
    switch (NVT) {
    case MVT::v2i32:
      return Select_ISD_SCALAR_TO_VECTOR_v2i32(N);
    case MVT::v4i32:
      return Select_ISD_SCALAR_TO_VECTOR_v4i32(N);
    case MVT::v2i64:
      return Select_ISD_SCALAR_TO_VECTOR_v2i64(N);
    case MVT::v4f32:
      return Select_ISD_SCALAR_TO_VECTOR_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_SCALAR_TO_VECTOR_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::SHL: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_SHL_i8(N);
    case MVT::i16:
      return Select_ISD_SHL_i16(N);
    case MVT::i32:
      return Select_ISD_SHL_i32(N);
    case MVT::i64:
      return Select_ISD_SHL_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::SIGN_EXTEND: {
    switch (NVT) {
    case MVT::i16:
      return Select_ISD_SIGN_EXTEND_i16(N);
    case MVT::i32:
      return Select_ISD_SIGN_EXTEND_i32(N);
    case MVT::i64:
      return Select_ISD_SIGN_EXTEND_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::SINT_TO_FP: {
    switch (NVT) {
    case MVT::f32:
      return Select_ISD_SINT_TO_FP_f32(N);
    case MVT::f64:
      return Select_ISD_SINT_TO_FP_f64(N);
    case MVT::v4f32:
      return Select_ISD_SINT_TO_FP_v4f32(N);
    default:
      break;
    }
    break;
  }
  case ISD::SRA: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_SRA_i8(N);
    case MVT::i16:
      return Select_ISD_SRA_i16(N);
    case MVT::i32:
      return Select_ISD_SRA_i32(N);
    case MVT::i64:
      return Select_ISD_SRA_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::SRL: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_SRL_i8(N);
    case MVT::i16:
      return Select_ISD_SRL_i16(N);
    case MVT::i32:
      return Select_ISD_SRL_i32(N);
    case MVT::i64:
      return Select_ISD_SRL_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::STORE: {
    switch (NVT) {
    default:
      return Select_ISD_STORE(N);
      break;
    }
    break;
  }
  case ISD::SUB: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_SUB_i8(N);
    case MVT::i16:
      return Select_ISD_SUB_i16(N);
    case MVT::i32:
      return Select_ISD_SUB_i32(N);
    case MVT::i64:
      return Select_ISD_SUB_i64(N);
    case MVT::v8i8:
      return Select_ISD_SUB_v8i8(N);
    case MVT::v4i16:
      return Select_ISD_SUB_v4i16(N);
    case MVT::v2i32:
      return Select_ISD_SUB_v2i32(N);
    case MVT::v1i64:
      return Select_ISD_SUB_v1i64(N);
    case MVT::v16i8:
      return Select_ISD_SUB_v16i8(N);
    case MVT::v8i16:
      return Select_ISD_SUB_v8i16(N);
    case MVT::v4i32:
      return Select_ISD_SUB_v4i32(N);
    case MVT::v2i64:
      return Select_ISD_SUB_v2i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::SUBC: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_SUBC_i32(N);
    case MVT::i64:
      return Select_ISD_SUBC_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::SUBE: {
    switch (NVT) {
    case MVT::i32:
      return Select_ISD_SUBE_i32(N);
    case MVT::i64:
      return Select_ISD_SUBE_i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::TRAP: {
    switch (NVT) {
    default:
      return Select_ISD_TRAP(N);
      break;
    }
    break;
  }
  case ISD::VECTOR_SHUFFLE: {
    switch (NVT) {
    case MVT::v8i8:
      return Select_ISD_VECTOR_SHUFFLE_v8i8(N);
    case MVT::v4i16:
      return Select_ISD_VECTOR_SHUFFLE_v4i16(N);
    case MVT::v2i32:
      return Select_ISD_VECTOR_SHUFFLE_v2i32(N);
    case MVT::v16i8:
      return Select_ISD_VECTOR_SHUFFLE_v16i8(N);
    case MVT::v8i16:
      return Select_ISD_VECTOR_SHUFFLE_v8i16(N);
    case MVT::v4i32:
      return Select_ISD_VECTOR_SHUFFLE_v4i32(N);
    case MVT::v2i64:
      return Select_ISD_VECTOR_SHUFFLE_v2i64(N);
    case MVT::v4f32:
      return Select_ISD_VECTOR_SHUFFLE_v4f32(N);
    case MVT::v2f64:
      return Select_ISD_VECTOR_SHUFFLE_v2f64(N);
    default:
      break;
    }
    break;
  }
  case ISD::XOR: {
    switch (NVT) {
    case MVT::i8:
      return Select_ISD_XOR_i8(N);
    case MVT::i16:
      return Select_ISD_XOR_i16(N);
    case MVT::i32:
      return Select_ISD_XOR_i32(N);
    case MVT::i64:
      return Select_ISD_XOR_i64(N);
    case MVT::v1i64:
      return Select_ISD_XOR_v1i64(N);
    case MVT::v2i64:
      return Select_ISD_XOR_v2i64(N);
    default:
      break;
    }
    break;
  }
  case ISD::ZERO_EXTEND: {
    switch (NVT) {
    case MVT::i16:
      return Select_ISD_ZERO_EXTEND_i16(N);
    case MVT::i32:
      return Select_ISD_ZERO_EXTEND_i32(N);
    case MVT::i64:
      return Select_ISD_ZERO_EXTEND_i64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::BRCOND: {
    switch (NVT) {
    default:
      return Select_X86ISD_BRCOND(N);
      break;
    }
    break;
  }
  case X86ISD::BSF: {
    switch (NVT) {
    case MVT::i16:
      return Select_X86ISD_BSF_i16(N);
    case MVT::i32:
      return Select_X86ISD_BSF_i32(N);
    case MVT::i64:
      return Select_X86ISD_BSF_i64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::BSR: {
    switch (NVT) {
    case MVT::i16:
      return Select_X86ISD_BSR_i16(N);
    case MVT::i32:
      return Select_X86ISD_BSR_i32(N);
    case MVT::i64:
      return Select_X86ISD_BSR_i64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::CALL: {
    switch (NVT) {
    default:
      return Select_X86ISD_CALL(N);
      break;
    }
    break;
  }
  case X86ISD::CMOV: {
    switch (NVT) {
    case MVT::i16:
      return Select_X86ISD_CMOV_i16(N);
    case MVT::i32:
      return Select_X86ISD_CMOV_i32(N);
    case MVT::i64:
      return Select_X86ISD_CMOV_i64(N);
    case MVT::f32:
      return Select_X86ISD_CMOV_f32(N);
    case MVT::f64:
      return Select_X86ISD_CMOV_f64(N);
    case MVT::f80:
      return Select_X86ISD_CMOV_f80(N);
    case MVT::v2i64:
      return Select_X86ISD_CMOV_v2i64(N);
    case MVT::v4f32:
      return Select_X86ISD_CMOV_v4f32(N);
    case MVT::v2f64:
      return Select_X86ISD_CMOV_v2f64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::CMP: {
    switch (NVT) {
    default:
      return Select_X86ISD_CMP(N);
      break;
    }
    break;
  }
  case X86ISD::COMI: {
    switch (NVT) {
    default:
      return Select_X86ISD_COMI(N);
      break;
    }
    break;
  }
  case X86ISD::EH_RETURN: {
    switch (NVT) {
    default:
      return Select_X86ISD_EH_RETURN(N);
      break;
    }
    break;
  }
  case X86ISD::FAND: {
    switch (NVT) {
    case MVT::f32:
      return Select_X86ISD_FAND_f32(N);
    case MVT::f64:
      return Select_X86ISD_FAND_f64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::FILD: {
    switch (NVT) {
    case MVT::f32:
      return Select_X86ISD_FILD_f32(N);
    case MVT::f64:
      return Select_X86ISD_FILD_f64(N);
    case MVT::f80:
      return Select_X86ISD_FILD_f80(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::FILD_FLAG: {
    switch (NVT) {
    case MVT::f64:
      return Select_X86ISD_FILD_FLAG_f64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::FLD: {
    switch (NVT) {
    case MVT::f32:
      return Select_X86ISD_FLD_f32(N);
    case MVT::f64:
      return Select_X86ISD_FLD_f64(N);
    case MVT::f80:
      return Select_X86ISD_FLD_f80(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::FMAX: {
    switch (NVT) {
    case MVT::f32:
      return Select_X86ISD_FMAX_f32(N);
    case MVT::f64:
      return Select_X86ISD_FMAX_f64(N);
    case MVT::v4f32:
      return Select_X86ISD_FMAX_v4f32(N);
    case MVT::v2f64:
      return Select_X86ISD_FMAX_v2f64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::FMIN: {
    switch (NVT) {
    case MVT::f32:
      return Select_X86ISD_FMIN_f32(N);
    case MVT::f64:
      return Select_X86ISD_FMIN_f64(N);
    case MVT::v4f32:
      return Select_X86ISD_FMIN_v4f32(N);
    case MVT::v2f64:
      return Select_X86ISD_FMIN_v2f64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::FNSTCW16m: {
    switch (NVT) {
    default:
      return Select_X86ISD_FNSTCW16m(N);
      break;
    }
    break;
  }
  case X86ISD::FOR: {
    switch (NVT) {
    case MVT::f32:
      return Select_X86ISD_FOR_f32(N);
    case MVT::f64:
      return Select_X86ISD_FOR_f64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::FP_TO_INT16_IN_MEM: {
    switch (NVT) {
    default:
      return Select_X86ISD_FP_TO_INT16_IN_MEM(N);
      break;
    }
    break;
  }
  case X86ISD::FP_TO_INT32_IN_MEM: {
    switch (NVT) {
    default:
      return Select_X86ISD_FP_TO_INT32_IN_MEM(N);
      break;
    }
    break;
  }
  case X86ISD::FP_TO_INT64_IN_MEM: {
    switch (NVT) {
    default:
      return Select_X86ISD_FP_TO_INT64_IN_MEM(N);
      break;
    }
    break;
  }
  case X86ISD::FRCP: {
    switch (NVT) {
    case MVT::f32:
      return Select_X86ISD_FRCP_f32(N);
    case MVT::v4f32:
      return Select_X86ISD_FRCP_v4f32(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::FRSQRT: {
    switch (NVT) {
    case MVT::f32:
      return Select_X86ISD_FRSQRT_f32(N);
    case MVT::v4f32:
      return Select_X86ISD_FRSQRT_v4f32(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::FSRL: {
    switch (NVT) {
    case MVT::v2f64:
      return Select_X86ISD_FSRL_v2f64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::FST: {
    switch (NVT) {
    default:
      return Select_X86ISD_FST(N);
      break;
    }
    break;
  }
  case X86ISD::FXOR: {
    switch (NVT) {
    case MVT::f32:
      return Select_X86ISD_FXOR_f32(N);
    case MVT::f64:
      return Select_X86ISD_FXOR_f64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::INSERTPS: {
    switch (NVT) {
    case MVT::v4f32:
      return Select_X86ISD_INSERTPS_v4f32(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::LCMPXCHG8_DAG: {
    switch (NVT) {
    default:
      return Select_X86ISD_LCMPXCHG8_DAG(N);
      break;
    }
    break;
  }
  case X86ISD::LCMPXCHG_DAG: {
    switch (NVT) {
    default:
      return Select_X86ISD_LCMPXCHG_DAG(N);
      break;
    }
    break;
  }
  case X86ISD::PEXTRB: {
    switch (NVT) {
    case MVT::i32:
      return Select_X86ISD_PEXTRB_i32(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::PEXTRW: {
    switch (NVT) {
    case MVT::i32:
      return Select_X86ISD_PEXTRW_i32(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::PINSRB: {
    switch (NVT) {
    case MVT::v16i8:
      return Select_X86ISD_PINSRB_v16i8(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::PINSRW: {
    switch (NVT) {
    case MVT::v4i16:
      return Select_X86ISD_PINSRW_v4i16(N);
    case MVT::v8i16:
      return Select_X86ISD_PINSRW_v8i16(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::RDTSC_DAG: {
    switch (NVT) {
    default:
      return Select_X86ISD_RDTSC_DAG(N);
      break;
    }
    break;
  }
  case X86ISD::REP_MOVS: {
    switch (NVT) {
    default:
      return Select_X86ISD_REP_MOVS(N);
      break;
    }
    break;
  }
  case X86ISD::REP_STOS: {
    switch (NVT) {
    default:
      return Select_X86ISD_REP_STOS(N);
      break;
    }
    break;
  }
  case X86ISD::RET_FLAG: {
    switch (NVT) {
    default:
      return Select_X86ISD_RET_FLAG(N);
      break;
    }
    break;
  }
  case X86ISD::SETCC: {
    switch (NVT) {
    case MVT::i8:
      return Select_X86ISD_SETCC_i8(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::SHLD: {
    switch (NVT) {
    case MVT::i16:
      return Select_X86ISD_SHLD_i16(N);
    case MVT::i32:
      return Select_X86ISD_SHLD_i32(N);
    case MVT::i64:
      return Select_X86ISD_SHLD_i64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::SHRD: {
    switch (NVT) {
    case MVT::i16:
      return Select_X86ISD_SHRD_i16(N);
    case MVT::i32:
      return Select_X86ISD_SHRD_i32(N);
    case MVT::i64:
      return Select_X86ISD_SHRD_i64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::TAILCALL: {
    switch (NVT) {
    default:
      return Select_X86ISD_TAILCALL(N);
      break;
    }
    break;
  }
  case X86ISD::TC_RETURN: {
    switch (NVT) {
    default:
      return Select_X86ISD_TC_RETURN(N);
      break;
    }
    break;
  }
  case X86ISD::THREAD_POINTER: {
    switch (NVT) {
    case MVT::i32:
      return Select_X86ISD_THREAD_POINTER_i32(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::TLSADDR: {
    switch (NVT) {
    case MVT::i32:
      return Select_X86ISD_TLSADDR_i32(N);
    case MVT::i64:
      return Select_X86ISD_TLSADDR_i64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::UCOMI: {
    switch (NVT) {
    default:
      return Select_X86ISD_UCOMI(N);
      break;
    }
    break;
  }
  case X86ISD::VZEXT_LOAD: {
    switch (NVT) {
    case MVT::v2i64:
      return Select_X86ISD_VZEXT_LOAD_v2i64(N);
    case MVT::v2f64:
      return Select_X86ISD_VZEXT_LOAD_v2f64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::VZEXT_MOVL: {
    switch (NVT) {
    case MVT::v8i8:
      return Select_X86ISD_VZEXT_MOVL_v8i8(N);
    case MVT::v4i16:
      return Select_X86ISD_VZEXT_MOVL_v4i16(N);
    case MVT::v2i32:
      return Select_X86ISD_VZEXT_MOVL_v2i32(N);
    case MVT::v4i32:
      return Select_X86ISD_VZEXT_MOVL_v4i32(N);
    case MVT::v2i64:
      return Select_X86ISD_VZEXT_MOVL_v2i64(N);
    case MVT::v4f32:
      return Select_X86ISD_VZEXT_MOVL_v4f32(N);
    case MVT::v2f64:
      return Select_X86ISD_VZEXT_MOVL_v2f64(N);
    default:
      break;
    }
    break;
  }
  case X86ISD::Wrapper: {
    switch (NVT) {
    case MVT::i32:
      return Select_X86ISD_Wrapper_i32(N);
    case MVT::i64:
      return Select_X86ISD_Wrapper_i64(N);
    default:
      break;
    }
    break;
  }
  } // end of big switch.

  cerr << "Cannot yet select: ";
  if (N.getOpcode() != ISD::INTRINSIC_W_CHAIN &&
      N.getOpcode() != ISD::INTRINSIC_WO_CHAIN &&
      N.getOpcode() != ISD::INTRINSIC_VOID) {
    N.Val->dump(CurDAG);
  } else {
    unsigned iid = cast<ConstantSDNode>(N.getOperand(N.getOperand(0).getValueType() == MVT::Other))->getValue();
    cerr << "intrinsic %"<< Intrinsic::getName((Intrinsic::ID)iid);
  }
  cerr << '\n';
  abort();
  return NULL;
}
